@@ -1,4 +1,4 @@
-/* ASSIST.C     (c) Copyright Roger Bowler, 1999-2003                */
+/* ASSIST.C     (c) Copyright Roger Bowler, 1999-2005                */
 /*              ESA/390 MVS Assist Routines                          */
 
 /*-------------------------------------------------------------------*/
@@ -8,11 +8,21 @@
 
 /*              Instruction decode rework - Jan Jaeger               */
 /*              Correct address wraparound - Jan Jaeger              */
-/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2003      */
+/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2005      */
 /*              Add dummy assist instruction - Jay Maynard,          */
 /*                  suggested by Brandon Hill                        */
 
 
+#include "hstdinc.h"
+
+#if !defined(_HENGINE_DLL_)
+#define _HENGINE_DLL_
+#endif
+
+#if !defined(_ASSIST_C_)
+#define _ASSIST_C_
+#endif
+
 #include "hercules.h"
 
 #include "opcode.h"
@@ -58,7 +68,7 @@ int     b1, b2;                         /* Values of base field      */
 VADR    effective_addr1,
         effective_addr2;                /* Effective addresses       */
 
-    SSE(inst, execflag, regs, b1, effective_addr1, b2, effective_addr2);
+    SSE(inst, regs, b1, effective_addr1, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -77,7 +87,7 @@ int     b1, b2;                         /* Values of base field      */
 VADR    effective_addr1,
         effective_addr2;                /* Effective addresses       */
 
-    SSE(inst, execflag, regs, b1, effective_addr1, b2, effective_addr2);
+    SSE(inst, regs, b1, effective_addr1, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -105,7 +115,7 @@ U32     lcpa;                           /* Logical CPU address       */
 VADR    newia;                          /* Unsuccessful branch addr  */
 int     acc_mode = 0;                   /* access mode to use        */
 
-    SSE(inst, execflag, regs, b1, effective_addr1, b2, effective_addr2);
+    SSE(inst, regs, b1, effective_addr1, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -163,13 +173,14 @@ int     acc_mode = 0;                   /* access mode to use        */
         newia = ARCH_DEP(vfetch4) ( lit_addr, acc_mode, regs );
 
         /* Save the link information in register 12 */
-        regs->GR_L(12) = regs->psw.IA;
+        regs->GR_L(12) = regs->psw.IA & ADDRESS_MAXWRAP(regs);
 
         /* Copy LITOLOC into register 13 to signify obtain failure */
         regs->GR_L(13) = newia;
 
         /* Update the PSW instruction address */
         regs->psw.IA = newia & ADDRESS_MAXWRAP(regs);
+        VALIDATE_AIA(regs);
     }
 
     /* Release main-storage access lock */
@@ -200,7 +211,7 @@ U32     lcpa;                           /* Logical CPU address       */
 VADR    newia;                          /* Unsuccessful branch addr  */
 int     acc_mode = 0;                   /* access mode to use        */
 
-    SSE(inst, execflag, regs, b1, effective_addr1, b2, effective_addr2);
+    SSE(inst, regs, b1, effective_addr1, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -259,13 +270,14 @@ int     acc_mode = 0;                   /* access mode to use        */
         newia = ARCH_DEP(vfetch4) ( lit_addr, acc_mode, regs );
 
         /* Save the link information in register 12 */
-        regs->GR_L(12) = regs->psw.IA;
+        regs->GR_L(12) = regs->psw.IA & ADDRESS_MAXWRAP(regs);
 
         /* Copy LITRLOC into register 13 to signify release failure */
         regs->GR_L(13) = newia;
 
         /* Update the PSW instruction address */
         regs->psw.IA = newia & ADDRESS_MAXWRAP(regs);
+        VALIDATE_AIA(regs);
     }
 
     /* Release main-storage access lock */
@@ -292,7 +304,7 @@ U32     lock;                           /* Lock value                */
 VADR    newia;                          /* Unsuccessful branch addr  */
 int     acc_mode = 0;                   /* access mode to use        */
 
-    SSE(inst, execflag, regs, b1, effective_addr1, b2, effective_addr2);
+    SSE(inst, regs, b1, effective_addr1, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -350,13 +362,14 @@ int     acc_mode = 0;                   /* access mode to use        */
         newia = ARCH_DEP(vfetch4) ( lit_addr, acc_mode, regs );
 
         /* Save the link information in register 12 */
-        regs->GR_L(12) = regs->psw.IA;
+        regs->GR_L(12) = regs->psw.IA & ADDRESS_MAXWRAP(regs);
 
         /* Copy LITOCMS into register 13 to signify obtain failure */
         regs->GR_L(13) = newia;
 
         /* Update the PSW instruction address */
         regs->psw.IA = newia & ADDRESS_MAXWRAP(regs);
+        VALIDATE_AIA(regs);
     }
 
     /* Release main-storage access lock */
@@ -386,7 +399,7 @@ U32     susp;                           /* Lock suspend queue        */
 VADR    newia;                          /* Unsuccessful branch addr  */
 int     acc_mode = 0;                   /* access mode to use        */
 
-    SSE(inst, execflag, regs, b1, effective_addr1, b2, effective_addr2);
+    SSE(inst, regs, b1, effective_addr1, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -444,13 +457,14 @@ int     acc_mode = 0;                   /* access mode to use        */
         newia = ARCH_DEP(vfetch4) ( lit_addr, acc_mode, regs );
 
         /* Save the link information in register 12 */
-        regs->GR_L(12) = regs->psw.IA;
+        regs->GR_L(12) = regs->psw.IA & ADDRESS_MAXWRAP(regs);
 
         /* Copy LITRCMS into register 13 to signify release failure */
         regs->GR_L(13) = newia;
 
         /* Update the PSW instruction address */
         regs->psw.IA = newia & ADDRESS_MAXWRAP(regs);
+        VALIDATE_AIA(regs);
     }
 
     /* Release main-storage access lock */
@@ -469,7 +483,7 @@ int     b1, b2;                         /* Values of base field      */
 VADR    effective_addr1,
         effective_addr2;                /* Effective addresses       */
 
-    SSE(inst, execflag, regs, b1, effective_addr1, b2, effective_addr2);
+    SSE(inst, regs, b1, effective_addr1, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -491,7 +505,7 @@ int     b1, b2;                         /* Values of base field      */
 VADR    effective_addr1,
         effective_addr2;                /* Effective addresses       */
 
-    SSE(inst, execflag, regs, b1, effective_addr1, b2, effective_addr2);
+    SSE(inst, regs, b1, effective_addr1, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -513,7 +527,7 @@ int     b1, b2;                         /* Values of base field      */
 VADR    effective_addr1,
         effective_addr2;                /* Effective addresses       */
 
-    SSE(inst, execflag, regs, b1, effective_addr1, b2, effective_addr2);
+    SSE(inst, regs, b1, effective_addr1, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -535,7 +549,7 @@ int     b1, b2;                         /* Values of base field      */
 VADR    effective_addr1,
         effective_addr2;                /* Effective addresses       */
 
-    SSE(inst, execflag, regs, b1, effective_addr1, b2, effective_addr2);
+    SSE(inst, regs, b1, effective_addr1, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -557,7 +571,7 @@ int     b1, b2;                         /* Values of base field      */
 VADR    effective_addr1,
         effective_addr2;                /* Effective addresses       */
 
-    SSE(inst, execflag, regs, b1, effective_addr1, b2, effective_addr2);
+    SSE(inst, regs, b1, effective_addr1, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -571,7 +585,7 @@ VADR    effective_addr1,
 
 
 /*-------------------------------------------------------------------*/
-/* E50C       - Trace SVC Return                               [SSE] */
+/* E50D       - Trace SVC Return                               [SSE] */
 /*-------------------------------------------------------------------*/
 DEF_INST(trace_svc_return)
 {
@@ -579,7 +593,7 @@ int     b1, b2;                         /* Values of base field      */
 VADR    effective_addr1,
         effective_addr2;                /* Effective addresses       */
 
-    SSE(inst, execflag, regs, b1, effective_addr1, b2, effective_addr2);
+    SSE(inst, regs, b1, effective_addr1, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
