@@ -1,4 +1,4 @@
-/* CARDRDR.C    (c) Copyright Roger Bowler, 1999-2003                */
+/* CARDRDR.C    (c) Copyright Roger Bowler, 1999-2005                */
 /*              ESA/390 Card Reader Device Handler                   */
 
 /*-------------------------------------------------------------------*/
@@ -6,10 +6,28 @@
 /* card reader devices.                                              */
 /*-------------------------------------------------------------------*/
 
+#include "hstdinc.h"
 #include "hercules.h"
 
 #include "devtype.h"
 
+#include "sockdev.h"
+
+
+#if defined(WIN32) && defined(OPTION_DYNAMIC_LOAD) && !defined(HDL_USE_LIBTOOL) && !defined(_MSVC_)
+ SYSBLK *psysblk;
+ #define sysblk (*psysblk)
+#endif
+
+
+/*-------------------------------------------------------------------*/
+/* ISW 2003/03/07                                                    */
+/* 3505 Byte 1 Sense Codes                                           */
+/*-------------------------------------------------------------------*/
+#define SENSE1_RDR_PERM         0x80 /* Permanent Err key depressed  */
+#define SENSE1_RDR_AUTORETRY    0x40 /* Don't know                   */
+#define SENSE1_RDR_MOTIONMF     0x20 /* Motion Malfunction           */
+#define SENSE1_RDR_RAIC         0x10 /* Retry After Intreq Cleared   */
 /*-------------------------------------------------------------------*/
 /* Internal macro definitions                                        */
 /*-------------------------------------------------------------------*/
@@ -19,7 +37,7 @@
 /*-------------------------------------------------------------------*/
 /* Initialize the device handler                                     */
 /*-------------------------------------------------------------------*/
-static int cardrdr_init_handler ( DEVBLK *dev, int argc, BYTE *argv[] )
+static int cardrdr_init_handler ( DEVBLK *dev, int argc, char *argv[] )
 {
 int     i;                              /* Array subscript           */
 int     fc;                             /* File counter              */
@@ -47,6 +65,9 @@ int     fc;                             /* File counter              */
     dev->cardrem = 0;
     dev->autopad = 0;
 
+    if(!sscanf(dev->typname,"%hx",&(dev->devtype)))
+        dev->devtype = 0x2501;
+
     fc = 0;
 
     if (dev->more_files) free (dev->more_files);
@@ -263,7 +284,9 @@ int     fc;                             /* File counter              */
 
     /* Set number of sense bytes */
 
-    dev->numsense = 1;
+    /* ISW 20030307 : Empirical knowledge : DOS/VS R34 Erep */
+    /*                indicates 4 bytes in 3505 sense       */
+    dev->numsense = 4;
 
     /* Initialize the device identifier bytes */
 
@@ -291,8 +314,8 @@ int     fc;                             /* File counter              */
 /*-------------------------------------------------------------------*/
 /* Query the device definition                                       */
 /*-------------------------------------------------------------------*/
-static void cardrdr_query_device (DEVBLK *dev, BYTE **class,
-                int buflen, BYTE *buffer)
+static void cardrdr_query_device (DEVBLK *dev, char **class,
+                int buflen, char *buffer)
 {
     *class = "RDR";
 
@@ -315,7 +338,10 @@ static int cardrdr_close_device ( DEVBLK *dev )
 {
     /* Close the device file */
 
-    if (dev->fh && fclose(dev->fh) != 0)
+    if (0
+        || (  dev->bs && close_socket( dev->fd ) < 0 )
+        || ( !dev->bs && dev->fh && fclose(dev->fh) != 0 )
+    )
     {
         logmsg (_("HHCRD011E Close error on file \"%s\": %s\n"),
             dev->filename, strerror(errno));
@@ -361,7 +387,7 @@ static int clear_cardrdr ( DEVBLK *dev )
         dev->multifile = 0;
         dev->ascii = 0;
         dev->ebcdic = 0;
-//        dev->rdreof = 0;
+//      dev->rdreof = 0;
         dev->trunc = 0;
         dev->autopad = 0;
     }
@@ -379,6 +405,7 @@ int     rc;                             /* Return code               */
 int     i;                              /* Array subscript           */
 int     len;                            /* Length of data            */
 BYTE    buf[160];                       /* Auto-detection buffer     */
+BYTE    pathname[MAX_PATH];             /* file path in host format  */
 
     *unitstat = 0;
 
@@ -390,68 +417,37 @@ BYTE    buf[160];                       /* Auto-detection buffer     */
 
         if (dev->fd == -1)
         {
+            if(dev->rdreof)
+            {
+                *unitstat=CSW_CE|CSW_DE|CSW_UX;
+                return -1;
+            }
             dev->sense[0] = SENSE_IR;
+            dev->sense[1] = SENSE1_RDR_RAIC; /* Retry when IntReq Cleared */
             *unitstat = CSW_CE | CSW_DE | CSW_UC;
             return -1;
         }
 
-        if (!dev->fh)
-        {
-            /*  GNU 'C' Library documentation for "fdopen":
-
-                  "The opentype argument is interpreted in the same
-                   way as for the fopen function (see section Opening
-                   Streams), except that the 'b' option is not permitted;
-                   this is because GNU makes no distinction between text
-                   and binary files. [...] You must make sure that the
-                   opentype argument matches the actual mode of the open
-                   file descriptor."
-
-                GNU 'C' Library documentation for "fopen":
-
-                  "The character 'b' in opentype has a standard meaning;
-                   it requests a binary stream rather than a text stream.
-                   But this makes no difference in POSIX systems (including
-                   the GNU system). If both `+' and 'b' are specified,
-                   they can appear in either order. See section Text and
-                   Binary Streams."
-
-                GNU 'C' Library documentation for "Text and Binary Streams":
-
-                  "When you open a stream, you can specify either a text
-                   stream or a binary stream. You indicate that you want
-                   a binary stream by specifying the 'b' modifier in the
-                   opentype argument to fopen; see section Opening Streams.
-                   Without this option, fopen opens the file as a text stream."
-
-                Note that even though it clearly states (for fdopen) that the
-                'b' option "is NOT permitted", we are assuming such is not true
-                since it accepts it (but simply ignores it) for fopen. Further-
-                more, since Windows (Cygwin?) DOES make a distinction between
-                binary and text files, we should specify the 'b' option in our
-                call to fdopen. (It has already been verified that this is safe
-                to do on Linux systems, so to play it safe (i.e. to prevent any
-                potential problems on Windows systems), we always specify 'b'.)
-            */
-
-            dev->fh = fdopen(dev->fd, "rb");
-        }
-
-        ASSERT(dev->fd != -1 && dev->fh);
-
         return 0;
     }
 
     /* Intervention required if device has no file name */
     if (dev->filename[0] == '\0')
     {
+        if(dev->rdreof)
+        {
+            *unitstat=CSW_CE|CSW_DE|CSW_UX;
+            return -1;
+        }
         dev->sense[0] = SENSE_IR;
+        dev->sense[1] = SENSE1_RDR_RAIC; /* Retry when IntReq Cleared */
         *unitstat = CSW_CE | CSW_DE | CSW_UC;
         return -1;
     }
 
     /* Open the device file */
-    rc = open (dev->filename, O_RDONLY | O_BINARY);
+    hostpath(pathname, dev->filename, sizeof(pathname));
+    rc = open (pathname, O_RDONLY | O_BINARY);
     if (rc < 0)
     {
         /* Handle open failure */
@@ -466,9 +462,7 @@ BYTE    buf[160];                       /* Auto-detection buffer     */
 
     /* Save the file descriptor in the device block */
     dev->fd = rc;
-    dev->fh = fdopen(dev->fd, "rb");    /* NOTE: see comments in
-                                           function "open_cardrdr"
-                                           regarding fdopen. */
+    dev->fh = fdopen(dev->fd, "rb");
 
     /* If neither EBCDIC nor ASCII was specified, attempt to
        detect the format by inspecting the first 160 bytes */
@@ -541,38 +535,44 @@ static int read_ebcdic ( DEVBLK *dev, BYTE *unitstat )
 int     rc;                             /* Return code               */
 
     /* Read 80 bytes of card image data into the device buffer */
-    rc = fread(dev->buf, 1, CARD_SIZE, dev->fh);
+    if (dev->bs)
+        rc = read_socket( dev->fd, dev->buf, CARD_SIZE );
+    else
+        rc = fread(dev->buf, 1, CARD_SIZE, dev->fh);
 
     if ((rc > 0) && (rc < CARD_SIZE) && dev->autopad)
     {
         memset(&dev->buf[rc], 0, CARD_SIZE - rc);
         rc = CARD_SIZE;
     }
-    else if (feof(dev->fh)) /* End of file */
+    else if /* Check for End of file */
+    (0
+        || ( dev->bs && rc <= 0)
+        || (!dev->bs && feof(dev->fh))
+    )
     {
-
-      /* Return unit exception or intervention required */
-      if (dev->rdreof)
+        /* Return unit exception or intervention required */
+        if (dev->rdreof)
         {
-          *unitstat = CSW_CE | CSW_DE | CSW_UX;
+            *unitstat = CSW_CE | CSW_DE | CSW_UX;
         }
-      else
+        else
         {
-          dev->sense[0] = SENSE_IR;
-          *unitstat = CSW_CE | CSW_DE | CSW_UC;
+            dev->sense[0] = SENSE_IR;
+            dev->sense[1] = SENSE1_RDR_RAIC; /* Retry when IntReq Cleared */
+            *unitstat = CSW_CE | CSW_DE | CSW_UC;
         }
 
-      /* Close the file and clear the file name and flags */
-
-      if (clear_cardrdr(dev) != 0)
+        /* Close the file and clear the file name and flags */
+        if (clear_cardrdr(dev) != 0)
         {
-          /* Set unit check with equipment check */
-          dev->sense[0] = SENSE_EC;
-          *unitstat = CSW_CE | CSW_DE | CSW_UC;
-          return -1;
+            /* Set unit check with equipment check */
+            dev->sense[0] = SENSE_EC;
+            *unitstat = CSW_CE | CSW_DE | CSW_UC;
+            return -1;
         }
 
-      return -2;
+        return -2;
     }
 
     /* Handle read error condition */
@@ -602,7 +602,7 @@ static int read_ascii ( DEVBLK *dev, BYTE *unitstat )
 {
 int     rc;                             /* Return code               */
 int     i;                              /* Array subscript           */
-BYTE    c;                              /* Input character           */
+BYTE    c = 0;                          /* Input character           */
 
     /* Prefill the card image with EBCDIC blanks */
     memset (dev->buf, HEX40, CARD_SIZE);
@@ -611,8 +611,16 @@ BYTE    c;                              /* Input character           */
     for (i = 0; ; )
     {
         /* Read next byte of card image */
-        rc = getc(dev->fh);
-        c = (BYTE)rc;
+        if (dev->bs)
+        {
+            BYTE b; rc = read_socket( dev->fd, &b, 1 );
+            if (rc <= 0) rc = EOF; else c = b;
+        }
+        else
+        {
+            rc = getc(dev->fh);
+            c = (BYTE)rc;
+        }
 
         /* Handle end-of-file condition */
         if (rc == EOF || c == '\x1A')
@@ -628,6 +636,7 @@ BYTE    c;                              /* Input character           */
             else
             {
                 dev->sense[0] = SENSE_IR;
+                dev->sense[1] = SENSE1_RDR_RAIC; /* Retry when IntReq Cleared */
                 *unitstat = CSW_CE | CSW_DE | CSW_UC;
             }
 
@@ -707,7 +716,8 @@ int     num;                            /* Number of bytes to move   */
     UNREFERENCED(ccwseq);
 
     /* Open the device file if necessary */
-    if ((dev->fd < 0 || !dev->fh) && !IS_CCW_SENSE(code))
+    if ( !IS_CCW_SENSE(code) &&
+        (dev->fd < 0 || (!dev->bs && !dev->fh)))
     {
         rc = open_cardrdr (dev, unitstat);
         if (rc) return;
@@ -716,6 +726,10 @@ int     num;                            /* Number of bytes to move   */
     /* Turn all read/feed commands into read, feed, select stacker 1 */
     if ((code & 0x17) == 0x02) code = 0x02;
 
+    /* Turn all feed-only commands into NOP. This is ugly, and should
+        really be thought out more. --JRM */
+    if ((code & 0x37) == 0x23) code = 0x03;
+
     /* Process depending on CCW opcode */
     switch (code) {
 
@@ -784,6 +798,18 @@ int     num;                            /* Number of bytes to move   */
         *residual = count - num;
         if (count < dev->numsense) *more = 1;
 
+        /* If sense is clear AND filename = "" OR sockdev and fd=-1 */
+        /* Put an IR sense - so that an unsolicited sense can see the intreq */
+        if(dev->sense[0]==0)
+        {
+            if(dev->filename[0]==0x00 ||
+                    (dev->bs && dev->fd==-1))
+            {
+                dev->sense[0] = SENSE_IR;
+                dev->sense[1] = SENSE1_RDR_RAIC; /* Retry when IntReq Cleared */
+            }
+        }
+
         /* Copy device sense bytes to channel I/O buffer */
         memcpy (iobuf, dev->sense, num);
 
@@ -823,10 +849,66 @@ int     num;                            /* Number of bytes to move   */
 } /* end function cardrdr_execute_ccw */
 
 
+#if defined(OPTION_DYNAMIC_LOAD)
+static
+#endif
 DEVHND cardrdr_device_hndinfo = {
-        &cardrdr_init_handler,
-        &cardrdr_execute_ccw,
-        &cardrdr_close_device,
-        &cardrdr_query_device,
-        NULL, NULL, NULL, NULL
+        &cardrdr_init_handler,         /* Device Initialisation      */
+        &cardrdr_execute_ccw,          /* Device CCW execute         */
+        &cardrdr_close_device,         /* Device Close               */
+        &cardrdr_query_device,         /* Device Query               */
+        NULL,                          /* Device Start channel pgm   */
+        NULL,                          /* Device End channel pgm     */
+        NULL,                          /* Device Resume channel pgm  */
+        NULL,                          /* Device Suspend channel pgm */
+        NULL,                          /* Device Read                */
+        NULL,                          /* Device Write               */
+        NULL,                          /* Device Query used          */
+        NULL,                          /* Device Reserve             */
+        NULL,                          /* Device Release             */
+        NULL,                          /* Immediate CCW Codes        */
+        NULL,                          /* Signal Adapter Input       */
+        NULL,                          /* Signal Adapter Output      */
+        NULL,                          /* Hercules suspend           */
+        NULL                           /* Hercules resume            */
 };
+
+/* Libtool static name colision resolution */
+/* note : lt_dlopen will look for symbol & modulename_LTX_symbol */
+#if !defined(HDL_BUILD_SHARED) && defined(HDL_USE_LIBTOOL)
+#define hdl_ddev hdt3505_LTX_hdl_ddev
+#define hdl_depc hdt3505_LTX_hdl_depc
+#define hdl_reso hdt3505_LTX_hdl_reso
+#define hdl_init hdt3505_LTX_hdl_init
+#define hdl_fini hdt3505_LTX_hdl_fini
+#endif
+
+
+#if defined(OPTION_DYNAMIC_LOAD)
+HDL_DEPENDENCY_SECTION;
+{
+     HDL_DEPENDENCY(HERCULES);
+     HDL_DEPENDENCY(DEVBLK);
+     HDL_DEPENDENCY(SYSBLK);
+}
+END_DEPENDENCY_SECTION;
+
+
+#if defined(WIN32) && !defined(HDL_USE_LIBTOOL) && !defined(_MSVC_)
+  #undef sysblk
+  HDL_RESOLVER_SECTION;
+  {
+    HDL_RESOLVE_PTRVAR( psysblk, sysblk );
+  }
+  END_RESOLVER_SECTION;
+#endif
+
+
+HDL_DEVICE_SECTION;
+{
+    HDL_DEVICE(1442, cardrdr_device_hndinfo );
+    HDL_DEVICE(2501, cardrdr_device_hndinfo );
+    HDL_DEVICE(3505, cardrdr_device_hndinfo );
+}
+END_DEVICE_SECTION;
+#endif
