@@ -2,24 +2,53 @@
 // Hercules IP Channel-to-Channel Support (CTCI)
 // ====================================================================
 //
-// Copyright    (C) Copyright James A. Pierson, 2002-2003
-//              (C) Copyright "Fish" (David B. Trout), 2002-2003
-//              (C) Copyright Roger Bowler, 2000-2003
+// Copyright    (C) Copyright James A. Pierson, 2002-2005
+//              (C) Copyright "Fish" (David B. Trout), 2002-2005
+//              (C) Copyright Roger Bowler, 2000-2005
 //
-// linux 2.4 modifications (c) Copyright Fritz Elfert, 2001-2003
+// linux 2.4 modifications (c) Copyright Fritz Elfert, 2001-2005
 //
 
-#if !defined(__APPLE__)
-
+#include "hstdinc.h"
 #include "hercules.h"
-#include "devtype.h"
 #include "ctcadpt.h"
 #include "tuntap.h"
-
 #include "hercifc.h"
-
 #include "opcode.h"
-#include <getopt.h>
+/* getopt dynamic linking kludge */
+#include "herc_getopt.h"
+
+/*-------------------------------------------------------------------*/
+/* Ivan Warren 20040227                                              */
+/* This table is used by channel.c to determine if a CCW code is an  */
+/* immediate command or not                                          */
+/* The tape is addressed in the DEVHND structure as 'DEVIMM immed'   */
+/* 0 : Command is NOT an immediate command                           */
+/* 1 : Command is an immediate command                               */
+/* Note : An immediate command is defined as a command which returns */
+/* CE (channel end) during initialisation (that is, no data is       */
+/* actually transfered. In this case, IL is not indicated for a CCW  */
+/* Format 0 or for a CCW Format 1 when IL Suppression Mode is in     */
+/* effect                                                            */
+/*-------------------------------------------------------------------*/
+
+static BYTE CTCI_Immed_Commands[256]=
+{ 0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,
+  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+  0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,
+  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
 
 // ====================================================================
 // Declarations
@@ -39,11 +68,24 @@ static int      ParseArgs( DEVBLK* pDEVBLK, PCTCBLK pCTCBLK,
 
 DEVHND ctci_device_hndinfo =
 {
-    &CTCI_Init,
-    &CTCI_ExecuteCCW,
-    &CTCI_Close,
-    &CTCI_Query,
-    NULL, NULL, NULL, NULL
+        &CTCI_Init,                    /* Device Initialisation      */
+        &CTCI_ExecuteCCW,              /* Device CCW execute         */
+        &CTCI_Close,                   /* Device Close               */
+        &CTCI_Query,                   /* Device Query               */
+        NULL,                          /* Device Start channel pgm   */
+        NULL,                          /* Device End channel pgm     */
+        NULL,                          /* Device Resume channel pgm  */
+        NULL,                          /* Device Suspend channel pgm */
+        NULL,                          /* Device Read                */
+        NULL,                          /* Device Write               */
+        NULL,                          /* Device Query used          */
+        NULL,                          /* Device Reserve             */
+        NULL,                          /* Device Release             */
+        CTCI_Immed_Commands,           /* Immediate CCW Codes        */
+        NULL,                          /* Signal Adapter Input       */
+        NULL,                          /* Signal Adapter Output      */
+        NULL,                          /* Hercules suspend           */
+        NULL                           /* Hercules resume            */
 };
 
 // ====================================================================
@@ -54,12 +96,44 @@ DEVHND ctci_device_hndinfo =
 // CTCI_Init
 //
 
-int  CTCI_Init( DEVBLK* pDEVBLK, int argc, BYTE *argv[] )
+#define CTC_DEVICES_IN_GROUP   2  // a read and write device
+
+int  CTCI_Init( DEVBLK* pDEVBLK, int argc, char *argv[] )
 {
     PCTCBLK         pWrkCTCBLK = NULL;  // Working CTCBLK
     PCTCBLK         pDevCTCBLK = NULL;  // Device  CTCBLK
-    DEVBLK*         pDevPair = NULL;    // Paired  DEVBLK
     int             rc = 0;             // Return code
+    int             nIFType;            // Interface type
+    int             nIFFlags;           // Interface flags
+
+    nIFType =               // Interface type
+        0
+        | IFF_TUN           // ("TUN", not "tap")
+        | IFF_NO_PI         // (no packet info)
+        ;
+
+    // ZZ FIXME: Technically, IFF_RUNNING should NOT be set by the user.
+    //           Only the interface itself should set IFF_RUNNING when-
+    //           ever it is successfully created/initialized (i.e. is
+    //           operational). Once it's operational (running), then it
+    //           may be enabled via IFF_UP. If it's not in IFF_RUNNING
+    //           state however, then IFF_UP cannot be set because the
+    //           interface is technically "broken" (not operational),
+    //           and non-operational (non-working) interfaces cannot be
+    //           enabled.  --  Fish, June 2004.
+
+    nIFFlags =              // Interface flags
+        0
+        | IFF_UP            // (interface has been enabled)
+        | IFF_RUNNING       // (interface is operational)
+        | IFF_BROADCAST     // (interface broadcast addr is valid)
+        ;
+
+    pDEVBLK->devtype = 0x3088;
+
+    // CTC is a group device
+    if(!group_device(pDEVBLK, CTC_DEVICES_IN_GROUP))
+        return 0;
 
     // Housekeeping
     pWrkCTCBLK = malloc( sizeof( CTCBLK ) );
@@ -77,144 +151,186 @@ int  CTCI_Init( DEVBLK* pDEVBLK, int argc, BYTE *argv[] )
     if( ParseArgs( pDEVBLK, pWrkCTCBLK, argc, (char**)argv ) != 0 )
     {
         free( pWrkCTCBLK );
+        pWrkCTCBLK = NULL;
         return -1;
     }
 
-    if( !pWrkCTCBLK->fOldFormat )
+    // Allocate the device CTCBLK and copy parsed information.
+
+    pDevCTCBLK = malloc( sizeof( CTCBLK ) );
+
+    if( !pDevCTCBLK )
     {
-        // Allocate the device CTCBLK and copy parsed information.
+        logmsg( _("HHCCT038E %4.4X: Unable to allocate CTCBLK\n"),
+                pDEVBLK->devnum );
+        free( pWrkCTCBLK );
+        pWrkCTCBLK = NULL;
+        return -1;
+    }
 
-        pDevCTCBLK = malloc( sizeof( CTCBLK ) );
+    memcpy( pDevCTCBLK, pWrkCTCBLK, sizeof( CTCBLK ) );
 
-        if( !pDevCTCBLK )
-        {
-            logmsg( _("HHCCT038E %4.4X: Unable to allocate CTCBLK\n"),
-                    pDEVBLK->devnum );
-            free( pWrkCTCBLK );
-            return -1;
-        }
+    // New format has only one device statement for both addresses
+    // We need to dynamically allocate the read device block
 
-        memcpy( pDevCTCBLK, pWrkCTCBLK, sizeof( CTCBLK ) );
+    pDevCTCBLK->pDEVBLK[0] = pDEVBLK->group->memdev[0];
+    pDevCTCBLK->pDEVBLK[1] = pDEVBLK->group->memdev[1];
 
-        // New format has only one device statement for both addresses
-        // We need to dynamically allocate the read device block
+    pDevCTCBLK->pDEVBLK[0]->dev_data = pDevCTCBLK;
+    pDevCTCBLK->pDEVBLK[1]->dev_data = pDevCTCBLK;
 
-        pDevCTCBLK->pDEVBLK[0] = NULL;
-        pDevCTCBLK->pDEVBLK[1] = pDEVBLK;
+    SetSIDInfo( pDevCTCBLK->pDEVBLK[0], 0x3088, 0x08, 0x3088, 0x01 );
+    SetSIDInfo( pDevCTCBLK->pDEVBLK[1], 0x3088, 0x08, 0x3088, 0x01 );
 
-        AddDevice( &pDevCTCBLK->pDEVBLK[0], pDEVBLK->devnum,
-                   "CTCI", &ctci_device_hndinfo );
+    pDevCTCBLK->pDEVBLK[0]->ctctype  = CTC_CTCI;
+    pDevCTCBLK->pDEVBLK[0]->ctcxmode = 1;
 
-        AddDevice( &pDevCTCBLK->pDEVBLK[1], pDEVBLK->devnum + 1,
-                   "CTCI", &ctci_device_hndinfo );
+    pDevCTCBLK->pDEVBLK[1]->ctctype  = CTC_CTCI;
+    pDevCTCBLK->pDEVBLK[1]->ctcxmode = 1;
 
-        pDevCTCBLK->pDEVBLK[0]->dev_data = pDevCTCBLK;
-        pDevCTCBLK->pDEVBLK[1]->dev_data = pDevCTCBLK;
+    pDevCTCBLK->sMTU                = atoi( pDevCTCBLK->szMTU );
+    pDevCTCBLK->iMaxFrameBufferSize = pDevCTCBLK->sMTU + sizeof( IP4FRM );
 
-        SetSIDInfo( pDevCTCBLK->pDEVBLK[0], 0x3088, 0x08, 0x3088, 0x01 );
-        SetSIDInfo( pDevCTCBLK->pDEVBLK[1], 0x3088, 0x08, 0x3088, 0x01 );
+    initialize_lock( &pDevCTCBLK->Lock );
+    initialize_lock( &pDevCTCBLK->EventLock );
+    initialize_condition( &pDevCTCBLK->Event );
 
-        pDEVBLK->ctctype    = CTC_CTCI;
-        pDEVBLK->ctcxmode   = 1;
+    // Give both Herc devices a reasonable name...
 
-        pDevPair = pDEVBLK;
+    strlcpy( pDevCTCBLK->pDEVBLK[0]->filename,
+             pDevCTCBLK->szTUNCharName,
+     sizeof( pDevCTCBLK->pDEVBLK[0]->filename ) );
+
+    strlcpy( pDevCTCBLK->pDEVBLK[1]->filename,
+             pDevCTCBLK->szTUNCharName,
+     sizeof( pDevCTCBLK->pDEVBLK[1]->filename ) );
+
+    rc = TUNTAP_CreateInterface( pDevCTCBLK->szTUNCharName,
+                                 IFF_TUN | IFF_NO_PI,
+                                 &pDevCTCBLK->fd,
+                                 pDevCTCBLK->szTUNDevName );
+
+    if( rc < 0 )
+    {
+        free( pWrkCTCBLK );
+        pWrkCTCBLK = NULL;
+        return -1;
     }
     else
     {
-        // Old format has paired device statements
-        // Find device block for paired CTC adapter device number
-        pDevPair = find_device_by_devnum( pDEVBLK->devnum ^ 0x01 );
-
-        // First pass through?
-        if( !pDevPair )
-        {
-            // Allocate the CTCBLK
-            pDevCTCBLK = malloc( sizeof( CTCBLK ) );
-
-            if( !pDevCTCBLK )
-            {
-                logmsg( _("HHCCT039E %4.4X: Unable to allocate CTCBLK\n"),
-                        pDEVBLK->devnum );
-                free( pWrkCTCBLK );
-                return -1;
-            }
-
-            memcpy( pDevCTCBLK, pWrkCTCBLK, sizeof( CTCBLK ) );
-
-            pDEVBLK->dev_data = pDevCTCBLK;
-
-            pDevCTCBLK->pDEVBLK[0] = pDEVBLK;
-        }
-        else
-        {
-            // Use CTCBLK from the paired DEVBLK
-            pDevCTCBLK = (PCTCBLK)pDevPair->dev_data;
-
-            pDEVBLK->dev_data  = pDevCTCBLK;
-
-            pDevCTCBLK->pDEVBLK[1] = pDEVBLK;
-        }
-
-        // Update SENSEID information
-        SetSIDInfo( pDEVBLK, 0x3088, 0x08, 0x3088, 0x01 );
-
-        pDEVBLK->ctctype             = CTC_CTCI;
-        pDEVBLK->ctcxmode            = 1;
+        logmsg(_("HHCCT073I %4.4X: TUN device %s opened\n"),
+                  pDevCTCBLK->pDEVBLK[0]->devnum,
+                  pDevCTCBLK->szTUNDevName);
     }
 
-    if( pDevPair )
+#ifdef OPTION_TUNTAP_SETMACADDR
+
+    if( !pDevCTCBLK->szMACAddress[0] )   // (if MAC address unspecified)
     {
-        // pDevPair is non-null if:
-        //   Old format and this is the 2nd pass or
-        //   New format unconditionally
+        // Build a default MAC addr based on the guest (destination) ip
+        // address so as to effectively *UNOFFICIALLY* assign ourselves
+        // the following Ethernet address block:
 
-        pDevCTCBLK->sMTU                = atoi( pDevCTCBLK->szMTU );
-        pDevCTCBLK->iMaxFrameBufferSize = pDevCTCBLK->sMTU + sizeof( IP4FRM );
+        /* (from: http://www.iana.org/assignments/ethernet-numbers)
+           (only the first 2 and last 2 paragraphs are of interest)
 
-        initialize_lock( &pDevCTCBLK->Lock );
-        initialize_lock( &pDevCTCBLK->EventLock );
-        initialize_condition( &pDevCTCBLK->Event );
+            IANA ETHERNET ADDRESS BLOCK - UNICAST USE
 
-        rc = TUNTAP_CreateInterface( pDevCTCBLK->szTUNCharName,
-                                     IFF_TUN | IFF_NO_PI,
-                                     &pDevCTCBLK->fd,
-                                     pDevCTCBLK->szTUNDevName );
+            The IANA owns an Ethernet address block which may be used for
+            unicast address asignments or other special purposes.
 
-        if( rc < 0 )
+            The IANA may assign unicast global IEEE 802 MAC address from it's
+            assigned OUI (00-00-5E) for use in IETF standard track protocols.  The
+            intended usage is for dynamic mapping between IP addresses and IEEE
+            802 MAC addresses.  These IEEE 802 MAC addresses are not to be
+            permanently assigned to any hardware interface, nor is this a
+            substitute for a network equipment supplier getting its own OUI.
+
+            ... (snipped)
+
+            Using this representation, the range of Internet Unicast addresses is:
+
+                   00-00-5E-00-00-00  to  00-00-5E-FF-FF-FF  in hex, ...
+
+            ... (snipped)
+
+            The low order 24 bits of these unicast addresses are assigned as
+            follows:
+
+            Dotted Decimal          Description                     Reference
+            ----------------------- ------------------------------- ---------
+            000.000.000-000.000.255 Reserved                        [IANA]
+            000.001.000-000.001.255 Virual Router Redundancy (VRRP) [Hinden]
+            000.002.000-127.255.255 Reserved                        [IANA]
+            128.000.000-255.255.255 Hercules TUNTAP (CTCI)          [Fish]
+        */
+
+        // Here's what we're basically doing:
+
+        //    00-00-5E-00-00-00  to  00-00-5E-00-00-FF  =  'Reserved' by IANA
+        //    00-00-5E-00-01-00  to  00-00-5E-00-01-FF  =  'VRRP' by Hinden
+        //    00-00-5E-00-02-00  to  00-00-5E-7F-FF-FF  =  (unassigned)
+        //    00-00-5E-80-00-00  to  00-00-5E-FF-FF-FF  =  'Hercules' by Fish
+
+        //    00-00-5E-00-00-00   (starting value)
+        //    00-00-5E-ip-ip-ip   (move in low-order 3 bytes of destination IP address)
+        //    00-00-5E-8p-ip-ip   ('OR' on the x'80' high-order bit)
+
+        in_addr_t  wrk_guest_ip_addr;
+
+        if ((in_addr_t)-1 != (wrk_guest_ip_addr = inet_addr( pDevCTCBLK->szGuestIPAddr )))
         {
-            free( pWrkCTCBLK );
-            return -1;
+            *(((BYTE*)&wrk_guest_ip_addr) + sizeof(wrk_guest_ip_addr) - 3 ) |= 0x80;
+
+            snprintf
+            (
+                pDevCTCBLK->szMACAddress,  sizeof( pDevCTCBLK->szMACAddress ),
+
+                "00:00:5E:%2.2X:%2.2X:%2.2X"
+
+                ,*(((BYTE*)&wrk_guest_ip_addr) + sizeof(wrk_guest_ip_addr) - 3 )
+                ,*(((BYTE*)&wrk_guest_ip_addr) + sizeof(wrk_guest_ip_addr) - 2 )
+                ,*(((BYTE*)&wrk_guest_ip_addr) + sizeof(wrk_guest_ip_addr) - 1 )
+            );
         }
+    }
 
-        TUNTAP_SetIPAddr  ( pDevCTCBLK->szTUNDevName,
-                            pDevCTCBLK->szDriveIPAddr );
+    TRACE
+    (
+        "** CTCI_Init: %4.4X (%s): IP %s  -->  default MAC %s\n"
 
-        TUNTAP_SetDestAddr( pDevCTCBLK->szTUNDevName,
-                            pDevCTCBLK->szGuestIPAddr );
+        ,pDevCTCBLK->pDEVBLK[0]->devnum
+        ,pDevCTCBLK->szTUNDevName
+        ,pDevCTCBLK->szGuestIPAddr
+        ,pDevCTCBLK->szMACAddress
+    );
 
-        TUNTAP_SetNetMask ( pDevCTCBLK->szTUNDevName,
-                            pDevCTCBLK->szNetMask );
-
-        TUNTAP_SetMTU     ( pDevCTCBLK->szTUNDevName,
-                            pDevCTCBLK->szMTU );
-
-#if defined( WIN32 )
-        if( pDevCTCBLK->szMACAddress[0] != '\0' )
-            TUNTAP_SetMACAddr( pDevCTCBLK->szTUNDevName,
-                               pDevCTCBLK->szMACAddress );
+    VERIFY( TUNTAP_SetMACAddr ( pDevCTCBLK->szTUNDevName, pDevCTCBLK->szMACAddress  ) == 0 );
 #endif
 
-        TUNTAP_SetFlags   ( pDevCTCBLK->szTUNDevName,
-                            IFF_UP | IFF_RUNNING | IFF_BROADCAST );
+    VERIFY( TUNTAP_SetIPAddr  ( pDevCTCBLK->szTUNDevName, pDevCTCBLK->szDriveIPAddr ) == 0 );
 
-        // Copy the fd to make panel.c happy
-        pDevCTCBLK->pDEVBLK[0]->fd =
-        pDevCTCBLK->pDEVBLK[1]->fd = pDevCTCBLK->fd;
+    VERIFY( TUNTAP_SetDestAddr( pDevCTCBLK->szTUNDevName, pDevCTCBLK->szGuestIPAddr ) == 0 );
 
-        create_thread( &pDevCTCBLK->tid, NULL, CTCI_ReadThread, pDevCTCBLK );
-    }
+#ifdef OPTION_TUNTAP_SETNETMASK
+    VERIFY( TUNTAP_SetNetMask ( pDevCTCBLK->szTUNDevName, pDevCTCBLK->szNetMask     ) == 0 );
+#endif
+
+    VERIFY( TUNTAP_SetMTU     ( pDevCTCBLK->szTUNDevName, pDevCTCBLK->szMTU         ) == 0 );
+
+    VERIFY( TUNTAP_SetFlags   ( pDevCTCBLK->szTUNDevName, nIFFlags                  ) == 0 );
+
+    // Copy the fd to make panel.c happy
+    pDevCTCBLK->pDEVBLK[0]->fd =
+    pDevCTCBLK->pDEVBLK[1]->fd = pDevCTCBLK->fd;
+
+    create_thread( &pDevCTCBLK->tid, NULL, CTCI_ReadThread, pDevCTCBLK );
+
+    pDevCTCBLK->pDEVBLK[0]->tid = pDevCTCBLK->tid;
+    pDevCTCBLK->pDEVBLK[1]->tid = pDevCTCBLK->tid;
 
     free( pWrkCTCBLK );
+    pWrkCTCBLK = NULL;
 
     return 0;
 }
@@ -443,23 +559,48 @@ void  CTCI_ExecuteCCW( DEVBLK* pDEVBLK, BYTE  bCode,
 
 int  CTCI_Close( DEVBLK* pDEVBLK )
 {
-    DEVBLK* pDevPair;
+    /* DEVBLK* pDEVBLK2; */
     PCTCBLK pCTCBLK  = (PCTCBLK)pDEVBLK->dev_data;
 
-    pDevPair = find_device_by_devnum( pDEVBLK->devnum ^ 0x01 );
-
     // Close the device file (if not already closed)
     if( pCTCBLK->fd >= 0 )
     {
-        TUNTAP_Close( pCTCBLK->fd );
+        // PROGRAMMING NOTE: there's currently no way to interrupt
+        // the "CTCI_ReadThread"s TUNTAP_Read of the adapter. Thus
+        // we must simply wait for CTCI_ReadThread to eventually
+        // notice that we're doing a close (via our setting of the
+        // fCloseInProgress flag). Its TUNTAP_Read will eventually
+        // timeout after a few seconds (currently 5, which is dif-
+        // ferent than the CTC_READ_TIMEOUT_SECS timeout value the
+        // CTCI_Read function uses) and will then do the close of
+        // the adapter for us (TUNTAP_Close) so we don't have to.
+        // All we need to do is ask it to exit (via our setting of
+        // the fCloseInProgress flag) and then wait for it to exit
+        // (which, as stated, could take up to a max of 5 seconds).
 
-        pCTCBLK->fd = -1;
-        pDEVBLK->fd = -1;           // indicate we're now closed
+        // All of this is simply because it's poor form to close a
+        // device from one thread while another thread is reading
+        // from it. Attempting to do so could trip a race condition
+        // wherein the internal i/o buffers used to process the
+        // read request could have been freed (by the close call)
+        // by the time the read request eventually gets serviced.
 
-        if( pDevPair )              // if paired device exists,
-            pDevPair->fd = -1;      // then it's now closed too.
+        // I'll eventually get around to addressing this issue in
+        // the next release of TunTap32, but for now, the threads
+        // doing the i/o must be the ones that do the closing.
+
+        TID tid = pCTCBLK->tid;
+        pCTCBLK->fCloseInProgress = 1;  // (ask read thread to exit)
+        signal_thread( tid, SIGUSR2 );   // (for non-Win32 platforms)
+//FIXME signal_thread not working for non-MSVC platforms
+#if defined(_MSVC_)
+        join_thread( tid, NULL );       // (wait for thread to end)
+#endif
+        detach_thread( tid );           // (wait for thread to end)
     }
 
+    pDEVBLK->fd = -1;           // indicate we're now closed
+
     return 0;
 }
 
@@ -468,12 +609,19 @@ int  CTCI_Close( DEVBLK* pDEVBLK )
 // CTCI_Query
 // -------------------------------------------------------------------
 
-void  CTCI_Query( DEVBLK* pDEVBLK, BYTE** ppszClass,
-                  int     iBufLen, BYTE*  pBuffer )
+void  CTCI_Query( DEVBLK* pDEVBLK, char** ppszClass,
+                  int     iBufLen, char*  pBuffer )
 {
     PCTCBLK     pCTCBLK  = (PCTCBLK)pDEVBLK->dev_data;
 
     *ppszClass = "CTCA";
+
+    if(!pCTCBLK)
+    {
+        strlcpy(pBuffer,"*Uninitialised",iBufLen);
+        return;
+    }
+
     snprintf( pBuffer, iBufLen, "CTCI %s/%s (%s)",
               pCTCBLK->szGuestIPAddr,
               pCTCBLK->szDriveIPAddr,
@@ -625,7 +773,7 @@ void  CTCI_Write( DEVBLK* pDEVBLK,   U16   sCount,
     // Check that CCW count is sufficient to contain block header
     if( sCount < sizeof( CTCIHDR ) )
     {
-        logmsg( _("HHCCT042E %4.4X Write CCW count %u is invalid\n"),
+        logmsg( _("HHCCT042E %4.4X: Write CCW count %u is invalid\n"),
                 pDEVBLK->devnum, sCount );
 
         pDEVBLK->sense[0] = SENSE_DC;
@@ -796,27 +944,31 @@ static void*  CTCI_ReadThread( PCTCBLK pCTCBLK )
 {
     DEVBLK*  pDEVBLK = pCTCBLK->pDEVBLK[0];
     int      iLength;
-    char     szBuff[2048];
+    BYTE     szBuff[2048];
+
+    // ZZ FIXME: Try to avoid race condition at startup with hercifc
+    SLEEP(10);
 
     pCTCBLK->pid = getpid();
 
-    while( 1 )
+    do
     {
         // Read frame from the TUN/TAP interface
-        iLength = TUNTAP_Read( pCTCBLK->fd, szBuff, 2048 );
+        iLength = TUNTAP_Read( pCTCBLK->fd, szBuff, sizeof(szBuff) );
 
         // Check for error condition
         if( iLength < 0 )
         {
-            if( pCTCBLK->fd != -1 )
-                logmsg( _("HHCCT048E %4.4X: Error reading from %s: %s\n"),
-                    pDEVBLK->devnum, pCTCBLK->szTUNDevName,
-                    strerror( errno ) );
-
-            break;
+            if( pCTCBLK->fd == -1 || pCTCBLK->fCloseInProgress )
+                break;
+            logmsg( _("HHCCT048E %4.4X: Error reading from %s: %s\n"),
+                pDEVBLK->devnum, pCTCBLK->szTUNDevName,
+                strerror( errno ) );
+            SLEEP(1);           // (purposeful long delay)
+            continue;
         }
 
-        if( iLength == 0 )
+        if( iLength == 0 )      // (probably EINTR; ignore)
             continue;
 
         if( pCTCBLK->fDebug )
@@ -827,12 +979,31 @@ static void*  CTCI_ReadThread( PCTCBLK pCTCBLK )
         }
 
         // Enqueue frame on buffer, if buffer is full, keep trying
-        while( !CTCI_EnqueueIPFrame( pDEVBLK, szBuff, iLength ) )
-            sched_yield();
-    }
+        while( CTCI_EnqueueIPFrame( pDEVBLK, szBuff, iLength ) < 0
+            && pCTCBLK->fd != -1 && !pCTCBLK->fCloseInProgress )
+        {
+            if( EMSGSIZE == errno )     // (if too large for buffer)
+            {
+                if( pCTCBLK->fDebug )
+                    logmsg( _("HHCCT072W %4.4X: Packet too big; dropped.\n"),
+                            pDEVBLK->devnum );
+                break;                  // (discard it...)
+            }
 
-    if( pCTCBLK->fd >= 0 )
-        CTCI_Close( pDEVBLK );
+            ASSERT( ENOBUFS == errno );
+
+            // Don't use sched_yield() here; use an actual non-dispatchable
+            // delay instead so as to allow another [possibly lower priority]
+            // thread to 'read' (remove) some packet(s) from our frame buffer.
+            usleep( CTC_DELAY_USECS );  // (wait a bit before retrying...)
+        }
+    }
+    while( pCTCBLK->fd != -1 && !pCTCBLK->fCloseInProgress );
+
+    // We must do the close since we were the one doing the i/o...
+
+    VERIFY( TUNTAP_Close( pCTCBLK->fd ) == 0 );
+    pCTCBLK->fd = -1;
 
     return NULL;
 }
@@ -844,7 +1015,12 @@ static void*  CTCI_ReadThread( PCTCBLK pCTCBLK )
 // Places the provided IP frame in the next available frame
 // slot in the adapter buffer.
 //
-
+// Returns:
+//
+//  0 == Success
+// -1 == Failure; errno = ENOBUFS:  No buffer space available
+//                        EMSGSIZE: Message too long
+//
 static int  CTCI_EnqueueIPFrame( DEVBLK* pDEVBLK,
                                  BYTE*   pData, size_t iSize )
 {
@@ -852,6 +1028,13 @@ static int  CTCI_EnqueueIPFrame( DEVBLK* pDEVBLK,
     PCTCISEG pSegment;
     PCTCBLK  pCTCBLK = (PCTCBLK)pDEVBLK->dev_data;
 
+    // Will frame NEVER fit into buffer??
+    if( iSize > MAX_CTCI_FRAME_SIZE )
+    {
+        errno = EMSGSIZE;   // Message too long
+        return -1;          // (-1==failure)
+    }
+
     obtain_lock( &pCTCBLK->Lock );
 
     // Ensure we dont overflow the buffer
@@ -859,10 +1042,11 @@ static int  CTCI_EnqueueIPFrame( DEVBLK* pDEVBLK,
           sizeof( CTCIHDR )     +       // Block Header
           sizeof( CTCISEG )     +       // Segment Header
           iSize +                       // Current packet
-          2 ) > 0x5000 )                // Block terminator
+          2 ) > CTC_FRAME_BUFFER_SIZE ) // Block terminator
     {
         release_lock( &pCTCBLK->Lock );
-        return 0;
+        errno = ENOBUFS;    // No buffer space available
+        return -1;          // (-1==failure)
     }
 
     // Fix-up Frame pointer
@@ -901,7 +1085,7 @@ static int  CTCI_EnqueueIPFrame( DEVBLK* pDEVBLK,
     signal_condition( &pCTCBLK->Event );
     release_lock( &pCTCBLK->EventLock );
 
-    return 1;
+    return 0;       // (0==success)
 }
 
 //
@@ -915,8 +1099,10 @@ static int  ParseArgs( DEVBLK* pDEVBLK, PCTCBLK pCTCBLK,
     int             iMTU;
     int             i;
     MAC             mac;                // Work area for MAC address
+#if defined(WIN32)
     int             iKernBuff;
     int             iIOBuff;
+#endif
 
     // Housekeeping
     memset( &addr, 0, sizeof( struct in_addr ) );
@@ -931,30 +1117,61 @@ static int  ParseArgs( DEVBLK* pDEVBLK, PCTCBLK pCTCBLK,
     strcpy( pCTCBLK->szTUNCharName,  "/dev/net/tun" );
 #endif
 
+#if defined( WIN32 )
     pCTCBLK->iKernBuff     = DEF_TT32DRV_BUFFSIZE_K * 1024;
     pCTCBLK->iIOBuff       = DEF_TT32DRV_BUFFSIZE_K * 1024;
+#endif
 
     // Initialize getopt's counter. This is necessary in the case
     // that getopt was used previously for another device.
-    optind      = 0;
 
+    OPTRESET();
+    optind      = 0;
+    // Check for correct number of arguments
+    if( argc < 2 )
+    {
+        logmsg( _("HHCCT056E %4.4X: Incorrect number of parameters\n"),
+               pDEVBLK->devnum );
+        return -1;
+    }
     // Compatability with old format configuration files needs to be
     // maintained. Old format statements have the tun character device
     // name as the second argument on Linux, or CTCI-W32 as the first
     // argument on Windows.
-    if( ( strncasecmp( argv[1], "/", 1 ) == 0 ) ||
-        ( strncasecmp( argv[0], "ctci-w32", 8 ) == 0 ) )
+    if( ( strncasecmp( argv[0], "/", 1 ) == 0 ) ||
+        ( strncasecmp( pDEVBLK->typname, "CTCI-W32", 8 ) == 0 ) )
     {
         pCTCBLK->fOldFormat = 1;
-        argc--; argv++;
     }
-	
+    else
+    {
+        // Build new argv list.
+        // getopt_long used to work on old format configuration statements
+        // because LCS was the first argument passed to the device
+        // initialization routine (and was interpreted by getopt*
+        // as the program name and ignored). Now that argv[0] is a valid
+        // argument, we need to shift the arguments and insert a dummy
+        // argv[0];
+
+        // Don't allow us to exceed the allocated storage (sanity check)
+        if( argc > (MAX_ARGS-1) )
+            argc = (MAX_ARGS-1);
+
+        for( i = argc; i > 0; i-- )
+            argv[i] = argv[i - 1];
+
+        argc++;
+        argv[0] = pDEVBLK->typname;
+    }
+
     // Parse any optional arguments if not old format
     while( !pCTCBLK->fOldFormat )
     {
-        int     iOpt;
         int     c;
 
+#if defined(HAVE_GETOPT_LONG)
+        int     iOpt;
+
         static struct option options[] =
         {
             { "dev",     1, NULL, 'n' },
@@ -970,6 +1187,9 @@ static int  ParseArgs( DEVBLK* pDEVBLK, PCTCBLK pCTCBLK,
         c = getopt_long( argc, argv,
                  "n:k:i:t:s:m:d",
                  options, &iOpt );
+#else /* defined(HAVE_GETOPT_LONG) */
+        c = getopt( argc, argv, "n:k:i:t:s:m:d");
+#endif /* defined(HAVE_GETOPT_LONG) */
 
         if( c == -1 ) // No more options found
             break;
@@ -977,7 +1197,7 @@ static int  ParseArgs( DEVBLK* pDEVBLK, PCTCBLK pCTCBLK,
         switch( c )
         {
         case 'n':     // Network Device
-#if defined( WIN32 )
+#if defined( OPTION_W32_CTCI )
             // This could be the IP or MAC address of the
             // host ethernet adapter.
             if( inet_aton( optarg, &addr ) == 0 )
@@ -990,7 +1210,7 @@ static int  ParseArgs( DEVBLK* pDEVBLK, PCTCBLK pCTCBLK,
                     return -1;
                 }
             }
-#else // !defined( WIN32 )
+#endif // defined( OPTION_W32_CTCI )
             // This is the file name of the special TUN/TAP character device
             if( strlen( optarg ) > sizeof( pCTCBLK->szTUNCharName ) - 1 )
             {
@@ -998,11 +1218,11 @@ static int  ParseArgs( DEVBLK* pDEVBLK, PCTCBLK pCTCBLK,
                     pDEVBLK->devnum, optarg );
                 return -1;
             }
-#endif // defined( WIN32 )
             strcpy( pCTCBLK->szTUNCharName, optarg );
             break;
 
         case 'k':     // Kernel Buffer Size (ignored if not Windows)
+#if defined( WIN32 )
             iKernBuff = atoi( optarg );
 
             if( iKernBuff < MIN_TT32DLL_BUFFSIZE_K    ||
@@ -1014,9 +1234,11 @@ static int  ParseArgs( DEVBLK* pDEVBLK, PCTCBLK pCTCBLK,
             }
 
             pCTCBLK->iKernBuff = iKernBuff * 1024;
+#endif
             break;
 
         case 'i':     // I/O Buffer Size (ignored if not Windows)
+#if defined( WIN32 )
             iIOBuff = atoi( optarg );
 
             if( iIOBuff < MIN_TT32DLL_BUFFSIZE_K    ||
@@ -1028,6 +1250,7 @@ static int  ParseArgs( DEVBLK* pDEVBLK, PCTCBLK pCTCBLK,
             }
 
             pCTCBLK->iIOBuff = iIOBuff * 1024;
+#endif
             break;
 
         case 't':     // MTU of point-to-point link (ignored if Windows)
@@ -1054,7 +1277,7 @@ static int  ParseArgs( DEVBLK* pDEVBLK, PCTCBLK pCTCBLK,
             strcpy( pCTCBLK->szNetMask, optarg );
             break;
 
-        case 'm':
+        case 'm':     // (ignored if not Windows)
             if( ParseMAC( optarg, mac ) != 0 )
             {
                 logmsg( _("HHCCT056E %4.4X: Invalid MAC address %s\n"),
@@ -1125,7 +1348,7 @@ static int  ParseArgs( DEVBLK* pDEVBLK, PCTCBLK pCTCBLK,
     }
     else // if( pCTCBLK->fOldFormat )
     {
-#if !defined( WIN32 )
+#if !defined( OPTION_W32_CTCI )
         // All arguments are non-optional in linux old-format
         // Old format has 5 and only 5 arguments
         if( argc != 5 )
@@ -1139,7 +1362,7 @@ static int  ParseArgs( DEVBLK* pDEVBLK, PCTCBLK pCTCBLK,
         if( **argv != '/' ||
             strlen( *argv ) > sizeof( pCTCBLK->szTUNCharName ) - 1 )
         {
-            logmsg( _("HHCCT061E %4.4X invalid device name %s\n"),
+            logmsg( _("HHCCT061E %4.4X: invalid device name %s\n"),
                 pDEVBLK->devnum, *argv );
             return -1;
         }
@@ -1203,7 +1426,7 @@ static int  ParseArgs( DEVBLK* pDEVBLK, PCTCBLK pCTCBLK,
                 pDEVBLK->devnum );
             return -1;
         }
-#else // defined( WIN32 )
+#else // defined( OPTION_W32_CTCI )
         // There are 2 non-optional arguments in the Windows old-format:
         //   Guest IP address and Gateway address.
         // There are also 2 additional optional arguments:
@@ -1240,17 +1463,17 @@ static int  ParseArgs( DEVBLK* pDEVBLK, PCTCBLK pCTCBLK,
 
                 strcpy( pCTCBLK->szTUNCharName, *argv );
 
-                // Kludge: This may look strange at first, but with 
-                // TunTap32, only the last byte of the "driver IP 
-                // address" is actually used. It's purpose is to 
-                // generate a unique MAC for the virtual interface. 
-                // Thus, having the same address for the adapter and 
-                // destination is not an issue. This used to be 
-                // generated from the guest IP address, I screwed up 
+                // Kludge: This may look strange at first, but with
+                // TunTap32, only the last 3 bytes of the "driver IP
+                // address" is actually used. It's purpose is to
+                // generate a unique MAC for the virtual interface.
+                // Thus, having the same address for the adapter and
+                // destination is not an issue. This used to be
+                // generated from the guest IP address, I screwed up
                 // TunTap32 V2. (JAP)
                 // This also fixes the confusing error messages from
                 // TunTap.c when a MAC is given for this argument.
-                
+
                 strcpy( pCTCBLK->szDriveIPAddr,
                         pCTCBLK->szGuestIPAddr );
 
@@ -1297,9 +1520,8 @@ static int  ParseArgs( DEVBLK* pDEVBLK, PCTCBLK pCTCBLK,
                 return -1;
             }
         }
-#endif // !defined( WIN32 )
+#endif // !defined( OPTION_W32_CTCI )
     }
 
     return 0;
 }
-#endif /* !defined(__APPLE__) */
