@@ -1,14 +1,12 @@
-// ====================================================================
 // Hercules Channel-to-Channel Emulation Support
 // ====================================================================
 //
-// Copyright (C) James A. Pierson, 2002-2003
-//               Roger Bowler, 2000-2003
+// Copyright (C) James A. Pierson, 2002-2005
+//               Roger Bowler, 2000-2005
 //
-// vmnet     (C) Copyright Willem Konynenberg, 2000-2003
-// CTCT      (C) Copyright Vic Cross, 2001-2003
+// vmnet     (C) Copyright Willem Konynenberg, 2000-2005
+// CTCT      (C) Copyright Vic Cross, 2001-2005
 //
-
 // Notes:
 //   This module contains the remaining CTC emulation modes that
 //   have not been moved to seperate modules. There is also logic
@@ -17,7 +15,10 @@
 //
 //   Please read README.NETWORKING for more info.
 //
-#if !defined(__APPLE__)
+#include "hstdinc.h"
+
+#define _CTCADPT_C_
+#define _HENGINE_DLL_
 
 #include "hercules.h"
 #include "devtype.h"
@@ -30,7 +31,7 @@
 // Declarations
 // ====================================================================
 
-static int      CTCT_Init( DEVBLK *dev, int argc, BYTE *argv[] );
+static int      CTCT_Init( DEVBLK *dev, int argc, char *argv[] );
 
 static void     CTCT_Read( DEVBLK* pDEVBLK,   U16   sCount,
                            BYTE*   pIOBuf,    BYTE* pUnitStat,
@@ -42,7 +43,7 @@ static void     CTCT_Write( DEVBLK* pDEVBLK,   U16   sCount,
 
 static void*    CTCT_ListenThread( void* argp );
 
-static int      VMNET_Init( DEVBLK *dev, int argc, BYTE *argv[] );
+static int      VMNET_Init( DEVBLK *dev, int argc, char *argv[] );
 
 static int      VMNET_Write( DEVBLK *dev, BYTE *iobuf,
                              U16 count, BYTE *unitstat );
@@ -68,34 +69,72 @@ CTCG_PARMBLK;
 
 DEVHND ctcadpt_device_hndinfo =
 {
-    &CTCX_Init,
-    &CTCX_ExecuteCCW,
-    &CTCX_Close,
-    &CTCX_Query,
-    NULL, NULL, NULL, NULL
+        &CTCX_Init,                    /* Device Initialisation      */
+        &CTCX_ExecuteCCW,              /* Device CCW execute         */
+        &CTCX_Close,                   /* Device Close               */
+        &CTCX_Query,                   /* Device Query               */
+        NULL,                          /* Device Start channel pgm   */
+        NULL,                          /* Device End channel pgm     */
+        NULL,                          /* Device Resume channel pgm  */
+        NULL,                          /* Device Suspend channel pgm */
+        NULL,                          /* Device Read                */
+        NULL,                          /* Device Write               */
+        NULL,                          /* Device Query used          */
+        NULL,                          /* Device Reserve             */
+        NULL,                          /* Device Release             */
+        NULL,                          /* Immediate CCW Codes        */
+        NULL,                          /* Signal Adapter Input       */
+        NULL,                          /* Signal Adapter Output      */
+        NULL,                          /* Hercules suspend           */
+        NULL                           /* Hercules resume            */
 };
 
 DEVHND ctct_device_hndinfo =
 {
-    &CTCT_Init,
-    &CTCX_ExecuteCCW,
-    &CTCX_Close,
-    &CTCX_Query,
-    NULL, NULL, NULL, NULL
+        &CTCT_Init,                    /* Device Initialisation      */
+        &CTCX_ExecuteCCW,              /* Device CCW execute         */
+        &CTCX_Close,                   /* Device Close               */
+        &CTCX_Query,                   /* Device Query               */
+        NULL,                          /* Device Start channel pgm   */
+        NULL,                          /* Device End channel pgm     */
+        NULL,                          /* Device Resume channel pgm  */
+        NULL,                          /* Device Suspend channel pgm */
+        NULL,                          /* Device Read                */
+        NULL,                          /* Device Write               */
+        NULL,                          /* Device Query used          */
+        NULL,                          /* Device Reserve             */
+        NULL,                          /* Device Release             */
+        NULL,                          /* Immediate CCW Codes        */
+        NULL,                          /* Signal Adapter Input       */
+        NULL,                          /* Signal Adapter Output      */
+        NULL,                          /* Hercules suspend           */
+        NULL                           /* Hercules resume            */
 };
 
 DEVHND vmnet_device_hndinfo =
 {
-    &VMNET_Init,
-    &CTCX_ExecuteCCW,
-    &CTCX_Close,
-    &CTCX_Query,
-    NULL, NULL, NULL, NULL
+        &VMNET_Init,                   /* Device Initialisation      */
+        &CTCX_ExecuteCCW,              /* Device CCW execute         */
+        &CTCX_Close,                   /* Device Close               */
+        &CTCX_Query,                   /* Device Query               */
+        NULL,                          /* Device Start channel pgm   */
+        NULL,                          /* Device End channel pgm     */
+        NULL,                          /* Device Resume channel pgm  */
+        NULL,                          /* Device Suspend channel pgm */
+        NULL,                          /* Device Read                */
+        NULL,                          /* Device Write               */
+        NULL,                          /* Device Query used          */
+        NULL,                          /* Device Reserve             */
+        NULL,                          /* Device Release             */
+        NULL,                          /* Immediate CCW Codes        */
+        NULL,                          /* Signal Adapter Input       */
+        NULL,                          /* Signal Adapter Output      */
+        NULL,                          /* Hercules suspend           */
+        NULL                           /* Hercules resume            */
 };
 
 extern DEVHND ctci_device_hndinfo;
 extern DEVHND lcs_device_hndinfo;
-extern DEVENT device_handler_table[];
 
 // ====================================================================
 // Primary Module Entry Points
@@ -105,51 +144,29 @@ extern DEVENT device_handler_table[];
 // Device Initialization Handler (Generic)
 // --------------------------------------------------------------------
 
-int  CTCX_Init( DEVBLK* pDEVBLK, int argc, BYTE *argv[] )
+int  CTCX_Init( DEVBLK* pDEVBLK, int argc, char *argv[] )
 {
-    // Allow for depreciated 3088 device type
-    if( strcasecmp( pDEVBLK->typname, "3088" ) == 0 )
-    {
-        // The first argument is the device emulation type
-        if( argc < 1 )
-        {
-            logmsg( _("HHCCT001E %4.4X: Incorrect number of parameters\n"),
-                pDEVBLK->devnum );
-            return -1;
-        }
+    pDEVBLK->devtype = 0x3088;
 
-        // Replace typname in DEVBLK with actual type.
-        if( strcasecmp( argv[0], "CTCI"     ) == 0 ||
-            strcasecmp( argv[0], "CTCI-W32" ) == 0 )
-            pDEVBLK->typname = "CTCI";
-        else if( strcasecmp( argv[0], "CTCT" ) == 0 )
-            pDEVBLK->typname = "CTCT";
-        else if( strcasecmp( argv[0], "LCS" ) == 0 )
-            pDEVBLK->typname = "LCS";
-        else
-            pDEVBLK->typname = "VMNET";
+    // The first argument is the device emulation type
+    if( argc < 1 )
+    {
+        logmsg( _("HHCCT001E %4.4X: Incorrect number of parameters\n"),
+            pDEVBLK->devnum );
+        return -1;
     }
 
-    // A little slight of hand here...
-    //
-    // The table in devtype.c points all CTC emulation types
-    // to ctcadpt_device_hndinfo. Based on the typnam member
-    // of DEVBLK, substitute the proper handler entries for
-    // this device.
-
-    if     ( strcasecmp( pDEVBLK->typname, "CTCI" ) == 0 )
-        pDEVBLK->hnd = &ctci_device_hndinfo;
-    else if( strcasecmp( pDEVBLK->typname, "CTCT"  ) == 0 )
-        pDEVBLK->hnd = &ctct_device_hndinfo;
-    else if( strcasecmp( pDEVBLK->typname, "LCS"   ) == 0 )
-        pDEVBLK->hnd = &lcs_device_hndinfo;
-    else if( strcasecmp( pDEVBLK->typname, "VMNET" ) == 0 )
-        pDEVBLK->hnd = &vmnet_device_hndinfo;
-    else
-        return -1;
-
-    // Now call the proper device init function
-    return (pDEVBLK->hnd->init)( pDEVBLK, argc, argv );
+    if((pDEVBLK->hnd = hdl_ghnd(argv[0])))
+    {
+        if(pDEVBLK->hnd->init == &CTCX_Init)
+            return -1;
+        free(pDEVBLK->typname);
+        pDEVBLK->typname = strdup(argv[0]);
+        return (pDEVBLK->hnd->init)( pDEVBLK, --argc, ++argv );
+    }
+    logmsg (_("HHCCT034E %s: Unrecognized/unsupported CTC emulation type\n"),
+        argv[0]);
+    return -1;
 }
 
 // -------------------------------------------------------------------
@@ -157,9 +174,9 @@ int  CTCX_Init( DEVBLK* pDEVBLK, int argc, BYTE *argv[] )
 // -------------------------------------------------------------------
 
 void  CTCX_Query( DEVBLK* pDEVBLK,
-                  BYTE**  ppszClass,
+                  char**  ppszClass,
                   int     iBufLen,
-                  BYTE*   pBuffer )
+                  char*   pBuffer )
 {
     *ppszClass = "CTCA";
     snprintf( pBuffer, iBufLen, "%s", pDEVBLK->filename );
@@ -174,7 +191,10 @@ int  CTCX_Close( DEVBLK* pDEVBLK )
     // Close the device file (if not already closed)
     if( pDEVBLK->fd >= 0 )
     {
-        close( pDEVBLK->fd );
+        if (socket_is_socket( pDEVBLK->fd ))
+            close_socket( pDEVBLK->fd );
+        else
+            close( pDEVBLK->fd );
         pDEVBLK->fd = -1;           // indicate we're now closed
     }
 
@@ -426,32 +446,28 @@ void  CTCX_ExecuteCCW( DEVBLK* pDEVBLK, BYTE  bCode,
 // CTCT_Init
 //
 
-static int  CTCT_Init( DEVBLK *dev, int argc, BYTE *argv[] )
+static int  CTCT_Init( DEVBLK *dev, int argc, char *argv[] )
 {
     int            rc;                 // Return code
     int            mtu;                // MTU size (binary)
     int            lport;              // Listen port (binary)
     int            rport;              // Destination port (binary)
-    BYTE*          listenp;            // Listening port number
-    BYTE*          remotep;            // Destination port number
-    BYTE*          mtusize;            // MTU size (characters)
-    BYTE*          remaddr;            // Remote IP address
+    char*          listenp;            // Listening port number
+    char*          remotep;            // Destination port number
+    char*          mtusize;            // MTU size (characters)
+    char*          remaddr;            // Remote IP address
     struct in_addr ipaddr;             // Work area for IP address
     BYTE           c;                  // Character work area
     TID            tid;                // Thread ID for server
     CTCG_PARMBLK   parm;               // Parameters for the server
-    BYTE           address[20]="";     // temp space for IP address
+    char           address[20]="";     // temp space for IP address
+
+    dev->devtype = 0x3088;
 
     dev->ctctype = CTC_CTCT;
 
     SetSIDInfo( dev, 0x3088, 0x08, 0x3088, 0x01 );
 
-    if( strcasecmp( argv[0], "CTCT" ) == 0 )
-    {
-        // Shift past emulation type
-        argc--; argv++;
-    }
-
     // Check for correct number of arguments
     if (argc != 4)
     {
@@ -535,7 +551,7 @@ static int  CTCT_Init( DEVBLK *dev, int argc, BYTE *argv[] )
     if( parm.listenfd < 0 )
     {
         logmsg( _("HHCCT007E %4.4X: Error creating socket: %s\n"),
-                dev->devnum, strerror( errno ) );
+                dev->devnum, strerror( HSO_errno ) );
         CTCX_Close( dev );
         return -1;
     }
@@ -555,7 +571,7 @@ static int  CTCT_Init( DEVBLK *dev, int argc, BYTE *argv[] )
     if( rc < 0 )
     {
         logmsg( _("HHCCT008E %4.4X: Error binding to socket: %s\n"),
-                dev->devnum, strerror( errno ) );
+                dev->devnum, strerror( HSO_errno ) );
         CTCX_Close( dev );
         return -1;
     }
@@ -579,14 +595,14 @@ static int  CTCT_Init( DEVBLK *dev, int argc, BYTE *argv[] )
                 dev->devnum, remaddr, remotep );
 
         // probably don't need to do this, not sure...
-        close( parm.listenfd );
+        close_socket( parm.listenfd );
 
         parm.listenfd = socket( AF_INET, SOCK_STREAM, 0 );
 
         if( parm.listenfd < 0 )
         {
             logmsg( _("HHCCT010E %4.4X: Error creating socket: %s\n"),
-                    dev->devnum, strerror( errno ) );
+                    dev->devnum, strerror( HSO_errno ) );
             CTCX_Close( dev );
             return -1;
         }
@@ -603,7 +619,7 @@ static int  CTCT_Init( DEVBLK *dev, int argc, BYTE *argv[] )
                   sizeof( parm.addr ) ) < 0 )
         {
             logmsg( _("HHCCT011E %4.4X: Error binding to socket: %s\n"),
-                    dev->devnum, strerror( errno ) );
+                    dev->devnum, strerror( HSO_errno ) );
             CTCX_Close( dev );
             return -1;
         }
@@ -611,7 +627,7 @@ static int  CTCT_Init( DEVBLK *dev, int argc, BYTE *argv[] )
         if( listen( parm.listenfd, 1 ) < 0 )
         {
             logmsg( _("HHCCT012E %4.4X: Error on call to listen: %s\n"),
-                    dev->devnum, strerror( errno ) );
+                    dev->devnum, strerror( HSO_errno ) );
             CTCX_Close( dev );
             return -1;
         }
@@ -658,7 +674,7 @@ static void  CTCT_Write( DEVBLK* pDEVBLK,   U16   sCount,
     // Check that CCW count is sufficient to contain block header
     if( sCount < sizeof( CTCIHDR ) )
     {
-        logmsg( _("HHCCT014E %4.4X Write CCW count %u is invalid\n"),
+        logmsg( _("HHCCT014E %4.4X: Write CCW count %u is invalid\n"),
                 pDEVBLK->devnum, sCount );
 
         pDEVBLK->sense[0] = SENSE_DC;
@@ -776,13 +792,13 @@ static void  CTCT_Write( DEVBLK* pDEVBLK,   U16   sCount,
         }
 
         // Write the IP packet
-        rc = write( pDEVBLK->fd, pSegment->bData, sDataLen );
+        rc = write_socket( pDEVBLK->fd, pSegment->bData, sDataLen );
 
         if( rc < 0 )
         {
             logmsg( _("HHCCT019E %4.4X: Error writing to %s: %s\n"),
                     pDEVBLK->devnum, pDEVBLK->filename,
-                    strerror( errno ) );
+                    strerror( HSO_errno ) );
 
             pDEVBLK->sense[0] = SENSE_EC;
             *pUnitStat        = CSW_CE | CSW_DE | CSW_UC;
@@ -840,11 +856,11 @@ static void  CTCT_Read( DEVBLK* pDEVBLK,   U16   sCount,
         return;
 
     case -1:
-        if( errno == EINTR )
+        if( HSO_errno == HSO_EINTR )
             return;
 
         logmsg( _("HHCCT020E %4.4X: Error reading from %s: %s\n"),
-                pDEVBLK->devnum, pDEVBLK->filename, strerror( errno ) );
+                pDEVBLK->devnum, pDEVBLK->filename, strerror( HSO_errno ) );
 
         pDEVBLK->sense[0] = SENSE_EC;
         *pUnitStat = CSW_CE | CSW_DE | CSW_UC;
@@ -855,13 +871,13 @@ static void  CTCT_Read( DEVBLK* pDEVBLK,   U16   sCount,
     }
 
     // Read an IP packet from the TUN device
-    iLength = read( pDEVBLK->fd, pDEVBLK->buf, pDEVBLK->bufsize );
+    iLength = read_socket( pDEVBLK->fd, pDEVBLK->buf, pDEVBLK->bufsize );
 
     // Check for other error condition
     if( iLength < 0 )
     {
         logmsg( _("HHCCT021E %4.4X: Error reading from %s: %s\n"),
-                pDEVBLK->devnum, pDEVBLK->filename, strerror( errno ) );
+                pDEVBLK->devnum, pDEVBLK->filename, strerror( HSO_errno ) );
         pDEVBLK->sense[0] = SENSE_EC;
         *pUnitStat = CSW_CE | CSW_DE | CSW_UC;
         return;
@@ -885,7 +901,8 @@ static void  CTCT_Read( DEVBLK* pDEVBLK,   U16   sCount,
     memset( pSegment, 0, iLength + sizeof( CTCISEG ) );
 
     // Update next frame offset
-    STORE_HW( pFrame->hwOffset, iLength + sizeof( CTCIHDR ) );
+    STORE_HW( pFrame->hwOffset, 
+              iLength + sizeof( CTCIHDR ) + sizeof( CTCISEG ) );
 
     // Store segment length
     STORE_HW( pSegment->hwLength, iLength + sizeof( CTCISEG ) );
@@ -897,7 +914,8 @@ static void  CTCT_Read( DEVBLK* pDEVBLK,   U16   sCount,
     memcpy( pSegment->bData, pDEVBLK->buf, iLength );
 
     // Fix-up frame pointer and terminate block
-    pFrame = (PCTCIHDR)( pIOBuf + sizeof( CTCIHDR ) + iLength );
+    pFrame = (PCTCIHDR)( pIOBuf + sizeof( CTCIHDR ) + 
+                         sizeof( CTCISEG ) + iLength );
     STORE_HW( pFrame->hwOffset, 0x0000 );
 
     // Calculate #of bytes returned including two slack bytes
@@ -950,11 +968,11 @@ static void*  CTCT_ListenThread( void* argp )
 
         if( strcmp( str, parm.dev->filename ) != 0 )
         {
-            logmsg( _("HHCCT023E %4.4X: Incorrect client or config error\n"),
-                    parm.dev->devnum );
-            logmsg( _("                 Config=%s, connecting client=%s\n"),
+            logmsg( _("HHCCT023E %4.4X: Incorrect client or config error\n"
+                      "                 Config=%s, connecting client=%s\n"),
+                    parm.dev->devnum,
                     parm.dev->filename, str);
-            close( connfd );
+            close_socket( connfd );
         }
         else
         {
@@ -968,10 +986,12 @@ static void*  CTCT_ListenThread( void* argp )
         // see the possibility for bad things to occur (eg if another
         // Hercules tries to connect).  This will also be fixed RSN.
     }
+
+    return NULL;    // make compiler happy
 }
 
 // ====================================================================
-// VMNET Support
+// VMNET Support -- written by Willem Konynenberg
 // ====================================================================
 
 /*-------------------------------------------------------------------*/
@@ -985,14 +1005,14 @@ static void*  CTCT_ListenThread( void* argp )
 /*-------------------------------------------------------------------*/
 /* Functions to support vmnet written by Willem Konynenberg          */
 /*-------------------------------------------------------------------*/
-static int start_vmnet(DEVBLK *dev, DEVBLK *xdev, int argc, BYTE *argv[])
+static int start_vmnet(DEVBLK *dev, DEVBLK *xdev, int argc, char *argv[])
 {
 int sockfd[2];
 int r, i;
-BYTE *ipaddress;
+char *ipaddress;
 
     if (argc < 2) {
-        logmsg (_("HHCCT024E 4.4X: Not enough arguments to start vmnet\n"),
+        logmsg (_("HHCCT024E %4.4X: Not enough arguments to start vmnet\n"),
                         dev->devnum);
         return -1;
     }
@@ -1025,7 +1045,7 @@ BYTE *ipaddress;
         }
 
         /* the ugly cast is to silence a compiler warning due to const */
-        execv (argv[0], (char *const *)argv);
+        execv (argv[0], (EXECV_ARG2_ARGV_T)argv);
 
         exit (1);
     }
@@ -1043,12 +1063,14 @@ BYTE *ipaddress;
     return 0;
 }
 
-static int VMNET_Init(DEVBLK *dev, int argc, BYTE *argv[])
+static int VMNET_Init(DEVBLK *dev, int argc, char *argv[])
 {
 U16             xdevnum;                /* Pair device devnum        */
 BYTE            c;                      /* tmp for scanf             */
 DEVBLK          *xdev;                  /* Pair device               */
 
+    dev->devtype = 0x3088;
+
     /* parameters for network CTC are:
      *    devnum of the other CTC device of the pair
      *    ipaddress
@@ -1171,7 +1193,7 @@ int n;
                     return -3;
                 logmsg (_("HHCCT033E %4.4X: Error: read: %s\n"),
                         dev->devnum, strerror(errno));
-                sleep(2);
+                SLEEP(2);
             }
         } while (n <= 0);
         dev->ctcrem = n;
@@ -1293,163 +1315,13 @@ int             lastlen = 2;            /* block length at last pckt */
     }
 }
 /*-------------------------------------------------------------------*/
-/* End of vmnet functions written by Willem Konynenberg              */
+/* End of VMNET functions written by Willem Konynenberg              */
 /*-------------------------------------------------------------------*/
 
 // ====================================================================
 // Support Functions
 // ====================================================================
 
-// ---------------------------------------------------------------------
-// AddDevice
-// ---------------------------------------------------------------------
-//
-// Creates an additional DEVBLK structure for LCS devices.
-//
-// Input:
-//      sDevNum   Unit address for the new DEVBLK
-//      sDevType  Device type for the new DEVBLK
-//
-// Output:
-//      ppDEVBLK  Address of a pointer to receive the new DEVBLK addr
-//
-
-void  AddDevice( DEVBLK**    ppDEVBLK,
-                 U16         sDevNum,
-                 const char* szDevType,
-                 DEVHND*     pDevHnd )
-{
-    DEVBLK*   pDev;                     // -> Device block
-    DEVBLK**  ppDev;                    // -> Device block address
-    DEVENT*   pDevEntry  = NULL;
-    int       fNewDEVBLK = 0;           // 1=Newly created devblk
-
-    // Check whether device number has already been defined
-    if( find_device_by_devnum( sDevNum ) != NULL )
-    {
-        logmsg( _("HHCCT034E device %4.4X already exists\n"), sDevNum );
-        return;
-    }
-
-    for( pDevEntry = device_handler_table; pDevEntry->hnd; pDevEntry++ )
-        if( !strcasecmp( pDevEntry->name, szDevType ) )
-            break;
-
-    if( !pDevEntry )
-    {
-        logmsg ( _("HHCCT035E AddDevice internal error (%s).\n" ),
-         szDevType );
-        return;
-    }
-
-    if( *ppDEVBLK == NULL )
-    {
-        // Attempt to reuse an existing device block
-        pDev = find_unused_device();
-
-        // If no device block is available, create a new one
-        if( pDev == NULL )
-        {
-            // Obtain a device block
-            pDev = (DEVBLK*)malloc( sizeof( DEVBLK ) );
-
-            if( pDev == NULL )
-            {
-                logmsg( _("HHCCT036E Cannot obtain device block "
-                          "for device %4.4X: %s\n"),
-                        sDevNum, strerror( errno ) );
-                return;
-            }
-
-            memset( pDev, 0, sizeof( DEVBLK ) );
-
-            // Indicate a newly allocated devblk
-            fNewDEVBLK = 1;
-
-            // Initialize the device lock and conditions
-            initialize_lock( &pDev->lock );
-            initialize_condition( &pDev->resumecond );
-
-            // Assign new subchannel number
-            pDev->subchan = sysblk.highsubchan++;
-        }
-
-        // Obtain the device lock
-        obtain_lock( &pDev->lock );
-    }
-    else
-        pDev = *ppDEVBLK;
-
-    // Initialize the device block
-    if( pDevHnd != NULL )
-        pDev->hnd = pDevHnd;
-    else
-        pDev->hnd = pDevEntry->hnd;
-
-    pDev->msgpipew = sysblk.msgpipew;
-    pDev->cpuprio  = sysblk.cpuprio;
-    pDev->devnum   = sDevNum;
-    pDev->chanset  = sDevNum >> 12;
-
-    if( pDev->chanset >= MAX_CPU_ENGINES )
-        pDev->chanset = MAX_CPU_ENGINES - 1;
-
-    pDev->devtype  = pDevEntry->type;
-    pDev->typname  = pDevEntry->name;
-
-    pDev->fd = -1;
-
-    // Initialize storage view
-    pDev->mainstor = sysblk.mainstor;
-    pDev->storkeys = sysblk.storkeys;
-    pDev->mainlim  = sysblk.mainsize - 1;
-
-    // Initialize the path management control word
-    pDev->pmcw.devnum[0] = pDev->devnum >> 8;
-    pDev->pmcw.devnum[1] = pDev->devnum & 0xFF;
-    pDev->pmcw.lpm       = 0x80;
-    pDev->pmcw.pim       = 0x80;
-    pDev->pmcw.pom       = 0xFF;
-    pDev->pmcw.pam       = 0x80;
-    pDev->pmcw.chpid[0]  = pDev->devnum >> 8;
-
-    // If we acquired a new device block, add it to the chain
-    if( fNewDEVBLK )
-    {
-        // Search for the last device block on the chain
-        for( ppDev   = &(sysblk.firstdev);
-             *ppDev != NULL;
-             ppDev   = &((*ppDev)->nextdev) );
-
-        // Add the new device block to the end of the chain
-        *ppDev = pDev;
-        pDev->nextdev = NULL;
-    }
-
-    // Mark device valid
-    pDev->pmcw.flag5 |= PMCW5_V;
-
-#ifdef _FEATURE_CHANNEL_SUBSYSTEM
-    // Indicate a CRW is pending for this device
-    pDev->crwpending = 1;
-#endif // _FEATURE_CHANNEL_SUBSYSTEM
-
-    if( *ppDEVBLK == NULL )
-    {
-        // Release device lock
-        release_lock( &pDev->lock );
-    }
-
-#ifdef _FEATURE_CHANNEL_SUBSYSTEM
-    // Signal machine check
-    machine_check_crwpend();
-#endif // _FEATURE_CHANNEL_SUBSYSTEM
-
-    *ppDEVBLK = pDev;
-
-    return;
-}
-
 // ---------------------------------------------------------------------
 // ParseMAC
 // ---------------------------------------------------------------------
@@ -1471,7 +1343,7 @@ void  AddDevice( DEVBLK**    ppDEVBLK,
 
 int             ParseMAC( char* pszMACAddr, BYTE* pbMACAddr )
 {
-    BYTE    work[((LCS_ADDR_LEN*3)-0)];
+    char    work[((LCS_ADDR_LEN*3)-0)];
     BYTE    sep;
     int     x, i;
 
@@ -1561,4 +1433,3 @@ void  packet_trace( BYTE* pAddr, int iLen )
         logmsg( " %s\n", print_chars );
     }
 }
-#endif /* !defined(__APPLE__) */
