@@ -2,7 +2,7 @@
 // Hercules Channel to Channel Emulation Support
 // ====================================================================
 //
-// Copyright (C) 2002-2003 by James A. Pierson
+// Copyright (C) 2002-2005 by James A. Pierson
 //
 
 #ifndef __CTCADPT_H_
@@ -27,62 +27,57 @@
 // External Declarations
 // --------------------------------------------------------------------
 
-extern int      CTCX_Init( DEVBLK* pDEVBLK, int argc, BYTE *argv[] );
+extern int      CTCX_Init( DEVBLK* pDEVBLK, int argc, char *argv[] );
 extern int      CTCX_Close( DEVBLK* pDEVBLK );
-extern void     CTCX_Query( DEVBLK* pDEVBLK, BYTE** ppszClass,
-                            int     iBufLen, BYTE*  pBuffer );
-extern void     CTCX_ExecuteCCW( DEVBLK* pDEVBLK, BYTE  bCode, 
-                                 BYTE    bFlags,  BYTE  bChained, 
-                                 U16     sCount,  BYTE  bPrevCode, 
-                                 int     iCCWSeq, BYTE* pIOBuf, 
-                                 BYTE*   pMore,   BYTE* pUnitStat, 
+extern void     CTCX_Query( DEVBLK* pDEVBLK, char** ppszClass,
+                            int     iBufLen, char*  pBuffer );
+extern void     CTCX_ExecuteCCW( DEVBLK* pDEVBLK, BYTE  bCode,
+                                 BYTE    bFlags,  BYTE  bChained,
+                                 U16     sCount,  BYTE  bPrevCode,
+                                 int     iCCWSeq, BYTE* pIOBuf,
+                                 BYTE*   pMore,   BYTE* pUnitStat,
                                  U16*    pResidual );
 
-extern int      CTCI_Init( DEVBLK* pDEVBLK, int argc, BYTE *argv[] );
+extern int      CTCI_Init( DEVBLK* pDEVBLK, int argc, char *argv[] );
 extern int      CTCI_Close( DEVBLK* pDEVBLK );
-extern void     CTCI_Query( DEVBLK* pDEVBLK, BYTE** ppszClass,
-                            int     iBufLen, BYTE*  pBuffer );
-extern void     CTCI_ExecuteCCW( DEVBLK* pDEVBLK, BYTE  bCode, 
-                                 BYTE    bFlags,  BYTE  bChained, 
-                                 U16     sCount,  BYTE  bPrevCode, 
-                                 int     iCCWSeq, BYTE* pIOBuf, 
-                                 BYTE*   pMore,   BYTE* pUnitStat, 
+extern void     CTCI_Query( DEVBLK* pDEVBLK, char** ppszClass,
+                            int     iBufLen, char*  pBuffer );
+extern void     CTCI_ExecuteCCW( DEVBLK* pDEVBLK, BYTE  bCode,
+                                 BYTE    bFlags,  BYTE  bChained,
+                                 U16     sCount,  BYTE  bPrevCode,
+                                 int     iCCWSeq, BYTE* pIOBuf,
+                                 BYTE*   pMore,   BYTE* pUnitStat,
                                  U16*    pResidual );
 
-extern void     CTCI_Read( DEVBLK* pDEVBLK,   U16   sCount, 
-                           BYTE*   pIOBuf,    BYTE* UnitStat, 
+extern void     CTCI_Read( DEVBLK* pDEVBLK,   U16   sCount,
+                           BYTE*   pIOBuf,    BYTE* UnitStat,
                            U16*    pResidual, BYTE* pMore );
-extern void     CTCI_Write( DEVBLK* pDEVBLK,   U16   sCount, 
-                            BYTE*   pIOBuf,    BYTE* UnitStat, 
+extern void     CTCI_Write( DEVBLK* pDEVBLK,   U16   sCount,
+                            BYTE*   pIOBuf,    BYTE* UnitStat,
                             U16*    pResidual );
 
-extern int      LCS_Init( DEVBLK* pDEVBLK, int argc, BYTE *argv[] );
+extern int      LCS_Init( DEVBLK* pDEVBLK, int argc, char *argv[] );
 extern int      LCS_Close( DEVBLK* pDEVBLK );
-extern void     LCS_Query( DEVBLK* pDEVBLK, BYTE** ppszClass,
-                           int     iBufLen, BYTE*  pBuffer );
-extern void     LCS_ExecuteCCW( DEVBLK* pDEVBLK, BYTE  bCode, 
-                                BYTE    bFlags,  BYTE  bChained, 
-                                U16     sCount,  BYTE  bPrevCode, 
-                                int     iCCWSeq, BYTE* pIOBuf, 
-                                BYTE*   pMore,   BYTE* pUnitStat, 
+extern void     LCS_Query( DEVBLK* pDEVBLK, char** ppszClass,
+                           int     iBufLen, char*  pBuffer );
+extern void     LCS_ExecuteCCW( DEVBLK* pDEVBLK, BYTE  bCode,
+                                BYTE    bFlags,  BYTE  bChained,
+                                U16     sCount,  BYTE  bPrevCode,
+                                int     iCCWSeq, BYTE* pIOBuf,
+                                BYTE*   pMore,   BYTE* pUnitStat,
                                 U16*    pResidual );
 
-extern void     LCS_Read( DEVBLK* pDEVBLK,   U16   sCount, 
-                          BYTE*   pIOBuf,    BYTE* UnitStat, 
+extern void     LCS_Read( DEVBLK* pDEVBLK,   U16   sCount,
+                          BYTE*   pIOBuf,    BYTE* UnitStat,
                           U16*    pResidual, BYTE* pMore );
-extern void     LCS_Write( DEVBLK* pDEVBLK,   U16   sCount, 
-                           BYTE*   pIOBuf,    BYTE* UnitStat, 
+extern void     LCS_Write( DEVBLK* pDEVBLK,   U16   sCount,
+                           BYTE*   pIOBuf,    BYTE* UnitStat,
                            U16*    pResidual );
 extern void     LCS_SDC( DEVBLK* pDEVBLK,   BYTE   bOpCode,
-                         U16     sCount,    BYTE*  pIOBuf,    
-                         BYTE*   UnitStat,  U16*   pResidual, 
+                         U16     sCount,    BYTE*  pIOBuf,
+                         BYTE*   UnitStat,  U16*   pResidual,
                          BYTE*   pMore );
 
-extern void     AddDevice( DEVBLK**    ppDEVBLK, 
-                           U16         sDevNum,  
-                           const char* szDevType,
-                           DEVHND*     pDevHnd );
-
 extern int      ParseMAC( char* pszMACAddr, BYTE* pbMACAddr );
 extern void     packet_trace( BYTE *addr, int len );
 
@@ -94,19 +89,31 @@ extern void     packet_trace( BYTE *addr, int len );
 #define FRAME_TYPE_ARP  0x0806
 #define FRAME_TYPE_SNA  0x80D5
 
+#if !(defined(IFHWADDRLEN))             // Only predefined on Linux
+#define IFHWADDRLEN 6                   // Ethernet MAC address length
+#endif /* !(defined(IFHWADDRLEN)) */
+
 typedef uint8_t MAC[IFHWADDRLEN];       // Data Type for MAC Addresses
 
+#if defined(_MSVC_)
+ #pragma pack(push)
+ #pragma pack(1)
+ #define ATTRIBUTE_PACKED
+#else
+ #define ATTRIBUTE_PACKED __attribute__((packed))
+#endif
+
 // ---------------------------------------------------------------------
 // Ethernet Frame Header
 // ---------------------------------------------------------------------
 
-struct _ETHFRM 
+struct _ETHFRM
 {
     MAC         bDestMAC;                // 0x00
     MAC         bSrcMAC;                 // 0x06
     HWORD       hwEthernetType;          // 0x0C
     BYTE        bData[0];                // 0x0E
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKED;
 
 typedef struct _ETHFRM ETHFRM, *PETHFRM;
 
@@ -114,23 +121,23 @@ typedef struct _ETHFRM ETHFRM, *PETHFRM;
 // IP Version 4 Frame Header ( Frame Type 0x0800 )
 // ---------------------------------------------------------------------
 
-struct  _IP4FRM 
+struct  _IP4FRM
 {
     BYTE        bVersion:4;              // 0x00
     BYTE        bHeaderLength:4;         // 0x00
     BYTE        bTOS;                    // 0x01
     HWORD       hwTotalLength;           // 0x02
     HWORD       hwIdentification;        // 0x04
-    U16         
+    U16
         bFlags:3,                        // 0x06
-        sFragmentOffset:13;        
+        sFragmentOffset:13;
     BYTE        bTTL;                    // 0x08
     BYTE        bProtocol;               // 0x09
     HWORD       hwChecksum;              // 0x0A
     U32         lSrcIP;                  // 0x0C
     U32         lDstIP;                  // 0x10
     BYTE        bData[0];                // 0x14
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKED;
 
 typedef struct _IP4FRM IP4FRM, *PIP4FRM;
 
@@ -138,7 +145,7 @@ typedef struct _IP4FRM IP4FRM, *PIP4FRM;
 // Address Resolution Protocol Frame ( Frame Type 0x0806 )
 // ---------------------------------------------------------------------
 
-struct  _ARPFRM 
+struct  _ARPFRM
 {
     HWORD       hwHardwareType;          // 0x00
     HWORD       hwProtocolType;          // 0x02
@@ -149,7 +156,7 @@ struct  _ARPFRM
     U32         lSendIPAddr;             // 0x12
     MAC         bTargEthAddr;            // 0x16
     U32         lTargIPAddr;             // 0x1C
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKED;
 
 typedef struct _ARPFRM ARPFRM, *PARPFRM;
 
@@ -162,7 +169,28 @@ typedef struct _ARPFRM ARPFRM, *PARPFRM;
 // CTCI Definitions
 // ====================================================================
 
-#define CTC_READ_TIMEOUT_SECS  (5)      // five seconds
+#define CTC_READ_TIMEOUT_SECS  (5)       // five seconds
+
+#define CTC_DELAY_USECS        (100000)  // 100 millisecond delay; used
+                                         // mostly by enqueue frame buffer
+                                         // full delay loop...
+
+#define CTC_FRAME_BUFFER_SIZE  (0x5000)  // 20K CTCI/LCS frame buffer
+
+#define MAX_CTCI_FRAME_SIZE     \
+    (                           \
+        CTC_FRAME_BUFFER_SIZE   \
+        - sizeof( CTCIHDR )     \
+        - sizeof( CTCISEG )     \
+        - 2                     \
+    )
+
+#define MAX_LCS_FRAME_SIZE      \
+    (                           \
+        CTC_FRAME_BUFFER_SIZE   \
+        - sizeof( PLCSETHFRM )  \
+        - 2                     \
+    )
 
 struct  _CTCBLK;
 struct  _CTCIHDR;
@@ -173,7 +201,7 @@ typedef struct _CTCIHDR CTCIHDR,*PCTCIHDR;
 typedef struct _CTCISEG CTCISEG,*PCTCISEG;
 
 // --------------------------------------------------------------------
-// CTCBLK - 
+// CTCBLK -
 // --------------------------------------------------------------------
 
 struct  _CTCBLK
@@ -183,10 +211,10 @@ struct  _CTCBLK
     pid_t       pid;                      // Read Thread pid
 
     DEVBLK*     pDEVBLK[2];               // 0 - Read subchannel
-                                          // 1 - Write cubchannel
+                                          // 1 - Write subchannel
 
     U16         iMaxFrameBufferSize;
-    BYTE        bFrameBuffer[0x5000];
+    BYTE        bFrameBuffer[CTC_FRAME_BUFFER_SIZE];
     U16         iFrameOffset;
     U16         sMTU;                     // Max MTU
 
@@ -199,8 +227,9 @@ struct  _CTCBLK
         fOldFormat:1,                     // Old Config Format
         fCreated:1,                       // Interface Created
         fStarted:1,                       // Startup Received
-        fDataPending:1;                   // Data is pending for
+        fDataPending:1,                   // Data is pending for
                                           //   read device
+        fCloseInProgress:1;               // Close in progress
 
     int         iKernBuff;                // Kernel buffer in K bytes.
     int         iIOBuff;                  // I/O buffer in K bytes.
@@ -221,7 +250,7 @@ struct _CTCIHDR                         // CTCI Block Header
 {
     HWORD   hwOffset;                   // Offset of next block
     BYTE    bData[0];                   // Beginning of data
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKED;
 
 struct _CTCISEG                         // CTCI Segment Header
 {
@@ -230,7 +259,7 @@ struct _CTCISEG                         // CTCI Segment Header
     HWORD   hwType;                     // Ethernet packet type
     HWORD   _reserved;                  // Unused, set to zeroes
     BYTE    bData[0];                   // Beginning of data
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKED;
 
 // ====================================================================
 // LCS Definitions
@@ -259,7 +288,7 @@ typedef struct  _LCSRTE     LCSRTE,     *PLCSRTE;
 typedef struct  _LCSHDR     LCSHDR,     *PLCSHDR;
 typedef struct  _LCSSTDFRM  LCSSTDFRM,  *PLCSSTDFRM;
 typedef struct  _LCSSTRTFRM LCSSTRTFRM, *PLCSSTRTFRM;
-typedef struct  _LCSQIPFRM  LCSQIPFRM,  *PLCSQIPFRM;  
+typedef struct  _LCSQIPFRM  LCSQIPFRM,  *PLCSQIPFRM;
 typedef struct  _LCSLSTFRM  LCSLSTFRM,  *PLCSLSTFRM;
 typedef struct  _LCSIPMPAIR LCSIPMPAIR, *PLCSIPMPAIR;
 typedef struct  _LCSIPMFRM  LCSIPMFRM,  *PLCSIPMFRM;
@@ -275,7 +304,7 @@ struct  _LCSDEV
     BYTE        bMode;                    // LCSDEV_MODE_XXXXX
     BYTE        bPort;                    // Relative Adapter No.
     BYTE        bType;                    // LCSDEV_TYPE_XXXXX
-    BYTE*       pszIPAddress;             // IP Address (string)
+    char*       pszIPAddress;             // IP Address (string)
     U32         lIPAddress;               // IP Address (binary)
 
     PLCSBLK     pLCSBLK;                  // -> LCSBLK
@@ -283,7 +312,7 @@ struct  _LCSDEV
                                           // 1 - Write cubchannel
 
     U16         iMaxFrameBufferSize;      // Device Buffer Size
-    BYTE        bFrameBuffer[0x5000];     // Device Buffer
+    BYTE        bFrameBuffer[CTC_FRAME_BUFFER_SIZE];
     U16         iFrameOffset;             // Curr Offset into Buffer
 
     LOCK        Lock;                     // Data LOCK
@@ -330,7 +359,8 @@ struct  _LCSPORT
       fLocalMAC:1,                        // MAC is specified in OAT
       fCreated:1,                         // Interface Created
       fStarted:1,                         // Startup Received
-      fRouteAdded:1;                      // Routing Added
+      fRouteAdded:1,                      // Routing Added
+      fCloseInProgress:1;                 // Close in progress
 
     int         fd;                       // TUN/TAP fd
     TID         tid;                      // Read Thread ID
@@ -368,6 +398,8 @@ struct  _LCSBLK
     BYTE
       fDebug:1;
 
+    int         icDevices;                // Number of devices
+
     PLCSDEV     pDevices;                 // -> Device chain
     LCSPORT     Port[LCS_MAX_PORTS];      // Port Blocks
 
@@ -392,7 +424,7 @@ struct _LCSHDR
 
     BYTE        bLanType;
     BYTE        bRelAdapterNo;
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKED;
 
 #define LCS_FRAME_TYPE_CNTL  0x00          // LCS command mode
 #define LCS_FRAME_TYPE_ENET  0x01
@@ -442,7 +474,7 @@ struct _LCSSTDFRM
     BYTE        bOperatorFlags[3];
     BYTE        _reserved[3];
     BYTE        bData[0];
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKED;
 
 // ---------------------------------------------------------------------
 // LCS Startup Command Frame
@@ -464,7 +496,7 @@ struct _LCSSTRTFRM
 
     HWORD       hwBufferSize;
     BYTE        _unused2[6];
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKED;
 
 // ---------------------------------------------------------------------
 // LCS Query IP Assists Frame
@@ -488,7 +520,7 @@ struct  _LCSQIPFRM
     HWORD       hwIPAssistsSupported;
     HWORD       hwIPAssistsEnabled;
     HWORD       hwIPVersion;
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKED;
 
 #define LCS_ARP_PROCESSING            0x0001
 #define LCS_INBOUND_CHECKSUM_SUPPORT  0x0002
@@ -527,7 +559,7 @@ struct  _LCSLSTFRM
     FWORD       fwRX_Errors;
     FWORD       fwRX_DiscardedNoBuffs;
     U32         fwRX_DiscardedTooLarge;
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKED;
 
 // ---------------------------------------------------------------------
 // LCS IPM Command Frame
@@ -538,7 +570,7 @@ struct  _LCSIPMPAIR
     U32         IP_Addr;
     MAC         MAC_Address;
     BYTE        _reserved[2];
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKED;
 
 #define MAX_IP_MAC_PAIRS      32
 
@@ -562,22 +594,26 @@ struct  _LCSIPMFRM
     U16         hwIPVersion;
     LCSIPMPAIR  IP_MAC_Pair[MAX_IP_MAC_PAIRS];
     U32         fwResponseData;
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKED;
 
 // ---------------------------------------------------------------------
 // LCS Ethernet Passthru Frame
 // ---------------------------------------------------------------------
 
-struct  _LCSETHFRM 
+struct  _LCSETHFRM
 {
     HWORD       hwOffset;
     BYTE        bType;
     BYTE        bSlot;
-    BYTE        bData[0];
-    MAC         bDestMAC;
-    MAC         bSrcMAC;
-    HWORD       hwEthernetType;
-} __attribute__ ((packed));
+    BYTE        bData[14];
+//  MAC         bDestMAC;
+//  MAC         bSrcMAC;
+//  HWORD       hwEthernetType;
+} ATTRIBUTE_PACKED;
+
+#if defined(_MSVC_)
+ #pragma pack(pop)
+#endif
 
 // ====================================================================
 // Inlines
@@ -623,7 +659,7 @@ static inline void SetCIWInfo( DEVBLK* pDEVBLK,
 
     pSIDInfo += 8;
     pSIDInfo += ( bOffset * 4 );
-    
+
     *pSIDInfo++ = bCIWType | 0x40;
     *pSIDInfo++ = bCIWOp;
     *pSIDInfo++ = (BYTE)(( wCIWCount >> 8 ) & 0x00FF );
