@@ -1,27 +1,28 @@
-/* DASDCOPY.C   (c) Copyright Roger Bowler, 1999-2003                */
+/* DASDCOPY.C   (c) Copyright Roger Bowler, 1999-2005                */
 /*       Copy a dasd file to another dasd file.                      */
 /*       Input file and output file may be compressed or not.        */
 /*       Files may be either ckd (or cckd) or fba (or cfba) but      */
 /*       file types (ckd/cckd or fba/cfba) may not be mixed.         */
 /*-------------------------------------------------------------------*/
 
+#include "hstdinc.h"
+
 #include "hercules.h"
 #include "dasdblks.h"
+#include "devtype.h"
+#include "opcode.h"
+
+#define FBA_BLKGRP_SIZE  (120 * 512)    /* Size of block group       */
+#define FBA_BLKS_PER_GRP        120     /* Blocks per group          */
 
 int syntax (char *);
 void status (int, int);
-int nulltrk(BYTE *, int, int);
-
-#ifdef EXTERNALGUI
-/* Special flag to indicate whether or not we're being
-   run under the control of the external GUI facility. */
-int  extgui = 0;
-#endif /*EXTERNALGUI*/
+int nulltrk(BYTE *, int, int, int);
 
 #define CKD      0x01
 #define CCKD     0x02
 #define FBA      0x04
-#define CFBA     0x06
+#define CFBA     0x08
 #define CKDMASK  0x03
 #define FBAMASK  0x0c
 #define COMPMASK 0x0a
@@ -46,15 +47,19 @@ char           *ifile, *ofile;          /* -> Input/Output file names*/
 char           *sfile=NULL;             /* -> Input shadow file name */
 CIFBLK         *icif, *ocif;            /* -> Input/Output CIFBLK    */
 DEVBLK         *idev, *odev;            /* -> Input/Output DEVBLK    */
+
 CKDDEV         *ckd=NULL;               /* -> CKD device table entry */
 FBADEV         *fba=NULL;               /* -> FBA device table entry */
 int             i, n, max;              /* Loop index, limits        */
 BYTE            unitstat;               /* Device unit status        */
-char            buf[512];               /* FBA i/o buffer            */
+char            msgbuf[512];            /* Message buffer            */
+size_t          fba_bytes_remaining=0;  /* FBA bytes to be copied    */
+int             nullfmt = CKDDASD_NULLTRK_FMT0; /* Null track format */
+BYTE            pathname[MAX_PATH];     /* file path in host format  */
 
 #if defined(ENABLE_NLS)
     setlocale(LC_ALL, "");
-    bindtextdomain(PACKAGE, LOCALEDIR);
+    bindtextdomain(PACKAGE, HERC_LOCALEDIR);
     textdomain(PACKAGE);
 #endif
 
@@ -63,11 +68,14 @@ char            buf[512];               /* FBA i/o buffer            */
     {
         extgui = 1;
         argc--;
+        setvbuf(stderr, NULL, _IONBF, 0);
+        setvbuf(stdout, NULL, _IONBF, 0);
     }
 #endif /*EXTERNALGUI*/
 
     /* Figure out processing based on the program name */
-    pgm = strrchr (argv[0], '/');
+    hostpath(pathname, argv[0], sizeof(pathname));
+    pgm = strrchr (pathname, '/');
     if (pgm) pgm++;
     else pgm = argv[0];
     strtok (pgm, ".");
@@ -98,8 +106,8 @@ char            buf[512];               /* FBA i/o buffer            */
         if (argv[0][0] != '-') break;
         if (strcmp(argv[0], "-v") == 0)
         {
-             snprintf (buf, 512, _("Hercules %s copy program "), pgm);
-             display_version (stderr, buf);
+             snprintf (msgbuf, 512, _("Hercules %s copy program "), pgm);
+             display_version (stderr, msgbuf, FALSE);
              return 0;
         }
         else if (strcmp(argv[0], "-h") == 0)
@@ -109,7 +117,7 @@ char            buf[512];               /* FBA i/o buffer            */
         }
         else if (strcmp(argv[0], "-q") == 0
               || strcmp(argv[0], "-quiet") == 0)
-            quiet = 1;  
+            quiet = 1;
         else if (strcmp(argv[0], "-r") == 0)
             r = 1;
 #ifdef CCKD_COMPRESS_ZLIB
@@ -178,7 +186,9 @@ char            buf[512];               /* FBA i/o buffer            */
     /* If we don't know what the input file is then find out */
     if (in == 0)
     {
-        fd = open (ifile, O_RDONLY|O_BINARY);
+        BYTE buf[8];
+        hostpath(pathname, ifile, sizeof(pathname));
+        fd = open (pathname, O_RDONLY|O_BINARY);
         if (fd < 0)
         {
             fprintf (stderr, _("HHCDC001E %s: %s open error: %s\n"),
@@ -233,7 +243,7 @@ char            buf[512];               /* FBA i/o buffer            */
 
     /* Perform sanity checks on the options */
     if ((in & CKDMASK) && !(out & CKDMASK)) return syntax(pgm);
-    if ((in & FBAMASK) && !(out & FBAMASK)) return syntax(pgm);        
+    if ((in & FBAMASK) && !(out & FBAMASK)) return syntax(pgm);
     if (sfile && !(in & COMPMASK)) return syntax(pgm);
     if (comp != 255 && !(out & COMPMASK)) return syntax(pgm);
     if (lfs && (out & COMPMASK)) return syntax(pgm);
@@ -255,12 +265,13 @@ char            buf[512];               /* FBA i/o buffer            */
         return -1;
     }
     idev = &icif->devblk;
+    if (idev->oslinux) nullfmt = CKDDASD_NULLTRK_FMT2;
 
     /* Calculate the number of tracks or blocks to copy */
     if (ckddasd)
     {
         if (cyls < 0) cyls = idev->ckdcyls;
-        else if (cyls == 0) cyls = (idev->ckdused)(idev);
+        else if (cyls == 0) cyls = (idev->hnd->used)(idev);
         ckd = dasd_lookup (DASD_CKDDEV, NULL, idev->devtype, cyls);
         if (ckd == NULL)
         {
@@ -278,8 +289,9 @@ char            buf[512];               /* FBA i/o buffer            */
     }
     else
     {
+        fba_bytes_remaining = idev->fbanumblk * idev->fbablksiz;
         if (blks < 0) blks = idev->fbanumblk;
-        else if (blks == 0) blks = (idev->fbaused)(idev);
+        else if (blks == 0) blks = (idev->hnd->used)(idev);
         fba = dasd_lookup (DASD_FBADEV, NULL, idev->devtype, blks);
         if (fba == NULL)
         {
@@ -291,12 +303,14 @@ char            buf[512];               /* FBA i/o buffer            */
         n = blks;
         max = idev->fbanumblk;
         if (max < n && out == CFBA) n = max;
+        n = (n + FBA_BLKS_PER_GRP - 1) / FBA_BLKS_PER_GRP;
+        max = (max + FBA_BLKS_PER_GRP - 1) / FBA_BLKS_PER_GRP;
     }
 
     /* Create the output file */
     if (ckddasd)
         rc = create_ckd(ofile, idev->devtype, idev->ckdheads,
-                        ckd->r1, cyls, "", comp, lfs, 1+r);
+                        ckd->r1, cyls, "", comp, lfs, 1+r, nullfmt);
     else
         rc = create_fba(ofile, idev->devtype, fba->size,
                         blks, "", comp, lfs, 1+r);
@@ -321,45 +335,73 @@ char            buf[512];               /* FBA i/o buffer            */
     odev = &ocif->devblk;
 
     /* Copy the files */
-    if (!quiet) printf (_("  %3d%% %7d of %d"), 0, 0, n);
+#ifdef EXTERNALGUI
+    if (extgui)
+        /* Notify GUI of total #of tracks or blocks being copied... */
+        fprintf (stderr, "TRKS=%d\n", n);
+    else
+#endif /*EXTERNALGUI*/
+    if (!quiet) printf ("  %3d%% %7d of %d", 0, 0, n);
     for (i = 0; i < n; i++)
     {
         /* Read a track or block */
         if (ckddasd)
         {
             if (i < max)
-                rc = (idev->ckdrdtrk)(idev, i / idev->ckdheads,
-                                      i % idev->ckdheads, &unitstat);
-            else rc = nulltrk(&idev->buf[idev->bufoff], i, idev->ckdheads);
+                rc = (idev->hnd->read)(idev, i, &unitstat);
+            else
+            {
+                memset (idev->buf, 0, idev->ckdtrksz);
+                rc = nulltrk(idev->buf, i, idev->ckdheads, nullfmt);
+            }
         }
         else
         {
             if (i < max)
-                rc = (idev->fbardblk)(idev, buf, fba->size, &unitstat);
+                rc = (idev->hnd->read)(idev, i, &unitstat);
             else
-                memset (buf, 0, 512);
+                memset (idev->buf, 0, FBA_BLKGRP_SIZE);
                 rc = 0;
         }
         if (rc < 0)
         {
             fprintf (stderr, _("HHCDC008E %s: %s read error %s %d "
-                               "stat=%2.2X\n"),
-                     pgm, ifile, ckddasd ? "track" : "block", i, unitstat);
-            close_image_file(icif); close_image_file(ocif);
-            return -1;
+                               "stat=%2.2X, null %s substituted\n"),
+                     pgm, ifile, ckddasd ? "track" : "block", i, unitstat,
+                     ckddasd ? "track" : "block");
+            if (ckddasd)
+                nulltrk(idev->buf, i, idev->ckdheads, nullfmt);
+            else
+                memset (idev->buf, 0, FBA_BLKGRP_SIZE);
+            if (!quiet)
+            {
+                printf (_("  %3d%% %7d of %d"), 0, 0, n);
+                status (i, n);
+            }
         }
 
         /* Write the track or block just read */
         if (ckddasd)
         {
-            rc = (odev->ckdrdtrk)(odev, i / idev->ckdheads,
-                  i % idev->ckdheads, &unitstat);
-            if (rc == 0)
-                rc = (odev->ckdupdtrk)(odev, &idev->buf[idev->bufoff],
+            rc = (odev->hnd->write)(odev, i, 0, idev->buf,
                       idev->ckdtrksz, &unitstat);
         }
         else
-            rc = (odev->fbawrblk)(odev, buf, fba->size, &unitstat);
+        {
+            if (fba_bytes_remaining >= (size_t)idev->buflen)
+            {
+                rc = (odev->hnd->write)(odev,  i, 0, idev->buf,
+                          idev->buflen, &unitstat);
+                fba_bytes_remaining -= (size_t)idev->buflen;
+            }
+            else
+            {
+                ASSERT(fba_bytes_remaining > 0 && (i+1) >= n);
+                rc = (odev->hnd->write)(odev,  i, 0, idev->buf,
+                          (int)fba_bytes_remaining, &unitstat);
+                fba_bytes_remaining = 0;
+            }
+        }
         if (rc < 0)
         {
             fprintf (stderr, _("HHCDC009E %s: %s write error %s %d "
@@ -373,36 +415,83 @@ char            buf[512];               /* FBA i/o buffer            */
         if (!quiet) status (i+1, n);
     }
 
-    if (!quiet) printf (_("\nHHCDC010I Copy successful !!!       \n"));
     close_image_file(icif); close_image_file(ocif);
-
+    if (!quiet) printf (_("\nHHCDC010I Copy successful !!!       \n"));
     return 0;
 }
 
 /*-------------------------------------------------------------------*/
 /* Build a null track image                                          */
 /*-------------------------------------------------------------------*/
-int nulltrk(BYTE *buf, int trk, int heads)
+int nulltrk(BYTE *buf, int trk, int heads, int nullfmt)
 {
-    BYTE cchh[4];
-    int  cyl, head;
+int             i;                      /* Loop counter              */
+CKDDASD_TRKHDR *trkhdr;                 /* -> Track header           */
+CKDDASD_RECHDR *rechdr;                 /* -> Record header          */
+U32             cyl;                    /* Cylinder number           */
+U32             head;                   /* Head number               */
+BYTE            r;                      /* Record number             */
+BYTE           *pos;                    /* -> Next position in buffer*/
     static BYTE eighthexFF[]={0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff};
 
     /* cylinder and head calculations */
     cyl = trk / heads;
     head = trk % heads;
-    cchh[0] = cyl >> 8;
-    cchh[1] = cyl & 0xFF;
-    cchh[2] = head >> 8;
-    cchh[3] = head & 0xFF;
 
-    /* A null track has a 5 byte track hdr, 8 byte r0 count,
-       8 byte r0 data and 8 ff's */
-    memset(buf, 0, 29);
-    memcpy (&buf[1], cchh, sizeof(cchh));
-    memcpy (&buf[5], cchh, sizeof(cchh));
-    buf[12] = 8;
-    memcpy (&buf[21], eighthexFF, 8);
+    /* Build the track header */
+    trkhdr = (CKDDASD_TRKHDR*)buf;
+    trkhdr->bin = 0;
+    store_hw(&trkhdr->cyl, cyl);
+    store_hw(&trkhdr->head, head);
+    pos = buf + CKDDASD_TRKHDR_SIZE;
+
+    /* Build record zero */
+    r = 0;
+    rechdr = (CKDDASD_RECHDR*)pos;
+    pos += CKDDASD_RECHDR_SIZE;
+    store_hw(&rechdr->cyl, cyl);
+    store_hw(&rechdr->head, head);
+    rechdr->rec = r;
+    rechdr->klen = 0;
+    store_hw(&rechdr->dlen, 8);
+    pos += 8;
+    r++;
+
+    /* Specific null track formatting */
+    if (nullfmt == CKDDASD_NULLTRK_FMT0)
+    {
+        rechdr = (CKDDASD_RECHDR*)pos;
+        pos += CKDDASD_RECHDR_SIZE;
+
+        store_hw(&rechdr->cyl, cyl);
+        store_hw(&rechdr->head, head);
+        rechdr->rec = r;
+        rechdr->klen = 0;
+        store_hw(&rechdr->dlen, 0);
+        r++;
+    }
+    else if (nullfmt == CKDDASD_NULLTRK_FMT2)
+    {
+        for (i = 0; i < 12; i++)
+        {
+            rechdr = (CKDDASD_RECHDR*)pos;
+            pos += CKDDASD_RECHDR_SIZE;
+
+            store_hw(&rechdr->cyl, cyl);
+            store_hw(&rechdr->head, head);
+            rechdr->rec = r;
+            rechdr->klen = 0;
+            store_hw(&rechdr->dlen, 4096);
+            r++;
+
+            pos += 4096;
+        }
+    }
+ 
+    /* Build the end of track marker */
+    memcpy (pos, eighthexFF, 8);
+    pos += 8;
+
     return 0;
 }
 
@@ -464,10 +553,12 @@ int syntax (char *pgm)
             "     -h                display this help and quit\n"
             "     -q                quiet mode, don't display status\n"
             "     -r                replace the output file if it exists\n"
-            "     -lfs              create single large output file\n"
+            "%s"
             "     -cyls  n          size of output file\n"
             "     -a                output file will have alt cyls\n"
-            ));
+            ),
+            (sizeof(OFF_T) > 4) ? _( "     -lfs              create single large output file\n" ) : ( "" )
+            );
     else if (strcmp(pgm, "fba2cfba") == 0)
         snprintf(usage,8192,_(
             "usage:  fba2cfba [-options] ifile ofile\n"
@@ -518,9 +609,11 @@ int syntax (char *pgm)
             "     -h                display this help and quit\n"
             "     -q                quiet mode, don't display status\n"
             "     -r                replace the output file if it exists\n"
-            "     -lfs              create single large output file\n"
+            "%s"
             "     -blks  n          size of output file\n"
-            ));
+            ),
+            (sizeof(OFF_T) > 4) ? _( "     -lfs              create single large output file\n" ) : ( "" )
+            );
     else
         snprintf(usage,8192,_(
             "usage:  %s [-options] ifile [sf=sfile] ofile\n"
@@ -542,7 +635,7 @@ int syntax (char *pgm)
             "     -blks  n          size of output fba file\n"
             "     -cyls  n          size of output ckd file\n"
             "     -a                output ckd file will have alt cyls\n"
-            "     -lfs              output ckd file will be a single file\n"
+            "%s"
             "                       even if it exceeds 2G in size\n"
             "     -o     type       output file type (CKD, CCKD, FBA, CFBA)\n"
             ),
@@ -561,6 +654,7 @@ int syntax (char *pgm)
 #else
             ""
 #endif
+            ,(sizeof(OFF_T) > 4) ? _( "     -lfs              output ckd file will be a single file\n" ) : ( "" )
             );
     printf (usage);
     return -1;
@@ -578,8 +672,8 @@ static char indic[] = "|/-\\";
     {
         if (i % 100) return;
         fprintf (stderr, "TRK=%d\n", i);
-        return; 
-    } 
+        return;
+    }
 #endif /*EXTERNALGUI*/
 //  if (i % 101 != 1) return;
     printf ("\r%c %3d%% %7d", indic[i%4], (int)((i*100.0)/n), i);
