@@ -1,4 +1,4 @@
-/* DASDINIT.C   (c) Copyright Roger Bowler, 1999-2003                */
+/* DASDINIT.C   (c) Copyright Roger Bowler, 1999-2005                */
 /*              Hercules DASD Utilities: DASD image builder          */
 
 /*-------------------------------------------------------------------*/
@@ -35,23 +35,51 @@
 /*              Shouldn't be specified if model is specified.        */ 
 /*                                                                   */
 /*-------------------------------------------------------------------*/
+#include "hstdinc.h"
 
 #include "hercules.h"
 #include "dasdblks.h"
 
-#ifdef EXTERNALGUI
-/* Special flag to indicate whether or not we're being
-   run under the control of the external GUI facility. */
-int  extgui = 0;
-#endif /*EXTERNALGUI*/
-
 /*-------------------------------------------------------------------*/
 /* Subroutine to display command syntax and exit                     */
 /*-------------------------------------------------------------------*/
 static void
-argexit ( int code )
+argexit ( int code, char *m )
 {
-    fprintf (stderr,
+
+    switch (code) {
+    case 0:
+        fprintf (stderr, "Invalid or unsupported option: %s\n",
+                 m ? m : "(null)");
+        break;
+    case 1:
+        fprintf (stderr, "Invalid or missing filename: %s\n",
+                 m ? m : "(null)");
+        break;
+    case 2:
+        fprintf (stderr, "Invalid or missing device type: %s\n",
+                 m ? m : "(null)");
+        break;
+    case 3:
+        fprintf (stderr, "Invalid or missing volser: %s\n",
+                 m ? m : "(null)");
+        break;
+    case 4:
+        fprintf (stderr, "Invalid or missing size: %s\n",
+                 m ? m : "(null)");
+        break;
+    case 5:
+        fprintf (stderr, "Invalid number of arguments\n");
+        break;
+    case 6:
+        fprintf (stderr, "`-linux' only supported for device type 3390\n");
+        break;
+    default:
+
+        display_version (stderr,
+                     "Hercules DASD image file creation program\n", FALSE);
+
+        fprintf (stderr,
 
 "Builds an empty dasd image file:\n\n"
 
@@ -59,18 +87,24 @@ argexit ( int code )
 
 "where:\n\n"
 
-#ifdef CCKD_COMPRESS_ZLIB
+"  -v         display version info and help\n"
+#ifdef HAVE_LIBZ
 "  -z         build compressed dasd image file using zlib\n"
 #endif
-#ifdef CCKD_COMPRESS_BZIP2
+#ifdef CCKD_BZIP2
 "  -bz2       build compressed dasd image file using bzip2\n"
 #endif
 "  -0         build compressed dasd image file with no compression\n"
-#if _FILE_OFFSET_BITS == 64 || defined(_LARGE_FILES)
-"  -lfs       build a large (uncompressed) dasd file\n"
-#endif
+);
+        if (sizeof(OFF_T) > 4) fprintf(stderr,
+"  -lfs       build a large (uncompressed) dasd file (if supported)\n"
+);
+        fprintf(stderr,
 "  -a         build dasd image file that includes alternate cylinders\n"
-"             (option ignored if size is manually specified)\n\n"
+"             (option ignored if size is manually specified)\n"
+
+"  -linux     null track images will look like linux dasdfmt'ed images\n"
+"             (3390 device type only)\n\n"
 
 "  filename   name of dasd image file to be created\n\n"
 
@@ -85,6 +119,8 @@ argexit ( int code )
 "             (required if model not specified else optional)\n"
 
 );
+        break;
+    }
     exit(code);
 } /* end function argexit */
 
@@ -102,57 +138,60 @@ U32     sectsize = 0;                   /* Sector size               */
 U16     devtype = 0;                    /* Device type               */
 BYTE    comp = 0xff;                    /* Compression algoritm      */
 BYTE    type = 0;                       /* C=CKD, F=FBA              */
-BYTE    fname[1024];                    /* File name                 */
-BYTE    volser[7];                      /* Volume serial number      */
+char    fname[1024];                    /* File name                 */
+char    volser[7];                      /* Volume serial number      */
 BYTE    c;                              /* Character work area       */
 CKDDEV *ckd;                            /* -> CKD device table entry */
 FBADEV *fba;                            /* -> FBA device table entry */
 int     lfs = 0;                        /* 1 = Build large file      */
+int     nullfmt = CKDDASD_NULLTRK_FMT1; /* Null track format type    */
 
 #ifdef EXTERNALGUI
     if (argc >= 1 && strncmp(argv[argc-1],"EXTERNALGUI",11) == 0)
     {
         extgui = 1;
         argc--;
+        setvbuf(stderr, NULL, _IONBF, 0);
+        setvbuf(stdout, NULL, _IONBF, 0);
     }
 #endif /*EXTERNALGUI*/
 
-    /* Display the program identification message */
-    display_version (stderr,
-                     "Hercules DASD image file creation program\n");
+    /* Display program identification and help */
+    if (argc <= 1 || (argc == 2 && !strcmp(argv[1], "-v")))
+        argexit(-1, NULL);
 
     /* Process optional arguments */
     for ( ; argc > 1 && argv[1][0] == '-'; argv++, argc--) 
     {
         if (strcmp("0", &argv[1][1]) == 0)
             comp = CCKD_COMPRESS_NONE;
-#ifdef CCKD_COMPRESS_ZLIB
+#ifdef HAVE_LIBZ
         else if (strcmp("z", &argv[1][1]) == 0)
             comp = CCKD_COMPRESS_ZLIB;
 #endif
-#ifdef CCKD_COMPRESS_BZIP2
+#ifdef CCKD_BZIP2
         else if (strcmp("bz2", &argv[1][1]) == 0)
             comp = CCKD_COMPRESS_BZIP2;
 #endif
         else if (strcmp("a", &argv[1][1]) == 0)
             altcylflag = 1;
-#if _FILE_OFFSET_BITS == 64 || defined(_LARGE_FILES)
-        else if (strcmp("lfs", &argv[1][1]) == 0)
+        else if (strcmp("lfs", &argv[1][1]) == 0 && sizeof(OFF_T) > 4)
             lfs = 1;
-#endif
-        else argexit(0);
+        else if (strcmp("linux", &argv[1][1]) == 0)
+            nullfmt = CKDDASD_NULLTRK_FMT2;
+        else argexit(0, argv[1]);
     }
 
     /* Check remaining number of arguments */
 
     if (argc < 4 || argc > 5)
-        argexit(5);
+        argexit(5, NULL);
 
     /* The first argument is the file name */
 
     if (!argv[1] || strlen(argv[1]) == 0
         || strlen(argv[1]) > sizeof(fname)-1)
-        argexit(1);
+        argexit(1, argv[1]);
 
     strcpy (fname, argv[1]);
 
@@ -160,7 +199,7 @@ int     lfs = 0;                        /* 1 = Build large file      */
        with or without the model number. */
 
     if (!argv[2])
-        argexit(2);
+        argexit(2, argv[2]);
     ckd = dasd_lookup (DASD_CKDDEV, argv[2], 0, 0);
     if (ckd != NULL)
     {
@@ -186,12 +225,12 @@ int     lfs = 0;                        /* 1 = Build large file      */
 
     if (!type)
         /* Specified model not found */
-        argexit(2);
+        argexit(2, argv[2]);
 
     /* The third argument is the volume serial number */
     if (!argv[3] || strlen(argv[3]) == 0
         || strlen(argv[3]) > sizeof(volser)-1)
-        argexit(3);
+        argexit(3, argv[3]);
 
     strcpy (volser, argv[3]);
     string_to_upper (volser);
@@ -199,21 +238,20 @@ int     lfs = 0;                        /* 1 = Build large file      */
     /* The fourth argument is the volume size */
     if (argc > 4)
     {
-        U32 requested_size = 0;
-
         if (argc > 5)
-            argexit(5);
+            argexit(5, NULL);
 
         if (!argv[4] || strlen(argv[4]) == 0
-            || sscanf(argv[4], "%u%c", &requested_size, &c) != 1)
-            argexit(4);
-
-        /* Use requested size only if no compression or FBA */
-        if (0xff == comp || type == 'F') size = requested_size;
+            || sscanf(argv[4], "%u%c", &size, &c) != 1)
+            argexit(4, argv[4]);
 
         altcylflag = 0;
     }
 
+    /* `-linux' only supported for 3390 device type */
+    if (nullfmt == CKDDASD_NULLTRK_FMT2 && devtype != 0x3390)
+        argexit(6, NULL);
+
     if (altcylflag)
         size += altsize;
 
@@ -221,7 +259,7 @@ int     lfs = 0;                        /* 1 = Build large file      */
 
     if (type == 'C')
         create_ckd (fname, devtype, heads, maxdlen, size, volser,
-                    comp, lfs, 0);
+                    comp, lfs, 0, nullfmt);
     else
         create_fba (fname, devtype, sectsize, size, volser, comp,
                     lfs, 0);
