@@ -1,4 +1,4 @@
-/* DASDISUP.C   (c) Copyright Roger Bowler, 1999-2003                */
+/* DASDISUP.C   (c) Copyright Roger Bowler, 1999-2005                */
 /*              Hercules DASD Utilities: IEHIOSUP                    */
 
 /*-------------------------------------------------------------------*/
@@ -11,6 +11,8 @@
 /* where: ckdfile is the name of the CKD image file                  */
 /*-------------------------------------------------------------------*/
 
+#include "hstdinc.h"
+
 #include "hercules.h"
 #include "dasdblks.h"
 
@@ -41,7 +43,7 @@ typedef struct _MEMINFO {
 /*-------------------------------------------------------------------*/
 
 /* List of first loads for Open/Close/EOV routines */
-static BYTE *firstload[] = {
+static char *firstload[] = {
         "IGC0001I",                     /* Open (SVC19)              */
         "IGC0002{",                     /* Close (SVC20)             */
         "IGC0002A",                     /* Stow (SVC21)              */
@@ -58,7 +60,7 @@ static BYTE *firstload[] = {
         NULL };                         /* End of list               */
 
 /* List of second loads for Open/Close/EOV routines */
-static BYTE *secondload[] = {
+static char *secondload[] = {
         "IFG019", "IGG019",             /* Open (SVC19)              */
         "IFG020", "IGG020",             /* Close (SVC20)             */
         "IGG021",                       /* Stow (SVC21)              */
@@ -74,12 +76,6 @@ static BYTE *secondload[] = {
 
 static  BYTE eighthexFF[] = {0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff};
 
-#ifdef EXTERNALGUI
-/* Special flag to indicate whether or not we're being
-   run under the control of the external GUI facility. */
-int  extgui = 0;
-#endif /*EXTERNALGUI*/
-
 #if 0
 /*-------------------------------------------------------------------*/
 /* Subroutine to process a member                                    */
@@ -107,6 +103,7 @@ FILE           *ofp;                    /* Output file pointer       */
 BYTE            ofname[256];            /* Output file name          */
 int             offset;                 /* Offset of record in buffer*/
 BYTE            card[81];               /* Logical record (ASCIIZ)   */
+BYTE            pathname[MAX_PATH];     /* ofname in host path format*/
 
     /* Build the output file name */
     memset (ofname, 0, sizeof(ofname));
@@ -115,7 +112,8 @@ BYTE            card[81];               /* Logical record (ASCIIZ)   */
     strcat (ofname, ".mac");
 
     /* Open the output file */
-    ofp = fopen (ofname, "w");
+    hostpath(pathname, ofname, sizeof(pathname));
+    ofp = fopen (pathname, "w");
     if (ofp == NULL)
     {
         fprintf (stderr,
@@ -230,7 +228,7 @@ int             k;                      /* Userdata halfword count   */
 BYTE           *dirptr;                 /* -> Next byte within block */
 int             dirrem;                 /* Number of bytes remaining */
 PDSDIR         *dirent;                 /* -> Directory entry        */
-BYTE            memnama[9];             /* Member name (ASCIIZ)      */
+char            memnama[9];             /* Member name (ASCIIZ)      */
 
     UNREFERENCED(cif);
     UNREFERENCED(noext);
@@ -241,7 +239,7 @@ BYTE            memnama[9];             /* Member name (ASCIIZ)      */
     dirrem = (dirptr[0] << 8) | dirptr[1];
     if (dirrem < 2 || dirrem > 256)
     {
-        fprintf (stdout, "HHCDS003E Directory block byte count is invalid\n");
+        fprintf (stdout, _("HHCDS003E Directory block byte count is invalid\n"));
         return -1;
     }
 
@@ -286,7 +284,7 @@ BYTE            memnama[9];             /* Member name (ASCIIZ)      */
             if (secondload[i] == NULL)
             {
                 fprintf (stdout,
-                        "HHCDS018I %s %s skipped\n",
+                        _("HHCDS018I %s %s skipped\n"),
                         memnama,
                         ((dirent->pds2indc & PDS2INDC_ALIAS) ?
                         "Alias" : "Member"));
@@ -299,7 +297,7 @@ BYTE            memnama[9];             /* Member name (ASCIIZ)      */
         if (n >= MAX_MEMBERS)
         {
             fprintf (stdout,
-                    "HHCDS004E Number of members exceeds MAX_MEMBERS\n");
+                    _("HHCDS004E Number of members exceeds MAX_MEMBERS\n"));
             return -1;
         }
 
@@ -308,7 +306,7 @@ BYTE            memnama[9];             /* Member name (ASCIIZ)      */
                 < 1)
         {
             fprintf (stdout,
-                    "HHCDS005E Member %s TTR count is zero\n", memnama);
+                    _("HHCDS005E Member %s TTR count is zero\n"), memnama);
             return -1;
         }
 
@@ -337,7 +335,7 @@ BYTE            memnama[9];             /* Member name (ASCIIZ)      */
         if ((dirent->pds2usrd[8] & 0x01) == 0 || totlen != txtlen)
         {
             fprintf (stdout,
-                    "HHCDS006W Member %s is not single text record\n",
+                    _("HHCDS006W Member %s is not single text record\n"),
                     memnama);
             memtab[n].multitxt = 1;
         }
@@ -346,8 +344,8 @@ BYTE            memnama[9];             /* Member name (ASCIIZ)      */
         if (totlen > 255*8)
         {
             fprintf (stdout,
-                    "HHCDS007W Member %s size %4.4X "
-                    "exceeds X\'7F8\' bytes\n",
+                    _("HHCDS007W Member %s size %4.4X "
+                    "exceeds X\'7F8\' bytes\n"),
                     memnama, totlen);
         }
 
@@ -355,8 +353,8 @@ BYTE            memnama[9];             /* Member name (ASCIIZ)      */
         if (totlen & 0x7)
         {
             fprintf (stdout,
-                    "HHCDS008W Member %s size %4.4X "
-                    "is not a multiple of 8\n",
+                    _("HHCDS008W Member %s size %4.4X "
+                    "is not a multiple of 8\n"),
                     memnama, totlen);
         }
 
@@ -395,11 +393,11 @@ int             trk;                    /* Relative track number     */
 int             xctloff;                /* Offset to XCTL table      */
 int             warn;                   /* 1=Flag TTRL difference    */
 BYTE           *blkptr;                 /* -> Text record data       */
-BYTE            memnama[9];             /* Member name (ASCIIZ)      */
+char            memnama[9];             /* Member name (ASCIIZ)      */
 BYTE            svcnum[3];              /* SVC number (EBCDIC)       */
 BYTE            prefix[3];              /* IGG/IFG prefix (EBCDIC)   */
 BYTE            refname[8];             /* Referred name (EBCDIC)    */
-BYTE            refnama[9];             /* Referred name (ASCIIZ)    */
+char            refnama[9];             /* Referred name (ASCIIZ)    */
 
     /* Extract the member name */
     make_asciiz (memnama, sizeof(memnama), memp->memname, 8);
@@ -409,14 +407,14 @@ BYTE            refnama[9];             /* Referred name (ASCIIZ)    */
     /* Skip the member if it is an alias */
     if (memp->alias)
     {
-        fprintf (stdout, "HHCDS009I Alias %s skipped\n", memnama);
+        fprintf (stdout, _("HHCDS009I Alias %s skipped\n"), memnama);
         return 0;
     }
 
     /* Skip the member if it has no XCTL table */
     if (memp->notable)
     {
-        fprintf (stdout, "HHCDS010I Member %s skipped\n", memnama);
+        fprintf (stdout, _("HHCDS010I Member %s skipped\n"), memnama);
         return 0;
     }
 
@@ -424,7 +422,7 @@ BYTE            refnama[9];             /* Referred name (ASCIIZ)    */
     if (memp->multitxt)
     {
         fprintf (stdout,
-                "HHCDS011E Member %s has multiple text records\n",
+                _("HHCDS011E Member %s has multiple text records\n"),
                 memnama);
         return -1;
     }
@@ -436,14 +434,14 @@ BYTE            refnama[9];             /* Referred name (ASCIIZ)    */
     if (rc < 0)
     {
         fprintf (stdout,
-                "HHCDS012E Member %s has invalid TTR %4.4X%2.2X\n",
+                _("HHCDS012E Member %s has invalid TTR %4.4X%2.2X\n"),
                 memnama, trk, rec);
         return -1;
     }
 
     fprintf (stdout,
-            "HHCDS013I Processing member %s text record TTR=%4.4X%2.2X "
-            "CCHHR=%4.4X%4.4X%2.2X\n",
+            _("HHCDS013I Processing member %s text record TTR=%4.4X%2.2X "
+            "CCHHR=%4.4X%4.4X%2.2X\n"),
             memnama, trk, rec, cyl, head, rec);
 
     /* Read the text record */
@@ -452,7 +450,7 @@ BYTE            refnama[9];             /* Referred name (ASCIIZ)    */
     if (rc != 0)
     {
         fprintf (stdout,
-                "HHCDS014E Member %s error reading TTR %4.4X%2.2X\n",
+                _("HHCDS014E Member %s error reading TTR %4.4X%2.2X\n"),
                 memnama, trk, rec);
         return -1;
     }
@@ -461,8 +459,8 @@ BYTE            refnama[9];             /* Referred name (ASCIIZ)    */
     if (len < 8 || len > 1024 || (len & 0x7))
     {
         fprintf (stdout,
-                "HHCDS015E Member %s TTR %4.4X%2.2X "
-                "text record length %4.4X is not valid\n",
+                _("HHCDS015E Member %s TTR %4.4X%2.2X "
+                "text record length %4.4X is not valid\n"),
                 memnama, trk, rec, len);
         return -1;
     }
@@ -471,9 +469,9 @@ BYTE            refnama[9];             /* Referred name (ASCIIZ)    */
     if (len != memp->dwdsize * 8)
     {
         fprintf (stdout,
-                "HHCDS016E Member %s TTR %4.4X%2.2X "
+                _("HHCDS016E Member %s TTR %4.4X%2.2X "
                 "text record length %4.4X does not match "
-                "length %4.4X in directory\n",
+                "length %4.4X in directory\n"),
                 memnama, trk, rec, len, memp->dwdsize * 8);
         return -1;
     }
@@ -516,8 +514,8 @@ BYTE            refnama[9];             /* Referred name (ASCIIZ)    */
         if (xctloff >= len - 10)
         {
             fprintf (stdout,
-                    "HHCDS017E Member %s TTR %4.4X%2.2X "
-                    "XCTL table improperly terminated\n",
+                    _("HHCDS017E Member %s TTR %4.4X%2.2X "
+                    "XCTL table improperly terminated\n"),
                     memnama, trk, rec);
             return -1;
         }
@@ -538,7 +536,7 @@ BYTE            refnama[9];             /* Referred name (ASCIIZ)    */
 
         /* Display XCTL table entry */
         fprintf (stdout,
-                "HHCDS019I In member %s: %s TTRL=%2.2X%2.2X%2.2X%2.2X",
+                _("HHCDS019I In member %s: %s TTRL=%2.2X%2.2X%2.2X%2.2X"),
                 memnama, refnama,
                 blkptr[xctloff+2], blkptr[xctloff+3],
                 blkptr[xctloff+4], blkptr[xctloff+5]);
@@ -606,8 +604,8 @@ int             cyl;                    /* Cylinder number           */
 int             head;                   /* Head number               */
 int             rec;                    /* Record number             */
 int             trk;                    /* Relative track number     */
-BYTE           *fname;                  /* -> CKD image file name    */
-BYTE           *sfname;                 /* -> CKD shadow file name   */
+char           *fname;                  /* -> CKD image file name    */
+char           *sfname;                 /* -> CKD shadow file name   */
 int             noext;                  /* Number of extents         */
 DSXTENT         extent[16];             /* Extent descriptor array   */
 BYTE           *blkptr;                 /* -> PDS directory block    */
@@ -615,17 +613,25 @@ CIFBLK         *cif;                    /* CKD image file descriptor */
 MEMINFO        *memtab;                 /* -> Member info array      */
 int             nmem = 0;               /* Number of array entries   */
 
+#if defined(ENABLE_NLS)
+    setlocale(LC_ALL, "");
+    bindtextdomain(PACKAGE, HERC_LOCALEDIR);
+    textdomain(PACKAGE);
+#endif
+
 #ifdef EXTERNALGUI
     if (argc >= 1 && strncmp(argv[argc-1],"EXTERNALGUI",11) == 0)
     {
         extgui = 1;
         argc--;
+        setvbuf(stderr, NULL, _IONBF, 0);
+        setvbuf(stdout, NULL, _IONBF, 0);
     }
 #endif /*EXTERNALGUI*/
 
     /* Display the program identification message */
     display_version (stderr,
-                     "Hercules IEHIOSUP program ");
+                     "Hercules IEHIOSUP program ", FALSE);
 
     /* Check the number of arguments */
     if (argc < 2 || argc > 3)
@@ -648,7 +654,7 @@ int             nmem = 0;               /* Number of array entries   */
     if (memtab == NULL)
     {
         fprintf (stdout,
-                "HHCDS001E Cannot obtain storage for member array: %s\n",
+                _("HHCDS001E Cannot obtain storage for member array: %s\n"),
                 strerror(errno));
         return -1;
     }
@@ -700,7 +706,7 @@ int             nmem = 0;               /* Number of array entries   */
     } /* end while */
 
     fprintf (stdout,
-            "HHCDS002I End of directory: %d members selected\n",
+            _("HHCDS002I End of directory: %d members selected\n"),
             nmem);
 
 #ifdef EXTERNALGUI
