@@ -1,13 +1,22 @@
-/* DASDTAB.C    (c) Copyright Roger Bowler, 1999-2003                */
+/* DASDTAB.C    (c) Copyright Roger Bowler, 1999-2005                */
 /*              Hercules Supported DASD definitions                  */
 
 /*-------------------------------------------------------------------*/
 /* This module contains the tables that define the attributes of     */
 /* each DASD device and control unit supported by Hercules.          */
-/* Routines are also provided to perform table lookup and to build   */
-/* to build the device identifier and characteristics areas.         */
+/* Routines are also provided to perform table lookup and build the  */
+/* device identifier and characteristics areas.                      */
+/*                                                                   */
+/* Note: source for most CKD/FBA device capacities take from SDI's   */
+/* device capacity page at: http://www.sdisw.com/dasd_capacity.html  */
+/* (used with permission)                                            */
 /*-------------------------------------------------------------------*/
 
+#include "hstdinc.h"
+
+#define _DASDTAB_C_
+#define _HDASD_DLL_
+
 #include "hercules.h"
 
 /*-------------------------------------------------------------------*/
@@ -15,8 +24,8 @@
 /*-------------------------------------------------------------------*/
 static CKDDEV ckdtab[] = {
 /* name         type model clas code prime a hd    r0    r1 har0   len sec    rps  f f1  f2   f3   f4 f5 f6  cu */
- {"2311",      0x2311,0x00,0x20,0x00,  200,2,10,    0, 3625,   0, 3625,  0,0x0000,-2,20, 61, 537, 512,  0,0,"2841"},
- {"2311-1",    0x2311,0x00,0x20,0x00,  200,2,10,    0, 3625,   0, 3625,  0,0x0000,-2,20, 61, 537, 512,  0,0,"2841"},
+ {"2311",      0x2311,0x00,0x20,0x00,  200,3,10,    0, 3625,   0, 3625,  0,0x0000,-2,20, 61, 537, 512,  0,0,"2841"},
+ {"2311-1",    0x2311,0x00,0x20,0x00,  200,3,10,    0, 3625,   0, 3625,  0,0x0000,-2,20, 61, 537, 512,  0,0,"2841"},
 
  {"2314",      0x2314,0x00,0x20,0x00,  200,3,20,    0, 7294,   0, 7294,  0,0x0000,-2,45,101,2137,2048,  0,0,"2841"},
  {"2314-1",    0x2314,0x00,0x20,0x00,  200,3,20,    0, 7294,   0, 7294,  0,0x0000,-2,45,101,2137,2048,  0,0,"2841"},
@@ -26,36 +35,40 @@ static CKDDEV ckdtab[] = {
  {"3330-2",    0x3330,0x11,0x20,0x00,  808,7,19,13165,13030, 135,13165,128,0x0000,-1,56,135,   0,   0,  0,0,"3830"},
  {"3330-11",   0x3330,0x11,0x20,0x00,  808,7,19,13165,13030, 135,13165,128,0x0000,-1,56,135,   0,   0,  0,0,"3830"},
 
- {"3340",      0x3340,0x00,0x20,0x00,  348,1,12, 8535, 8368, 167, 8535, 64,0x0000,-1,75,167,   0,   0,  0,0,"3830"},
- {"3340-1",    0x3340,0x00,0x20,0x00,  348,1,12, 8535, 8368, 167, 8535, 64,0x0000,-1,75,167,   0,   0,  0,0,"3830"},
- {"3340-35",   0x3340,0x00,0x20,0x00,  348,1,12, 8535, 8368, 167, 8535, 64,0x0000,-1,75,167,   0,   0,  0,0,"3830"},
- {"3340-2",    0x3340,0x00,0x20,0x00,  696,2,12, 8535, 8368, 167, 8535, 64,0x0000,-1,75,167,   0,   0,  0,0,"3830"},
- {"3340-70",   0x3340,0x00,0x20,0x00,  696,2,12, 8535, 8368, 167, 8535, 64,0x0000,-1,75,167,   0,   0,  0,0,"3830"},
+ {"3340",      0x3340,0x01,0x20,0x00,  348,1,12, 8535, 8368, 167, 8535, 64,0x0000,-1,75,167,   0,   0,  0,0,"3830"},
+ {"3340-1",    0x3340,0x01,0x20,0x00,  348,1,12, 8535, 8368, 167, 8535, 64,0x0000,-1,75,167,   0,   0,  0,0,"3830"},
+ {"3340-35",   0x3340,0x01,0x20,0x00,  348,1,12, 8535, 8368, 167, 8535, 64,0x0000,-1,75,167,   0,   0,  0,0,"3830"},
+ {"3340-2",    0x3340,0x02,0x20,0x00,  696,2,12, 8535, 8368, 167, 8535, 64,0x0000,-1,75,167,   0,   0,  0,0,"3830"},
+ {"3340-70",   0x3340,0x02,0x20,0x00,  696,2,12, 8535, 8368, 167, 8535, 64,0x0000,-1,75,167,   0,   0,  0,0,"3830"},
 
  {"3350",      0x3350,0x00,0x20,0x00,  555,5,30,19254,19069, 185,19254,128,0x0000,-1,82,185,   0,   0,  0,0,"3830"},
  {"3350-1",    0x3350,0x00,0x20,0x00,  555,5,30,19254,19069, 185,19254,128,0x0000,-1,82,185,   0,   0,  0,0,"3830"},
 
- {"3375",      0x3375,0x02,0x20,0x0e,  959,3,12,36000,35616, 832,36000,196,0x5007, 1, 32,384,160,   0,  0,0,"3880"},
- {"3375-1",    0x3375,0x02,0x20,0x0e,  959,3,12,36000,35616, 832,36000,196,0x5007, 1, 32,384,160,   0,  0,0,"3880"},
-
- {"3380",      0x3380,0x02,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
- {"3380-1",    0x3380,0x02,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
- {"3380-A",    0x3380,0x02,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
- {"3380-B",    0x3380,0x02,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"}, 
- {"3380-D",    0x3380,0x02,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
- {"3380-J",    0x3380,0x02,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
- {"3380-2",    0x3380,0x0a,0x20,0x0e, 1770,2,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
- {"3380-E",    0x3380,0x0a,0x20,0x0e, 1770,2,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
- {"3380-3",    0x3380,0x1e,0x20,0x0e, 2655,3,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
- {"3380-K",    0x3380,0x1e,0x20,0x0e, 2655,3,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
- {"EMC3380K+", 0x3380,0x1e,0x20,0x0e, 3339,3,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
- {"EMC3380K++",0x3380,0x1e,0x20,0x0e, 3993,3,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
+ {"3375",      0x3375,0x02,0x20,0x0e,  959,3,12,36000,35616, 832,36000,196,0x5007, 1, 0x20,0x01,0x08,0x00,0x0a,0x00,"3880"},
+ {"3375-1",    0x3375,0x02,0x20,0x0e,  959,3,12,36000,35616, 832,36000,196,0x5007, 1, 0x20,0x01,0x08,0x00,0x0a,0x00,"3880"},
+ 
+ {"3380",      0x3380,0x16,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
+ {"3380-1",    0x3380,0x16,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
+ {"3380-A",    0x3380,0x16,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
+ {"3380-B",    0x3380,0x16,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
+ {"3380-D",    0x3380,0x16,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
+ {"3380-J",    0x3380,0x16,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
+ {"3380-2",    0x3380,0x0a,0x20,0x0e, 1770,2,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
+ {"3380-E",    0x3380,0x0a,0x20,0x0e, 1770,2,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
+ {"3380-3",    0x3380,0x1e,0x20,0x0e, 2655,3,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
+ {"3380-K",    0x3380,0x1e,0x20,0x0e, 2655,3,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
+ {"EMC3380K+", 0x3380,0x1e,0x20,0x0e, 3339,3,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
+ {"EMC3380K++",0x3380,0x1e,0x20,0x0e, 3993,3,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
 
  {"3390",      0x3390,0x02,0x20,0x26, 1113,1,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990"},
  {"3390-1",    0x3390,0x02,0x20,0x26, 1113,1,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990"},
  {"3390-2",    0x3390,0x06,0x20,0x27, 2226,1,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990"},
  {"3390-3",    0x3390,0x0a,0x20,0x24, 3339,1,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990"},
  {"3390-9",    0x3390,0x0c,0x20,0x32,10017,3,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990"},
+ {"3390-27",   0x3390,0x0c,0x20,0x32,32760,3,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990"},
+ {"3390-J",    0x3390,0x0c,0x20,0x32,32760,3,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990"},
+ {"3390-54",   0x3390,0x0c,0x20,0x32,65520,3,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990"},
+ {"3390-JJ",   0x3390,0x0c,0x20,0x32,65520,3,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990"},
 
  {"9345",      0x9345,0x04,0x20,0x04, 1440,0,15,48174,46456,1184,48280,213,0x8b07, 2, 34,18,   7,   6,116,6,"9343"},
  {"9345-1",    0x9345,0x04,0x20,0x04, 1440,0,15,48174,46456,1184,48280,213,0x8b07, 2, 34,18,   7,   6,116,6,"9343"},
@@ -70,8 +83,10 @@ static CKDCU ckdcutab[] = {
 /* name          type model code features   ciws -------- */
  {"2841",       0x2841,0x00,0x00,0x50000103,0,0,0,0,0,0,0,0},
  {"3830",       0x3830,0x02,0x00,0x50000103,0,0,0,0,0,0,0,0},
- {"3880",       0x3880,0x03,0x10,0x50000003,0,0,0,0,0,0,0,0},
+ {"3880",       0x3880,0x05,0x09,0x80000000,0,0,0,0,0,0,0,0},
  {"3990",       0x3990,0xc2,0x10,0xd0000002,0x40fa0100,0,0,0,0,0,0,0},
+ {"3990-3",     0x3990,0xec,0x06,0xd000009e,0x40fa0100,0x41270004,0x423e0040,0,0,0,0,0},
+ {"3990-6",     0x3990,0xe9,0x15,0xd00010fe,0x40fa0100,0x41270004,0x423e0060,0,0,0,0,0},
  {"9343",       0x9343,0xe0,0x11,0x80000000,0,0,0,0,0,0,0,0}
 } ;
 #define CKDCU_NUM (sizeof(ckdcutab)/CKDCU_SIZE)
@@ -81,8 +96,8 @@ static CKDCU ckdcutab[] = {
 /*-------------------------------------------------------------------*/
 static FBADEV fbatab[] = {
 /* name          devt class type mdl  bpg bpp size   blks   cu     */
- {"3310",       0x3310,0x21,0x01,0x01, 32,352,512, 126016,0x4331},
- {"3310-1",     0x3310,0x21,0x01,0x01, 32,352,512, 126016,0x4331},
+ {"3310",       0x3310,0x21,0x01,0x01, 32,352,512, 125664,0x4331},
+ {"3310-1",     0x3310,0x21,0x01,0x01, 32,352,512, 125664,0x4331},
  {"3310-x",     0x3310,0x21,0x01,0x01, 32,352,512,      0,0x4331},
 
  {"3370",       0x3370,0x21,0x02,0x00, 62,744,512, 558000,0x3880},
@@ -133,7 +148,7 @@ static FBADEV fbatab[] = {
 /*-------------------------------------------------------------------*/
 /* Lookup a table entry either by name or type                       */
 /*-------------------------------------------------------------------*/
-void *dasd_lookup (int dtype, BYTE *name, U32 devt, U32 size)
+DLL_EXPORT void *dasd_lookup (int dtype, char *name, U32 devt, U32 size)
 {
 U32 i;                                  /* Loop Index                */
 
@@ -274,20 +289,20 @@ int altcyls;                            /* Number alternate cyls     */
     devchar[13] = (cyls - altcyls) & 0xff;
     devchar[14] = (ckd->heads >> 8) & 0xff;
     devchar[15] = ckd->heads & 0xff;
-    devchar[16] = ckd->sectors;
-    devchar[17] = (ckd->len >> 16) & 0xff;
+    devchar[16] = (BYTE)(ckd->sectors);
+    devchar[17] = 0; // (ckd->len >> 16) & 0xff;
     devchar[18] = (ckd->len >> 8) & 0xff;
     devchar[19] = ckd->len & 0xff;
     devchar[20] = (ckd->har0 >> 8) & 0xff;
     devchar[21] = ckd->har0 & 0xff;
     if (ckd->formula > 0)
     {
-        devchar[22] = ckd->formula;
-        devchar[23] = ckd->f1;
-        devchar[24] = ckd->f2;
-        devchar[25] = ckd->f3;
-        devchar[26] = ckd->f4;
-        devchar[27] = ckd->f5;
+        devchar[22] = (BYTE)(ckd->formula);
+        devchar[23] = (BYTE)(ckd->f1);
+        devchar[24] = (BYTE)(ckd->f2);
+        devchar[25] = (BYTE)(ckd->f3);
+        devchar[26] = (BYTE)(ckd->f4);
+        devchar[27] = (BYTE)(ckd->f5);
     }
     else
     {
@@ -328,7 +343,7 @@ int altcyls;                            /* Number alternate cyls     */
     devchar[45] = ckd->r0 & 0xff;
     devchar[46] = 0;
     devchar[47] = 0;
-    devchar[48] = ckd->f6;
+    devchar[48] = (BYTE)(ckd->f6);
     devchar[49] = (ckd->rpscalc >> 8) & 0xff;
     devchar[50] = ckd->rpscalc & 0xff;
     devchar[56] = 0xff;         // real CU type code
