@@ -1,4 +1,4 @@
-/* DECIMAL.C    (c) Copyright Roger Bowler, 1991-2003                */
+/* DECIMAL.C    (c) Copyright Roger Bowler, 1991-2005                */
 /*              ESA/390 Packed Decimal Routines                      */
 
 /*-------------------------------------------------------------------*/
@@ -16,9 +16,20 @@
 /* Add trialrun to ED and EDMK                   Jan Jaeger 19/07/00 */
 /* Fix random MP bug - Mario Bezzi                                   */
 /* Clear DXC on data exception - Peter Kuschnerus                V209*/
-/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2003      */
+/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2005      */
 /* TP instruction - Roger Bowler                            08/02/01 */
+/* packed_to_binary subroutine - Roger Bowler               29/06/03 */
+/* binary_to_packed subroutine - Roger Bowler              02jul2003 */
 /*-------------------------------------------------------------------*/
+#include "hstdinc.h"
+
+#if !defined(_HENGINE_DLL_)
+#define _HENGINE_DLL_
+#endif
+
+#if !defined(_DECIMAL_C_)
+#define _DECIMAL_C_
+#endif
 
 #include "hercules.h"
 
@@ -36,6 +47,177 @@
 #define MAX_DECIMAL_LENGTH      16
 #define MAX_DECIMAL_DIGITS      (((MAX_DECIMAL_LENGTH)*2)-1)
 
+/*-------------------------------------------------------------------*/
+/* Convert packed decimal number to binary                           */
+/*                                                                   */
+/* This subroutine is called by the CVB/CVBY/CVBG instructions.      */
+/* It performs the conversion of a 8-byte or 16-byte packed          */
+/* decimal number into a 64-bit signed binary result.                */
+/* This routine is not architecture-dependent; all of its operands   */
+/* are contained in work areas passed by the architecture-dependent  */
+/* instruction routines which handle all main-storage accesses and   */
+/* possible program checks.                                          */
+/*                                                                   */
+/* Input:                                                            */
+/*      dec     An 8 or 16 byte area containing a copy of the        */
+/*              packed decimal storage operand.                      */
+/*      len     Length-1 (in bytes) of the packed decimal input      */
+/*              (7 for CVB/CVBY or 15 for CVBG).                     */
+/* Output:                                                           */
+/*      result  Points to an U64 field which will receive the        */
+/*              result as a 64-bit signed binary number.             */
+/*      ovf     Points to an int field which will be set to 1 if     */
+/*              the result overflows 63 bits plus sign, else 0.      */
+/*              If overflow occurs, the result field will contain    */
+/*              the rightmost 64 bits of the result.                 */
+/*      dxf     Points to an int field which will be set to 1 if     */
+/*              invalid digits or sign were detected, else 0.        */
+/*              The result field is not set if the dxf is set to 1.  */
+/*-------------------------------------------------------------------*/
+void packed_to_binary (BYTE *dec, int len, U64 *result,
+                        int *ovf, int *dxf)
+{
+U64     dreg;                           /* 64-bit result accumulator */
+int     i;                              /* Loop counter              */
+int     h, d=0;                         /* Decimal digits            */
+U64     oreg = 0;                       /* 64 bit overflow work reg  */
+
+    /* Initialize result flags */
+    *ovf = 0;
+    *dxf = 0;
+
+    /* Initialize 64-bit result accumulator */
+    dreg = 0;
+
+    /* Convert decimal digits to binary */
+    for (i = 0; i <= len; i++)
+    {
+        /* Isolate high-order and low-order digits */
+        h = (dec[i] & 0xF0) >> 4;
+        d = dec[i] & 0x0F;
+
+        /* Data exception if high-order digit is invalid */
+        if (h > 9)
+        {
+            *dxf = 1;
+            return;
+        }
+
+        /* Accumulate high-order digit into result */
+        dreg *= 10;
+        dreg += h;
+
+        /* Set overflow indicator if an overflow has occurred */
+        if(dreg < oreg)
+            *ovf = 1;
+
+        /* Save current value */
+        oreg = dreg;
+
+        /* Check for valid low-order digit or sign */
+        if (i < len)
+        {
+            /* Data exception if low-order digit is invalid */
+            if (d > 9)
+            {
+                *dxf = 1;
+                return;
+            }
+
+            /* Accumulate low-order digit into result */
+            dreg *= 10;
+            dreg += d;
+        }
+        else
+        {
+            /* Data exception if sign is invalid */
+            if (d < 10)
+            {
+                *dxf = 1;
+                return;
+            }
+        }
+
+    } /* end for(i) */
+
+    /* Result is negative if sign is X'B' or X'D' */
+    if (d == 0x0B || d == 0x0D)
+    {
+        if( (S64)dreg == -1LL )
+            *ovf = 1;
+        dreg = -((S64)dreg);
+    }
+
+    /* Set result field and return */
+    *result = dreg;
+
+} /* end function packed_to_binary */
+
+/*-------------------------------------------------------------------*/
+/* Convert binary number to packed decimal                           */
+/*                                                                   */
+/* This subroutine is called by the CVD/CVDY/CVDG instructions.      */
+/* It performs the conversion of a 64-bit signed binary number       */
+/* to a 16-byte packed decimal result. Since the maximum 63 bit      */
+/* number is less than 31 decimal digits, overflow cannot occur.     */
+/* Similarly, the maximum 31 bit number is less than 15 decimal      */
+/* digits, therefore CVD/CVDY can safely use the rightmost eight     */
+/* bytes of the packed decimal result without risk of overflow.      */
+/*                                                                   */
+/* This routine is not architecture-dependent; all of its operands   */
+/* are contained in work areas passed by the architecture-dependent  */
+/* instruction routines which handle all main-storage accesses and   */
+/* possible program checks.                                          */
+/*                                                                   */
+/* Input:                                                            */
+/*      bin     Binary number (63 bits plus sign)                    */
+/* Output:                                                           */
+/*      result  Points to a 16-byte field which will receive the     */
+/*              result as a packed decimal number (31 digits + sign) */
+/*-------------------------------------------------------------------*/
+void binary_to_packed (S64 bin, BYTE *result)
+{
+int     i;                              /* Array subscript           */
+int     d;                              /* Decimal digit or sign     */
+
+    /* Special case when input is maximum negative value */
+    if ((U64)bin == 0x8000000000000000ULL)
+    {
+        memcpy (result,
+                "\x00\x00\x00\x00\x00\x00\x92\x23"
+                "\x37\x20\x36\x85\x47\x75\x80\x8D",
+                16);
+    }
+    else
+    {
+        /* Load absolute value and generate sign */
+        if ((U64)bin < 0x8000000000000000ULL)
+        {
+            /* Value is positive */
+            d = 0x0C;
+        }
+        else
+        {
+            /* Value is negative */
+            bin = -bin;
+            d = 0x0D;
+        }
+
+        /* Store sign and decimal digits from right to left */
+        memset (result, 0, 16);
+        for (i = 16 - 1; d != 0 || bin != 0; i--)
+        {
+            result[i] = d;
+            d = bin % 10;
+            bin /= 10;
+            result[i] |= (d << 4);
+            d = bin % 10;
+            bin /= 10;
+        }
+    }
+
+} /* end function(binary_to_packed) */
+
 /*-------------------------------------------------------------------*/
 /* Add two decimal byte strings as unsigned decimal numbers          */
 /*                                                                   */
@@ -488,7 +670,7 @@ BYTE    dec3[MAX_DECIMAL_DIGITS];       /* Work area for result      */
 int     count1, count2, count3;         /* Significant digit counters*/
 int     sign1, sign2, sign3;            /* Sign of operands & result */
 
-    SS(inst, execflag, regs, l1, l2, b1, effective_addr1,
+    SS(inst, regs, l1, l2, b1, effective_addr1,
                                      b2, effective_addr2);
 
     /* Load operands into work areas */
@@ -541,7 +723,7 @@ int     sign1, sign2, sign3;            /* Sign of operands & result */
     regs->psw.cc = cc;
 
     /* Program check if overflow and PSW program mask is set */
-    if (cc == 3 && regs->psw.domask)
+    if (cc == 3 && DOMASK(&regs->psw))
         ARCH_DEP(program_interrupt) (regs, PGM_DECIMAL_OVERFLOW_EXCEPTION);
 
 } /* end DEF_INST(add_decimal) */
@@ -562,7 +744,7 @@ int     count1, count2;                 /* Significant digit counters*/
 int     sign1, sign2;                   /* Sign of each operand      */
 int     rc;                             /* Return code               */
 
-    SS(inst, execflag, regs, l1, l2, b1, effective_addr1,
+    SS(inst, regs, l1, l2, b1, effective_addr1,
                                      b2, effective_addr2);
 
     /* Load operands into work areas */
@@ -622,7 +804,7 @@ int     count1, count2;                 /* Significant digit counters*/
 int     sign1, sign2;                   /* Sign of operands          */
 int     signq, signr;                   /* Sign of quotient/remainder*/
 
-    SS(inst, execflag, regs, l1, l2, b1, effective_addr1,
+    SS(inst, regs, l1, l2, b1, effective_addr1,
                                      b2, effective_addr2);
 
     /* Program check if the second operand length exceeds 15 digits
@@ -695,7 +877,7 @@ BYTE    fbyte;                          /* Fill byte                 */
 BYTE    pbyte;                          /* Pattern byte              */
 BYTE    rbyte;                          /* Result byte               */
 
-    SS_L(inst, execflag, regs, l, b1, effective_addr1,
+    SS_L(inst, regs, l, b1, effective_addr1,
                                   b2, effective_addr2);
 
     /* If addr1 crosses page, make sure both pages are accessable */
@@ -877,7 +1059,7 @@ int     d;                              /* Decimal digit             */
 int     i1, i2, i3;                     /* Array subscripts          */
 int     carry;                          /* Carry indicator           */
 
-    SS(inst, execflag, regs, l1, l2, b1, effective_addr1,
+    SS(inst, regs, l1, l2, b1, effective_addr1,
                                      b2, effective_addr2);
 
     /* Program check if the second operand length exceeds 15 digits
@@ -943,7 +1125,7 @@ int     i, j;                           /* Array subscripts          */
 int     d;                              /* Decimal digit             */
 int     carry;                          /* Carry indicator           */
 
-    SS(inst, execflag, regs, l1, i3, b1, effective_addr1,
+    SS(inst, regs, l1, i3, b1, effective_addr1,
                                      b2, effective_addr2);
 
     /* Load operand into work area */
@@ -1012,7 +1194,7 @@ int     carry;                          /* Carry indicator           */
     regs->psw.cc = cc;
 
     /* Program check if overflow and PSW program mask is set */
-    if (cc == 3 && regs->psw.domask)
+    if (cc == 3 && DOMASK(&regs->psw))
         ARCH_DEP(program_interrupt) (regs, PGM_DECIMAL_OVERFLOW_EXCEPTION);
 
 } /* end DEF_INST(shift_and_round_decimal) */
@@ -1034,7 +1216,7 @@ BYTE    dec3[MAX_DECIMAL_DIGITS];       /* Work area for result      */
 int     count1, count2, count3;         /* Significant digit counters*/
 int     sign1, sign2, sign3;            /* Sign of operands & result */
 
-    SS(inst, execflag, regs, l1, l2, b1, effective_addr1,
+    SS(inst, regs, l1, l2, b1, effective_addr1,
                                      b2, effective_addr2);
 
     /* Load operands into work areas */
@@ -1087,7 +1269,7 @@ int     sign1, sign2, sign3;            /* Sign of operands & result */
     regs->psw.cc = cc;
 
     /* Program check if overflow and PSW program mask is set */
-    if (cc == 3 && regs->psw.domask)
+    if (cc == 3 && DOMASK(&regs->psw))
         ARCH_DEP(program_interrupt) (regs, PGM_DECIMAL_OVERFLOW_EXCEPTION);
 
 } /* end DEF_INST(subtract_decimal) */
@@ -1107,7 +1289,7 @@ BYTE    dec[MAX_DECIMAL_DIGITS];        /* Work area for operand     */
 int     count;                          /* Significant digit counter */
 int     sign;                           /* Sign                      */
 
-    SS(inst, execflag, regs, l1, l2, b1, effective_addr1,
+    SS(inst, regs, l1, l2, b1, effective_addr1,
                                      b2, effective_addr2);
 
     /* Load second operand into work area */
@@ -1131,7 +1313,7 @@ int     sign;                           /* Sign                      */
     regs->psw.cc = cc;
 
     /* Program check if overflow and PSW program mask is set */
-    if (cc == 3 && regs->psw.domask)
+    if (cc == 3 && DOMASK(&regs->psw))
         ARCH_DEP(program_interrupt) (regs, PGM_DECIMAL_OVERFLOW_EXCEPTION);
 
 } /* end DEF_INST(zero_and_add) */
@@ -1150,7 +1332,7 @@ int     i;                              /* Array subscript           */
 int     cc = 0;                         /* Condition code            */
 BYTE    pack[MAX_DECIMAL_LENGTH];       /* Packed decimal work area  */
 
-    RSL(inst, execflag, regs, l1, b1, effective_addr1);
+    RSL(inst, regs, l1, b1, effective_addr1);
 
     /* Fetch the packed decimal operand into the work area */
     ARCH_DEP(vfetchc) (pack, l1, effective_addr1, b1, regs);
