@@ -1,4 +1,4 @@
-/* DIAGMSSF.C   (c) Copyright Jan Jaeger, 1999-2003                  */
+/* DIAGMSSF.C   (c) Copyright Jan Jaeger, 1999-2005                  */
 /*              ESA/390 Diagnose Functions                           */
 
 /*-------------------------------------------------------------------*/
@@ -8,9 +8,19 @@
 /* LPAR RMF interface call                                           */
 /*                                                                   */
 /*                                             04/12/1999 Jan Jaeger */
-/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2003      */
+/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2005      */
 /*-------------------------------------------------------------------*/
 
+#include "hstdinc.h"
+
+#if !defined(_HENGINE_DLL_)
+#define _HENGINE_DLL_
+#endif
+
+#if !defined(_DIAGMSSF_C_)
+#define _DIAGMSSF_C_
+#endif
+
 #include "hercules.h"
 
 #include "opcode.h"
@@ -147,7 +157,7 @@ typedef struct _DIAG204_HDR {
                                            0x0005 under VM           */
         HWORD   physcpu;                /* Number of phys CP's       */
         HWORD   offown;                 /* Offset to own partition   */
-        DWORD   diagstck;               /* TOD of last diag204       */
+        DBLWRD  diagstck;               /* TOD of last diag204       */
     } DIAG204_HDR;
 
 typedef struct _DIAG204_PART {
@@ -162,8 +172,8 @@ typedef struct _DIAG204_PART_CPU {
         HWORD   cpaddr;                 /* CP address                */
         HWORD   resv2[2];
         HWORD   relshare;               /* Relative share            */
-        DWORD   totdispatch;            /* Total dispatch time       */
-        DWORD   effdispatch;            /* Effective dispatch time   */
+        DBLWRD  totdispatch;            /* Total dispatch time       */
+        DBLWRD  effdispatch;            /* Effective dispatch time   */
     } DIAG204_PART_CPU;
 
 
@@ -264,36 +274,24 @@ DEVBLK            *dev;                /* Device block pointer       */
             spccbconfig->hex01 = 0x01;
 
             /* Set CPU array count and offset in SPCCB */
-#ifdef _FEATURE_CPU_RECONFIG
-            STORE_HW(spccbconfig->toticpu,MAX_CPU_ENGINES);
-#else /*!_FEATURE_CPU_RECONFIG*/
-            STORE_HW(spccbconfig->toticpu,sysblk.numcpu);
-#endif /*!_FEATURE_CPU_RECONFIG*/
+            STORE_HW(spccbconfig->toticpu,MAX_CPU);
             offset = sizeof(SPCCB_HEADER) + sizeof(SPCCB_CONFIG_INFO);
             STORE_HW(spccbconfig->officpu,offset);
 
             /* Set HSA array count and offset in SPCCB */
             STORE_HW(spccbconfig->tothsa,0);
-#ifdef _FEATURE_CPU_RECONFIG
-            offset += sizeof(SPCCB_CPU_INFO) * MAX_CPU_ENGINES;
-#else /*!_FEATURE_CPU_RECONFIG*/
-            offset += sizeof(SPCCB_CPU_INFO) * sysblk.numcpu;
-#endif /*!_FEATURE_CPU_RECONFIG*/
+            offset += sizeof(SPCCB_CPU_INFO) * MAX_CPU;
             STORE_HW(spccbconfig->offhsa,offset);
 
             /* Move IPL load parameter to SPCCB */
-            memcpy (spccbconfig->loadparm, sysblk.loadparm, 8);
+            get_loadparm (spccbconfig->loadparm);
 
             /* Build the CPU information array after the SCP info */
             spccbcpu = (SPCCB_CPU_INFO*)(spccbconfig+1);
-#ifdef _FEATURE_CPU_RECONFIG
-            for (i = 0; i < MAX_CPU_ENGINES; i++, spccbcpu++)
-#else /*!_FEATURE_CPU_RECONFIG*/
-            for (i = 0; i < sysblk.numcpu; i++, spccbcpu++)
-#endif /*!_FEATURE_CPU_RECONFIG*/
+            for (i = 0; i < MAX_CPU; i++, spccbcpu++)
             {
                 memset (spccbcpu, 0, sizeof(SPCCB_CPU_INFO));
-                spccbcpu->cpuaddr = sysblk.regs[i].cpuad;
+                spccbcpu->cpuaddr = i;
                 spccbcpu->todid = 0;
             }
 
@@ -367,38 +365,39 @@ DIAG204_PART      *partinfo;           /* Partition info             */
 DIAG204_PART_CPU  *cpuinfo;            /* CPU info                   */
 RADR              abs;                 /* abs addr of data area      */
 U64               dreg;                /* work doubleword            */
+U64               tdis = 0, teff = 0;
 U32               i;                   /* loop counter               */
 struct rusage     usage;               /* RMF type data              */
-static char       lparname[] = "HERCULES";
-static char       physical[] = "PHYSICAL";
+static BYTE       physical[8] =
+              {0xD7,0xC8,0xE8,0xE2,0xC9,0xC3,0xC1,0xD3}; /* PHYSICAL */
 static U64        diag204tod;          /* last diag204 tod           */
 
-    abs = APPLY_PREFIXING (regs->GR(r1), regs->PX);
-
-    /* Program check if RMF data is not on a page boundary */
-    if ( (abs & PAGEFRAME_BYTEMASK) != 0x000)
-        ARCH_DEP(program_interrupt) (regs, PGM_SPECIFICATION_EXCEPTION);
-
-    /* Program check if RMF data area is outside main storage */
-    if ( abs > regs->mainlim )
-        ARCH_DEP(program_interrupt) (regs, PGM_ADDRESSING_EXCEPTION);
-
     /* Test DIAG204 command word */
     switch (regs->GR_L(r2)) {
 
     case 0x04:
 
+        abs = APPLY_PREFIXING (regs->GR_L(r1), regs->PX);
+
+        /* Program check if RMF data is not on a page boundary */
+        if ( (abs & PAGEFRAME_BYTEMASK) != 0x000)
+            ARCH_DEP(program_interrupt) (regs, PGM_SPECIFICATION_EXCEPTION);
+
+        /* Program check if RMF data area is outside main storage */
+        if ( abs > regs->mainlim )
+            ARCH_DEP(program_interrupt) (regs, PGM_ADDRESSING_EXCEPTION);
+
         /* Obtain the TOD clock update lock */
         obtain_lock (&sysblk.todlock);
 
         /* Update the TOD clock */
-        update_TOD_clock();
+        update_tod_clock();
 
         /* save last diag204 tod */
         dreg = diag204tod;
 
         /* Retrieve the TOD clock value and shift out the epoch */
-        diag204tod = (sysblk.todclk + regs->todoffset) << 8;
+        diag204tod = TOD_CLOCK(regs) << 8;
 
         /* Release the TOD clock update lock */
         release_lock (&sysblk.todlock);
@@ -412,7 +411,7 @@ static U64        diag204tod;          /* last diag204 tod           */
         memset(hdrinfo, 0, sizeof(DIAG204_HDR));
         hdrinfo->numpart = 1;
         hdrinfo->flags = DIAG204_PHYSICAL_PRESENT;
-        STORE_HW(hdrinfo->physcpu,sysblk.numcpu);
+        STORE_HW(hdrinfo->physcpu,sysblk.cpus);
         STORE_HW(hdrinfo->offown,sizeof(DIAG204_HDR));
         STORE_DW(hdrinfo->diagstck,dreg);
 
@@ -420,33 +419,28 @@ static U64        diag204tod;          /* last diag204 tod           */
         partinfo = (DIAG204_PART*)(hdrinfo + 1);
         memset(partinfo, 0, sizeof(DIAG204_PART));
         partinfo->partnum = 1;
-        partinfo->virtcpu = sysblk.numcpu;
-        for(i = 0; i < sizeof(partinfo->partname); i++)
-            partinfo->partname[i] = host_to_guest((int)lparname[i]);
+        partinfo->virtcpu = sysblk.cpus;
+        get_lparname(partinfo->partname);
 
         /* hercules cpu's */
         getrusage(RUSAGE_SELF,&usage);
         cpuinfo = (DIAG204_PART_CPU*)(partinfo + 1);
-#ifdef _FEATURE_CPU_RECONFIG
-        for(i = 0; i < MAX_CPU_ENGINES;i++)
-          if(sysblk.regs[i].cpuonline)
-#else /*!_FEATURE_CPU_RECONFIG*/
-        for(i = 0; i < sysblk.numcpu;i++)
-#endif /*!_FEATURE_CPU_RECONFIG*/
-        {
-            memset(cpuinfo, 0, sizeof(DIAG204_PART_CPU));
-            STORE_HW(cpuinfo->cpaddr,sysblk.regs[i].cpuad);
-            STORE_HW(cpuinfo->relshare,100);
-            dreg = (U64)(usage.ru_utime.tv_sec + usage.ru_stime.tv_sec) / sysblk.numcpu;
-            dreg = dreg * 1000000 + (i ? 0 : usage.ru_utime.tv_usec + usage.ru_stime.tv_usec);
-            dreg <<= 12;
-            STORE_DW(cpuinfo->totdispatch,dreg);
-            dreg = (U64)(usage.ru_utime.tv_sec) / sysblk.numcpu;
-            dreg = dreg * 1000000 + (i ? 0 : usage.ru_utime.tv_usec );
-            dreg <<= 12;
-            STORE_DW(cpuinfo->effdispatch,dreg);
-            cpuinfo += 1;
-        }
+        for(i = 0; i < MAX_CPU; i++)
+          if (IS_CPU_ONLINE(i))
+          {
+              memset(cpuinfo, 0, sizeof(DIAG204_PART_CPU));
+              STORE_HW(cpuinfo->cpaddr,sysblk.regs[i]->cpuad);
+              STORE_HW(cpuinfo->relshare,100);
+              dreg = (U64)(usage.ru_utime.tv_sec + usage.ru_stime.tv_sec) / sysblk.cpus;
+              dreg = (dreg * 1000000) + (i ? 0 : (usage.ru_utime.tv_usec + usage.ru_stime.tv_usec));
+              tdis += dreg;
+              STORE_DW(cpuinfo->totdispatch,dreg);
+              dreg = (U64)(usage.ru_utime.tv_sec) / sysblk.cpus;
+              dreg = (dreg * 1000000) + (i ? 0 : (usage.ru_utime.tv_usec) );
+              teff += dreg;
+              STORE_DW(cpuinfo->effdispatch,dreg);
+              cpuinfo += 1;
+          }
 
         /* lpar management */
         getrusage(RUSAGE_CHILDREN,&usage);
@@ -454,16 +448,18 @@ static U64        diag204tod;          /* last diag204 tod           */
         memset(partinfo, 0, sizeof(DIAG204_PART));
         partinfo->partnum = 0;
         partinfo->virtcpu = 1;
-        for(i = 0; i < sizeof(partinfo->partname); i++)
-            partinfo->partname[i] = host_to_guest((int)physical[i]);
+        memcpy(partinfo->partname,physical,sizeof(physical));
         cpuinfo = (DIAG204_PART_CPU*)(partinfo + 1);
         memset(cpuinfo, 0, sizeof(DIAG204_PART_CPU));
 //      STORE_HW(cpuinfo->cpaddr,0);
-        dreg = (U64)(usage.ru_utime.tv_sec + usage.ru_stime.tv_sec) / sysblk.numcpu;
-        dreg = dreg * 1000000 + (i ? 0 : usage.ru_utime.tv_usec + usage.ru_stime.tv_usec);
-        dreg <<= 12;
-        STORE_DW(cpuinfo->totdispatch,dreg);
-//      cpuinfo->effdispatch = 0;
+        dreg = (U64)(usage.ru_utime.tv_sec + usage.ru_stime.tv_sec);
+        dreg = (dreg * 1000000) + (usage.ru_utime.tv_usec + usage.ru_stime.tv_usec);
+    tdis += dreg;
+        STORE_DW(cpuinfo->totdispatch,tdis);
+        dreg = (U64)(usage.ru_utime.tv_sec);
+        dreg = (dreg * 1000000) + (usage.ru_utime.tv_usec);
+    teff += dreg;
+        STORE_DW(cpuinfo->effdispatch,teff);
 
         regs->GR_L(r2) = 0;
 
