@@ -1,4 +1,4 @@
-/* DIAGNOSE.C   (c) Copyright Roger Bowler, 2000-2003                */
+/* DIAGNOSE.C   (c) Copyright Roger Bowler, 2000-2005                */
 /*              ESA/390 Diagnose Functions                           */
 
 /*-------------------------------------------------------------------*/
@@ -9,13 +9,24 @@
 /* Additional credits:                                               */
 /*      Hercules-specific diagnose calls by Jay Maynard.             */
 /*      Set/reset bad frame indicator call by Jan Jaeger.            */
-/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2003      */
+/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2005      */
 /*-------------------------------------------------------------------*/
+#include "hstdinc.h"
+
+#if !defined(_HENGINE_DLL_)
+#define _HENGINE_DLL_
+#endif
+
+#if !defined(_DIAGNOSE_C_)
+#define _DIAGNOSE_C_
+#endif
 
 #include "hercules.h"
 
 #include "opcode.h"
 
+#include "inline.h"
+
 #if !defined(_DIAGNOSE_H)
 
 #define _DIAGNOSE_H
@@ -23,18 +34,66 @@
 /*-------------------------------------------------------------------*/
 /* Internal macro definitions                                        */
 /*-------------------------------------------------------------------*/
-#define SPACE   ((BYTE)' ')
 
 #endif /*!defined(_DIAGNOSE_H)*/
 
+#if defined(OPTION_DYNAMIC_LOAD) && defined(FEATURE_HERCULES_DIAGCALLS)
+
+void ARCH_DEP(diagf14_call)(int r1, int r3, REGS *regs)
+{
+char name[32+1];
+char entry[64];
+unsigned int  i;
+void (*dllcall)(int, int, REGS *);
+
+static char *prefix[] = {
+#if defined(_370)
+    "s370_diagf14_",
+#endif
+#if defined(_390)
+    "s390_diagf14_",
+#endif
+#if defined(_900)
+    "z900_diagf14_"
+#endif
+    };
+
+    ARCH_DEP(vfetchc) (name,sizeof(name)-2, regs->GR(r1), USE_REAL_ADDR, regs);
+
+    for(i = 0; i < sizeof(name)-1; i++)
+    {
+        name[i] = guest_to_host(name[i]);
+        if(!isprint(name[i]) || isspace(name[i]))
+        {
+            name[i] = '\0';
+            break;
+        }
+    }
+    /* Ensure string terminator */
+    name[i] = '\0';
+    strcpy(entry,prefix[regs->arch_mode]);
+    strcat(entry,name);
+
+    if( (dllcall = HDL_FINDSYM(entry)) )
+        dllcall(r1, r3, regs);
+    else
+        ARCH_DEP(program_interrupt) (regs, PGM_SPECIFICATION_EXCEPTION);
+
+}
+#endif /*defined(OPTION_DYNAMIC_LOAD)*/
+
 /*-------------------------------------------------------------------*/
 /* Diagnose instruction                                              */
 /*-------------------------------------------------------------------*/
-void ARCH_DEP(diagnose_call) (U32 code, int r1, int r2, REGS *regs)
+void ARCH_DEP(diagnose_call) (VADR effective_addr2, int b2,
+                              int r1, int r2, REGS *regs)
 {
 #ifdef FEATURE_HERCULES_DIAGCALLS
-U32             n;                      /* 32-bit operand value      */
+U32   n;                                /* 32-bit operand value      */
 #endif /*FEATURE_HERCULES_DIAGCALLS*/
+U32   code;
+
+    code = effective_addr2;
 
     switch(code) {
 
@@ -50,20 +109,27 @@ U32             n;                      /* 32-bit operand value      */
         break;
 #endif
 
+
     case 0x01F:
     /*---------------------------------------------------------------*/
     /* Diagnose 01F: Power Off                                       */
     /*---------------------------------------------------------------*/
 
+        /* If diag8cmd is not enabled then we are not allowed
+         * to manipulate the real machine i.e. hercules itself
+         */
+        if(!sysblk.diag8cmd)
+            ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
+
         /* The poweroff diagnose is only valid on the 9221 */
-        if((sysblk.cpuid >> 16 & 0xFFFF) != 0x9221
+        if(((sysblk.cpuid >> 16 & 0xFFFF) != 0x9221 )
           /* and r1/r2 must contain C'POWEROFF' in EBCDIC */
           || regs->GR_L(r1) != 0xD7D6E6C5
           || regs->GR_L(r2) != 0xD9D6C6C6)
             ARCH_DEP(program_interrupt) (regs, PGM_SPECIFICATION_EXCEPTION);
 
         regs->cpustate = CPUSTATE_STOPPING;
-        ON_IC_CPU_NOT_STARTED(regs);
+        ON_IC_INTERRUPT(regs);
 
         /* Release the configuration */
         release_config();
@@ -126,6 +192,12 @@ U32             n;                      /* 32-bit operand value      */
     /*---------------------------------------------------------------*/
     /* Diagnose 008: Virtual Console Function                        */
     /*---------------------------------------------------------------*/
+        /* If diag8cmd is not enabled then we are not allowed
+         * to manipulate the real machine i.e. hercules itself
+         */
+        if(!sysblk.diag8cmd)
+            ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
+
         /* Process CP command and set condition code */
         regs->psw.cc = ARCH_DEP(cpcmd_call) (r1, r2, regs);
         break;
@@ -210,6 +282,31 @@ U32             n;                      /* 32-bit operand value      */
         regs->psw.cc = ARCH_DEP(diag_ppagerel) (r1, r2, regs);
         break;
 
+
+    case 0x220:
+    /*---------------------------------------------------------------*/
+    /* Diagnose 220: TOD Epoch                                       */
+    /*---------------------------------------------------------------*/
+        ODD_CHECK(r2, regs);
+
+        switch(regs->GR_L(r1))
+        {
+            case 0:
+                /* Obtain TOD features */
+                regs->GR_L(r2)  =0xc0000000;
+                regs->GR_L(r2+1)=0x00000000;
+                break;
+            case 1:
+                /* Obtain TOD offset to real TOD in R2, R2+1 */
+                regs->GR_L(r2)  = (regs->tod_epoch >> 24) & 0xFFFFFFFF;
+                regs->GR_L(r2+1)= (regs->tod_epoch << 8) & 0xFFFFFFFF; 
+                break;
+            default:
+                ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
+        }
+        break;
+
+
     case 0x23C:
     /*---------------------------------------------------------------*/
     /* Diagnose 23C: Address Space Services                          */
@@ -247,6 +344,12 @@ U32             n;                      /* 32-bit operand value      */
     /*---------------------------------------------------------------*/
     /* Diagnose F00: Hercules normal mode                            */
     /*---------------------------------------------------------------*/
+        /* If diag8cmd is not enabled then we are not allowed
+         * to manipulate the real machine i.e. hercules itself
+         */
+        if(!sysblk.diag8cmd)
+            ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
+
         sysblk.inststep = 0;
         SET_IC_TRACE;
         break;
@@ -255,8 +358,14 @@ U32             n;                      /* 32-bit operand value      */
     /*---------------------------------------------------------------*/
     /* Diagnose F04: Hercules single step mode                       */
     /*---------------------------------------------------------------*/
+        /* If diag8cmd is not enabled then we are not allowed
+         * to manipulate the real machine i.e. hercules itself
+         */
+        if(!sysblk.diag8cmd)
+            ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
+
         sysblk.inststep = 1;
-        ON_IC_TRACE;
+        SET_IC_TRACE;
         break;
 
     case 0xF08:
@@ -270,6 +379,12 @@ U32             n;                      /* 32-bit operand value      */
     /*---------------------------------------------------------------*/
     /* Diagnose F0C: Set/reset bad frame indicator                   */
     /*---------------------------------------------------------------*/
+        /* If diag8cmd is not enabled then we are not allowed
+         * to manipulate the real machine i.e. hercules itself
+         */
+        if(!sysblk.diag8cmd)
+            ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
+
         /* Load 4K block address from R2 register */
         n = regs->GR_L(r2) & ADDRESS_MAXWRAP(regs);
 
@@ -293,10 +408,25 @@ U32             n;                      /* 32-bit operand value      */
     /*---------------------------------------------------------------*/
     /* Diagnose F10: Hercules CPU stop                               */
     /*---------------------------------------------------------------*/
+        /* If diag8cmd is not enabled then we are not allowed
+         * to manipulate the real machine i.e. hercules itself
+         */
+        if(!sysblk.diag8cmd)
+            ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
+
         regs->cpustate = CPUSTATE_STOPPING;
-        ON_IC_CPU_NOT_STARTED(regs);
+        ON_IC_INTERRUPT(regs);
         break;
 
+#if defined(OPTION_DYNAMIC_LOAD)
+    case 0xF14:
+    /*---------------------------------------------------------------*/
+    /* Diagnose F14: Hercules DLL interface                          */
+    /*---------------------------------------------------------------*/
+        ARCH_DEP(diagf14_call) (r1, r2, regs);
+        break;
+#endif /*defined(OPTION_DYNAMIC_LOAD)*/
+
 #if !defined(NO_SIGABEND_HANDLER)
     /* The following diagnose calls cause a exigent (non-repressible)
        machine check, and are used for test purposes only *JJ */
@@ -339,7 +469,7 @@ U32             n;                      /* 32-bit operand value      */
     /*---------------------------------------------------------------*/
     /* Diagnose FFC: Simulate Wait                                   */
     /*---------------------------------------------------------------*/
-        sleep(300);
+        SLEEP(300);
         break;
 #endif /*!defined(NO_SIGABEND_HANDLER)*/
 
@@ -349,10 +479,48 @@ U32             n;                      /* 32-bit operand value      */
     /*---------------------------------------------------------------*/
     /* Diagnose xxx: Invalid function code                           */
     /*---------------------------------------------------------------*/
+
+        if( HDC4(debug_diagnose, code, r1, r2, regs) )
+            return;
+
+    /*  Power Off diagnose on 4361, 9371, 9373, 9375, 9377, 9221:    */
+    /*                                                               */
+    /*          DS 0H                                                */
+    /*          DC X'8302',S(SHUTDATA)     MUST BE R1 AND R2         */
+    /*          ...                                                  */
+    /*          DS 0H                                                */
+    /* SHUTDATA DC X'0000FFFF'             MUST BE X'0000FFFF'       */
+
+    if (0 == r1 && 2 == r2
+         && (sysblk.cpuid >> 56 & 0xFF) != 0xFF
+         && ((sysblk.cpuid >> 16 & 0xFFFF) == 0x4361
+          || (sysblk.cpuid >> 16 & 0xFFF9) == 0x9371    /* (937X) */
+          || (sysblk.cpuid >> 16 & 0xFFFF) == 0x9221)
+       )
+    {
+        if (0x0000FFFF == ARCH_DEP(vfetch4)(effective_addr2, b2, regs))
+        {
+            /* If diag8cmd is not enabled then we are not allowed
+             * to manipulate the real machine i.e. hercules itself
+             */
+        if(!sysblk.diag8cmd)
+            ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
+
+            regs->cpustate = CPUSTATE_STOPPING;
+            ON_IC_INTERRUPT(regs);
+
+            /* Release the configuration */
+            release_config();
+
+            /* Power Off: exit hercules */
+            exit(0);
+        }
+    }
+
 #if defined(FEATURE_S370_CHANNEL) && defined(OPTION_NOP_MODEL158_DIAGNOSE)
-      if((sysblk.cpuid >> 16 & 0xFFFF) != 0x0158)
+        if((sysblk.cpuid >> 16 & 0xFFFF) != 0x0158)
 #endif
-        ARCH_DEP(program_interrupt) (regs, PGM_SPECIFICATION_EXCEPTION);
+        ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
         return;
 
     } /* end switch(code) */
