@@ -1,4 +1,4 @@
-/* DMAP2HRC.C   (c) Copyright Jay Maynard, 2001-2003                 */
+/* DMAP2HRC.C   (c) Copyright Jay Maynard, 2001-2005                 */
 /*              Convert P/390 DEVMAP to Hercules config file         */
 
 /*-------------------------------------------------------------------*/
@@ -7,6 +7,8 @@
 /* the format Hercules uses for its .cnf file.                       */
 /*-------------------------------------------------------------------*/
 
+#include "hstdinc.h"
+
 #include "hercules.h"
 
 /*-------------------------------------------------------------------*/
@@ -29,29 +31,23 @@ typedef struct _DEVMAP_CTLR {
 typedef struct _DEVMAP_DEV {
         BYTE    highaddr;               /* High order dev addr byte  */
         BYTE    lowaddr;                /* Low order dev addr byte   */
-        BYTE    type[4];        /* Type of device            */
+        char    type[4];        /* Type of device            */
     union {
         struct {                    /* Disk devices:             */
             BYTE filler1[4];        /* filler                    */
             BYTE volser[6];         /* Volume serial             */
             BYTE filler2[2];        /* more filler               */
-            BYTE filename[45];      /* name of file on disk      */
+            char filename[45];      /* name of file on disk      */
             BYTE flags;             /* flag byte                 */
         } disk;
         struct {                    /* Other devices:            */
             BYTE filler1[7];        /* fill bytes                */
-            BYTE filename[50];      /* device filename           */
+            char filename[50];      /* device filename           */
             BYTE flags;             /* flag byte                 */
         } other;
     } parms;
     } DEVMAP_DEV;
 
-#if defined(EXTERNALGUI)
-/* Special flag to indicate whether or not we're being
-   run under the control of the external GUI facility. */
-int  extgui = 0;
-#endif /*defined(EXTERNALGUI)*/
-
 /*-------------------------------------------------------------------*/
 /* DEVMAP2CNF main entry point                                       */
 /*-------------------------------------------------------------------*/
@@ -59,25 +55,28 @@ int main (int argc, char *argv[])
 {
 int             i;                      /* Array subscript           */
 int             len;                    /* Length of actual read     */
-BYTE           *filename;               /* -> Input file name        */
+char           *filename;               /* -> Input file name        */
 int             infd = -1;              /* Input file descriptor     */
 DEVMAP_CTLR     controller;             /* Controller record         */
-DEVMAP_DEV      device;         /* Device record             */
-BYTE        output_type[5];     /* Device type to print      */
-BYTE           *output_filename;    /* -> filename to print      */
-int     more_devices;       /* More devices this ctlr?   */
+DEVMAP_DEV      device;                 /* Device record             */
+char            output_type[5];         /* Device type to print      */
+char           *output_filename;        /* -> filename to print      */
+int             more_devices;           /* More devices this ctlr?   */
+BYTE            pathname[MAX_PATH];     /* file path in host format  */
 
 #ifdef EXTERNALGUI
     if (argc >= 1 && strncmp(argv[argc-1],"EXTERNALGUI",11) == 0)
     {
         extgui = 1;
         argc--;
+        setvbuf(stderr, NULL, _IONBF, 0);
+        setvbuf(stdout, NULL, _IONBF, 0);
     }
 #endif /*EXTERNALGUI*/
 
     /* Display the program identification message */
     display_version (stderr,
-                     "P/390 DEVMAP to Hercules conversion program\n");
+                     "P/390 DEVMAP to Hercules conversion program\n", FALSE);
 
     /* The only argument is the DEVMAP file name */
     if (argc == 2 && argv[1] != NULL)
@@ -91,7 +90,8 @@ int     more_devices;       /* More devices this ctlr?   */
     }
 
     /* Open the devmap file */
-    infd = open (filename, O_RDONLY | O_BINARY);
+    hostpath(pathname, filename, sizeof(pathname));
+    infd = open (pathname, O_RDONLY | O_BINARY);
     if (infd < 0)
     {
         fprintf (stderr,"dmap2hrc: Error opening %s: %s\n",
