@@ -1,8 +1,8 @@
-/* EXTERNAL.C   (c) Copyright Roger Bowler, 1999-2003                */
+/* EXTERNAL.C   (c) Copyright Roger Bowler, 1999-2005                */
 /*              ESA/390 External Interrupt and Timer                 */
 
-/* Interpretive Execution - (c) Copyright Jan Jaeger, 1999-2003      */
-/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2003      */
+/* Interpretive Execution - (c) Copyright Jan Jaeger, 1999-2005      */
+/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2005      */
 
 /*-------------------------------------------------------------------*/
 /* This module implements external interrupt, timer, and signalling  */
@@ -23,6 +23,16 @@
 /*      Modifications for Interpretive Execution (SIE) by Jan Jaeger */
 /*-------------------------------------------------------------------*/
 
+#include "hstdinc.h"
+
+#if !defined(_HENGINE_DLL_)
+#define _HENGINE_DLL_
+#endif
+
+#if !defined(_EXTERNAL_C_)
+#define _EXTERNAL_C_
+#endif
+
 #include "hercules.h"
 
 #include "opcode.h"
@@ -34,8 +44,17 @@
 /* Synchronize broadcast request                                     */
 /* Input:                                                            */
 /*      regs    A pointer to the CPU register context                */
+/*      code    A mask indicating which function to perform.         */
+/*              More than 1 function may be requested:               */
+/*              PTLB  - 0x01 - purge TLB            (CSP)            */
+/*              PALB  - 0x02 - purge ALB            (CSP)            */
+/*              PTLBE - 0x04 - purge TLB entry      (IPTE)           */
+/*              If the code is zero then we are entering on the      */
+/*              behalf of another CPU and the function to be         */
+/*              performed is in sysblk.broadcast_code.               */
+/*      pfra    Real address if PTLBE request                        */
 /*                                                                   */
-/* The intlock MUST be held when calling synchronize_broadcast()     */
+/* The intlock MUST be held on entry                                 */
 /*                                                                   */
 /* Signals all other CPU's to perform a requested function           */
 /* synchronously, such as purging the ALB and TLB buffers.           */
@@ -43,84 +62,73 @@
 /* all other CPU's have performed the requested action.         *JJ  */
 /*                                                                   */
 /*-------------------------------------------------------------------*/
-void ARCH_DEP(synchronize_broadcast) (REGS *regs, int code, U64 pfra)
+void ARCH_DEP(synchronize_broadcast) (REGS *regs, int code, RADR pfra)
 {
 U32     i;                              /* Array subscript           */
 REGS   *realregs;                       /* Real REGS if guest        */
+REGS   *tregs;                          /* Target regs               */
 
-    realregs =
-#if defined(_FEATURE_SIE)
-               regs->sie_state ? regs->hostregs :
-#endif /*defined(_FEATURE_SIE)*/
-                                                  regs;
+    realregs = SIE_MODE(regs) ? regs->hostregs : regs;
 
-#if MAX_CPU_ENGINES > 1
-    if (code > 0)
-    {
-        obtain_lock (&sysblk.intlock);
-        while (IS_IC_BROADCAST_ON)
-            if (IS_IC_BROADCAST(realregs))
-                ARCH_DEP(synchronize_broadcast)(realregs, 0, 0);
-            else
-            {
-                release_lock (&sysblk.intlock);
-                sched_yield();
-                obtain_lock (&sysblk.intlock);
-            }
-        ON_IC_BROADCAST;
-        sysblk.broadcast_mask = sysblk.started_mask;
-        sysblk.broadcast_code = code;
-        sysblk.broadcast_pfra = pfra;
-    }
-#else /* MAX_CPU_ENGINES > 1 */
-    sysblk.broadcast_code = code;
-    sysblk.broadcast_pfra = pfra;
-#endif /* MAX_CPU_ENGINES > 1 */
+    do {
+        if (code)
+        {
+            /* Complete any outstanding broadcasts */
+            while (sysblk.broadcast_count)
+                ARCH_DEP(synchronize_broadcast) (realregs, 0, 0);
 
-    /* Purge TLB */
-    if (sysblk.broadcast_code & BROADCAST_PTLB)
-        ARCH_DEP(purge_tlb) (realregs);
+            /* Turn broadcast bit on for all started CPUs */
+            for (i = 0; i < MAX_CPU; i++)
+                if (IS_CPU_ONLINE(i))
+                {
+                    tregs = sysblk.regs[i];
+                    if ( sysblk.started_mask & BIT(tregs->cpuad) )
+                    {
+                        ON_IC_BROADCAST(tregs);
+                        sysblk.broadcast_count++;
+                    }
+                }
+            sysblk.broadcast_code = code;
+            sysblk.BROADCAST_PFRA = pfra;
+            WAKEUP_CPUS_MASK(sysblk.waiting_mask);
+        }
+
+        /* Perform the requested functions */
+        if (IS_IC_BROADCAST(realregs))
+        {
+            /* Purge TLB */
+            if (sysblk.broadcast_code & BROADCAST_PTLB)
+                ARCH_DEP(purge_tlb) (realregs);
+
+            /* Purge TLB entries */
+            if (sysblk.broadcast_code & BROADCAST_PTLBE)
+                ARCH_DEP(purge_tlbe) (realregs, sysblk.BROADCAST_PFRA);
 
 #if defined(FEATURE_ACCESS_REGISTERS)
-    /* Purge ALB */
-    if (sysblk.broadcast_code & BROADCAST_PALB)
-        ARCH_DEP(purge_alb) (realregs);
+            /* Purge ALB */
+            if (sysblk.broadcast_code & BROADCAST_PALB)
+                ARCH_DEP(purge_alb) (realregs);
 #endif /*defined(FEATURE_ACCESS_REGISTERS)*/
 
-    /* Invalidate TLB entries */
-    if (sysblk.broadcast_code & BROADCAST_ITLB)
-    {
-        for (i = 0; i < (sizeof(regs->tlb)/sizeof(TLBE)); i++)
-            if ((regs->tlb[i].TLB_PTE & PAGETAB_PFRA) == sysblk.broadcast_pfra
-              && regs->tlb[i].valid)
-                    regs->tlb[i].valid = 0;
-        for (i = 0; i < (sizeof(regs->tlb)/sizeof(TLBE)) && realregs != regs; i++)
-            if ((realregs->tlb[i].TLB_PTE & PAGETAB_PFRA) == sysblk.broadcast_pfra
-              && realregs->tlb[i].valid)
-                    realregs->tlb[i].valid = 0;
-    }
-
-#if MAX_CPU_ENGINES > 1
-    /* Wait for the other cpus */
-    sysblk.broadcast_mask &= ~realregs->cpumask;
-    if (code > 0)
-    {
-        if (sysblk.broadcast_mask != 0)
-        {
-            WAKEUP_WAITING_CPUS(ALL_CPUS, CPUSTATE_STARTED);
-            wait_condition (&sysblk.broadcast_cond, &sysblk.intlock);
+            OFF_IC_BROADCAST(realregs);
+            sysblk.broadcast_count--;
         }
-        OFF_IC_BROADCAST;
-        release_lock (&sysblk.intlock);
-    }
-    else
-    {
-        if (sysblk.broadcast_mask == 0)
-            broadcast_condition (&sysblk.broadcast_cond);
-        else
+
+        /* Wait for the other CPUs.  If count is zero and code is
+         * non-zero then there are no other CPUs
+         */
+        if (sysblk.broadcast_count)
             wait_condition (&sysblk.broadcast_cond, &sysblk.intlock);
-    }
-#endif /*MAX_CPU_ENGINES > 1*/
+        else if (!code)
+            broadcast_condition (&sysblk.broadcast_cond);
+
+        /* If we were the instigating CPU and there are other CPUs
+         * then we waited.  While we were waiting, another CPU may
+         * have instigated a synchronize broadcast request.
+         */
+        code = 0;
+
+    } while (sysblk.broadcast_count);
 
 } /* end function synchronize_broadcast */
 
@@ -134,32 +142,29 @@ RADR    pfx;
 PSA     *psa;
 int     rc;
 
-    /* reset the cpuint indicator */
-    RESET_IC_CPUINT(regs);
-
 #if defined(_FEATURE_SIE)
     /* Set the main storage reference and change bits */
-    if(regs->sie_state
-#if defined(FEATURE_EXPEDITED_SIE_SUBSET)
-                       && !(regs->siebk->s & SIE_S_EXP_TIMER)
-#endif /*defined(FEATURE_EXPEDITED_SIE_SUBSET)*/
+    if(SIE_MODE(regs)
+#if defined(_FEATURE_EXPEDITED_SIE_SUBSET)
+                       && !SIE_FEATB(regs, S, EXP_TIMER)
+#endif /*defined(_FEATURE_EXPEDITED_SIE_SUBSET)*/
 #if defined(_FEATURE_EXTERNAL_INTERRUPT_ASSIST)
-                       && !(regs->siebk->ec[0] & SIE_EC0_EXTA)
+                       && !SIE_FEATB(regs, EC0, EXTA)
 #endif
                                                             )
     {
         /* Point to SIE copy of PSA in state descriptor */
-        psa = (void*)(regs->hostregs->mainstor + regs->sie_state + SIE_IP_PSA_OFFSET);
-        STORAGE_KEY(regs->sie_state, regs->hostregs) |= (STORKEY_REF | STORKEY_CHANGE);
+        psa = (void*)(regs->hostregs->mainstor + SIE_STATE(regs) + SIE_IP_PSA_OFFSET);
+        STORAGE_KEY(SIE_STATE(regs), regs->hostregs) |= (STORKEY_REF | STORKEY_CHANGE);
     }
     else
 #endif /*defined(_FEATURE_SIE)*/
     {
         /* Point to PSA in main storage */
         pfx = regs->PX;
-#if defined(FEATURE_EXPEDITED_SIE_SUBSET)
+#if defined(_FEATURE_EXPEDITED_SIE_SUBSET)
         SIE_TRANSLATE(&pfx, ACCTYPE_SIE, regs);
-#endif /*defined(FEATURE_EXPEDITED_SIE_SUBSET)*/
+#endif /*defined(_FEATURE_EXPEDITED_SIE_SUBSET)*/
         psa = (void*)(regs->mainstor + pfx);
         STORAGE_KEY(pfx, regs) |= (STORKEY_REF | STORKEY_CHANGE);
     }
@@ -174,20 +179,18 @@ int     rc;
 
 #if defined(FEATURE_BCMODE)
     /* For ECMODE, store external interrupt code at PSA+X'86' */
-    if ( regs->psw.ecmode )
+    if ( ECMODE(&regs->psw) )
 #endif /*defined(FEATURE_BCMODE)*/
         STORE_HW(psa->extint,code);
 
-#if defined(_FEATURE_SIE)
-    if(!regs->sie_state
-#if defined(FEATURE_EXPEDITED_SIE_SUBSET)
-                       || (regs->siebk->s & SIE_S_EXP_TIMER)
-#endif /*defined(FEATURE_EXPEDITED_SIE_SUBSET)*/
+    if ( !SIE_MODE(regs)
+#if defined(_FEATURE_EXPEDITED_SIE_SUBSET)
+                       || SIE_FEATB(regs, S, EXP_TIMER)
+#endif /*defined(_FEATURE_EXPEDITED_SIE_SUBSET)*/
 #if defined(_FEATURE_EXTERNAL_INTERRUPT_ASSIST)
-                       || (regs->siebk->ec[0] & SIE_EC0_EXTA)
+                       || SIE_FEATB(regs, EC0, EXTA)
 #endif
-                                                            )
-#endif /*defined(_FEATURE_SIE)*/
+       )
     {
         /* Store current PSW at PSA+X'18' */
         ARCH_DEP(store_psw) (regs, psa->extold);
@@ -204,18 +207,16 @@ int     rc;
 
     release_lock(&sysblk.intlock);
 
-#if defined(_FEATURE_SIE)
-    if(regs->sie_state
-#if defined(FEATURE_EXPEDITED_SIE_SUBSET)
-                       && !(regs->siebk->s & SIE_S_EXP_TIMER)
-#endif /*defined(FEATURE_EXPEDITED_SIE_SUBSET)*/
+    if ( SIE_MODE(regs)
+#if defined(_FEATURE_EXPEDITED_SIE_SUBSET)
+                       && !SIE_FEATB(regs, S, EXP_TIMER)
+#endif /*defined(_FEATURE_EXPEDITED_SIE_SUBSET)*/
 #if defined(_FEATURE_EXTERNAL_INTERRUPT_ASSIST)
-                       && !(regs->siebk->ec[0] & SIE_EC0_EXTA)
+                       && !SIE_FEATB(regs, EC0, EXTA)
 #endif
-                                                            )
+       )
         longjmp (regs->progjmp, SIE_INTERCEPT_EXT);
     else
-#endif /*defined(_FEATURE_SIE)*/
         longjmp (regs->progjmp, SIE_NO_INTERCEPT);
 
 } /* end function external_interrupt */
@@ -242,11 +243,7 @@ PSA    *psa;                            /* -> Prefixed storage area  */
 U16     cpuad;                          /* Originating CPU address   */
 
     /* External interrupt if console interrupt key was depressed */
-    if (OPEN_IC_INTKEY(regs)
-#if defined(_FEATURE_SIE)
-        && !regs->sie_state
-#endif /*!defined(_FEATURE_SIE)*/
-        )
+    if ( OPEN_IC_INTKEY(regs) && !SIE_MODE(regs) )
     {
         logmsg (_("HHCCP023I External interrupt: Interrupt key\n"));
 
@@ -263,7 +260,7 @@ U16     cpuad;                          /* Originating CPU address   */
         /* Find first CPU which generated a malfunction alert */
         for (cpuad = 0; regs->malfcpu[cpuad] == 0; cpuad++)
         {
-            if (cpuad >= MAX_CPU_ENGINES)
+            if (cpuad >= MAX_CPU)
             {
                 OFF_IC_MALFALT(regs);
                 return;
@@ -283,7 +280,7 @@ U16     cpuad;                          /* Originating CPU address   */
         /* Reset emergency signal pending flag if there are
            no other CPUs which generated emergency signal */
         OFF_IC_MALFALT(regs);
-        while (++cpuad < MAX_CPU_ENGINES)
+        while (++cpuad < MAX_CPU)
         {
             if (regs->malfcpu[cpuad])
             {
@@ -303,7 +300,7 @@ U16     cpuad;                          /* Originating CPU address   */
         /* Find first CPU which generated an emergency signal */
         for (cpuad = 0; regs->emercpu[cpuad] == 0; cpuad++)
         {
-            if (cpuad >= MAX_CPU_ENGINES)
+            if (cpuad >= MAX_CPU)
             {
                 OFF_IC_EMERSIG(regs);
                 return;
@@ -323,7 +320,7 @@ U16     cpuad;                          /* Originating CPU address   */
         /* Reset emergency signal pending flag if there are
            no other CPUs which generated emergency signal */
         OFF_IC_EMERSIG(regs);
-        while (++cpuad < MAX_CPU_ENGINES)
+        while (++cpuad < MAX_CPU)
         {
             if (regs->emercpu[cpuad])
             {
@@ -354,7 +351,7 @@ U16     cpuad;                          /* Originating CPU address   */
     }
 
     /* External interrupt if TOD clock exceeds clock comparator */
-    if ((sysblk.todclk + regs->todoffset) > regs->clkc
+    if ( TOD_CLOCK(regs) > regs->clkc
         && sysblk.insttrace == 0
         && sysblk.inststep == 0
         && OPEN_IC_CLKC(regs) )
@@ -372,7 +369,7 @@ U16     cpuad;                          /* Originating CPU address   */
     {
         if (sysblk.insttrace || sysblk.inststep)
         {
-            logmsg (_("HHCCP025I External interrupt: CPU timer=%16.16llX\n"),
+            logmsg (_("HHCCP025I External interrupt: CPU timer=%16.16" I64_FMT "X\n"),
                     (long long)regs->ptimer);
         }
         ARCH_DEP(external_interrupt) (EXT_CPU_TIMER_INTERRUPT, regs);
@@ -382,26 +379,48 @@ U16     cpuad;                          /* Originating CPU address   */
 #if defined(FEATURE_INTERVAL_TIMER)
     if (OPEN_IC_ITIMER(regs)
 #if defined(_FEATURE_SIE)
-        && !(regs->sie_state
-          && (regs->siebk->m & SIE_M_ITMOF))
+        && !(SIE_STATB(regs, M, ITMOF))
 #endif /*defined(_FEATURE_SIE)*/
         )
     {
+#if defined(FEATURE_ECPSVM)
+        if(ecpsvm_virttmr_ext(regs)==0)
+        {
+            regs->vtimerint=0;
+            /* If no Real Int Timer Int pending */
+            /* Otherwise, we keep the Int Timer */
+            /* status on                        */
+            if(!regs->rtimerint)
+            {
+                OFF_IC_ITIMER(regs);
+            }
+            ARCH_DEP(external_interrupt) (EXT_VINTERVAL_TIMER_INTERRUPT,regs);
+        }
+#endif
         if (sysblk.insttrace || sysblk.inststep)
         {
             logmsg (_("HHCCP026I External interrupt: Interval timer\n"));
         }
+        /* NOTE : Virtual Interval Timer may still be there.. */
+        /*        but we will have to wait for the next timer */
+        /*        pass , because if regs->vtimerint is still  */
+        /*        1, it means the conditions were not right   */
+        /*        to present the interrupt...                 */
         OFF_IC_ITIMER(regs);
-        ARCH_DEP(external_interrupt) (EXT_INTERVAL_TIMER_INTERRUPT, regs);
+#if defined(FEATURE_ECPSVM)
+        if(regs->rtimerint)
+        {
+            regs->rtimerint=0;
+#endif
+            ARCH_DEP(external_interrupt) (EXT_INTERVAL_TIMER_INTERRUPT, regs);
+#if defined(FEATURE_ECPSVM)
+        }
+#endif
     }
 #endif /*FEATURE_INTERVAL_TIMER*/
 
     /* External interrupt if service signal is pending */
-    if (OPEN_IC_SERVSIG(regs)
-#if defined(_FEATURE_SIE)
-        && !regs->sie_state
-#endif /*!defined(_FEATURE_SIE)*/
-        )
+    if ( OPEN_IC_SERVSIG(regs) && !SIE_MODE(regs) )
     {
         /* Apply prefixing if the parameter is a storage address */
         if ( (sysblk.servparm & SERVSIG_ADDR) )
@@ -425,9 +444,6 @@ U16     cpuad;                          /* Originating CPU address   */
         ARCH_DEP(external_interrupt) (EXT_SERVICE_SIGNAL_INTERRUPT, regs);
     }
 
-    /* reset the cpuint indicator */
-    RESET_IC_CPUINT(regs);
-
 } /* end function perform_external_interrupt */
 
 
@@ -437,6 +453,10 @@ U16     cpuad;                          /* Originating CPU address   */
 /*      sregs   Register context of CPU whose status is to be stored */
 /*      aaddr   A valid absolute address of a 512-byte block into    */
 /*              which status is to be stored                         */
+/*              For an implicit store status, or an operator         */
+/*              initiated store status the absolute address will be  */
+/*              zero, for a store status at address order the        */
+/*              supplied address will be nonzero                     */
 /*-------------------------------------------------------------------*/
 void ARCH_DEP(store_status) (REGS *ssreg, RADR aaddr)
 {
@@ -444,6 +464,23 @@ U64     dreg;                           /* Double register work area */
 int     i;                              /* Array subscript           */
 PSA     *sspsa;                         /* -> Store status area      */
 
+    /* Set reference and change bits */
+    STORAGE_KEY(aaddr, ssreg) |= (STORKEY_REF | STORKEY_CHANGE);
+#if defined(FEATURE_ESAME)
+    /* The ESAME PSA is two pages in size */
+    if(!aaddr)
+        STORAGE_KEY(aaddr + 4096, ssreg) |= (STORKEY_REF | STORKEY_CHANGE);
+#endif /*defined(FEATURE_ESAME)*/
+
+#if defined(FEATURE_ESAME)
+    /* For store status at address, we must ajust the PSA offset */
+    /* ZZ THIS TEST IS NOT CONCLUSIVE */
+    if(aaddr != 0 && aaddr != ssreg->PX)
+        aaddr -= 512 + 4096 ;
+#endif
+
+    aaddr &= 0x7FFFFE00;
+
     /* Point to the PSA into which status is to be stored */
     sspsa = (void*)(ssreg->mainstor + aaddr);
 
@@ -452,7 +489,11 @@ PSA     *sspsa;                         /* -> Store status area      */
     STORE_DW(sspsa->storeptmr, ssreg->ptimer);
 
     /* Store clock comparator in bytes 224-231 */
+#if defined(FEATURE_ESAME)
+    STORE_DW(sspsa->storeclkc, ssreg->clkc);
+#else /*defined(FEATURE_ESAME)*/
     STORE_DW(sspsa->storeclkc, ssreg->clkc << 8);
+#endif /*defined(FEATURE_ESAME)*/
 
     /* Store PSW in bytes 256-263 */
     ARCH_DEP(store_psw) (ssreg, sspsa->storepsw);
@@ -469,10 +510,12 @@ PSA     *sspsa;                         /* -> Store status area      */
 #endif /*defined(FEATURE_ESAME)*/
 
 #if defined(_900)
+    /* Only store the arch mode indicator for a PSA type store status */
+    if(!aaddr)
 #if defined(FEATURE_ESAME)
-    sspsa->arch = 1;
+        sspsa->arch = 1;
 #else /*defined(FEATURE_ESAME)*/
-    sspsa->arch = 0;
+        sspsa->arch = 0;
 #endif /*defined(FEATURE_ESAME)*/
 #endif /*defined(_900)*/
 
