@@ -1,11 +1,13 @@
-/* FEATCHK.H    (c) Copyright Jan Jaeger, 2000-2003          */
-/*      Feature definition consistency checks            */
-
+/* FEATCHK.H    (c) Copyright Jan Jaeger, 2000-2005                  */
 /*-------------------------------------------------------------------*/
-/* Perform various checks on feature combinations, and set       */
-/* additional flags to percolate certain features such as        */
-/* SIE down to lower architecture levels such that these         */
-/* can include emulation support                     */
+/*                                                                   */
+/*           Feature definition consistency checks                   */
+/*                                                                   */
+/*  Perform various checks on feature combinations, and set          */
+/*  additional flags to percolate certain features such as           */
+/*  SIE down to lower architecture levels such that these            */
+/*  can include emulation support                                    */
+/*                                                                   */
 /*-------------------------------------------------------------------*/
 
 #if defined(FEATCHK_CHECK_ALL)
@@ -25,7 +27,7 @@
  #endif
 #endif
 
-/* _FEATURE_MULTIPLE_CONTROLLED_DATA_SPACE is used for host 
+/* _FEATURE_MULTIPLE_CONTROLLED_DATA_SPACE is used for host
    related processing issues, FEATURE_MULTIPLE_CONTROLLED_DATA_SPACE
    is defined only in ESA/390 mode. MCDS is an ESA/390
    feature that is supported under z/Architecture SIE */
@@ -57,6 +59,10 @@
  #define _FEATURE_EXPANDED_STORAGE
 #endif
 
+#if defined(FEATURE_ECPSVM)
+ #define _FEATURE_ECPSVM
+#endif
+
 #if defined(_FEATURE_SIE) && defined(FEATURE_STORAGE_KEY_ASSIST)
  #define _FEATURE_STORAGE_KEY_ASSIST
 #endif
@@ -93,6 +99,14 @@
  #define _FEATURE_EXTERNAL_INTERRUPT_ASSIST
 #endif
 
+#if defined(FEATURE_MESSAGE_SECURITY_ASSIST)
+ #define _FEATURE_MESSAGE_SECURITY_ASSIST
+#endif
+
+#if defined(FEATURE_ASN_AND_LX_REUSE)
+ #define _FEATURE_ASN_AND_LX_REUSE
+#endif
+
 #undef _VSTORE_C_STATIC
 #if !defined(OPTION_NO_INLINE_VSTORE)
  #define _VSTORE_C_STATIC static inline
@@ -168,7 +182,7 @@
  #define GEN_MAXARCH    3+2
 #elif defined(_ARCHMODE2)
  #define GEN_MAXARCH    2+2
-#else 
+#else
  #define GEN_MAXARCH    1+2
 #endif
 
@@ -176,14 +190,15 @@
  #error OPTION_390_MODE must be enabled for OPTION_900_MODE
 #endif
 
+
 #else /*!defined(FEATCHK_CHECK_ALL)*/
 
 /* When ESAME is installed then all instructions
    marked N3 in the reference are also available
    in ESA/390 mode */
-#if defined(_900)
+#if defined(_900) && (__GEN_ARCH == 390)
  #define FEATURE_ESAME_N3_ESA390
-#endif /*defined(_900)*/
+#endif
 
 #if !defined(FEATURE_2K_STORAGE_KEYS) \
  && !defined(FEATURE_4K_STORAGE_KEYS)
@@ -204,7 +219,7 @@
 #if defined(_900) && defined(FEATURE_VECTOR_FACILITY)
  #error Vector Facility not supported on ESAME capable processors
 #endif
- 
+
 #if defined(FEATURE_MOVE_PAGE_FACILITY_2) \
  && !defined(FEATURE_4K_STORAGE_KEYS)
  #error Move page facility cannot be defined with 2K storage keys
@@ -251,6 +266,11 @@
  #error I/O Assist Feature only supported with SIE
 #endif
 
+#if defined(FEATURE_IO_ASSIST) \
+ && !defined(_FEATURE_REGION_RELOCATE)
+ #error Region Relocate Facility required for IO Assist
+#endif
+
 #if defined(FEATURE_EXTERNAL_INTERRUPT_ASSIST) \
  && !defined(_FEATURE_SIE)
  #error External Interruption assist only supported with SIE
@@ -261,6 +281,15 @@
  #error Expedited SIE Subset only supported with SIE
 #endif
 
+#if defined(FEATURE_ASN_AND_LX_REUSE)
+ #if !defined(FEATURE_DUAL_ADDRESS_SPACE)
+  #error ASN-and-LX-Reuse requires Dual Address-Space feature
+ #endif
+ #if !defined(FEATURE_ESAME)
+  #error ASN-and-LX-Reuse is only supported with ESAME
+ #endif
+#endif
+
 #if defined(FEATURE_ESAME) \
  && defined(FEATURE_VECTOR_FACILITY)
  #error Vector Facility not supported in ESAME mode
@@ -284,10 +313,42 @@
  #endif
 #endif
 
+#if defined(FEATURE_FPS_EXTENSIONS) \
+ && !defined(FEATURE_BINARY_FLOATING_POINT)
+ #error FP support extensions requires binary floating point
+#endif
+
+#if defined(FEATURE_HFP_MULTIPLY_ADD_SUBTRACT) \
+ && !defined(FEATURE_HEXADECIMAL_FLOATING_POINT)
+ #error HFP multiply add/subtract requires hexadecimal floating point
+#endif
+
+#if defined(FEATURE_HFP_UNNORMALIZED_EXTENSION) \
+ && !defined(FEATURE_HEXADECIMAL_FLOATING_POINT)
+ #error HFP unnormalized extension requires hexadecimal floating point
+#endif
+
 #if defined(FEATURE_PER2) && !defined(FEATURE_PER)
  #error FEATURE_PER must be defined when using FEATURE_PER2
 #endif
 
+#if defined(FEATURE_PER3) && !defined(FEATURE_PER)
+ #error FEATURE_PER must be defined when using FEATURE_PER3
+#endif
+
+
+#if defined(FEATURE_MESSAGE_SECURITY_ASSIST)
+ #if defined(_370)
+  #define _370_FEATURE_MESSAGE_SECURITY_ASSIST
+ #endif
+ #if defined(_390)
+  #define _390_FEATURE_MESSAGE_SECURITY_ASSIST
+ #endif
+ #if defined(_900)
+  #define _900_FEATURE_MESSAGE_SECURITY_ASSIST
+ #endif
+#endif /*defined(FEATURE_MESSAGE_SECURITY_ASSIST)*/
+
 
 #endif /*!defined(FEATALL_CHECKALL)*/
 
