@@ -1,23 +1,19 @@
 ////////////////////////////////////////////////////////////////////////////////////
 //         fishhang.c           verify/debug proper Hercules LOCK handling...
 ////////////////////////////////////////////////////////////////////////////////////
-// (c) Copyright "Fish" (David B. Trout), 2002-2003. Released under the Q Public License
+// (c) Copyright "Fish" (David B. Trout), 2002-2005. Released under the Q Public License
 // (http://www.conmicro.cx/hercules/herclic.html) as modifications to Hercules.
 ////////////////////////////////////////////////////////////////////////////////////
 
-#if defined(HAVE_CONFIG_H)
-#include <config.h>     // (needed to set FISH_HANG flag!)
-#endif
+#include "hstdinc.h"
 
-#if !defined(FISH_HANG)
-int dummy = 0;
-#else // defined(FISH_HANG)
+#define _FISHHANG_C_
+#define _HUTIL_DLL_
 
-#include <windows.h>    // (standard WIN32)
-#include <stdio.h>      // (need "fprintf")
-#include <malloc.h>     // (need "malloc")
-#include "linklist.h"   // (linked list macros)
-#include "fishhang.h"   // (prototypes for this module)
+#include "hercules.h"
+#include "fishhang.h"       // (prototypes for this module)
+
+#if defined(FISH_HANG)
 
 ////////////////////////////////////////////////////////////////////////////////////
 // Global variables...
@@ -32,7 +28,27 @@ CRITICAL_SECTION                                ListsLock;
 #define LockFishHang()   (EnterCriticalSection(&ListsLock))
 #define UnlockFishHang() (LeaveCriticalSection(&ListsLock))
 
-BOOL  bFishHangAtExit = FALSE;  // (set before exiting to disable some error checking)
+// bFishHangAtExit is set before exiting to disable some error checking...
+
+DLL_EXPORT
+BOOL  bFishHangAtExit = FALSE;
+
+//////////////////////////////////////////////////////////////////////////////////
+// We need to use our own private report o/p stream since
+// the logger thread uses fthreads and thus fishhang too!
+
+FILE*  fh_report_stream = NULL;
+
+#define  logmsg  FishHang_Printf
+
+DLL_EXPORT
+void FishHang_Printf( const char* pszFormat, ... )
+{
+    va_list   vl;
+    va_start( vl, pszFormat );
+    vfprintf( fh_report_stream, pszFormat, vl );
+    fflush( fh_report_stream );
+}
 
 //////////////////////////////////////////////////////////////////////////////////
 // Thread information...   (locks it owns, which lock/event it's waiting on, etc)
@@ -41,7 +57,7 @@ typedef struct _tagFishThread
 {
     DWORD       dwCreatingThreadID;     // threadid that created it
     SYSTEMTIME  timeCreated;            // time created
-    char*       pszFileCreated;         // source file that created it
+    const char* pszFileCreated;         // source file that created it
     int         nLineCreated;           // line number of source file
 
     //////////////////////////////////////////////////////////////////
@@ -59,7 +75,7 @@ typedef struct _tagFishThread
     void*       pWhatWaiting;           // ptr to FISH_LOCK last attempted to be signalled
                                         // -or- to FISH_EVENT last waited to be posted
                                         // (NOTE: must manually cast to whichever type)
-    char*       pszFileWaiting;         // source file where attempt/wait occured
+    const char* pszFileWaiting;         // source file where attempt/wait occured
     int         nLineWaiting;           // line number of source file
     SYSTEMTIME  timeWaiting;            // time when attempt/wait occured
 }
@@ -72,7 +88,7 @@ typedef struct _tagFishLock
 {
     DWORD       dwCreatingThreadID;     // threadid that created it
     SYSTEMTIME  timeCreated;            // time created
-    char*       pszFileCreated;         // source file that created it
+    const char* pszFileCreated;         // source file that created it
     int         nLineCreated;           // line number of source file
 
     //////////////////////////////////////////////////////////////////
@@ -94,7 +110,7 @@ typedef struct _tagFishEvent
 {
     DWORD       dwCreatingThreadID;     // threadid that created it
     SYSTEMTIME  timeCreated;            // time created
-    char*       pszFileCreated;         // source file that created it
+    const char* pszFileCreated;         // source file that created it
     int         nLineCreated;           // line number of source file
 
     //////////////////////////////////////////////////////////////////
@@ -104,28 +120,47 @@ typedef struct _tagFishEvent
 
     FISH_THREAD*  pWhoSet;              // ptr to FISH_THREAD that set it
     SYSTEMTIME    timeSet;              // time set
-    char*         pszFileSet;           // source file that set it
+    const char*   pszFileSet;           // source file that set it
     int           nLineSet;             // line number of source file
 
     FISH_THREAD*  pWhoReset;            // ptr to FISH_THREAD that reset it
     SYSTEMTIME    timeReset;            // time reset
-    char*         pszFileReset;         // source file that reset it
+    const char*   pszFileReset;         // source file that reset it
     int           nLineReset;           // line number of source file
 }
 FISH_EVENT;
 
+/////////////////////////////////////////////////////////////////////////////
+// Macro to remove Microsoft's stupid fullpath from __FILE__ name...
+
+#define FIXFILENAME( filename )           \
+    do { if ( filename ) {                \
+    char* p = strrchr( filename, '\\' );  \
+    if (!p) p = strrchr( filename, '/' ); \
+    if (p) filename = p+1; } } while (0)
+
+
 /////////////////////////////////////////////////////////////////////////////
 // (forward references...)
 
-FISH_THREAD* CreateFISH_THREAD(char* pszFileCreated,int nLineCreated);
+FISH_THREAD* CreateFISH_THREAD( const char* pszFileCreated, const int nLineCreated );
 
 /////////////////////////////////////////////////////////////////////////////
 // Initialize global variables...
 
-void FishHangInit(char* pszFileCreated, int nLineCreated)
+DLL_EXPORT
+void FishHangInit( const char* pszFileCreated, const int nLineCreated )
 {
     FISH_THREAD*  pFISH_THREAD;
 
+    FIXFILENAME(pszFileCreated);
+
+    if ( !( fh_report_stream = fopen( "FishHangReport.txt", "w" ) ) )
+    {
+        perror( "FishHang report o/p file open failure" );
+        abort();
+    }
+
     InitializeListHead(&ThreadsListHead);
     InitializeListHead(&LocksListHead);
     InitializeListHead(&EventsListHead);
@@ -133,7 +168,7 @@ void FishHangInit(char* pszFileCreated, int nLineCreated)
 
     if (!(pFISH_THREAD = CreateFISH_THREAD(pszFileCreated,nLineCreated)))
     {
-        fprintf(stdout,"** FishHangInit: CreateFISH_THREAD failed\n");
+        logmsg("** FishHangInit: CreateFISH_THREAD failed\n");
         exit(-1);
     }
 
@@ -144,6 +179,7 @@ void FishHangInit(char* pszFileCreated, int nLineCreated)
 /////////////////////////////////////////////////////////////////////////////
 // Indicate we're about to exit...  (disables some error checking)
 
+DLL_EXPORT
 void FishHangAtExit()
 {
     bFishHangAtExit = TRUE;
@@ -152,21 +188,21 @@ void FishHangAtExit()
 /////////////////////////////////////////////////////////////////////////////
 // Internal abort function...
 
-#define FishHangAbort(file,line)                                            \
-{                                                                           \
-    FishHangAtExit();   /* (disable further error checking) */              \
-    UnlockFishHang();   /* (prevent deadlocking ourselves!) */              \
-    fprintf(stdout,"** FishHangAbort called from %s(%d)\n",(file),(line));  \
-    Sleep(100);         /* (give system time to display messages) */        \
-    exit(-1);           /* (immediately terminate entire process) */        \
+#define FishHangAbort(file,line)                                        \
+{                                                                       \
+    FishHangAtExit();   /* (disable further error checking) */          \
+    UnlockFishHang();   /* (prevent deadlocking ourselves!) */          \
+    logmsg("** FishHangAbort called from %s(%d)\n",(file),(line));      \
+    Sleep(100);         /* (give system time to display messages) */    \
+    exit(-1);           /* (immediately terminate entire process) */    \
 }
 
 /////////////////////////////////////////////////////////////////////////////
 
 FISH_THREAD*  CreateFISH_THREAD
 (
-    char*  pszFileCreated,  // source file that created it;
-    int    nLineCreated     // line number of source file;
+    const char*  pszFileCreated,  // source file that created it;
+    const int    nLineCreated     // line number of source file;
 )
 {
     FISH_THREAD*  pFISH_THREAD = malloc(sizeof(FISH_THREAD));
@@ -175,6 +211,8 @@ FISH_THREAD*  CreateFISH_THREAD
 
     GetSystemTime(&pFISH_THREAD->timeCreated);
 
+    FIXFILENAME(pszFileCreated);
+
     pFISH_THREAD->dwCreatingThreadID = GetCurrentThreadId();
     pFISH_THREAD->pszFileCreated     = pszFileCreated;
     pFISH_THREAD->nLineCreated       = nLineCreated;
@@ -199,8 +237,8 @@ FISH_THREAD*  CreateFISH_THREAD
 
 FISH_LOCK*  CreateFISH_LOCK
 (
-    char*  pszFileCreated,  // source file that created it;
-    int    nLineCreated     // line number of source file;
+    const char*  pszFileCreated,  // source file that created it;
+    const int    nLineCreated     // line number of source file;
 )
 {
     FISH_LOCK*  pFISH_LOCK = malloc(sizeof(FISH_LOCK));
@@ -209,6 +247,8 @@ FISH_LOCK*  CreateFISH_LOCK
 
     GetSystemTime(&pFISH_LOCK->timeCreated);
 
+    FIXFILENAME(pszFileCreated);
+
     pFISH_LOCK->dwCreatingThreadID = GetCurrentThreadId();
     pFISH_LOCK->pszFileCreated     = pszFileCreated;
     pFISH_LOCK->nLineCreated       = nLineCreated;
@@ -227,8 +267,8 @@ FISH_LOCK*  CreateFISH_LOCK
 
 FISH_EVENT*  CreateFISH_EVENT
 (
-    char*   pszFileCreated,     // source file that created it;
-    int     nLineCreated        // line number of source file;
+    const char*   pszFileCreated,     // source file that created it;
+    const int     nLineCreated        // line number of source file;
 )
 {
     FISH_EVENT*  pFISH_EVENT = malloc(sizeof(FISH_EVENT));
@@ -237,6 +277,8 @@ FISH_EVENT*  CreateFISH_EVENT
 
     GetSystemTime(&pFISH_EVENT->timeCreated);
 
+    FIXFILENAME(pszFileCreated);
+
     pFISH_EVENT->dwCreatingThreadID = GetCurrentThreadId();
     pFISH_EVENT->pszFileCreated     = pszFileCreated;
     pFISH_EVENT->nLineCreated       = nLineCreated;
@@ -366,7 +408,7 @@ char*  PrintFISH_THREAD
         ThreadLockListLink = NULL;
     }
 
-    sprintf(PrintFISH_THREADBuffer,
+    snprintf(PrintFISH_THREADBuffer,sizeof(PrintFISH_THREADBuffer),
         "THREAD @ %8.8X\n"
         "         bWaitingForLock    = %s\n"
         "         bTryingForLock     = %s\n"
@@ -405,6 +447,8 @@ char*  PrintFISH_THREAD
             (int)ThreadListLink
         );
 
+    PrintFISH_THREADBuffer[ sizeof(PrintFISH_THREADBuffer) - 1 ] = 0;
+
     return PrintFISH_THREADBuffer;
 }
 
@@ -451,7 +495,7 @@ char*  PrintFISH_LOCK
     {
         if (ThreadLockListLink)
         {
-            fprintf(stdout,"** PrintFISH_LOCK:  **OOPS!**  "
+            logmsg("** PrintFISH_LOCK:  **OOPS!**  "
                 "Lock %8.8X not owned, "
                 "but ThreadLockListLink not NULL!\n",
                 (int)pFISH_LOCK
@@ -459,7 +503,7 @@ char*  PrintFISH_LOCK
         }
     }
 
-    sprintf(PrintFISH_LOCKBuffer,
+    snprintf(PrintFISH_LOCKBuffer,sizeof(PrintFISH_LOCKBuffer),
         "LOCK @ %8.8X\n"
         "       pOwningThread      = %8.8X\n"
         "       nLockedDepth       = %d\n"
@@ -485,6 +529,8 @@ char*  PrintFISH_LOCK
             (int)LockListLink
         );
 
+    PrintFISH_LOCKBuffer[ sizeof(PrintFISH_LOCKBuffer) - 1 ] = 0;
+
     return PrintFISH_LOCKBuffer;
 }
 
@@ -510,7 +556,7 @@ char*  PrintFISH_EVENT
     }
     else EventsListLink = (FISH_EVENT*) &EventsListHead;
 
-    sprintf(PrintFISH_EVENTBuffer,
+    snprintf(PrintFISH_EVENTBuffer,sizeof(PrintFISH_EVENTBuffer),
         "EVENT @ %8.8X\n"
         "        timeSet            = %2.2d:%2.2d:%2.2d.%3.3d\n"
         "        pszFileSet         = %s\n"
@@ -552,6 +598,8 @@ char*  PrintFISH_EVENT
             (int)EventsListLink
         );
 
+    PrintFISH_EVENTBuffer[ sizeof(PrintFISH_EVENTBuffer) - 1 ] = 0;
+
     return PrintFISH_EVENTBuffer;
 }
 
@@ -571,14 +619,14 @@ void  PrintAllFISH_THREADs()
     }
     else pFISH_THREAD = (FISH_THREAD*) &ThreadsListHead;
 
-    fprintf(stdout,"\nTHREAD LIST ANCHOR @ %8.8X --> %8.8X\n\n",
+    logmsg("\nTHREAD LIST ANCHOR @ %8.8X --> %8.8X\n\n",
         (int)&ThreadsListHead,(int)pFISH_THREAD);
 
     while (pListEntry != &ThreadsListHead)
     {
         pFISH_THREAD = CONTAINING_RECORD(pListEntry,FISH_THREAD,ThreadListLink);
         pListEntry = pListEntry->Flink;
-        fprintf(stdout,"%s\n",PrintFISH_THREAD(pFISH_THREAD));
+        logmsg("%s\n",PrintFISH_THREAD(pFISH_THREAD));
     }
 }
 
@@ -598,14 +646,14 @@ void  PrintAllFISH_LOCKs()
     }
     else pFISH_LOCK = (FISH_LOCK*) &LocksListHead;
 
-    fprintf(stdout,"\nLOCK LIST ANCHOR @ %8.8X --> %8.8X\n\n",
+    logmsg("\nLOCK LIST ANCHOR @ %8.8X --> %8.8X\n\n",
         (int)&LocksListHead,(int)pFISH_LOCK);
 
     while (pListEntry != &LocksListHead)
     {
         pFISH_LOCK = CONTAINING_RECORD(pListEntry,FISH_LOCK,LockListLink);
         pListEntry = pListEntry->Flink;
-        fprintf(stdout,"%s\n",PrintFISH_LOCK(pFISH_LOCK));
+        logmsg("%s\n",PrintFISH_LOCK(pFISH_LOCK));
     }
 }
 
@@ -625,23 +673,24 @@ void  PrintAllFISH_EVENTs()
     }
     else pFISH_EVENT = (FISH_EVENT*) &EventsListHead;
 
-    fprintf(stdout,"\nEVENT LIST ANCHOR @ %8.8X --> %8.8X\n\n",
+    logmsg("\nEVENT LIST ANCHOR @ %8.8X --> %8.8X\n\n",
         (int)&EventsListHead,(int)pFISH_EVENT);
 
     while (pListEntry != &EventsListHead)
     {
         pFISH_EVENT = CONTAINING_RECORD(pListEntry,FISH_EVENT,EventsListLink);
         pListEntry = pListEntry->Flink;
-        fprintf(stdout,"%s\n",PrintFISH_EVENT(pFISH_EVENT));
+        logmsg("%s\n",PrintFISH_EVENT(pFISH_EVENT));
     }
 }
 
 /////////////////////////////////////////////////////////////////////////////
 
+DLL_EXPORT
 HANDLE FishHang_CreateThread
 (
-    char*  pszFileCreated,  // source file that created it
-    int    nLineCreated,    // line number of source file
+    const char*  pszFileCreated,  // source file that created it
+    const int    nLineCreated,    // line number of source file
 
     LPSECURITY_ATTRIBUTES   lpThreadAttributes, // pointer to security attributes
     DWORD                   dwStackSize,        // initial thread stack size
@@ -654,6 +703,8 @@ HANDLE FishHang_CreateThread
     FISH_THREAD*  pFISH_THREAD;
     HANDLE        hThread;
 
+    FIXFILENAME(pszFileCreated);
+
     if (!(pFISH_THREAD = CreateFISH_THREAD(pszFileCreated,nLineCreated))) return NULL;
 
     hThread = CreateThread
@@ -684,33 +735,36 @@ HANDLE FishHang_CreateThread
 
 /////////////////////////////////////////////////////////////////////////////
 
+DLL_EXPORT
 void FishHang_InitializeCriticalSection
 (
-    char*  pszFileCreated,  // source file that created it
-    int    nLineCreated,    // line number of source file
+    const char*  pszFileCreated,  // source file that created it
+    const int    nLineCreated,    // line number of source file
 
     LPCRITICAL_SECTION lpCriticalSection   // address of critical section object
 )
 {
     FISH_LOCK*  pFISH_LOCK;
 
+    FIXFILENAME(pszFileCreated);
+
     LockFishHang();
 
     pFISH_LOCK = FindFISH_LOCK(lpCriticalSection);
 
     if (pFISH_LOCK)
     {
-        fprintf(stdout,
+        logmsg(
             "** ERROR ** FISH_LOCK %8.8X already initialized!\n",
             (int)pFISH_LOCK);
-        fprintf(stdout,"%s(%d)\n",pszFileCreated,nLineCreated);
-        fprintf(stdout,"%s\n",PrintFISH_LOCK(pFISH_LOCK));
+        logmsg("%s(%d)\n",pszFileCreated,nLineCreated);
+        logmsg("%s\n",PrintFISH_LOCK(pFISH_LOCK));
         FishHangAbort(pszFileCreated,nLineCreated);
     }
 
     if (!(pFISH_LOCK = CreateFISH_LOCK(pszFileCreated,nLineCreated)))
     {
-        fprintf(stdout,"** FishHang_InitializeCriticalSection: CreateFISH_LOCK failed!\n");
+        logmsg("** FishHang_InitializeCriticalSection: CreateFISH_LOCK failed!\n");
         FishHangAbort(pszFileCreated,nLineCreated);
     }
 
@@ -738,7 +792,7 @@ BOOL GetThreadAndLockPtrs
 
     if (!*ppFISH_THREAD)
     {
-        fprintf(stdout,
+        logmsg(
             "** GetThreadAndLockPtrs: FindFISH_THREAD(%8.8X) failed!\n",
             (int)GetCurrentThreadId());
         PrintAllFISH_THREADs();     // (caller will abort)
@@ -746,7 +800,7 @@ BOOL GetThreadAndLockPtrs
 
     if (!*ppFISH_LOCK)
     {
-        fprintf(stdout,
+        logmsg(
             "** GetThreadAndLockPtrs: FindFISH_LOCK(%8.8X) failed!\n",
             (int)lpCriticalSection);
         PrintAllFISH_LOCKs();       // (caller will abort)
@@ -772,7 +826,7 @@ BOOL GetThreadAndEventPtrs
 
     if (!*ppFISH_THREAD)
     {
-        fprintf(stdout,
+        logmsg(
             "** GetThreadAndEventPtrs: FindFISH_THREAD(%8.8X) failed!\n",
             (int)GetCurrentThreadId());
         PrintAllFISH_THREADs();     // (caller will abort)
@@ -780,7 +834,7 @@ BOOL GetThreadAndEventPtrs
 
     if (!*ppFISH_EVENT)
     {
-        fprintf(stdout,
+        logmsg(
             "** GetThreadAndEventPtrs: FindFISH_EVENT(%8.8X) failed!\n",
             (int)hEvent);
         PrintAllFISH_EVENTs();      // (caller will abort)
@@ -795,8 +849,8 @@ BOOL GetThreadAndEventPtrs
 BOOL  PrintDeadlock
 (
     FISH_THREAD*  pForThisFISH_THREAD,  // thread to check
-    char*         pszFile,              // source file (may be NULL)
-    int           nLine                 // line number of source file
+    const char*         pszFile,        // source file (may be NULL)
+    const int           nLine           // line number of source file
 )
 {
     LIST_ENTRY*   pListEntry;
@@ -804,9 +858,10 @@ BOOL  PrintDeadlock
     FISH_LOCK*    pWhatWaiting;
     FISH_THREAD*  pOwningThread;
 
-    // For each thread that is waiting for a lock (pFISH_THREAD),
-    // see if the thread that owns the lock (pOwningThread) is waiting
-    // for any of the locks that the first thread (pFISH_THREAD) already owns...
+    FIXFILENAME(pszFile);
+
+    // Technique: for each thread that is waiting for a lock, chase the lock owner
+    // chain to see if it leads back to the thread we started with...
 
     pListEntry = ThreadsListHead.Flink;
 
@@ -823,68 +878,87 @@ BOOL  PrintDeadlock
         if (pForThisFISH_THREAD &&
             pFISH_THREAD != pForThisFISH_THREAD) continue;
 
-        // If thread isn't waiting for a lock, no deadlock possible; skip it.
+        // If the thread isn't waiting for a lock, then no deadlock possible; skip it.
 
-        if (!pFISH_THREAD->bWaitingForLock) continue;
+        if (!pFISH_THREAD->bWaitingForLock)
+        {
+            // If the caller was only interested in checking this one thread
+            // then since it's not waiting for any locks, we're done...
 
-        // This thread (pFISH_THREAD) is waiting for a lock. Chase all the locks
-        // for the thread that currently owns the lock (pOwningThread) and see if
-        // maybe it is waiting for any locks that THIS thread (pFISH_THREAD) owns...
+            if (pForThisFISH_THREAD)
+                return FALSE;               // (that was easy!)
 
-        pOwningThread = pFISH_THREAD;   // (so we can chase our chains...)
+            continue;
+        }
+
+        // This thread (pFISH_THREAD) is waiting for a lock. Chase the lock owner
+        // chain to see if it eventually leads back to the current thread. If so,
+        // then we've found a deadlock...
+
+        // (I.e. If this thread A is waiting for lock X and lock X is owned by thread
+        // B and thread B is itself waiting for lock Y and lock Y is owned by thread C
+        // and thread C is itself waiting for lock Z and lock Z is owned by thread A
+        // (the thread we started with), then we have found a deadlock situation. If
+        // any of the threads in the chain are not waiting for a lock. then of course
+        // no deadlock is possible so we simply move on to the next thread).
+
+        // Get a pointer to the lock that this thread is waiting for...
+
+        pWhatWaiting = (FISH_LOCK*) pFISH_THREAD->pWhatWaiting;
 
         for (;;)
         {
-            // Get a pointer to the lock that is being waited on...
+            // (Note: if the pOwningThread variable is NULL (i.e. lock not currently
+            // owned by anyone), then the thread isn't actually waiting for a lock at
+            // all. All a NULL pOwningThread variable means is the thread was simply
+            // interrupted in the middle of its FishHang_EnterCriticalSection call,
+            // and if had NOT been interrupted, it WOULD have grabbed that lock.)
+
+            // Grab the owner of the lock the current thread is trying to acquire...
+
+            pOwningThread = pWhatWaiting->pOwningThread;    // (next owner in chain)
+
+            if (!pOwningThread ||                   // (if there is no lock owner, or)
+                !pOwningThread->bWaitingForLock)    // (owner not waiting for any locks)
+                break;                              // (then go on to next thread)
+
+            // Grab which lock the current lock-owner is trying to acquire...
 
             pWhatWaiting = (FISH_LOCK*) pOwningThread->pWhatWaiting;
 
-            // Get a pointer to the thread that OWNS the lock being waited on...
+            // If we've circled around such that the current lock-owner is the very
+            // same thread we began with, then we've detected a deadlock situation...
 
-            pOwningThread = pWhatWaiting->pOwningThread;
+            if (pOwningThread != pFISH_THREAD)      // (back to where we started?)
+                continue;                           // (nope, go on to next owner)
 
-            // If the lock is not owned by anyone or if the thread that owns it
-            // is not waiting for another lock, then no deadlock is possible...
+            logmsg(">>>>>>> Deadlock detected! <<<<<<<\n");
+            if (pszFile) logmsg("%s(%d)\n",pszFile,nLine);
+            logmsg("\n");
 
-            if (!pOwningThread || !pOwningThread->bWaitingForLock) break;
+            // Now do the same thing we already just did, but THIS time print
+            // each thread that's participating in the deadlock and the lock
+            // they're each waiting on...
 
-            // If the thread that owns the lock that the lock owner is waiting on
-            // is not the thread currently being processed (pFISH_THREAD), then it's
-            // not a deadlock for the thread currently being processed. (Note: if it
-            // is a part of some other deadlocak situation, we'll detect it when we
-            // eventually process the other participating thread.)
+            pWhatWaiting = (FISH_LOCK*) pFISH_THREAD->pWhatWaiting;
 
-            if (pOwningThread != pFISH_THREAD) continue;
+            logmsg("%s",PrintFISH_THREAD(pFISH_THREAD));
+            logmsg("%s",PrintFISH_LOCK(pWhatWaiting));
 
-            // Ah-HA! This thread is waiting for a lock that WE own! Oops!
-
-            fprintf(stdout,">>>>>>> Deadlock detected! <<<<<<<\n");
-            if (pszFile) fprintf(stdout,"%s(%d)\n",pszFile,nLine);
-            fprintf(stdout,"\n");
-
-            for (;;)
+            do
             {
-                // Display the thread that owns the lock causing the deadlock...
-
-                fprintf(stdout,"%s",PrintFISH_THREAD(pOwningThread));
-
-                // Display the lock that's causing the deadlock (the one WE own)...
-
-                pWhatWaiting = (FISH_LOCK*) pOwningThread->pWhatWaiting;
-                fprintf(stdout,"%s",PrintFISH_LOCK(pWhatWaiting));
-
-                // Now switch over to the other participant in the deadlock
-                // and display the same information from that perspective...
+                logmsg("\n");
 
                 pOwningThread = pWhatWaiting->pOwningThread;
-                if (pOwningThread != pFISH_THREAD) continue;
+                pWhatWaiting = (FISH_LOCK*) pOwningThread->pWhatWaiting;
 
-                // The deadlock information for both participating threads
-                // has been displayed. Return to caller so it can abort.
-
-                fprintf(stdout,"\n");
-                return TRUE;            // (deadlock detected!)
+                logmsg("%s",PrintFISH_THREAD(pOwningThread));
+                logmsg("%s",PrintFISH_LOCK(pWhatWaiting));
             }
+            while (pWhatWaiting->pOwningThread != pFISH_THREAD);
+
+            logmsg("\n");
+            return TRUE;            // (deadlock detected!)
         }
     }
 
@@ -893,10 +967,11 @@ BOOL  PrintDeadlock
 
 /////////////////////////////////////////////////////////////////////////////
 
+DLL_EXPORT
 void FishHang_EnterCriticalSection
 (
-    char*  pszFileWaiting,  // source file that attempted it
-    int    nLineWaiting,    // line number of source file
+    const char*  pszFileWaiting,  // source file that attempted it
+    const int    nLineWaiting,    // line number of source file
 
     LPCRITICAL_SECTION lpCriticalSection   // address of critical section object
 )
@@ -904,6 +979,8 @@ void FishHang_EnterCriticalSection
     FISH_THREAD*  pFISH_THREAD;
     FISH_LOCK*    pFISH_LOCK;
 
+    FIXFILENAME(pszFileWaiting);
+
     LockFishHang();
 
     if (!GetThreadAndLockPtrs(&pFISH_THREAD,&pFISH_LOCK,lpCriticalSection))
@@ -945,10 +1022,11 @@ void FishHang_EnterCriticalSection
 
 /////////////////////////////////////////////////////////////////////////////
 
+DLL_EXPORT
 BOOL FishHang_TryEnterCriticalSection
 (
-    char*  pszFileWaiting,  // source file that attempted it
-    int    nLineWaiting,    // line number of source file
+    const char*  pszFileWaiting,  // source file that attempted it
+    const int    nLineWaiting,    // line number of source file
 
     LPCRITICAL_SECTION lpCriticalSection   // address of critical section object
 )
@@ -957,6 +1035,8 @@ BOOL FishHang_TryEnterCriticalSection
     FISH_LOCK*    pFISH_LOCK;
     BOOL          bSuccess;
 
+    FIXFILENAME(pszFileWaiting);
+
     LockFishHang();
 
     if (!GetThreadAndLockPtrs(&pFISH_THREAD,&pFISH_LOCK,lpCriticalSection))
@@ -1004,10 +1084,11 @@ BOOL FishHang_TryEnterCriticalSection
 
 /////////////////////////////////////////////////////////////////////////////
 
+DLL_EXPORT
 void FishHang_LeaveCriticalSection
 (
-    char*  pszFileReleasing,    // source file that attempted it
-    int    nLineReleasing,      // line number of source file
+    const char*  pszFileReleasing,    // source file that attempted it
+    const int    nLineReleasing,      // line number of source file
 
     LPCRITICAL_SECTION lpCriticalSection   // address of critical section object
 )
@@ -1015,6 +1096,8 @@ void FishHang_LeaveCriticalSection
     FISH_THREAD*  pFISH_THREAD;
     FISH_LOCK*    pFISH_LOCK;
 
+    FIXFILENAME(pszFileReleasing);
+
     LockFishHang();
 
     if (!GetThreadAndLockPtrs(&pFISH_THREAD,&pFISH_LOCK,lpCriticalSection))
@@ -1022,7 +1105,7 @@ void FishHang_LeaveCriticalSection
 
     if (!bFishHangAtExit && pFISH_LOCK->pOwningThread != pFISH_THREAD)
     {
-        fprintf(stdout,
+        logmsg(
             "\n** ERROR ** FISH_THREAD %8.8X "
             "releasing FISH_LOCK %8.8X "
             "owned by FISH_THREAD %8.8X!\n"
@@ -1054,7 +1137,7 @@ void FishHang_LeaveCriticalSection
     {
         if (pFISH_LOCK->nLockedDepth < 0)
         {
-            fprintf(stdout,
+            logmsg(
                 "\n** ERROR ** FISH_THREAD %8.8X "
                 "attempted to release FISH_LOCK %8.8X "
                 "one too many times!?!\n"
@@ -1088,10 +1171,11 @@ void FishHang_LeaveCriticalSection
 
 /////////////////////////////////////////////////////////////////////////////
 
+DLL_EXPORT
 HANDLE FishHang_CreateEvent
 (
-    char*  pszFileCreated,  // source file that created it
-    int    nLineCreated,    // line number of source file
+    const char*  pszFileCreated,  // source file that created it
+    const int    nLineCreated,    // line number of source file
 
     LPSECURITY_ATTRIBUTES  lpEventAttributes,   // pointer to security attributes
     BOOL                   bManualReset,        // flag for manual-reset event
@@ -1102,9 +1186,11 @@ HANDLE FishHang_CreateEvent
     FISH_EVENT*  pFISH_EVENT;
     HANDLE       hEvent;
 
+    FIXFILENAME(pszFileCreated);
+
     if (!(pFISH_EVENT = CreateFISH_EVENT(pszFileCreated,nLineCreated)))
     {
-        fprintf(stdout,"** FishHang_CreateEvent: CreateFISH_EVENT failed\n");
+        logmsg("** FishHang_CreateEvent: CreateFISH_EVENT failed\n");
         exit(-1);
     }
 
@@ -1126,10 +1212,11 @@ HANDLE FishHang_CreateEvent
 
 /////////////////////////////////////////////////////////////////////////////
 
+DLL_EXPORT
 BOOL FishHang_SetEvent
 (
-    char*  pszFileSet,      // source file that set it
-    int    nLineSet,        // line number of source file
+    const char*  pszFileSet,      // source file that set it
+    const int    nLineSet,        // line number of source file
 
     HANDLE  hEvent          // handle to event object
 )
@@ -1137,6 +1224,8 @@ BOOL FishHang_SetEvent
     FISH_THREAD*  pFISH_THREAD;
     FISH_EVENT*   pFISH_EVENT;
 
+    FIXFILENAME(pszFileSet);
+
     LockFishHang();
 
     if (!GetThreadAndEventPtrs(&pFISH_THREAD,&pFISH_EVENT,hEvent))
@@ -1155,10 +1244,11 @@ BOOL FishHang_SetEvent
 
 /////////////////////////////////////////////////////////////////////////////
 
+DLL_EXPORT
 BOOL FishHang_ResetEvent
 (
-    char*  pszFileReset,    // source file that reset it
-    int    nLineReset,      // line number of source file
+    const char*  pszFileReset,    // source file that reset it
+    const int    nLineReset,      // line number of source file
 
     HANDLE  hEvent          // handle to event object
 )
@@ -1166,6 +1256,8 @@ BOOL FishHang_ResetEvent
     FISH_THREAD*  pFISH_THREAD;
     FISH_EVENT*   pFISH_EVENT;
 
+    FIXFILENAME(pszFileReset);
+
     LockFishHang();
 
     if (!GetThreadAndEventPtrs(&pFISH_THREAD,&pFISH_EVENT,hEvent))
@@ -1184,10 +1276,11 @@ BOOL FishHang_ResetEvent
 
 /////////////////////////////////////////////////////////////////////////////
 
+DLL_EXPORT
 BOOL FishHang_PulseEvent
 (
-    char*  pszFilePosted,   // source file that signalled it
-    int    nLinePosted,     // line number of source file
+    const char*  pszFilePosted,   // source file that signalled it
+    const int    nLinePosted,     // line number of source file
 
     HANDLE  hEvent          // handle to event object
 )
@@ -1195,6 +1288,8 @@ BOOL FishHang_PulseEvent
     FISH_THREAD*  pFISH_THREAD;
     FISH_EVENT*   pFISH_EVENT;
 
+    FIXFILENAME(pszFilePosted);
+
     LockFishHang();
 
     if (!GetThreadAndEventPtrs(&pFISH_THREAD,&pFISH_EVENT,hEvent))
@@ -1217,10 +1312,11 @@ BOOL FishHang_PulseEvent
 
 /////////////////////////////////////////////////////////////////////////////
 
+DLL_EXPORT
 BOOL FishHang_CloseHandle   // ** NOTE: only events for right now **
 (
-    char*  pszFileClosed,   // source file that closed it
-    int    nLineClosed,     // line number of source file
+    const char*  pszFileClosed,   // source file that closed it
+    const int    nLineClosed,     // line number of source file
 
     HANDLE  hEvent          // handle to event object
 )
@@ -1230,6 +1326,8 @@ BOOL FishHang_CloseHandle   // ** NOTE: only events for right now **
     LIST_ENTRY*   pListEntry;
     FISH_THREAD*  pWaitingFISH_THREAD;
 
+    FIXFILENAME(pszFileClosed);
+
     LockFishHang();
 
     if (!GetThreadAndEventPtrs(&pThisFISH_THREAD,&pFISH_EVENT,hEvent))
@@ -1245,7 +1343,7 @@ BOOL FishHang_CloseHandle   // ** NOTE: only events for right now **
         if (!pWaitingFISH_THREAD->bWaitingForEvent ||
             pWaitingFISH_THREAD->pWhatWaiting != pFISH_EVENT) continue;
 
-        fprintf(stdout,
+        logmsg(
             "\n** ERROR ** %s(%d); FISH_THREAD %8.8X "
             "is closing FISH_EVENT %8.8X "
             "that FISH_THREAD %8.8X is still waiting on!\n",
@@ -1254,7 +1352,7 @@ BOOL FishHang_CloseHandle   // ** NOTE: only events for right now **
             (int)pWaitingFISH_THREAD
             );
 
-        fprintf(stdout,"\n%s\n%s\n%s\n",
+        logmsg("\n%s\n%s\n%s\n",
             PrintFISH_THREAD(pThisFISH_THREAD),
             PrintFISH_EVENT(pFISH_EVENT),
             PrintFISH_THREAD(pWaitingFISH_THREAD)
@@ -1273,10 +1371,11 @@ BOOL FishHang_CloseHandle   // ** NOTE: only events for right now **
 
 /////////////////////////////////////////////////////////////////////////////
 
+DLL_EXPORT
 DWORD FishHang_WaitForSingleObject  // ** NOTE: only events for right now **
 (
-    char*  pszFileWaiting,  // source file that attempted it
-    int    nLineWaiting,    // line number of source file
+    const char*  pszFileWaiting,  // source file that attempted it
+    const int    nLineWaiting,    // line number of source file
 
     HANDLE  hEvent,         // handle to event to wait for
     DWORD   dwMilliseconds  // time-out interval in milliseconds
@@ -1286,6 +1385,8 @@ DWORD FishHang_WaitForSingleObject  // ** NOTE: only events for right now **
     FISH_EVENT*   pFISH_EVENT;
     DWORD         dwWaitRetCode;
 
+    FIXFILENAME(pszFileWaiting);
+
     LockFishHang();
 
     if (!GetThreadAndEventPtrs(&pFISH_THREAD,&pFISH_EVENT,hEvent))
@@ -1311,24 +1412,31 @@ DWORD FishHang_WaitForSingleObject  // ** NOTE: only events for right now **
 
 /////////////////////////////////////////////////////////////////////////////
 
+DLL_EXPORT
 void  FishHangReport()
 {
     LockFishHang();
-    fprintf(stdout,"\n--------------- BEGIN FishHangReport ---------------\n\n");
+
+    logmsg("\n--------------- BEGIN FishHangReport ---------------\n\n");
+
     PrintDeadlock(NULL,NULL,0);
+
     PrintAllFISH_THREADs();
     PrintAllFISH_LOCKs();
     PrintAllFISH_EVENTs();
-    fprintf(stdout,"\n---------------- END FishHangReport ----------------\n\n");
+
+    logmsg("\n---------------- END FishHangReport ----------------\n\n");
+
     UnlockFishHang();
 }
 
 /////////////////////////////////////////////////////////////////////////////
 
+DLL_EXPORT
 void FishHang_DeleteCriticalSection
 (
-    char*  pszFileDeleting, // source file that's deleting it
-    int    nLineDeleting,   // line number of source file
+    const char*  pszFileDeleting, // source file that's deleting it
+    const int    nLineDeleting,   // line number of source file
 
     LPCRITICAL_SECTION lpCriticalSection   // address of critical section object
 )
@@ -1336,6 +1444,8 @@ void FishHang_DeleteCriticalSection
     FISH_THREAD*  pFISH_THREAD;
     FISH_LOCK*    pFISH_LOCK;
 
+    FIXFILENAME(pszFileDeleting);
+
     LockFishHang();
 
     if (!GetThreadAndLockPtrs(&pFISH_THREAD,&pFISH_LOCK,lpCriticalSection))
@@ -1343,7 +1453,7 @@ void FishHang_DeleteCriticalSection
 
     if (!bFishHangAtExit && pFISH_LOCK->nLockedDepth)
     {
-        fprintf(stdout,
+        logmsg(
             "\n** ERROR ** FISH_THREAD %8.8X "
             "deleting still-locked FISH_LOCK %8.8X!\n",
             (int)pFISH_THREAD,
@@ -1352,7 +1462,7 @@ void FishHang_DeleteCriticalSection
 
         if (pFISH_LOCK->pOwningThread != pFISH_THREAD)
         {
-            fprintf(stdout,
+            logmsg(
                 "** ERROR ** FISH_LOCK %8.8X "
                 "owned by FISH_THREAD %8.8X!\n",
                 (int)pFISH_LOCK,
@@ -1360,9 +1470,9 @@ void FishHang_DeleteCriticalSection
                 );
         }
 
-        fprintf(stdout,"%s(%d)\n\n",pszFileDeleting,nLineDeleting);
-        fprintf(stdout,"%s\n",PrintFISH_THREAD(pFISH_THREAD));
-        fprintf(stdout,"%s\n",PrintFISH_LOCK(pFISH_LOCK));
+        logmsg("%s(%d)\n\n",pszFileDeleting,nLineDeleting);
+        logmsg("%s\n",PrintFISH_THREAD(pFISH_THREAD));
+        logmsg("%s\n",PrintFISH_LOCK(pFISH_LOCK));
     }
 
     RemoveListEntry(&pFISH_LOCK->LockListLink);
@@ -1379,6 +1489,7 @@ void FishHang_DeleteCriticalSection
 
 /////////////////////////////////////////////////////////////////////////////
 
+DLL_EXPORT
 void FishHang_ExitThread
 (
     DWORD dwExitCode    // exit code for this thread
@@ -1393,7 +1504,7 @@ void FishHang_ExitThread
     if (!(pFISH_THREAD = FindFISH_THREAD(GetCurrentThreadId())))
     {
         UnlockFishHang();
-        fprintf(stdout,"** FishHang_ExitThread: FindFISH_THREAD failed!\n");
+        logmsg("** FishHang_ExitThread: FindFISH_THREAD failed!\n");
         if (bFishHangAtExit) ExitThread(dwExitCode);
         exit(-1);
     }
@@ -1403,14 +1514,14 @@ void FishHang_ExitThread
 
     if (!bFishHangAtExit && !IsListEmpty(&pFISH_THREAD->ThreadLockListHead))
     {
-        fprintf(stdout,
+        logmsg(
             "\n** ERROR ** FISH_THREAD %8.8X "
             "exiting with locks still owned!\n\n",
             (int)pFISH_THREAD);
 
         RemoveListEntry(&pFISH_THREAD->ThreadListLink);
         InsertListHead(&ThreadsListHead,&pFISH_THREAD->ThreadListLink);
-        fprintf(stdout,"%s\n",PrintFISH_THREAD(pFISH_THREAD));
+        logmsg("%s\n",PrintFISH_THREAD(pFISH_THREAD));
 
         pListEntry = pFISH_THREAD->ThreadLockListHead.Flink;
 
@@ -1418,7 +1529,7 @@ void FishHang_ExitThread
         {
             pFISH_LOCK = CONTAINING_RECORD(pListEntry,FISH_LOCK,ThreadLockListLink);
             pListEntry = pListEntry->Flink;
-            fprintf(stdout,"%s\n",PrintFISH_LOCK(pFISH_LOCK));
+            logmsg("%s\n",PrintFISH_LOCK(pFISH_LOCK));
         }
     }
 
@@ -1430,3 +1541,4 @@ void FishHang_ExitThread
 /////////////////////////////////////////////////////////////////////////////
 
 #endif // !defined(FISH_HANG)
+
