@@ -2,8 +2,8 @@
 // Hercules Interface Configuration Program
 // ====================================================================
 //
-// Copyright    (C) Copyright Roger Bowler, 2000-2003
-//              (C) Copyright James A. Pierson, 2002-2003
+// Copyright    (C) Copyright Roger Bowler, 2000-2005
+//              (C) Copyright James A. Pierson, 2002-2005
 //
 // Based on code originally written by Roger Bowler
 // Modified to communicate via unix sockets.
@@ -20,6 +20,8 @@
 //
 
 #include "hercules.h"
+
+#if defined(BUILD_HERCIFC)
 #include "hercifc.h"
 
 // --------------------------------------------------------------------
@@ -28,13 +30,14 @@
 
 int main( int argc, char **argv )
 {
-    BYTE*       pszProgName  = NULL;    // Name of this program
-    BYTE*       pOp          = NULL;    // Operation text
-    BYTE*       pIF          = NULL;    // -> interface name
+    char*       pszProgName  = NULL;    // Name of this program
+    char*       pOp          = NULL;    // Operation text
+    char*       pIF          = NULL;    // -> interface name
     void*       pArg         = NULL;    // -> ifreq or rtentry 
     CTLREQ      ctlreq;                 // Request Buffer
     int         sockfd;                 // Socket descriptor
     int         rc;                     // Return code
+    pid_t       ppid;                   // Parent's PID
     char        szMsgBuffer[255];
 
     UNREFERENCED( argc );
@@ -61,6 +64,8 @@ int main( int argc, char **argv )
         exit( 2 );
     }
 
+    ppid = getppid();
+
     // Process ioctl messages from Hercules
     while( 1 )
     {
@@ -76,6 +81,15 @@ int main( int argc, char **argv )
             exit( 3 );
         }
 
+        if( ppid != getppid() )
+        {
+            sleep( 1 ); // Let other messages go first
+            fprintf( stderr,
+                     _("HHCIF007E %s: Hercules disappeared!! .. exiting\n"),
+                     pszProgName);
+            exit( 4 );
+        }
+
         switch( ctlreq.iCtlOp )
         {
         case SIOCSIFADDR:
@@ -90,12 +104,6 @@ int main( int argc, char **argv )
             pIF  = ctlreq.iru.ifreq.ifr_name;
             break;
 
-        case SIOCSIFNETMASK:
-            pOp  = "SIOCSIFNETMASK";
-            pArg = &ctlreq.iru.ifreq;
-            pIF  = ctlreq.iru.ifreq.ifr_name;
-            break;
-
         case SIOCSIFFLAGS:
             pOp  = "SIOCSIFFLAGS";
             pArg = &ctlreq.iru.ifreq;
@@ -108,12 +116,6 @@ int main( int argc, char **argv )
             pIF  = ctlreq.iru.ifreq.ifr_name;
             break;
 
-        case SIOCSIFHWADDR:
-            pOp  = "SIOCSIFHWADDR";
-            pArg = &ctlreq.iru.ifreq;
-            pIF  = ctlreq.iru.ifreq.ifr_name;
-            break;
-
         case SIOCADDMULTI:
             pOp  = "SIOCADDMULTI";
             pArg = &ctlreq.iru.ifreq;
@@ -126,20 +128,34 @@ int main( int argc, char **argv )
             pIF  = ctlreq.iru.ifreq.ifr_name;
             break;
 
+#ifdef OPTION_TUNTAP_SETNETMASK
+        case SIOCSIFNETMASK:
+            pOp  = "SIOCSIFNETMASK";
+            pArg = &ctlreq.iru.ifreq;
+            pIF  = ctlreq.iru.ifreq.ifr_name;
+            break;
+#endif
+#ifdef OPTION_TUNTAP_SETMACADDR
+        case SIOCSIFHWADDR:
+            pOp  = "SIOCSIFHWADDR";
+            pArg = &ctlreq.iru.ifreq;
+            pIF  = ctlreq.iru.ifreq.ifr_name;
+            break;
+#endif
+#ifdef OPTION_TUNTAP_DELADD_ROUTES
         case SIOCADDRT:
             pOp  = "SIOCADDRT";
             pArg = &ctlreq.iru.rtentry;
             pIF  = ctlreq.szIFName;
             ctlreq.iru.rtentry.rt_dev = ctlreq.szIFName;
             break;
-
         case SIOCDELRT:
             pOp  = "SIOCDELRT";
             pArg = &ctlreq.iru.rtentry;
             pIF  = ctlreq.szIFName;
             ctlreq.iru.rtentry.rt_dev = ctlreq.szIFName;
             break;
-
+#endif
         case CTLREQ_OP_DONE:
             close( STDIN_FILENO  );
             close( STDOUT_FILENO );
@@ -147,8 +163,8 @@ int main( int argc, char **argv )
             exit( 0 );
 
         default:
-            sprintf( szMsgBuffer,
-                     _("HHCIF004W %s: Unknown request: %8.8X.\n"),
+            snprintf( szMsgBuffer,sizeof(szMsgBuffer),
+                     _("HHCIF004W %s: Unknown request: %8.8lX.\n"),
                      pszProgName, ctlreq.iCtlOp );
             
             write( STDERR_FILENO, szMsgBuffer, strlen( szMsgBuffer ) );
@@ -156,11 +172,19 @@ int main( int argc, char **argv )
             continue;
         }
             
-        rc = ioctl( sockfd, ctlreq.iCtlOp, pArg );
+#if defined(DEBUG) || defined(_DEBUG)
+        snprintf( szMsgBuffer,sizeof(szMsgBuffer),
+                 _("HHCIF006I %s: Doing %s on %s\n"),
+                 pszProgName, pOp, pIF);
+
+        write( STDERR_FILENO, szMsgBuffer, strlen( szMsgBuffer ) );
+#endif /*defined(DEBUG) || defined(_DEBUG)*/
     
+        rc = ioctl( sockfd, ctlreq.iCtlOp, pArg );
+
         if( rc < 0 )
         {
-            sprintf( szMsgBuffer,
+            snprintf( szMsgBuffer,sizeof(szMsgBuffer),
                      _("HHCIF005E %s: ioctl error doing %s on %s: %s\n"),
                      pszProgName, pOp, pIF, strerror( errno ) );
             
@@ -172,5 +196,4 @@ int main( int argc, char **argv )
     return 0;
 }
 
-
-
+#endif // defined(BUILD_HERCIFC)
