@@ -1,7 +1,7 @@
 /*
 || ----------------------------------------------------------------------------
 ||
-|| HETLIB.C     (c) Copyright Leland Lucius, 2000-2003
+|| HETINIT.C    (c) Copyright Leland Lucius, 2000-2005
 ||              Released under terms of the Q Public License.
 ||
 || Creates IEHINITT or NL format Hercules Emulated Tapes.
@@ -9,16 +9,13 @@
 || ----------------------------------------------------------------------------
 */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "hstdinc.h"
 
+#include "hercules.h"
 #include "hetlib.h"
 #include "sllib.h"
-#include "hercules.h"
-#include <fcntl.h>
-#include <unistd.h>
-#include <time.h>
+#include "herc_getopt.h"
+
 /*
 || Local constant data
 */
@@ -31,12 +28,6 @@ static const char help[] =
     "  -i  create an IEHINITT formatted tape (default: on)\n"
     "  -n  create an NL tape\n";
 
-#ifdef EXTERNALGUI
-/* Special flag to indicate whether or not we're being
-   run under the control of the external GUI facility. */
-int extgui = 0;
-#endif /*EXTERNALGUI*/
-
 /*
 || Prints usage information
 */
@@ -46,6 +37,17 @@ usage( char *name )
     printf( help, name, name );
 }
 
+/*
+|| Subroutine to convert a null-terminated string to upper case
+*/
+void het_string_to_upper (char *source)
+{
+int i;
+
+    for (i = 0; source[i] != '\0'; i++)
+        source[i] = toupper(source[i]);
+}
+
 /*
 || Standard main() function
 */
@@ -62,11 +64,15 @@ main( int argc, char *argv[] )
     char *o_owner;
     char *o_volser;
 
+    set_codepage( NULL );
+
 #ifdef EXTERNALGUI
     if (argc >= 1 && strncmp(argv[argc-1],"EXTERNALGUI",11) == 0)
     {
         extgui = 1;
         argc--;
+        setvbuf(stderr, NULL, _IONBF, 0);
+        setvbuf(stdout, NULL, _IONBF, 0);
     }
 #endif /*EXTERNALGUI*/
 
@@ -80,7 +86,7 @@ main( int argc, char *argv[] )
     o_volser = NULL;
 
     /* Display the program identification message */
-    display_version (stderr, "Hercules HET IEHINITT program ");
+    display_version (stderr, "Hercules HET IEHINITT program ", FALSE);
 
     while( TRUE )
     {
@@ -154,6 +160,12 @@ main( int argc, char *argv[] )
         }
     }
 
+    if( o_volser )
+        het_string_to_upper( o_volser );
+
+    if( o_owner )
+        het_string_to_upper( o_owner );
+
     rc = het_open( &hetb, o_filename, HETOPEN_CREATE );
     if( rc < 0 )
     {
@@ -170,7 +182,13 @@ main( int argc, char *argv[] )
 
     if( o_iehinitt )
     {
-        sl_vol1( &lab, o_volser, o_owner );
+        rc = sl_vol1( &lab, o_volser, o_owner );
+        if( rc < 0 )
+        {
+            printf( "%s\n", sl_error(rc) );
+            goto exit;
+        }
+
         rc = het_write( hetb, &lab, sizeof( lab ) );
         if( rc < 0 )
         {
@@ -178,7 +196,13 @@ main( int argc, char *argv[] )
             goto exit;
         }
 
-        sl_hdr1( &lab, SL_INITDSN, NULL, 0, 0, NULL, 0 );
+        rc = sl_hdr1( &lab, SL_INITDSN, NULL, 0, 0, NULL, 0 );
+        if( rc < 0 )
+        {
+            printf( "%s\n", sl_error(rc) );
+            goto exit;
+        }
+
         rc = het_write( hetb, &lab, sizeof( lab ) );
         if( rc < 0 )
         {
