@@ -1,30 +1,21 @@
 /*
 || ----------------------------------------------------------------------------
 ||
-|| HETLIB.C     (c) Copyright Leland Lucius, 2000-2003
+|| HETLIB.C     (c) Copyright Leland Lucius, 2000-2005
 ||              Released under terms of the Q Public License.
 ||
-|| Library for managing Hercules Emulated Tapes.  
+|| Library for managing Hercules Emulated Tapes.
 ||
 || ----------------------------------------------------------------------------
 */
 
-#include "hetlib.h"
-#include <sys/types.h>
-#include <limits.h>
-#include <stdio.h>
-#include <errno.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <fcntl.h>
-#include <sys/stat.h>
+#include "hstdinc.h"
 
-#if defined(HAVE_LIBZ)
-#include <zlib.h>
-#endif
-#if defined( HET_BZIP2 )
-#include <bzlib.h>
-#endif /* defined( HET_BZIP2 ) */
+#define _HETLIB_C_
+#define _HTAPE_DLL_
+
+#include "hercules.h"
+#include "hetlib.h"
 
 #undef HETDEBUGR
 #undef HETDEBUGW
@@ -32,7 +23,7 @@
 /*
 || Local constant data
 */
-static const char *het_errstr[] = 
+static const char *het_errstr[] =
 {
     "No error",
     "File error",
@@ -65,30 +56,37 @@ static const char *het_errstr[] =
 
     NAME
             het_open - Open an HET format file
-        
+
     SYNOPSIS
             #include "hetlib.h"
 
             int het_open( HETB **hetb, char *filename, int flags )
-        
+
     DESCRIPTION
             The het_open() function opens the file indicated by the "filename"
             parameter and, if successful, places the address of an HETB at
             the location pointed to by the "hetb" parameter.
-            
+
             Currently, "HETOPEN_CREATE" is the only flag available and has
             the same function as the O_CREAT flag of the open(3) function.
-            
+
+        @ISW@ Added flag HETOPEN_READONLY
+
+        HETOPEN_CREATE and HETOPEN_READONLY are mutually exclusive.
+
+        When HETOPEN_READONLY is set, the het file must exist.
+        It is opened read only. Any attempt to write will fail.
+
     RETURN VALUE
             If no errors are detected then the return value will be >= 0
             and the address of the newly allocated HETB will be place at the
             "hetb" location.
-            
+
             If an error occurs, then the return value will be < 0 and will be
             one of the following:
-            
+
             HETE_NOMEM          Insufficient memory to allocate an HETB
-            
+
             HETE_ERROR          File system error - check errno(3)
 
             For other possible errors, see:
@@ -105,7 +103,7 @@ static const char *het_errstr[] =
             //
 
             #include "hetlib.h"
-            
+
             int main( int argc, char *argv[] )
             {
                 HETB *hetb;
@@ -131,19 +129,22 @@ static const char *het_errstr[] =
 
 ==DOC==*/
 
-int 
+DLL_EXPORT int
 het_open( HETB **hetb, char *filename, int flags )
 {
     HETB *thetb;
     char *omode;
-    int rc;
-    int fd;
+    int   rc;
+    int   fd;
+    int   oflags;
+    BYTE  pathname[MAX_PATH];
 
     /*
     || Initialize
     */
     *hetb = NULL;
-    
+    hostpath(pathname, filename, sizeof(pathname));
+
     /*
     || Allocate a new HETB
     */
@@ -162,24 +163,34 @@ het_open( HETB **hetb, char *filename, int flags )
     thetb->level      = HETDFLT_LEVEL;
     thetb->chksize    = HETDFLT_CHKSIZE;
 
+    /*
+    || clear HETOPEN_CREATE if HETOPEN_READONLY is specified
+    */
+    if(flags & HETOPEN_READONLY)
+    {
+        flags&=~HETOPEN_CREATE;
+    }
     /*
     || Translate HET create flag to filesystem flag
     */
-    flags = ( ( flags & HETOPEN_CREATE ) ? O_CREAT : 0 );
+    oflags = ( ( flags & HETOPEN_CREATE ) ? O_CREAT : 0 );
 
     /*
     || Open the tape file
     */
     omode = "r+b";
-    fd = open( filename, O_RDWR | flags, S_IRUSR | S_IWUSR | S_IRGRP );
-    if( fd == -1 && (errno == EROFS || errno == EACCES) )
+    if(!(flags & HETOPEN_READONLY))
+    {
+        fd = open( pathname, O_RDWR | O_BINARY | oflags, S_IRUSR | S_IWUSR | S_IRGRP );
+    }
+    if( (flags & HETOPEN_READONLY) || (fd == -1 && (errno == EROFS || errno == EACCES) ) )
     {
         /*
         || Retry open if file resides on readonly file system
         */
         omode = "rb";
         thetb->writeprotect = TRUE;
-        fd = open( filename, O_RDONLY, S_IRUSR | S_IWUSR | S_IRGRP );
+        fd = open( pathname, O_RDONLY | O_BINARY, S_IRUSR | S_IWUSR | S_IRGRP );
     }
 
     /*
@@ -250,21 +261,21 @@ het_open( HETB **hetb, char *filename, int flags )
 
     NAME
             het_close - Close an HET file
-        
+
     SYNOPSIS
             #include "hetlib.h"
 
             int het_close( HETB **hetb )
-        
+
     DESCRIPTION
             The het_close() function closes an HET file and releases the
             HETB.
-            
+
     RETURN VALUE
             If no errors are detected then the return value will be >= 0
             and the location specified by the "hetb" parameter will be set
             to NULL.
-            
+
             If an error occurs, then the return value will be < 0.  At this
             time, no errors will be returned.
 
@@ -274,7 +285,7 @@ het_open( HETB **hetb, char *filename, int flags )
             //
 
             #include "hetlib.h"
-            
+
             int main( int argc, char *argv[] )
             {
                 HETB *hetb;
@@ -300,10 +311,10 @@ het_open( HETB **hetb, char *filename, int flags )
 
 ==DOC==*/
 
-int 
+DLL_EXPORT int
 het_close( HETB **hetb )
 {
-    
+
     /*
     || Only free the HETB if we have one
     */
@@ -331,27 +342,27 @@ het_close( HETB **hetb )
 
     NAME
             het_cntl - Control HET file behavior
-        
+
     SYNOPSIS
             #include "hetlib.h"
 
             int het_cntl( HETB *hetb, int func, unsigned long val )
-        
+
     DESCRIPTION
             The het_cntl() function allows you to get/set several values
             that control how an HET file behaves.  The value of the "val"
             parameter depends on the function code.
 
             The possible modes are:
-            
+
             HETCNTL_GET         Should be ORed (|) with the function codes
                                 to retrieve the current setting.  (Default)
-                            
+
             HETCNTL_SET         Should be ORed (|) with the function codes
                                 to set a new value.
-                            
+
             The possible function codes are:
-            
+
             HETCNTL_COMPRESS    val=TRUE to enable write compression (see notes)
                                 Values:     FALSE (disable)
                                             TRUE (enable)
@@ -361,22 +372,22 @@ het_close( HETB **hetb )
                                 Values:     FALSE (diable)
                                             TRUE (enable)
                                 Default:    HETDFLT_DECOMPRESS (TRUE)
-            
+
             HETCNTL_METHOD      val=Compression method to use
                                 Values:     HETMETH_ZLIB (1)
                                             HETMETH_BZLIB (2)
                                 Default:    HETDFLT_METHOD (HETMETH_ZLIB)
-            
+
             HETCNTL_LEVEL       val=Level of compression
                                 Min:        HETMIN_LEVEL (1)
                                 Max:        HETMAX_LEVEL (9)
                                 Default:    HETDFLT_LEVEL (4)
-                                
+
             HETCNTL_CHUNKSIZE   val=Size of output chunks (see notes)
                                 Min:        HETMIN_CHUNKSIZE (4096)
                                 Max:        HETMAX_CHUNKSIZE (65535)
                                 Default:    HETDFLT_CHUNKSIZE (65535)
-            
+
     RETURN VALUE
             If no errors are detected then the return value will be either
             the current setting for a "get" request or >= 0 for a "set"
@@ -408,7 +419,7 @@ het_close( HETB **hetb )
             #include "hetlib.h"
 
             char data[] = "This is a test";
-            
+
             int main( int argc, char *argv[] )
             {
                 HETB *hetb;
@@ -443,11 +454,11 @@ het_close( HETB **hetb )
 
 ==DOC==*/
 
-int 
+DLL_EXPORT int
 het_cntl( HETB *hetb, int func, unsigned long val )
 {
     int mode;
-    
+
     /*
     || Isolate the mode
     */
@@ -466,7 +477,7 @@ het_cntl( HETB *hetb, int func, unsigned long val )
 
             hetb->compress = ( val ? TRUE : FALSE );
         break;
-        
+
         case HETCNTL_DECOMPRESS:
             if( mode == HETCNTL_GET )
             {
@@ -475,7 +486,7 @@ het_cntl( HETB *hetb, int func, unsigned long val )
 
             hetb->decompress = ( val ? TRUE : FALSE );
         break;
-        
+
         case HETCNTL_METHOD:
             if( mode == HETCNTL_GET )
             {
@@ -503,7 +514,7 @@ het_cntl( HETB *hetb, int func, unsigned long val )
 
             hetb->level = val;
         break;
-        
+
         case HETCNTL_CHUNKSIZE:
             if( mode == HETCNTL_GET )
             {
@@ -517,7 +528,7 @@ het_cntl( HETB *hetb, int func, unsigned long val )
 
             hetb->chksize = val;
         break;
-        
+
         default:
             return( HETE_BADFUNC );
         break;
@@ -534,15 +545,15 @@ het_cntl( HETB *hetb, int func, unsigned long val )
 
     NAME
             het_read_header - Retrieve the next chunk header from an HET file
-        
+
     SYNOPSIS
             #include "hetlib.h"
 
             int het_read_header( HETB *hetb )
-        
+
     DESCRIPTION
             Retrieves the next chunk header and stores it in the HETB.
-            
+
     RETURN VALUE
             If no errors are detected then the return value will be >= 0
             and the current block count will be incremented.
@@ -601,7 +612,7 @@ het_cntl( HETB *hetb, int func, unsigned long val )
 
 ==DOC==*/
 
-int
+DLL_EXPORT int
 het_read_header( HETB *hetb )
 {
     int rc;
@@ -658,16 +669,16 @@ het_read_header( HETB *hetb )
 
     NAME
             het_read - Retrieve the next block from an HET file
-        
+
     SYNOPSIS
             #include "hetlib.h"
 
             int het_read( HETB *hetb,  void *sbuf )
-        
+
     DESCRIPTION
             Read the next block of data into the "sbuf" memory location.  The
             length of "sbuf" should be at least HETMAX_BLOCKSIZE bytes.
-            
+
     RETURN VALUE
             If no errors are detected then the return value will be the
             size of the block read.  This will be either the compressed or
@@ -680,13 +691,13 @@ het_read_header( HETB *hetb )
             HETE_ERROR          File system error - check errno(3)
 
             HETE_BADBOR         Beginning of record expected but not found
-            
+
             HETE_BADCOMPRESS    Compression mismatch between related chunks
-            
+
             HETE_OVERFLOW       Record too large for buffer
-            
+
             HETE_PREMEOF        Premature EOF on file
-            
+
             HETE_DECERR         Decompression error (stored in errno(3))
 
             HETE_UNKMETH        Unknown compression method encountered
@@ -733,7 +744,7 @@ het_read_header( HETB *hetb )
 
 ==DOC==*/
 
-int
+DLL_EXPORT int
 het_read( HETB *hetb, void *sbuf )
 {
     char *tptr;
@@ -868,24 +879,24 @@ het_read( HETB *hetb, void *sbuf )
 #if defined(HAVE_LIBZ)
             case HETHDR_FLAGS1_ZLIB:
                 slen = HETMAX_BLOCKSIZE;
-    
-                rc = uncompress( sbuf, &slen, tbuf, tlen );
+
+                rc = uncompress( sbuf, &slen, (unsigned char *)tbuf, tlen );
                 if( rc != Z_OK )
                 {
                     errno = rc;
                     return( HETE_DECERR );
                 }
-    
+
                 tlen = slen;
             break;
 #endif
-    
+
 #if defined( HET_BZIP2 )
             case HETHDR_FLAGS1_BZLIB:
                 slen = HETMAX_BLOCKSIZE;
-    
+
                 rc = BZ2_bzBuffToBuffDecompress( sbuf,
-                                                 (unsigned int *) &slen,
+                                                 (void *) &slen,
                                                  tbuf,
                                                  tlen,
                                                  0,
@@ -895,11 +906,11 @@ het_read( HETB *hetb, void *sbuf )
                     errno = rc;
                     return( HETE_DECERR );
                 }
-    
+
                 tlen = slen;
             break;
 #endif /* defined( HET_BZIP2 ) */
-    
+
             default:
                 return( HETE_UNKMETH );
             break;
@@ -921,15 +932,15 @@ het_read( HETB *hetb, void *sbuf )
 
     NAME
             het_write_header - Write a chunk header to an HET file
-        
+
     SYNOPSIS
             #include "hetlib.h"
 
             int het_write_header( HETB *hetb, int len, int flags1, int flags2 )
-        
+
     DESCRIPTION
             Constructs and writes a chunk header to an HET file.
-            
+
     RETURN VALUE
             If no errors are detected then the return value will be >= 0
             and the current block count will be incremented.
@@ -991,7 +1002,8 @@ het_read( HETB *hetb, void *sbuf )
 int
 het_write_header( HETB *hetb, int len, int flags1, int flags2 )
 {
-    int rc;
+    int    rc;
+    OFF_T  rcoff;
 
 #if defined( HETDEBUGW )
     printf("write hdr: pl=%d, cl=%d, f1=%02x, f2=%02x\n",
@@ -1029,30 +1041,30 @@ het_write_header( HETB *hetb, int len, int flags1, int flags2 )
     */
     if( !hetb->readlast )
     {
-        fseek( hetb->fd, 0, SEEK_CUR );
+        FSEEK( hetb->fd, 0, SEEK_CUR );
         hetb->readlast = FALSE;
     }
-        
+
     /*
     || If this is the first write, truncate the file
     */
     if( !hetb->truncated )
     {
-        rc = ftell( hetb->fd );
+        rcoff = FTELL( hetb->fd );
+        if( rcoff == -1 )
+        {
+            return( HETE_ERROR );
+        }
+
+        rc = FTRUNCATE( fileno( hetb->fd ), rcoff );
         if( rc == -1 )
         {
             return( HETE_ERROR );
         }
-        
-        rc = ftruncate( fileno( hetb->fd ), rc );
-        if( rc == -1 )
-        {
-            return( HETE_ERROR );
-        }
-        
+
         hetb->truncated = TRUE;
     }
-        
+
     /*
     || Construct the header
     */
@@ -1100,7 +1112,7 @@ het_write_header( HETB *hetb, int len, int flags1, int flags2 )
             Writes a block of data specified by "sbuf" with a length of "slen"
             to an HET file.  Depending on the current HETCNTL_COMPRESS setting,
             the data may be compressed prior to writing.
-            
+
     RETURN VALUE
             If no errors are detected then the return value will be the
             size of the block written.  This will be either the compressed or
@@ -1113,9 +1125,9 @@ het_write_header( HETB *hetb, int len, int flags1, int flags2 )
             HETE_ERROR          File system error - check errno(3)
 
             HETE_BADLEN         "slen" parameter out of range
-            
+
             HETE_BADCOMPRESS    Compression mismatch between related chunks
-            
+
             For other possible errors, see:
                 het_write_header()
 
@@ -1127,7 +1139,7 @@ het_write_header( HETB *hetb, int len, int flags1, int flags2 )
             #include "hetlib.h"
 
             char data[] = "This is a test";
-            
+
             int main( int argc, char *argv[] )
             {
                 HETB *hetb;
@@ -1158,13 +1170,15 @@ het_write_header( HETB *hetb, int len, int flags1, int flags2 )
 
 ==DOC==*/
 
-int
+DLL_EXPORT int
 het_write( HETB *hetb, void *sbuf, int slen )
 {
     int rc;
     int flags;
     unsigned long tlen;
-    char tbuf[ HETMAX_BLOCKSIZE ];
+#if defined(HAVE_LIBZ) || defined( HET_BZIP2 )
+    char tbuf[ ((((HETMAX_BLOCKSIZE * 1001) + 999) / 1000) + 12) ];
+#endif
 
     /*
     || Validate
@@ -1195,7 +1209,7 @@ het_write( HETB *hetb, void *sbuf, int slen )
             case HETHDR_FLAGS1_ZLIB:
                 tlen = sizeof( tbuf );
 
-                rc = compress2( tbuf, &tlen, sbuf, slen, hetb->level );
+                rc = compress2( (unsigned char *)tbuf, &tlen, sbuf, slen, hetb->level );
                 if( rc != Z_OK )
                 {
                     errno = rc;
@@ -1216,7 +1230,7 @@ het_write( HETB *hetb, void *sbuf, int slen )
                 tlen = sizeof( tbuf );
 
                 rc = BZ2_bzBuffToBuffCompress( tbuf,
-                                               (unsigned int *) &tlen,
+                                               (void *) &tlen,
                                                sbuf,
                                                slen,
                                                hetb->level,
@@ -1279,11 +1293,16 @@ het_write( HETB *hetb, void *sbuf, int slen )
         {
             return( HETE_ERROR );
         }
-        
+
         /*
         || Bump pointers and turn off BOR flag
         */
-        sbuf += tlen;
+        {
+            char    *csbuf;
+            csbuf=(char *)sbuf;
+            csbuf+=tlen;
+            sbuf=(void *)csbuf;
+        }
         slen -= tlen;
         flags &= (~HETHDR_FLAGS1_BOR);
     }
@@ -1299,16 +1318,16 @@ het_write( HETB *hetb, void *sbuf, int slen )
 
     NAME
             het_tapemark - Write a tape mark to an HET file
-        
+
     SYNOPSIS
             #include "hetlib.h"
 
             int het_tapemark( HETB *hetb )
-        
+
     DESCRIPTION
             Writes a special chunk header to an HET file to simulate a tape
             mark.
-            
+
     RETURN VALUE
             If no errors are detected then the return value will be >= 0.
 
@@ -1352,7 +1371,7 @@ het_write( HETB *hetb, void *sbuf, int slen )
 
 ==DOC==*/
 
-int
+DLL_EXPORT int
 het_tapemark( HETB *hetb )
 {
     int rc;
@@ -1376,16 +1395,16 @@ het_tapemark( HETB *hetb )
 
     NAME
             het_locate - Locate a block within an HET file
-        
+
     SYNOPSIS
             #include "hetlib.h"
 
             int het_locate( HETB *hetb, int block )
-        
+
     DESCRIPTION
             Repositions the HET file to the start of the block specified by
             the "block" parameter.
-            
+
     RETURN VALUE
             If no errors are detected then the return value will be >= 0 and
             represents the new current block number.
@@ -1435,11 +1454,11 @@ het_tapemark( HETB *hetb )
 
 ==DOC==*/
 
-int
+DLL_EXPORT int
 het_locate( HETB *hetb, int block )
 {
     int rc;
-    
+
     /*
     || Start the search from the beginning
     */
@@ -1463,21 +1482,21 @@ het_locate( HETB *hetb, int block )
 
     return( hetb->cblk );
 }
-    
+
 /*==DOC==
 
     NAME
             het_bsb - Backspace a block in an HET file
-        
+
     SYNOPSIS
             #include "hetlib.h"
 
             int het_bsb( HETB *hetb )
-        
+
     DESCRIPTION
-            Repositions the current block pointer in an HET file to the 
+            Repositions the current block pointer in an HET file to the
             previous block.
-            
+
     RETURN VALUE
             If no errors are detected then the return value will be >= 0 and
             will be the new block number.
@@ -1488,7 +1507,7 @@ het_locate( HETB *hetb, int block )
             HETE_ERROR          File system error - check errno(3)
 
             HETE_BOT            Beginning of tape
-            
+
             HETE_TAPEMARK       Tape mark encountered
 
             For other possible errors, see:
@@ -1536,14 +1555,14 @@ het_locate( HETB *hetb, int block )
 
 ==DOC==*/
 
-int
+DLL_EXPORT int
 het_bsb( HETB *hetb )
 {
     int rc;
     int newblk;
     int offset;
     int tapemark = FALSE;
-    
+
     /*
     || Error if at BOT
     */
@@ -1568,7 +1587,7 @@ het_bsb( HETB *hetb )
     /*
     || Calculate offset to get back to beginning of current block
     */
-    offset = -( HETHDR_CLEN( hetb ) + sizeof( HETHDR ) );
+    offset = -((int)( HETHDR_CLEN( hetb ) + sizeof( HETHDR ) ));
 
     /*
     || Search backwards an entire block.  If the block is a tapemark, we can't
@@ -1580,7 +1599,7 @@ het_bsb( HETB *hetb )
         /*
         || Reposition to start of chunk
         */
-        rc = fseek( hetb->fd,
+        rc = FSEEK( hetb->fd,
                     offset,
                     SEEK_CUR );
         if( rc == -1 )
@@ -1600,7 +1619,7 @@ het_bsb( HETB *hetb )
         /*
         || Recalculate offset
         */
-        offset = -( HETHDR_PLEN( hetb ) + ( sizeof( HETHDR ) * 2 ) );
+        offset = -((int)( HETHDR_PLEN( hetb ) + ( sizeof( HETHDR ) * 2 ) ));
     }
     while( hetb->chdr.flags1 & !( HETHDR_FLAGS1_BOR | HETHDR_FLAGS1_TAPEMARK ) );
 
@@ -1613,7 +1632,7 @@ het_bsb( HETB *hetb )
     || Reposition to chunk header preceding this one so we can load keep the
     || chdr in the HET current.
     */
-    rc = fseek( hetb->fd,
+    rc = FSEEK( hetb->fd,
                 offset,
                 SEEK_CUR );
     if( rc == -1 )
@@ -1633,7 +1652,7 @@ het_bsb( HETB *hetb )
     /*
     || Finally reposition back to the where we should be
     */
-    rc = fseek( hetb->fd,
+    rc = FSEEK( hetb->fd,
                 HETHDR_CLEN( hetb ),
                 SEEK_CUR );
     if( rc == -1 )
@@ -1664,21 +1683,21 @@ het_bsb( HETB *hetb )
     */
     return( hetb->cblk );
 }
-    
+
 /*==DOC==
 
     NAME
             het_fsb - Foward space a block in an HET file
-        
+
     SYNOPSIS
             #include "hetlib.h"
 
             int het_fsb( HETB *hetb )
-        
+
     DESCRIPTION
-            Repositions the current block pointer in an HET file to the 
+            Repositions the current block pointer in an HET file to the
             next block.
-            
+
     RETURN VALUE
             If no errors are detected then the return value will be >= 0 and
             will be the new block number.
@@ -1728,11 +1747,11 @@ het_bsb( HETB *hetb )
 
 ==DOC==*/
 
-int
+DLL_EXPORT int
 het_fsb( HETB *hetb )
 {
     int rc;
-    
+
     /*
     || Loop until we've processed an entire block
     */
@@ -1750,7 +1769,7 @@ het_fsb( HETB *hetb )
         /*
         || Seek to next chunk
         */
-        rc = fseek( hetb->fd,
+        rc = FSEEK( hetb->fd,
                     HETHDR_CLEN( hetb ),
                     SEEK_CUR );
         if( rc == -1 )
@@ -1770,21 +1789,21 @@ het_fsb( HETB *hetb )
     */
     return( hetb->cblk );
 }
-    
+
 /*==DOC==
 
     NAME
             het_bsf - Backspace a file in an HET file
-        
+
     SYNOPSIS
             #include "hetlib.h"
 
             int het_bsf( HETB *hetb )
-        
+
     DESCRIPTION
-            Repositions the current block pointer in an HET file to the 
+            Repositions the current block pointer in an HET file to the
             previous tapemark.
-            
+
     RETURN VALUE
             If no errors are detected then the return value will be >= 0 and
             will be the new block number.
@@ -1834,11 +1853,11 @@ het_fsb( HETB *hetb )
 
 ==DOC==*/
 
-int
+DLL_EXPORT int
 het_bsf( HETB *hetb )
 {
     int rc;
-    
+
     /*
     || Backspace until we hit a tapemark
     */
@@ -1847,7 +1866,7 @@ het_bsf( HETB *hetb )
         rc = het_bsb( hetb );
     }
     while( rc >= 0 );
-    
+
     /*
     || Success
     */
@@ -1861,21 +1880,21 @@ het_bsf( HETB *hetb )
     */
     return( rc );
 }
-    
+
 /*==DOC==
 
     NAME
             het_fsf - Forward space a file in an HET file
-        
+
     SYNOPSIS
             #include "hetlib.h"
 
             int het_fsf( HETB *hetb )
-        
+
     DESCRIPTION
-            Repositions the current block pointer in an HET file to the 
+            Repositions the current block pointer in an HET file to the
             next tapemark.
-            
+
     RETURN VALUE
             If no errors are detected then the return value will be >= 0 and
             will be the new block number.
@@ -1921,11 +1940,11 @@ het_bsf( HETB *hetb )
 
 ==DOC==*/
 
-int
+DLL_EXPORT int
 het_fsf( HETB *hetb )
 {
     int rc;
-    
+
     /*
     || Forward space until we hit a tapemark
     */
@@ -1934,7 +1953,7 @@ het_fsf( HETB *hetb )
         rc = het_fsb( hetb );
     }
     while( rc >= 0 );
-    
+
     /*
     || Success
     */
@@ -1952,24 +1971,24 @@ het_fsf( HETB *hetb )
 /*==DOC==
 
     NAME
-            het_rewind - Rewind an HET file 
-        
+            het_rewind - Rewind an HET file
+
     SYNOPSIS
             #include "hetlib.h"
 
             int het_rewind( HETB *hetb )
-        
+
     DESCRIPTION
-            Repositions the current block pointer in an HET file to the 
+            Repositions the current block pointer in an HET file to the
             load point.
-            
+
     RETURN VALUE
             If no errors are detected then the return value will be >= 0 and
             represents the new block number (always 0).
 
             If an error occurs, then the return value will be < 0 and will be
             one of the following:
-            
+
             HETE_ERROR          File system error - check errno(3)
 
     EXAMPLE
@@ -2009,15 +2028,15 @@ het_fsf( HETB *hetb )
 
 ==DOC==*/
 
-int
+DLL_EXPORT int
 het_rewind( HETB *hetb )
 {
     int rc;
-    
+
     /*
     || Just seek to the beginning of the file
     */
-    rc = fseek( hetb->fd,
+    rc = FSEEK( hetb->fd,
                 0,
                 SEEK_SET );
     if( rc == -1 )
@@ -2050,16 +2069,16 @@ het_rewind( HETB *hetb )
 
     NAME
             het_error - Returns a text message for an HET error code
-        
+
     SYNOPSIS
             #include "hetlib.h"
 
             char *het_error( int rc )
-        
+
     DESCRIPTION
             Simply returns a pointer to a string that describes the error
             code passed in the "rc" parameter.
-            
+
     RETURN VALUE
             The return value is always valid and no errors are returned.
 
@@ -2083,7 +2102,7 @@ het_rewind( HETB *hetb )
 
 ==DOC==*/
 
-const char *
+DLL_EXPORT const char *
 het_error( int rc )
 {
     /*
@@ -2093,7 +2112,7 @@ het_error( int rc )
     {
         rc = 0;
     }
-    
+
     /*
     || Turn it into an index
     */
@@ -2112,3 +2131,71 @@ het_error( int rc )
     */
     return( het_errstr[ rc ] );
 }
+/*==DOC==
+
+    NAME
+            het_tell - Returns the current read/write pointer offset
+
+    SYNOPSIS
+            #include "hetlib.h"
+
+            OFF_T het_tell( HETB *hetb )
+
+    DESCRIPTION
+            Returns a OFF_T describing the actual read/write cursor
+            within the HET file
+
+    RETURN VALUE
+            >=0 The actual cursor offset
+            <0 An error occured.
+                Possible errors are :
+                HETE_ERROR - File system error occured
+
+    EXAMPLE
+            //
+            // Get the current HET pointer
+            //
+
+            #include "hetlib.h"
+
+            int main( int argc, char *argv[] )
+            {
+                HETB *hetb;
+                int rc;
+                OFF_T rwptr;
+
+                rc = het_open( &hetb, argv[ 1 ], 0 );
+                if( rc >= 0 )
+                {
+                    rwptr = het_tell( hetb );
+                    if( rwptr >= 0 )
+                    {
+                        printf( "Current offset is %" I64_FMT "d\n" , (U64)rwptr);
+                    }
+                }
+
+                if( rc < 0 )
+                {
+                    printf( "HETLIB error: %d\n", rc );
+                }
+
+                het_close( &hetb );
+
+                return( 0 );
+            }
+
+    SEE ALSO
+
+==DOC==*/
+
+DLL_EXPORT
+OFF_T
+het_tell( HETB *hetb )
+{
+    OFF_T rwptr = FTELL( hetb->fd );
+    if ( rwptr < 0 )
+    {
+        return HETE_ERROR;
+    }
+    return rwptr;
+}
