@@ -1,25 +1,20 @@
 /*
 || ----------------------------------------------------------------------------
 ||
-|| HETMAP.C     (c) Copyright Leland Lucius, 2000-2003
+|| HETMAP.C     (c) Copyright Leland Lucius, 2000-2005
 ||              Released under terms of the Q Public License.
 ||
-|| Displays information about the structure of a Hercules Emulated Tape. 
+|| Displays information about the structure of a Hercules Emulated Tape.
 ||
 || ----------------------------------------------------------------------------
 */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "hstdinc.h"
 
-#include <unistd.h>
-#include <fcntl.h>
-#include <string.h>
-#include <stdlib.h>
+#include "hercules.h"
 #include "hetlib.h"
 #include "sllib.h"
-#include "hercules.h"
+#include "herc_getopt.h"
 
 /*
 || Local constant data
@@ -36,11 +31,8 @@ static const char help[] =
     "  -l  print only label information (default: off)\n";
 
 #ifdef EXTERNALGUI
-/* Special flag to indicate whether or not we're being
-   run under the control of the external GUI facility. */
-int extgui = 0;
 /* Previous reported file position */
-static long prevpos = 0;
+static OFF_T prevpos = 0;
 /* Report progress every this many bytes */
 #define PROGRESS_MASK (~0x3FFFF /* 256K */)
 #endif /*EXTERNALGUI*/
@@ -166,13 +158,15 @@ main( int argc, char *argv[] )
     {
         extgui = 1;
         argc--;
+        setvbuf(stderr, NULL, _IONBF, 0);
+        setvbuf(stdout, NULL, _IONBF, 0);
     }
 #endif /*EXTERNALGUI*/
 
     opts = O_ALL;
 
     /* Display the program identification message */
-    display_version (stderr, "Hercules HET map program ");
+    display_version (stderr, "Hercules HET map program ", FALSE);
 
     while( TRUE )
     {
@@ -253,11 +247,11 @@ main( int argc, char *argv[] )
         if( extgui )
         {
             /* Report progress every nnnK */
-            long curpos = ftell( hetb->fd );
+            OFF_T curpos = FTELL( hetb->fd );
             if( ( curpos & PROGRESS_MASK ) != ( prevpos & PROGRESS_MASK ) )
             {
                 prevpos = curpos;
-                fprintf( stderr, "IPOS=%ld\n", curpos );
+                fprintf( stderr, "IPOS=%" I64_FMT "d\n", (U64)curpos );
             }
         }
 #endif /*EXTERNALGUI*/
@@ -297,7 +291,7 @@ main( int argc, char *argv[] )
             cminsz = 0;
             cmaxsz = 0;
             cbytes = 0;
-            
+
             continue;
         }
 
