@@ -1,7 +1,7 @@
 /*
 || ----------------------------------------------------------------------------
 ||
-|| HETUPD.C     (c) Copyright Leland Lucius, 2000-2003
+|| HETUPD.C     (c) Copyright Leland Lucius, 2000-2005
 ||              Released under terms of the Q Public License.
 ||
 || Copy/update Hercules Emulated Tapes while allowing various modifications
@@ -11,19 +11,12 @@
 || ----------------------------------------------------------------------------
 */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#include <unistd.h>
-#include <fcntl.h>
-#include <string.h>
-#include <stdlib.h>
-#include <limits.h>
+#include "hstdinc.h"
 
+#include "hercules.h"
 #include "hetlib.h"
 #include "sllib.h"
-#include "hercules.h"
+#include "herc_getopt.h"
 
 /*
 || Local volatile data
@@ -40,11 +33,8 @@ static int dorename     = FALSE;
 static HETB *s_hetb     = NULL;
 static HETB *d_hetb     = NULL;
 #ifdef EXTERNALGUI
-/* Special flag to indicate whether or not we're being
-   run under the control of the external GUI facility. */
-int extgui = 0;
 /* Previous reported file position */
-static long prevpos = 0;
+static OFF_T prevpos = 0;
 /* Report progress every this many bytes */
 #define PROGRESS_MASK (~0x3FFFF /* 256K */)
 #endif /*EXTERNALGUI*/
@@ -53,7 +43,7 @@ static long prevpos = 0;
 || Local constant data
 */
 static const char help[] =
-    "%s - Print a map of an HET tape file\n\n"
+    "%s - Updates the compression of a Hercules Emulated Tape file.\n\n"
     "Usage: %s [options] source [dest]\n\n"
     "Options:\n"
     "  -1   compress fast\n"
@@ -97,7 +87,7 @@ closetapes( int rc )
 
     het_close( &d_hetb );
     het_close( &s_hetb );
-    
+
     if( dorename )
     {
         if( rc >= 0 )
@@ -132,11 +122,11 @@ copytape( void )
         if( extgui )
         {
             /* Report progress every nnnK */
-            long curpos = ftell( s_hetb->fd );
+            OFF_T curpos = FTELL( s_hetb->fd );
             if( ( curpos & PROGRESS_MASK ) != ( prevpos & PROGRESS_MASK ) )
             {
                 prevpos = curpos;
-                fprintf( stderr, "IPOS=%ld\n", curpos );
+                fprintf( stderr, "IPOS=%" I64_FMT "d\n", (U64)curpos );
             }
         }
 #endif /*EXTERNALGUI*/
@@ -241,7 +231,7 @@ opentapes( void )
         printf( "Compression level  : %d\n",
             het_cntl( d_hetb, HETCNTL_LEVEL, 0 ) );
     }
-        
+
 exit:
 
     if( rc < 0 )
@@ -269,6 +259,8 @@ main( int argc, char *argv[] )
     {
         extgui = 1;
         argc--;
+        setvbuf(stderr, NULL, _IONBF, 0);
+        setvbuf(stdout, NULL, _IONBF, 0);
     }
 #endif /*EXTERNALGUI*/
 
@@ -276,7 +268,7 @@ main( int argc, char *argv[] )
     d_hetb = NULL;
 
     /* Display the program identification message */
-    display_version (stderr, "Hercules HET copy/update program ");
+    display_version (stderr, "Hercules HET copy/update program ", FALSE);
 
     while( TRUE )
     {
@@ -357,11 +349,11 @@ main( int argc, char *argv[] )
             o_dname = toname;
             dorename = TRUE;
         break;
-        
+
         case 2:
             o_dname = argv[ optind + 1 ];
         break;
-        
+
         default:
             usage( argv[ 0 ] );
             exit( 1 );
@@ -388,6 +380,6 @@ main( int argc, char *argv[] )
     }
 
     closetapes( rc );
-            
+
     return 0;
 }
