@@ -1,35 +1,320 @@
-/* HSCMISC.C    (c) Copyright Roger Bowler, 1999-2003                */
-/*              (c) Copyright Jan Jaeger, 1999-2003                  */
-/*              Misc. system command routines                        */
+/* HSCMISC.C    Misc. system command routines                        */
+/*                                                                   */
+/*              (c) Copyright Roger Bowler, 1999-2005                */
+/*              (c) Copyright Jan Jaeger, 1999-2005                  */
 
+#include "hstdinc.h"
+
+#if !defined(_HENGINE_DLL_)
+#define _HENGINE_DLL_
+#endif
+
+#if !defined(_HSCMISC_C_)
+#define _HSCMISC_C_
+#endif
 
 #include "hercules.h"
-
 #include "devtype.h"
-
 #include "opcode.h"
-
 #include "inline.h"
 
+#define  DISPLAY_INSTRUCTION_OPERANDS
 
 #if !defined(_HSCMISC_C)
 #define _HSCMISC_C
 
+/*-------------------------------------------------------------------*/
+/*                System Shutdown Processing                         */
+/*-------------------------------------------------------------------*/
+
+/* The following 'sigq' functions are responsible for ensuring all of
+   the CPUs are stopped ("quiesced") before continuing with Hercules
+   shutdown processing, and should NEVER be called directly. Instead,
+   they are called by the main 'do_shutdown' (or 'do_shutdown_wait')
+   function(s) (defined further below) as needed and/or appropriate.
+*/
+static int wait_sigq_pending = 0;
+
+static int is_wait_sigq_pending()
+{
+int pending;
+
+    obtain_lock(&sysblk.intlock);
+    pending = wait_sigq_pending;
+    release_lock(&sysblk.intlock);
+
+    return pending;
+}
+
+static void wait_sigq_resp()
+{
+int pending, i;
+    /* Wait for all CPU's to stop */
+    do
+    {
+        obtain_lock(&sysblk.intlock);
+        wait_sigq_pending = 0;
+        for (i = 0; i < MAX_CPU_ENGINES; i++)
+        if (IS_CPU_ONLINE(i)
+          && sysblk.regs[i]->cpustate != CPUSTATE_STOPPED)
+            wait_sigq_pending = 1;
+        pending = wait_sigq_pending;
+        release_lock(&sysblk.intlock);
+
+        if(pending)
+            SLEEP(1);
+    }
+    while(is_wait_sigq_pending());
+}
+
+static void cancel_wait_sigq()
+{
+    obtain_lock(&sysblk.intlock);
+    wait_sigq_pending = 0;
+    release_lock(&sysblk.intlock);
+}
+
+/*                       do_shutdown_now
+
+   This is the main shutdown processing function. It is NEVER called
+   directly, but is instead ONLY called by either the 'do_shutdown'
+   or 'do_shutdown_wait' functions after all CPUs have been stopped.
+
+   It is responsible for releasing the device configuration and then
+   calling the Hercules Dynamic Loader "hdl_shut" function to invoke
+   all registered "Hercules at-exit/termination functions" (similar
+   to 'atexit' but unique to Hercules) (to perform any other needed
+   miscellaneous shutdown related processing).
+
+   Only after the above three tasks have been completed (stopping the
+   CPUs, releasing the device configuration, calling registered term-
+   ination routines/functions) can Hercules then be safely terminated.
+
+   Note too that, *technically*, this function *should* wait for *all*
+   other threads to finish terminating first before either exiting or
+   returning back to the caller, but we don't currently enforce that
+   (since that's *really* what hdl_adsc + hdl_shut is designed for!).
+
+   At the moment, as long as the three previously mentioned three most
+   important shutdown tasks have been completed (stop cpus, release
+   device config, call term funcs), then we consider the brunt of our
+   shutdown processing to be completed and thus exit (or return back
+   to the caller to let them exit instead). If there happen to be any
+   threads still running when that happens, they will be automatically
+   terminated by the operating sytem as normal when a process exits.
+
+   SO... If there are any threads that must be terminated completely
+   and cleanly before Hercules can safely terminate, then you better
+   add code to this function to ENSURE that your thread is terminated
+   properly! (and/or add a call to 'hdl_adsc' at the appropriate place
+   in the startup sequence). For this purpose, the use of "join_thread"
+   is *strongly* encouraged as it *ensures* that your thread will not
+   continue until the thread in question has completely exited first.
+*/
+static void do_shutdown_now()
+{
+    logmsg("HHCIN900I Begin Hercules shutdown\n");
+
+    ASSERT( !sysblk.shutfini );  // (sanity check)
+
+    sysblk.shutfini = 0;  // (shutdown NOT finished yet)
+    sysblk.shutdown = 1;  // (system shutdown initiated)
+
+    logmsg("HHCIN901I Releasing configuration\n");
+    {
+        release_config();
+    }
+    logmsg("HHCIN902I Configuration release complete\n");
+
+    logmsg("HHCIN903I Calling termination routines\n");
+    {
+        hdl_shut();
+    }
+    logmsg("HHCIN904I All termination routines complete\n");
+
+    /*
+    logmsg("HHCIN905I Terminating threads\n");
+    {
+        // (none we really care about at the moment...)
+    }
+    logmsg("HHCIN906I Threads terminations complete\n");
+    */
+
+    logmsg("HHCIN909I Hercules shutdown complete\n");
+    sysblk.shutfini = 1;    // (shutdown is now complete)
+
+    //                     PROGRAMMING NOTE
+
+    // If we're NOT in "daemon_mode" (i.e. panel_display in control),
+    // -OR- if a daemon_task DOES exist, then THEY are in control of
+    // shutdown; THEY are responsible for exiting the system whenever
+    // THEY feel it's proper to do so (by simply returning back to the
+    // caller thereby allowing 'main' to return back to the operating
+    // system).
+
+    // OTHEWRWISE we ARE in "daemon_mode", but a daemon_task does NOT
+    // exist, which means the main thread (tail end of 'impl.c') is
+    // stuck in a loop reading log messages and writing them to the
+    // logfile, so we need to do the exiting here since it obviously
+    // cannot.
+
+    if ( sysblk.daemon_mode
+#if defined(OPTION_DYNAMIC_LOAD)
+         && !daemon_task
+#endif /*defined(OPTION_DYNAMIC_LOAD)*/
+       )
+    {
+#if defined(FISH_HANG)
+        FishHangAtExit();
+#endif
+#ifdef _MSVC_
+        socket_deinit();
+#endif
+#ifdef DEBUG
+        fprintf(stdout, _("DO_SHUTDOWN_NOW EXIT\n"));
+#endif
+        fprintf(stdout, _("HHCIN099I Hercules terminated\n"));
+        fflush(stdout);
+        exit(0);
+    }
+}
+
+/*                     do_shutdown_wait
+
+   This function simply waits for the CPUs to stop and then calls
+   the above do_shutdown_now function to perform the actual shutdown
+   (which releases the device configuration, etc)
+*/
+static void do_shutdown_wait()
+{
+    logmsg(_("HHCIN098I Shutdown initiated\n"));
+    wait_sigq_resp();
+    do_shutdown_now();
+}
+
+/*                 *****  do_shutdown  *****
+
+   This is the main system shutdown function, and the ONLY function
+   that should EVER be called to shut the system down. It calls one
+   or more of the above static helper functions as needed.
+*/
+void do_shutdown()
+{
+TID tid;
+    if(is_wait_sigq_pending())
+        cancel_wait_sigq();
+    else
+        if(can_signal_quiesce() && !signal_quiesce(0,0))
+            create_thread(&tid, &sysblk.detattr, do_shutdown_wait, NULL);
+        else
+            do_shutdown_now();
+}
+/*-------------------------------------------------------------------*/
+/* The following 2 routines display an array of 32/64 registers      */
+/* 1st parameter is the register type (GR, CR, AR, etc..)            */
+/* 2nd parameter is the CPU Address involved                         */
+/* 3rd parameter is an array of 32/64 bit regs                       */
+/* NOTE : 32 bit regs are displayed 4 by 4, while 64 bit regs are    */
+/*        displayed 2 by 2. Change the modulo if to change this      */
+/*        behaviour.                                                 */
+/* These routines are intended to be invoked by display_regs,        */
+/* display_cregs and display_aregs                                   */
+/* Ivan Warren 2005/11/07                                            */
+/*-------------------------------------------------------------------*/
+static void display_regs32(char *hdr,U16 cpuad,U32 *r,int numcpus)
+{
+    int i;
+    for(i=0;i<16;i++)
+    {
+        if(!(i%4))
+        {
+            if(i)
+            {
+                logmsg("\n");
+            }
+            if(numcpus>1)
+            {
+                logmsg("CPU%4.4X: ",cpuad);
+            }
+        }
+        if(i%4)
+        {
+            logmsg("  ");
+        }
+        logmsg("%s%2.2d=%8.8"I32_FMT"X",hdr,i,r[i]);
+    }
+    logmsg("\n");
+}
+
+#if defined(_900)
+
+static void display_regs64(char *hdr,U16 cpuad,U64 *r,int numcpus)
+{
+    int i;
+    int rpl;
+    if(numcpus>1)
+    {
+        rpl=2;
+    }
+    else
+    {
+        rpl=4;
+    }
+    for(i=0;i<16;i++)
+    {
+        if(!(i%rpl))
+        {
+            if(i)
+            {
+                logmsg("\n");
+            }
+            if(numcpus>1)
+            {
+                logmsg("CPU%4.4X: ",cpuad);
+            }
+        }
+        if(i%rpl)
+        {
+            logmsg(" ");
+        }
+        logmsg("%s%1.1X=%16.16"I64_FMT"X",hdr,i,r[i]);
+    }
+    logmsg("\n");
+}
+
+#endif
+
 /*-------------------------------------------------------------------*/
 /* Display general purpose registers                                 */
 /*-------------------------------------------------------------------*/
 void display_regs (REGS *regs)
 {
-int     i;
+    int i;
+    U32 gprs[16];
+#if defined(_900)
+    U64 ggprs[16];
+#endif
 
+#if defined(_900)
     if(regs->arch_mode != ARCH_900)
-        for (i = 0; i < 16; i++)
-            logmsg ("GR%2.2d=%8.8X%s", i, regs->GR_L(i),
-                ((i & 0x03) == 0x03) ? "\n" : "\t");
+    {
+#endif
+        for(i=0;i<16;i++)
+        {
+            gprs[i]=regs->GR_L(i);
+        }
+        display_regs32("GR",regs->cpuad,gprs,sysblk.cpus);
+#if defined(_900)
+    }
     else
-        for (i = 0; i < 16; i++)
-            logmsg ("R%1.1X=%16.16llX%s", i, (long long)regs->GR_G(i),
-                ((i & 0x03) == 0x03) ? "\n" : " ");
+    {
+        for(i=0;i<16;i++)
+        {
+            ggprs[i]=regs->GR_G(i);
+        }
+        display_regs64("R",regs->cpuad,ggprs,sysblk.cpus);
+    }
+#endif
 
 } /* end function display_regs */
 
@@ -39,16 +324,32 @@ int     i;
 /*-------------------------------------------------------------------*/
 void display_cregs (REGS *regs)
 {
-int     i;
+    int i;
+    U32 crs[16];
+#if defined(_900)
+    U64 gcrs[16];
+#endif
 
+#if defined(_900)
     if(regs->arch_mode != ARCH_900)
-        for (i = 0; i < 16; i++)
-            logmsg ("CR%2.2d=%8.8X%s", i, regs->CR_L(i),
-                ((i & 0x03) == 0x03) ? "\n" : "\t");
+    {
+#endif
+        for(i=0;i<16;i++)
+        {
+            crs[i]=regs->CR_L(i);
+        }
+        display_regs32("CR",regs->cpuad,crs,sysblk.cpus);
+#if defined(_900)
+    }
     else
-        for (i = 0; i < 16; i++)
-            logmsg ("C%1.1X=%16.16llX%s", i, (long long)regs->CR_G(i),
-                ((i & 0x03) == 0x03) ? "\n" : " ");
+    {
+        for(i=0;i<16;i++)
+        {
+            gcrs[i]=regs->CR_G(i);
+        }
+        display_regs64("C",regs->cpuad,gcrs,sysblk.cpus);
+    }
+#endif
 
 } /* end function display_cregs */
 
@@ -58,11 +359,14 @@ int     i;
 /*-------------------------------------------------------------------*/
 void display_aregs (REGS *regs)
 {
-int     i;
+    int i;
+    U32 ars[16];
 
-    for (i = 0; i < 16; i++)
-        logmsg ("AR%2.2d=%8.8X%s", i, regs->AR(i),
-            ((i & 0x03) == 0x03) ? "\n" : "\t");
+    for(i=0;i<16;i++)
+    {
+        ars[i]=regs->AR(i);
+    }
+    display_regs32("AR",regs->cpuad,ars,sysblk.cpus);
 
 } /* end function display_aregs */
 
@@ -74,29 +378,34 @@ void display_fregs (REGS *regs)
 {
 
     if(regs->CR(0) & CR0_AFP)
+        logmsg
+        (
+            "CPU%4.4X:  FPR0=%8.8X %8.8X   FPR1=%8.8X %8.8X\n"
+            "CPU%4.4X:  FPR2=%8.8X %8.8X   FPR3=%8.8X %8.8X\n"
+            "CPU%4.4X:  FPR4=%8.8X %8.8X   FPR5=%8.8X %8.8X\n"
+            "CPU%4.4X:  FPR6=%8.8X %8.8X   FPR7=%8.8X %8.8X\n"
+            "CPU%4.4X:  FPR8=%8.8X %8.8X   FPR9=%8.8X %8.8X\n"
+            "CPU%4.4X:  FPRa=%8.8X %8.8X   FPRb=%8.8X %8.8X\n"
+            "CPU%4.4X:  FPRc=%8.8X %8.8X   FPRd=%8.8X %8.8X\n"
+            "CPU%4.4X:  FPRe=%8.8X %8.8X   FPRf=%8.8X %8.8X\n"
 
-    logmsg ("FPR0=%8.8X %8.8X\t\tFPR1=%8.8X %8.8X\n"
-            "FPR2=%8.8X %8.8X\t\tFPR3=%8.8X %8.8X\n"
-            "FPR4=%8.8X %8.8X\t\tFPR5=%8.8X %8.8X\n"
-            "FPR6=%8.8X %8.8X\t\tFPR7=%8.8X %8.8X\n"
-            "FPR8=%8.8X %8.8X\t\tFPR9=%8.8X %8.8X\n"
-            "FPRa=%8.8X %8.8X\t\tFPRb=%8.8X %8.8X\n"
-            "FPRc=%8.8X %8.8X\t\tFPRd=%8.8X %8.8X\n"
-            "FPRe=%8.8X %8.8X\t\tFPRf=%8.8X %8.8X\n",
-            regs->fpr[0], regs->fpr[1], regs->fpr[2], regs->fpr[3],
-            regs->fpr[4], regs->fpr[5], regs->fpr[6], regs->fpr[7],
-            regs->fpr[8], regs->fpr[9], regs->fpr[10], regs->fpr[11],
-            regs->fpr[12], regs->fpr[13], regs->fpr[14], regs->fpr[15],
-            regs->fpr[16], regs->fpr[17], regs->fpr[18], regs->fpr[19],
-            regs->fpr[20], regs->fpr[21], regs->fpr[22], regs->fpr[23],
-            regs->fpr[24], regs->fpr[25], regs->fpr[26], regs->fpr[27],
-            regs->fpr[28], regs->fpr[29], regs->fpr[30], regs->fpr[31]);
+            ,regs->cpuad, regs->fpr[0],  regs->fpr[1],  regs->fpr[2],  regs->fpr[3]
+            ,regs->cpuad, regs->fpr[4],  regs->fpr[5],  regs->fpr[6],  regs->fpr[7]
+            ,regs->cpuad, regs->fpr[8],  regs->fpr[9],  regs->fpr[10], regs->fpr[11]
+            ,regs->cpuad, regs->fpr[12], regs->fpr[13], regs->fpr[14], regs->fpr[15]
+            ,regs->cpuad, regs->fpr[16], regs->fpr[17], regs->fpr[18], regs->fpr[19]
+            ,regs->cpuad, regs->fpr[20], regs->fpr[21], regs->fpr[22], regs->fpr[23]
+            ,regs->cpuad, regs->fpr[24], regs->fpr[25], regs->fpr[26], regs->fpr[27]
+            ,regs->cpuad, regs->fpr[28], regs->fpr[29], regs->fpr[30], regs->fpr[31]
+        );
     else
-
-    logmsg ("FPR0=%8.8X %8.8X\t\tFPR2=%8.8X %8.8X\n"
-            "FPR4=%8.8X %8.8X\t\tFPR6=%8.8X %8.8X\n",
-            regs->fpr[0], regs->fpr[1], regs->fpr[2], regs->fpr[3],
-            regs->fpr[4], regs->fpr[5], regs->fpr[6], regs->fpr[7]);
+        logmsg
+        (
+            "CPU%4.4X:  FPR0=%8.8X %8.8X   FPR2=%8.8X %8.8X\n"
+            "CPU%4.4X:  FPR4=%8.8X %8.8X   FPR6=%8.8X %8.8X\n"
+            ,regs->cpuad, regs->fpr[0], regs->fpr[1], regs->fpr[2], regs->fpr[3]
+            ,regs->cpuad, regs->fpr[4], regs->fpr[5], regs->fpr[6], regs->fpr[7]
+        );
 
 } /* end function display_fregs */
 
@@ -170,7 +479,7 @@ void display_subchannel (DEVBLK *dev)
 /*           start/end/value are returned in saddr, eaddr, newval    */
 /*      -1 = error message issued                                    */
 /*-------------------------------------------------------------------*/
-static int parse_range (BYTE *operand, U64 maxadr, U64 *sadrp,
+static int parse_range (char *operand, U64 maxadr, U64 *sadrp,
                         U64 *eadrp, BYTE *newval)
 {
 U64     opnd1, opnd2;                   /* Address/length operands   */
@@ -178,14 +487,15 @@ U64     saddr, eaddr;                   /* Range start/end addresses */
 int     rc;                             /* Return code               */
 int     n;                              /* Number of bytes altered   */
 int     h1, h2;                         /* Hexadecimal digits        */
-BYTE   *s;                              /* Alteration value pointer  */
+char    *s;                             /* Alteration value pointer  */
 BYTE    delim;                          /* Operand delimiter         */
 BYTE    c;                              /* Character work area       */
 
-    rc = sscanf(operand, "%Lx%c%Lx%c", &opnd1, &delim, &opnd2, &c);
+    rc = sscanf(operand, "%"I64_FMT"x%c%"I64_FMT"x%c",
+                &opnd1, &delim, &opnd2, &c);
 
     /* Process storage alteration operand */
-    if (rc > 2 && delim == '=')
+    if (rc > 2 && delim == '=' && newval)
     {
         s = strchr (operand, '=');
         for (n = 0; n < 32;)
@@ -295,21 +605,22 @@ void get_connected_client (DEVBLK* dev, char** pclientip, char** pclientname)
 /*      if a translation exception occurs), the translation is       */
 /*      performed using a temporary copy of the CPU registers.       */
 /*-------------------------------------------------------------------*/
-static U16 ARCH_DEP(virt_to_abs) (RADR *aaptr, int *siptr,
+static U16 ARCH_DEP(virt_to_abs) (RADR *raptr, int *siptr,
                         VADR vaddr, int arn, REGS *regs, int acctype)
 {
-RADR    raddr;                          /* Real address              */
-RADR    aaddr;
+RADR    raddr;
 int     icode;
-int  private = 0;
-int  protect = 0;
-U16  xcode;
 REGS    gregs, hgregs;
 
+    // ZZFIXME:  Win32 builds emits bad code here
+    //           so we have the next stmt:
+    //           (is that still true??)
+    if (!regs) return 0;
+
     gregs = *regs;
     gregs.ghostregs = 1;
 
-    if(gregs.sie_state)
+    if(SIE_MODE(&gregs))
     {
         hgregs = *gregs.hostregs;
         gregs.hostregs = &hgregs;
@@ -317,42 +628,18 @@ REGS    gregs, hgregs;
     }
 
     hgregs.ghostregs = 1;
-        
-    if(!(icode = setjmp(gregs.progjmp)))
+
+    if( !(icode = setjmp(gregs.progjmp)) )
     {
-//      hgregs.progjmp = gregs.progjmp;
         memcpy(&hgregs.progjmp,&gregs.progjmp,sizeof(jmp_buf));
-
-        /* Convert logical address to real address */
-        if (REAL_MODE(&gregs.psw))
-            raddr = vaddr;
-        else {
-            /* Return condition code 3 if translation exception */
-            if (ARCH_DEP(translate_addr) (vaddr, arn, &gregs,
-                                            acctype, &raddr,
-                                            &xcode, &private, &protect,
-                                            siptr))
-                return xcode;
-
-            /* Convert real address to absolute address */
-            aaddr = APPLY_PREFIXING (raddr, gregs.PX);
-
-            /* Program check if absolute address is outside main storage */
-            if (aaddr > gregs.mainlim)
-                return PGM_ADDRESSING_EXCEPTION;
-
-            SIE_TRANSLATE(&aaddr, ACCTYPE_SIE, &gregs);
-    
-            /* Program check if absolute address is outside main storage */
-            if (aaddr > gregs.mainlim)
-                return PGM_ADDRESSING_EXCEPTION;
-
-        }
+        ARCH_DEP(logical_to_main) (vaddr, arn, &gregs, acctype, 0);
+        raddr = SIE_MODE(&gregs) ? hgregs.dat.raddr : gregs.dat.raddr;
     }
     else
         return icode;
 
-    *aaptr = aaddr;
+    *siptr = gregs.dat.stid;
+    *raptr = raddr;
 
     return 0;
 
@@ -364,23 +651,19 @@ REGS    gregs, hgregs;
 /* Prefixes display by Rxxxxx: if draflag is 1                       */
 /* Returns number of characters placed in display buffer             */
 /*-------------------------------------------------------------------*/
-static int ARCH_DEP(display_real) (REGS *regs, RADR raddr, BYTE *buf,
+static int ARCH_DEP(display_real) (REGS *regs, RADR raddr, char *buf,
                                     int draflag)
 {
 RADR    aaddr;                          /* Absolute storage address  */
 int     i, j;                           /* Loop counters             */
 int     n = 0;                          /* Number of bytes in buffer */
-BYTE    hbuf[40];                       /* Hexadecimal buffer        */
+char    hbuf[40];                       /* Hexadecimal buffer        */
 BYTE    cbuf[17];                       /* Character buffer          */
 BYTE    c;                              /* Character work area       */
 
     if (draflag)
     {
-      #if defined(FEATURE_ESAME)
-        n = sprintf (buf, "R:%16.16llX:", (long long)raddr);
-      #else /*!defined(FEATURE_ESAME)*/
-        n = sprintf (buf, "R:%8.8X:", (U32)raddr);
-      #endif /*!defined(FEATURE_ESAME)*/
+        n = sprintf (buf, "R:"F_RADR":", raddr);
     }
 
     aaddr = APPLY_PREFIXING (raddr, regs->PX);
@@ -416,7 +699,7 @@ BYTE    c;                              /* Character work area       */
 /* Display virtual storage (up to 16 bytes, or until end of page)    */
 /* Returns number of characters placed in display buffer             */
 /*-------------------------------------------------------------------*/
-static int ARCH_DEP(display_virt) (REGS *regs, VADR vaddr, BYTE *buf,
+static int ARCH_DEP(display_virt) (REGS *regs, VADR vaddr, char *buf,
                                     int ar, int acctype)
 {
 RADR    raddr;                          /* Real address              */
@@ -424,11 +707,8 @@ int     n;                              /* Number of bytes in buffer */
 int     stid;                           /* Segment table indication  */
 U16     xcode;                          /* Exception code            */
 
-  #if defined(FEATURE_ESAME)
-    n = sprintf (buf, "V:%16.16llX:", (long long)vaddr);
-  #else /*!defined(FEATURE_ESAME)*/
-    n = sprintf (buf, "V:%8.8X:", vaddr);
-  #endif /*!defined(FEATURE_ESAME)*/
+    n = sprintf (buf, "%c:"F_VADR":", ar == USE_REAL_ADDR ? 'R' : 'V',
+                             vaddr);
     xcode = ARCH_DEP(virt_to_abs) (&raddr, &stid,
                                     vaddr, ar, regs, acctype);
     if (xcode == 0)
@@ -443,10 +723,111 @@ U16     xcode;                          /* Exception code            */
 } /* end function display_virt */
 
 
+/*-------------------------------------------------------------------*/
+/* Disassemble real                                                  */
+/*-------------------------------------------------------------------*/
+static void ARCH_DEP(disasm_stor) (REGS *regs, char *opnd)
+{
+U64     saddr, eaddr;                   /* Range start/end addresses */
+U64     maxadr;                         /* Highest real storage addr */
+RADR    raddr;                          /* Real storage address      */
+RADR    aaddr;                          /* Absolute storage address  */
+int     stid = -1;
+int     len;                            /* Number of bytes to alter  */
+int     i;                              /* Loop counter              */
+int     ilc;
+BYTE    inst[6];                        /* Storage alteration value  */
+BYTE    opcode;
+U16     xcode;
+char    type;
+
+    /* Set limit for address range */
+  #if defined(FEATURE_ESAME)
+    maxadr = 0xFFFFFFFFFFFFFFFFULL;
+  #else /*!defined(FEATURE_ESAME)*/
+    maxadr = 0x7FFFFFFF;
+  #endif /*!defined(FEATURE_ESAME)*/
+
+    while((opnd && *opnd != '\0') &&
+      (*opnd == ' ' || *opnd == '\t'))
+        opnd++;
+
+    if(REAL_MODE(&regs->psw))
+        type = 'R';
+    else
+        type = 'V';
+
+    switch(toupper(*opnd)) {
+        case 'R': /* real */
+        case 'V': /* virtual */
+        case 'P': /* primary */
+        case 'H': /* home */
+          type = toupper(*opnd);
+          opnd++;
+    }
+
+    /* Parse the range or alteration operand */
+    len = parse_range (opnd, maxadr, &saddr, &eaddr, NULL);
+    if (len < 0) return;
+
+    /* Display real storage */
+    for (i = 0; i < 999 && saddr <= eaddr; i++)
+    {
+
+        if(type == 'R')
+            raddr = saddr;
+        else
+        {
+            if((xcode = ARCH_DEP(virt_to_abs) (&raddr, &stid, saddr, 0, regs, ACCTYPE_INSTFETCH) ))
+            {
+                logmsg(_("Storage not accessible code = %4.4X\n"), xcode);
+                return;
+            }
+        }
+
+        aaddr = APPLY_PREFIXING (raddr, regs->PX);
+        if (aaddr > regs->mainlim)
+        {
+            logmsg(_("Addressing exception\n"));
+            return;
+        }
+
+        opcode = regs->mainstor[aaddr];
+        ilc = ILC(opcode);
+
+        if (aaddr + ilc > regs->mainlim)
+        {
+            logmsg(_("Addressing exception\n"));
+            return;
+        }
+
+        memcpy(inst, regs->mainstor + aaddr, ilc);
+        logmsg("%c" F_RADR ": %2.2X%2.2X",
+          stid == TEA_ST_PRIMARY ? 'P' :
+          stid == TEA_ST_HOME ? 'H' :
+          stid == TEA_ST_SECNDRY ? 'S' : 'R',
+          raddr, inst[0], inst[1]);
+        if(ilc > 2)
+        {
+            logmsg("%2.2X%2.2X", inst[2], inst[3]);
+            if(ilc > 4)
+                logmsg("%2.2X%2.2X ", inst[4], inst[5]);
+            else
+                logmsg("     ");
+        }
+        else
+            logmsg("         ");
+        DISASM_INSTRUCTION(inst);
+        saddr += ilc;
+    } /* end for(i) */
+
+} /* end function disasm_stor */
+
+
 /*-------------------------------------------------------------------*/
 /* Process real storage alter/display command                        */
 /*-------------------------------------------------------------------*/
-static void ARCH_DEP(alter_display_real) (BYTE *opnd, REGS *regs)
+static void ARCH_DEP(alter_display_real) (char *opnd, REGS *regs)
 {
 U64     saddr, eaddr;                   /* Range start/end addresses */
 U64     maxadr;                         /* Highest real storage addr */
@@ -455,7 +836,7 @@ RADR    aaddr;                          /* Absolute storage address  */
 int     len;                            /* Number of bytes to alter  */
 int     i;                              /* Loop counter              */
 BYTE    newval[32];                     /* Storage alteration value  */
-BYTE    buf[100];                       /* Message buffer            */
+char    buf[100];                       /* Message buffer            */
 
     /* Set limit for address range */
   #if defined(FEATURE_ESAME)
@@ -495,7 +876,7 @@ BYTE    buf[100];                       /* Message buffer            */
 /*-------------------------------------------------------------------*/
 /* Process virtual storage alter/display command                     */
 /*-------------------------------------------------------------------*/
-static void ARCH_DEP(alter_display_virt) (BYTE *opnd, REGS *regs)
+static void ARCH_DEP(alter_display_virt) (char *opnd, REGS *regs)
 {
 U64     saddr, eaddr;                   /* Range start/end addresses */
 U64     maxadr;                         /* Highest virt storage addr */
@@ -509,7 +890,7 @@ int     n;                              /* Number of bytes in buffer */
 int     arn = 0;                        /* Access register number    */
 U16     xcode;                          /* Exception code            */
 BYTE    newval[32];                     /* Storage alteration value  */
-BYTE    buf[100];                       /* Message buffer            */
+char    buf[100];                       /* Message buffer            */
 
     /* Set limit for address range */
   #if defined(FEATURE_ESAME)
@@ -547,11 +928,7 @@ BYTE    buf[100];                       /* Message buffer            */
         {
             xcode = ARCH_DEP(virt_to_abs) (&raddr, &stid, vaddr, arn,
                                             regs, ACCTYPE_LRA);
-          #if defined(FEATURE_ESAME)
-            n = sprintf (buf, "V:%16.16llX ", (long long)vaddr);
-          #else /*!defined(FEATURE_ESAME)*/
-            n = sprintf (buf, "V:%8.8X ", vaddr);
-          #endif /*!defined(FEATURE_ESAME)*/
+            n = sprintf (buf, "V:"F_VADR" ", vaddr);
             if (stid == TEA_ST_PRIMARY)
                 n += sprintf (buf+n, "(primary)");
             else if (stid == TEA_ST_SECNDRY)
@@ -561,11 +938,7 @@ BYTE    buf[100];                       /* Message buffer            */
             else
                 n += sprintf (buf+n, "(AR%2.2d)", arn);
             if (xcode == 0)
-          #if defined(FEATURE_ESAME)
-                n += sprintf (buf+n, " R:%16.16llX", (long long)raddr);
-          #else /*!defined(FEATURE_ESAME)*/
-                n += sprintf (buf+n, " R:%8.8X", (U32)raddr);
-          #endif /*!defined(FEATURE_ESAME)*/
+                n += sprintf (buf+n, " R:"F_RADR, raddr);
             logmsg ("%s\n", buf);
         }
         ARCH_DEP(display_virt) (regs, vaddr, buf, arn, ACCTYPE_LRA);
@@ -588,11 +961,11 @@ int     ilc;                            /* Instruction length        */
 int     b1=-1, b2=-1, x1;               /* Register numbers          */
 VADR    addr1 = 0, addr2 = 0;           /* Operand addresses         */
 #endif /*DISPLAY_INSTRUCTION_OPERANDS*/
-BYTE    buf[100];                       /* Message buffer            */
+char    buf[256];                       /* Message buffer            */
 int     n;                              /* Number of bytes in buffer */
 
   #if defined(_FEATURE_SIE)
-    if(regs->sie_state)
+    if(SIE_MODE(regs))
         logmsg(_("SIE: "));
   #endif /*defined(_FEATURE_SIE)*/
 
@@ -608,8 +981,16 @@ int     n;                              /* Number of bytes in buffer */
 
     /* Display the PSW */
     memset (qword, 0x00, sizeof(qword));
-    ARCH_DEP(store_psw) (regs, qword);
-    n = sprintf (buf,
+    copy_psw (regs, qword);
+    if(sysblk.cpus>1)
+    {
+        n=sprintf(buf,"CPU%4.4X:  ",regs->cpuad);
+    }
+    else
+    {
+        n=0;
+    }
+    n += sprintf (buf+n,
                 "PSW=%2.2X%2.2X%2.2X%2.2X %2.2X%2.2X%2.2X%2.2X ",
                 qword[0], qword[1], qword[2], qword[3],
                 qword[4], qword[5], qword[6], qword[7]);
@@ -630,7 +1011,7 @@ int     n;                              /* Number of bytes in buffer */
 
     /* Extract the opcode and determine the instruction length */
     opcode = inst[0];
-    ilc = (opcode < 0x40) ? 2 : (opcode < 0xC0) ? 4 : 6;
+    ilc = ILC(opcode);
 
     /* Display the instruction */
     n += sprintf (buf+n, "INST=%2.2X%2.2X", inst[0], inst[1]);
@@ -718,29 +1099,40 @@ int     n;                              /* Number of bytes in buffer */
     /* Display storage at first storage operand location */
     if (b1 >= 0)
     {
-        if (REAL_MODE(&regs->psw))
-            n = ARCH_DEP(display_real) (regs, addr1, buf, 1);
+        if(REAL_MODE(&regs->psw))
+            n = ARCH_DEP(display_virt) (regs, addr1, buf, USE_REAL_ADDR,
+                                                ACCTYPE_READ);
         else
             n = ARCH_DEP(display_virt) (regs, addr1, buf, b1,
                                 (opcode == 0x44 ? ACCTYPE_INSTFETCH :
                                  opcode == 0xB1 ? ACCTYPE_LRA :
                                                   ACCTYPE_READ));
+        if(sysblk.cpus>1)
+        {
+            logmsg ("CPU%4.4X:  ", regs->cpuad);
+        }
         logmsg ("%s\n", buf);
     }
 
     /* Display storage at second storage operand location */
     if (b2 >= 0)
     {
-        if (REAL_MODE(&regs->psw)
+        if(
+            (REAL_MODE(&regs->psw)
             || (opcode == 0xB2 && inst[1] == 0x4B) /*LURA*/
             || (opcode == 0xB2 && inst[1] == 0x46) /*STURA*/
             || (opcode == 0xB9 && inst[1] == 0x05) /*LURAG*/
-            || (opcode == 0xB9 && inst[1] == 0x25)) /*STURG*/
-            n = ARCH_DEP(display_real) (regs, addr2, buf, 1);
+            || (opcode == 0xB9 && inst[1] == 0x25))) /*STURG*/
+            n = ARCH_DEP(display_virt) (regs, addr2, buf, USE_REAL_ADDR,
+                                                ACCTYPE_READ);
         else
             n = ARCH_DEP(display_virt) (regs, addr2, buf, b2,
                                         ACCTYPE_READ);
 
+        if(sysblk.cpus>1)
+        {
+            logmsg ("CPU%4.4X:  ", regs->cpuad);
+        }
         logmsg ("%s\n", buf);
     }
 
@@ -757,6 +1149,8 @@ int     n;                              /* Number of bytes in buffer */
     if (!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         display_aregs (regs);
 
+    // display_fregs (regs)
+
 } /* end function display_inst */
 
 
@@ -777,7 +1171,7 @@ int     n;                              /* Number of bytes in buffer */
 /*-------------------------------------------------------------------*/
 /* Wrappers for architecture-dependent functions                     */
 /*-------------------------------------------------------------------*/
-void alter_display_real (BYTE *opnd, REGS *regs)
+void alter_display_real (char *opnd, REGS *regs)
 {
     switch(sysblk.arch_mode) {
 #if defined(_370)
@@ -797,7 +1191,7 @@ void alter_display_real (BYTE *opnd, REGS *regs)
 } /* end function alter_display_real */
 
 
-void alter_display_virt (BYTE *opnd, REGS *regs)
+void alter_display_virt (char *opnd, REGS *regs)
 {
     switch(sysblk.arch_mode) {
 #if defined(_370)
@@ -840,15 +1234,53 @@ void display_inst(REGS *regs, BYTE *inst)
 }
 
 
+void disasm_stor(REGS *regs, char *opnd)
+{
+    switch(regs->arch_mode) {
+#if defined(_370)
+        case ARCH_370:
+            s370_disasm_stor(regs,opnd);
+            break;
+#endif
+#if defined(_390)
+        case ARCH_390:
+            s390_disasm_stor(regs,opnd);
+            break;
+#endif
+#if defined(_900)
+        case ARCH_900:
+            z900_disasm_stor(regs,opnd);
+            break;
+#endif
+    }
+
+}
+
 /*-------------------------------------------------------------------*/
 /* Execute a Unix or Windows command                                 */
 /* Returns the system command status code                            */
 /*-------------------------------------------------------------------*/
 int herc_system (char* command)
 {
-#ifdef WIN32
+
+#if HOW_TO_IMPLEMENT_SH_COMMAND == USE_ANSI_SYSTEM_API_FOR_SH_COMMAND
+
     return system(command);
-#else /* !WIN32 */
+
+#elif HOW_TO_IMPLEMENT_SH_COMMAND == USE_W32_POOR_MANS_FORK
+
+  #define  SHELL_CMD_SHIM_PGM   "conspawn "
+
+    int rc = strlen(SHELL_CMD_SHIM_PGM) + strlen(command) + 1;
+    char* pszNewCommandLine = malloc( rc );
+    strlcpy( pszNewCommandLine, SHELL_CMD_SHIM_PGM, rc );
+    strlcat( pszNewCommandLine, command,            rc );
+    rc = w32_poor_mans_fork( pszNewCommandLine, NULL );
+    free( pszNewCommandLine );
+    return rc;
+
+#elif HOW_TO_IMPLEMENT_SH_COMMAND == USE_FORK_API_FOR_SH_COMMAND
+
 extern char **environ;
 int pid, status;
 
@@ -864,9 +1296,8 @@ int pid, status;
     {
         char *argv[4];
 
-        dup2(sysblk.msgpiper, STDIN_FILENO);
-        dup2(fileno(sysblk.msgpipew), STDOUT_FILENO);
-        dup2(fileno(sysblk.msgpipew), STDERR_FILENO);
+        /* Redirect stderr (screen) to hercules log task */
+        dup2(STDOUT_FILENO, STDERR_FILENO);
 
         /* Drop ROOT authority (saved uid) */
         SETMODE(TERM);
@@ -889,7 +1320,9 @@ int pid, status;
         } else
             return status;
     } while(1);
-#endif /* !WIN32 */
+#else
+  #error 'HOW_TO_IMPLEMENT_SH_COMMAND' not #defined correctly
+#endif
 } /* end function herc_system */
 
 #endif /*!defined(_GEN_ARCH)*/
