@@ -1,4 +1,4 @@
-/* HTTPSERV.C   (c)Copyright Jan Jaeger, 2002-2003                   */
+/* HTTPSERV.C   (c)Copyright Jan Jaeger, 2002-2005                   */
 /*              HTTP Server                                          */
 
 /* This file contains all code required for the HTTP server,         */
@@ -27,8 +27,11 @@
 /*                                                                   */
 /*                                           Jan Jaeger - 28/03/2002 */
 
+#include "hstdinc.h"
+
 #include "hercules.h"
 #include "httpmisc.h"
+#include "hostinfo.h"
 
 
 #if defined(OPTION_HTTP_SERVER)
@@ -37,33 +40,39 @@
 extern CGITAB cgidir[];
 
 
-static CONTYP mime_types[] = {
+static MIMETAB mime_types[] = {
     { NULL,    NULL },                            /* No suffix entry */
-    { "txt",   "text/plain" },
-    { "jcl",   "text/plain" },
-    { "gif",   "image/gif"  },
-    { "jpg",   "image/jpeg" },
-    { "css",   "text/css"   },
-    { "html",  "text/html"  },
-    { "htm",   "text/html"  },
+    { "txt",   "text/plain"                },
+    { "jcl",   "text/plain"                },
+    { "gif",   "image/gif"                 },
+    { "jpg",   "image/jpeg"                },
+    { "css",   "text/css"                  },
+    { "html",  "text/html"                 },
+    { "htm",   "text/html"                 },
+/* This one should be:
+    { "ico",   "image/vnd.microsoft.icon"  },
+   but Apache 2 sets it as: */
+    { "ico",   "image/x-icon"              },
+/* so we'll go with what's actually in use. --JRM */
     { NULL,    NULL } };                     /* Default suffix entry */
 
-
 int html_include(WEBBLK *webblk, char *filename)
 {
     FILE *inclfile;
-    char fullname[1024];
-    char buffer[1024];
+    char fullname[HTTP_PATH_LENGTH];
+    char buffer[HTTP_PATH_LENGTH];
     int ret;
 
-    strncpy(fullname,sysblk.httproot,1024);
-    inclfile = fopen(strncat(fullname,filename,1024),"r");
+    strlcpy( fullname, sysblk.httproot, sizeof(fullname) );
+    strlcat( fullname, filename,        sizeof(fullname) );
+
+    inclfile = fopen(fullname,"rb");
 
     if (!inclfile)
     {
         logmsg(_("HHCHT011E html_include: Cannot open %s: %s\n"),
           fullname,strerror(errno));
-        fprintf(webblk->hsock,_("ERROR: Cannot open %s: %s\n"),
+        hprintf(webblk->sock,_("ERROR: Cannot open %s: %s\n"),
           filename,strerror(errno));
         return FALSE;
     }
@@ -72,30 +81,29 @@ int html_include(WEBBLK *webblk, char *filename)
     {
         ret = fread(buffer, 1, sizeof(buffer), inclfile);
         if (ret <= 0) break;
-        fwrite(buffer, 1, ret, webblk->hsock);
+        hwrite(webblk->sock,buffer, ret);
     }
 
     fclose(inclfile);
     return TRUE;
 }
 
-
 void html_header(WEBBLK *webblk)
 {
     if (webblk->request_type != REQTYPE_POST)
-        fprintf(webblk->hsock,"Expires: 0\n");
+        hprintf(webblk->sock,"Expires: 0\n");
 
-    fprintf(webblk->hsock,"Content-type: text/html;\n\n");
+    hprintf(webblk->sock,"Content-type: text/html;\n\n");
 
     if (!html_include(webblk,HTML_HEADER))
-        fprintf(webblk->hsock,"<HTML>\n<HEAD>\n<TITLE>Hercules</TITLE>\n</HEAD>\n<BODY>\n\n");
+        hprintf(webblk->sock,"<HTML>\n<HEAD>\n<TITLE>Hercules</TITLE>\n</HEAD>\n<BODY>\n\n");
 }
 
 
 void html_footer(WEBBLK *webblk)
 {
     if (!html_include(webblk,HTML_FOOTER))
-        fprintf(webblk->hsock,"\n</BODY>\n</HTML>\n");
+        hprintf(webblk->sock,"\n</BODY>\n</HTML>\n");
 }
 
 
@@ -104,7 +112,7 @@ static void http_exit(WEBBLK *webblk)
 CGIVAR *cgivar;
     if(webblk)
     {
-        fclose(webblk->hsock);
+        close_socket(webblk->sock);
         if(webblk->user) free(webblk->user);
         if(webblk->request) free(webblk->request);
         cgivar = webblk->cgivar;
@@ -124,9 +132,9 @@ CGIVAR *cgivar;
 
 static void http_error(WEBBLK *webblk, char *err, char *header, char *info)
 {
-    fprintf(webblk->hsock,"HTTP/1.0 %s\n%sConnection: close\n"
+    hprintf(webblk->sock,"HTTP/1.0 %s\n%sConnection: close\n"
                           "Content-Type: text/html\n\n"
-                          "<HTML><HEAD><TITLE>%s</TITLE></HEAD>" 
+                          "<HTML><HEAD><TITLE>%s</TITLE></HEAD>"
                           "<BODY><H1>%s</H1><P>%s</BODY></HTML>\n\n",
                           err, header, err, err, info);
     http_exit(webblk);
@@ -200,7 +208,7 @@ static char *http_unescape(char *buffer)
             pointer++;
             continue;
         }
-    
+
         if (lownibble >= '0' && lownibble <= '9')
             lownibble = lownibble - '0';
         else if (lownibble >= 'A' && lownibble <= 'F')
@@ -212,13 +220,13 @@ static char *http_unescape(char *buffer)
             pointer++;
             continue;
         }
-                        
+
         *pointer = (highnibble<<4) | lownibble;
 
         memcpy(pointer+1, pointer+3, strlen(pointer+3)+1);
 
         pointer++;
-    } 
+    }
 
     return buffer;
 }
@@ -236,12 +244,12 @@ CGIVAR **cgivar;
          cgivar = &((*cgivar)->next));
 
     for (name = strtok_r(qstring,"&; ",&strtok_str);
-         name; 
+         name;
          name = strtok_r(NULL,"&; ",&strtok_str))
     {
         if(!(value = strchr(name,'=')))
             continue;
-                        
+
         *value++ = '\0';
 
         (*cgivar) = malloc(sizeof(CGIVAR));
@@ -277,22 +285,27 @@ char *http_variable(WEBBLK *webblk, char *name, int type)
 
 static void http_verify_path(WEBBLK *webblk, char *path)
 {
-    char resolved_base[1024];
-    char resolved_path[1024];
+    char resolved_path[HTTP_PATH_LENGTH];
+#if 0
     int i;
 
-    realpath(sysblk.httproot,resolved_base); strncat(resolved_base,"/",1024);
-    realpath(path,resolved_path);
-
     for (i = 0; path[i]; i++)
         if (!isalnum((int)path[i]) && !strchr("/.-_", path[i]))
             http_error(webblk, "404 File Not Found","",
                                "Illegal character in filename");
+#endif
 
-    if(strncmp(resolved_base,resolved_path,strlen(resolved_base)))
+    if (!realpath( path, resolved_path ))
         http_error(webblk, "404 File Not Found","",
                            "Invalid pathname");
 
+    // The following verifies the specified file does not lie
+    // outside the specified httproot (Note: sysblk.httproot
+    // was previously resolved to an absolute path by config.c)
+
+    if (strncmp( sysblk.httproot, resolved_path, strlen(sysblk.httproot)))
+        http_error(webblk, "404 File Not Found","",
+                           "Invalid pathname");
 }
 
 
@@ -312,7 +325,7 @@ static int http_authenticate(WEBBLK *webblk, char *type, char *userpass)
                 *pointer = 0;
                 user = userpass;
                 passwd = pointer+1;
-    
+
                 /* Hardcoded userid and password in configuration file */
                 if(sysblk.httpuser && sysblk.httppass)
                 {
@@ -335,9 +348,9 @@ static int http_authenticate(WEBBLK *webblk, char *type, char *userpass)
 // ZZ No password check is being performed yet...
                     if((pass = getpwnam(user))
                       &&
-                       (pass->pw_uid == 0 
+                       (pass->pw_uid == 0
                           || pass->pw_uid == getuid()))
-                    { 
+                    {
                         webblk->user = strdup(user);
                         return TRUE;
                     }
@@ -355,20 +368,20 @@ static int http_authenticate(WEBBLK *webblk, char *type, char *userpass)
 
 static void http_download(WEBBLK *webblk, char *filename)
 {
-    char buffer[1024];
+    char buffer[HTTP_PATH_LENGTH];
     char tbuf[80];
     int fd, length;
     char *filetype;
-    char fullname[1024];
+    char fullname[HTTP_PATH_LENGTH];
     struct stat st;
-    CONTYP *mime_type = mime_types;
+    MIMETAB *mime_type = mime_types;
 
-    strncpy(fullname,sysblk.httproot,1024);
-    strncat(fullname,filename,1024);
+    strlcpy( fullname, sysblk.httproot, sizeof(fullname) );
+    strlcat( fullname, filename,        sizeof(fullname) );
 
     http_verify_path(webblk,fullname);
 
-    if(stat(fullname,&st)) 
+    if(stat(fullname,&st))
         http_error(webblk, "404 File Not Found","",
                            strerror(errno));
 
@@ -376,35 +389,35 @@ static void http_download(WEBBLK *webblk, char *filename)
         http_error(webblk, "404 File Not Found","",
                            "The requested file is not a regular file");
 
-    fd = open(fullname,O_RDONLY,0);
+    fd = open(fullname,O_RDONLY|O_BINARY,0);
     if (fd == -1)
         http_error(webblk, "404 File Not Found","",
                            strerror(errno));
 
-    fprintf(webblk->hsock,"HTTP/1.0 200 OK\n");
+    hprintf(webblk->sock,"HTTP/1.0 200 OK\n");
     if ((filetype = strrchr(filename,'.')))
         for(mime_type++;mime_type->suffix
           && strcasecmp(mime_type->suffix,filetype + 1);
           mime_type++);
     if(mime_type->type)
-        fprintf(webblk->hsock,"Content-Type: %s\n", mime_type->type);
+        hprintf(webblk->sock,"Content-Type: %s\n", mime_type->type);
 
-    fprintf(webblk->hsock,"Expires: %s\n",
+    hprintf(webblk->sock,"Expires: %s\n",
       http_timestring(tbuf,sizeof(tbuf),time(NULL)+HTML_STATIC_EXPIRY_TIME));
 
-    fprintf(webblk->hsock,"Content-Length: %d\n\n", (int)st.st_size);
+    hprintf(webblk->sock,"Content-Length: %d\n\n", (int)st.st_size);
     while ((length = read(fd, buffer, sizeof(buffer))) > 0)
-            fwrite(buffer, 1, length, webblk->hsock);
+            hwrite(webblk->sock,buffer, length);
     close(fd);
     http_exit(webblk);
 }
 
 
-static void *http_request(FILE *hsock)
+static void *http_request(int sock)
 {
     WEBBLK *webblk;
     int authok = !sysblk.httpauth;
-    char line[1024];
+    char line[HTTP_PATH_LENGTH];
     char *url = NULL;
     char *pointer;
     char *strtok_str;
@@ -415,9 +428,9 @@ static void *http_request(FILE *hsock)
         http_exit(webblk);
 
     memset(webblk,0,sizeof(WEBBLK));
-    webblk->hsock = hsock;
+    webblk->sock = sock;
 
-    while (fgets(line, sizeof(line), webblk->hsock))
+    while (hgets(line, sizeof(line), webblk->sock))
     {
         if (*line == '\r' || *line == '\n')
             break;
@@ -479,10 +492,10 @@ static void *http_request(FILE *hsock)
         int i;
             for(i = 0; i < content_length; i++)
             {
-                *pointer = fgetc(webblk->hsock);
+                *pointer = hgetc(webblk->sock);
                 if(*pointer != '\n' && *pointer != '\r')
                     pointer++;
-            } 
+            }
             *pointer = '\0';
             http_interpret_variable_string(webblk, post_arg, VARTYPE_POST);
             free(post_arg);
@@ -490,7 +503,7 @@ static void *http_request(FILE *hsock)
     }
 
     if (!authok)
-        http_error(webblk, "401 Authorization Required", 
+        http_error(webblk, "401 Authorization Required",
                            "WWW-Authenticate: Basic realm=\"HERCULES\"\n",
                            "You must be authenticated to use this service");
 
@@ -529,14 +542,30 @@ static void *http_request(FILE *hsock)
         if(!strcmp(cgient->path, url))
         {
         char tbuf[80];
-            fprintf(webblk->hsock,"HTTP/1.0 200 OK\nConnection: close\n");
-            fprintf(webblk->hsock,"Date: %s\n",
+            hprintf(webblk->sock,"HTTP/1.0 200 OK\nConnection: close\n");
+            hprintf(webblk->sock,"Date: %s\n",
               http_timestring(tbuf,sizeof(tbuf),time(NULL)));
             (cgient->cgibin) (webblk);
             http_exit(webblk);
         }
     }
 
+#if defined(OPTION_DYNAMIC_LOAD)
+    {
+    zz_cgibin dyncgi;
+
+        if( (dyncgi = HDL_FINDSYM(webblk->baseurl)) )
+        {
+        char tbuf[80];
+            hprintf(webblk->sock,"HTTP/1.0 200 OK\nConnection: close\n");
+            hprintf(webblk->sock,"Date: %s\n",
+              http_timestring(tbuf,sizeof(tbuf),time(NULL)));
+            dyncgi(webblk);
+            http_exit(webblk);
+        }
+    }
+#endif /*defined(OPTION_DYNAMIC_LOAD)*/
+
     http_error(webblk, "404 File Not Found","",
                        "The requested file was not found");
 
@@ -549,7 +578,6 @@ void *http_server (void *arg)
 int                     rc;             /* Return code               */
 int                     lsock;          /* Socket for listening      */
 int                     csock;          /* Socket for conversation   */
-FILE                   *hsock;          /* Socket for conversation   */
 struct sockaddr_in      server;         /* Server address structure  */
 fd_set                  selset;         /* Read bit map for select   */
 int                     optval;         /* Argument for setsockopt   */
@@ -562,19 +590,71 @@ TID                     httptid;        /* Negotiation thread id     */
             "tid="TIDPAT", pid=%d\n"),
             thread_id(), getpid());
 
+
+    /* If the HTTP root directory is not specified,
+       use a reasonable default */
+    if (!sysblk.httproot)
+        {
+#if defined(_MSVC_)
+        char process_dir[HTTP_PATH_LENGTH];
+        if (get_process_directory(process_dir,HTTP_PATH_LENGTH) > 0)
+        {
+            strlcat(process_dir,"\\html",HTTP_PATH_LENGTH);
+            sysblk.httproot = strdup(process_dir);
+        }
+        else
+#endif /*defined(WIN32)*/
+        sysblk.httproot = strdup(HTTP_ROOT);
+    }
+
+    /* Convert the specified HTTPROOT value to an absolute path
+       ending with a '/' and save in sysblk.httproot. */
+    {
+        char absolute_httproot_path[HTTP_PATH_LENGTH];
+        char save_working_directory[HTTP_PATH_LENGTH];
+        int  rc;
+#if defined(_MSVC_)
+        rc = expand_environ_vars( sysblk.httproot, absolute_httproot_path,
+            sizeof(absolute_httproot_path) );
+        if (rc == 0)
+            free(sysblk.httproot); sysblk.httproot = strdup(absolute_httproot_path);
+#endif /* defined(_MSVC_) */
+        if (!realpath(sysblk.httproot,absolute_httproot_path))
+        {
+            logmsg( _("HHCCF066E Invalid HTTPROOT: \"%s\": %s\n"),
+                   sysblk.httproot, strerror(errno));
+            return NULL;
+        }
+        VERIFY(getcwd(save_working_directory,sizeof(save_working_directory)));
+        rc = chdir(absolute_httproot_path);     // (verify path)
+        VERIFY(!chdir(save_working_directory)); // (restore cwd)
+        if (rc != 0)
+        {
+            logmsg( _("HHCCF066E Invalid HTTPROOT: \"%s\": %s\n"),
+                   absolute_httproot_path, strerror(errno));
+            return NULL;
+        }
+        /* Append trailing [back]slash, but only if needed */
+        rc = strlen(absolute_httproot_path);
+        if (absolute_httproot_path[rc-1] != *HTTP_PS)
+            strlcat(absolute_httproot_path,HTTP_PS,sizeof(absolute_httproot_path));
+        free(sysblk.httproot); sysblk.httproot = strdup(absolute_httproot_path);
+        logmsg(_("HHCHT013I Using HTTPROOT directory \"%s\"\n"),sysblk.httproot);
+    }
+
     /* Obtain a socket */
-    lsock = socket (AF_INET, SOCK_STREAM, 0);
+    lsock = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);
 
     if (lsock < 0)
     {
-        logmsg(_("HHCHT002E socket: %s\n"), strerror(errno));
+        logmsg(_("HHCHT002E socket: %s\n"), strerror(HSO_errno));
         return NULL;
     }
 
     /* Allow previous instance of socket to be reused */
     optval = 1;
     setsockopt (lsock, SOL_SOCKET, SO_REUSEADDR,
-                &optval, sizeof(optval));
+                (void*)&optval, sizeof(optval));
 
     /* Prepare the sockaddr structure for the bind */
     memset (&server, 0, sizeof(server));
@@ -588,16 +668,16 @@ TID                     httptid;        /* Negotiation thread id     */
     {
         rc = bind (lsock, (struct sockaddr *)&server, sizeof(server));
 
-        if (rc == 0 || errno != EADDRINUSE) break;
+        if (rc == 0 || HSO_errno != HSO_EADDRINUSE) break;
 
         logmsg (_("HHCHT003W Waiting for port %u to become free\n"),
                 sysblk.httpport);
-        sleep(10);
+        SLEEP(10);
     } /* end while */
 
     if (rc != 0)
     {
-        logmsg(_("HHCHT004E bind: %s\n"), strerror(errno));
+        logmsg(_("HHCHT004E bind: %s\n"), strerror(HSO_errno));
         return NULL;
     }
 
@@ -606,14 +686,14 @@ TID                     httptid;        /* Negotiation thread id     */
 
     if (rc < 0)
     {
-        logmsg(_("HHCHT005E listen: %s\n"), strerror(errno));
+        logmsg(_("HHCHT005E listen: %s\n"), strerror(HSO_errno));
         return NULL;
     }
 
     logmsg(_("HHCHT006I Waiting for HTTP requests on port %u\n"),
             sysblk.httpport);
 
-    /* Handle connection requests and attention interrupts */
+    /* Handle http requests */
     while (TRUE) {
 
         /* Initialize the select parameters */
@@ -621,51 +701,36 @@ TID                     httptid;        /* Negotiation thread id     */
         FD_SET (lsock, &selset);
 
         /* Wait for a file descriptor to become ready */
-#if defined(WIN32)
-    {
-        struct timeval tv={0,500000};   /* half a second */
-        rc = select ( lsock+1, &selset, NULL, NULL, &tv );
-    }
-#else /*!defined(WIN32)*/
         rc = select ( lsock+1, &selset, NULL, NULL, NULL );
-#endif /*!defined(WIN32)*/
 
         if (rc == 0) continue;
 
         if (rc < 0 )
         {
-            if (errno == EINTR) continue;
-            logmsg(_("HHCHT007E select: %s\n"), strerror(errno));
+            if (HSO_errno == HSO_EINTR) continue;
+            logmsg(_("HHCHT007E select: %s\n"), strerror(HSO_errno));
             break;
         }
 
-        /* If a client connection request has arrived then accept it */
+        /* If a http request has arrived then accept it */
         if (FD_ISSET(lsock, &selset))
         {
-            /* Accept a connection and create conversation socket */
+            /* Accept the connection and create conversation socket */
             csock = accept (lsock, NULL, NULL);
 
             if (csock < 0)
             {
-                logmsg(_("HHCHT008E accept: %s\n"), strerror(errno));
+                logmsg(_("HHCHT008E accept: %s\n"), strerror(HSO_errno));
                 continue;
             }
 
-            if(!(hsock = fdopen(csock,"r+")))
-            {
-                logmsg(_("HHCHT009E fdopen: %s\n"),strerror(errno));
-                close(csock);
-                continue;
-            }
-
-            /* Create a thread to complete the client connection */
+            /* Create a thread to execute the http request */
             if ( create_thread (&httptid, &sysblk.detattr,
-                                http_request, hsock) )
+                                http_request, (void *)(long)csock) )
             {
                 logmsg(_("HHCHT010E http_request create_thread: %s\n"),
                         strerror(errno));
-                fclose (hsock);
-                close (csock);
+                close_socket (csock);
             }
 
         } /* end if(lsock) */
@@ -673,7 +738,7 @@ TID                     httptid;        /* Negotiation thread id     */
     } /* end while */
 
     /* Close the listening socket */
-    close (lsock);
+    close_socket (lsock);
 
     return NULL;
 
