@@ -1,8 +1,8 @@
-/* IO.C         (c) Copyright Roger Bowler, 1994-2003                */
+/* IO.C         (c) Copyright Roger Bowler, 1994-2005                */
 /*              ESA/390 CPU Emulator                                 */
 
-/* Interpretive Execution - (c) Copyright Jan Jaeger, 1999-2003      */
-/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2003      */
+/* Interpretive Execution - (c) Copyright Jan Jaeger, 1999-2005      */
+/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2005      */
 
 /*-------------------------------------------------------------------*/
 /* This module implements all I/O instructions of the                */
@@ -30,6 +30,16 @@
 /*      64-bit IDAW support - Roger Bowler v209                  @IWZ*/
 /*-------------------------------------------------------------------*/
 
+#include "hstdinc.h"
+
+#if !defined(_HENGINE_DLL_)
+#define _HENGINE_DLL_
+#endif
+
+#if !defined(_IO_C_)
+#define _IO_C_
+#endif
+
 #include "hercules.h"
 
 #include "opcode.h"
@@ -47,12 +57,12 @@ int     b2;                             /* Effective addr base       */
 VADR    effective_addr2;                /* Effective address         */
 DEVBLK *dev;                            /* -> device block           */
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
 #if defined(_FEATURE_IO_ASSIST)
-    if(regs->sie_state && !regs->sie_pref && !(regs->siebk->ec[0] & SIE_EC0_IOA))
+    if(SIE_STATNB(regs, EC0, IOA) && !regs->sie_pref)
 #endif
        SIE_INTERCEPT(regs);
 
@@ -93,12 +103,12 @@ int     b2;                             /* Effective addr base       */
 VADR    effective_addr2;                /* Effective address         */
 DEVBLK *dev;                            /* -> device block           */
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
 #if defined(_FEATURE_IO_ASSIST)
-    if(regs->sie_state && !regs->sie_pref && !(regs->siebk->ec[0] & SIE_EC0_IOA))
+    if(SIE_STATNB(regs, EC0, IOA) && !regs->sie_pref)
 #endif
        SIE_INTERCEPT(regs);
 
@@ -138,7 +148,7 @@ VADR    effective_addr2;                /* Effective address         */
 DEVBLK *dev;                            /* -> device block           */
 PMCW    pmcw;                           /* Path management ctl word  */
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -158,7 +168,6 @@ PMCW    pmcw;                           /* Path management ctl word  */
         || (pmcw.flag25 & PMCW25_VISC)
         || (pmcw.flag27 & PMCW27_I)
 #endif
-        || (pmcw.flag25 & PMCW25_RESV)
         || (pmcw.flag26 != 0)
         || (pmcw.flag27 & PMCW27_RESV))
         ARCH_DEP(program_interrupt) (regs, PGM_OPERAND_EXCEPTION);
@@ -188,20 +197,21 @@ PMCW    pmcw;                           /* Path management ctl word  */
     PERFORM_SERIALIZATION (regs);
     PERFORM_CHKPT_SYNC (regs);
 
+    /* Obtain the device lock */
+    obtain_lock (&dev->lock);
+
     /* Condition code 1 if subchannel is status pending
        with other than intermediate status */
     if ((dev->scsw.flag3 & SCSW3_SC_PEND)
       && !(dev->scsw.flag3 & SCSW3_SC_INTER))
     {
         regs->psw.cc = 1;
+        release_lock (&dev->lock);
         return;
     }
 
-    /* Obtain the device lock */
-    obtain_lock (&dev->lock);
-
     /* Condition code 2 if subchannel is busy */
-    if (dev->busy || dev->pending)
+    if (dev->busy || IOPENDING(dev))
     {
         regs->psw.cc = 2;
         release_lock (&dev->lock);
@@ -222,7 +232,8 @@ PMCW    pmcw;                           /* Path management ctl word  */
     memcpy (dev->pmcw.intparm, pmcw.intparm, sizeof(FWORD));
 
     /* Update the ISC and A fields */
-    dev->pmcw.flag4 = pmcw.flag4;
+    dev->pmcw.flag4 &= ~(PMCW4_ISC | PMCW4_A);
+    dev->pmcw.flag4 |= (pmcw.flag4 & (PMCW4_ISC | PMCW4_A));
 
     /* Update the path management (LPM and POM) fields */
     dev->pmcw.lpm = pmcw.lpm;
@@ -230,10 +241,12 @@ PMCW    pmcw;                           /* Path management ctl word  */
 
     /* Update zone, VISC, I and S bit */
     dev->pmcw.zone = pmcw.zone;
-    dev->pmcw.flag25 = pmcw.flag25;
+    dev->pmcw.flag25 &= ~(PMCW25_VISC);
+    dev->pmcw.flag25 |= (pmcw.flag25 & PMCW25_VISC);
     dev->pmcw.flag26 = pmcw.flag26;
     dev->pmcw.flag27 = pmcw.flag27;
 
+#if defined(_FEATURE_IO_ASSIST)
     /* Relate the device storage view to the requested zone */
     { RADR mso, msl;
         mso = sysblk.zpb[dev->pmcw.zone].mso << 20;
@@ -247,6 +260,7 @@ PMCW    pmcw;                           /* Path management ctl word  */
         dev->mainlim = msl - mso;
         dev->storkeys = &(STORAGE_KEY(mso, &sysblk));
     }
+#endif /*defined(_FEATURE_IO_ASSIST)*/
 
     /* Set device priority from the interruption subclass bits */
     dev->priority = (dev->pmcw.flag4 & PMCW4_ISC) >> 3;
@@ -268,7 +282,7 @@ int     b2;                             /* Base of effective addr    */
 VADR    effective_addr2;                /* Effective address         */
 BYTE    chpid;
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -278,14 +292,14 @@ BYTE    chpid;
     if(regs->GR_L(1) & 0xFFFFFF00)
         ARCH_DEP(program_interrupt) (regs, PGM_OPERAND_EXCEPTION);
 
-    chpid = effective_addr2 & 0xFF;
+    chpid = regs->GR_L(1) & 0xFF;
 
     if( !(regs->psw.cc = chp_reset(chpid)) )
     {
         obtain_lock(&sysblk.intlock);
         sysblk.chp_reset[chpid/32] |= 0x80000000 >> (chpid % 32);
         ON_IC_CHANRPT;
-        WAKEUP_WAITING_CPU (ALL_CPUS, CPUSTATE_STARTED);
+        WAKEUP_CPUS_MASK (sysblk.waiting_mask);
         release_lock (&sysblk.intlock);
     }
 
@@ -303,12 +317,12 @@ int     b2;                             /* Effective addr base       */
 VADR    effective_addr2;                /* Effective address         */
 DEVBLK *dev;                            /* -> device block           */
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
 #if defined(_FEATURE_IO_ASSIST)
-    if(regs->sie_state && !regs->sie_pref && !(regs->siebk->ec[0] & SIE_EC0_IOA))
+    if(SIE_STATNB(regs, EC0, IOA) && !regs->sie_pref)
 #endif
        SIE_INTERCEPT(regs);
 
@@ -347,7 +361,7 @@ DEF_INST(set_address_limit)
 int     b2;                             /* Effective addr base       */
 VADR    effective_addr2;                /* Effective address         */
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -368,12 +382,12 @@ DEF_INST(set_channel_monitor)
 int     b2;                             /* Effective addr base       */
 VADR    effective_addr2;                /* Effective address         */
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
 #if defined(_FEATURE_IO_ASSIST)
-    if(regs->sie_state && !regs->sie_pref && !(regs->siebk->ec[0] & SIE_EC0_IOA))
+    if(SIE_STATNB(regs, EC0, IOA) && !regs->sie_pref)
 #endif
         SIE_INTERCEPT(regs);
 
@@ -389,7 +403,7 @@ VADR    effective_addr2;                /* Effective address         */
 
 #if defined(_FEATURE_IO_ASSIST)
     /* Virtual use of I/O Assist features must be intercepted */
-    if(regs->sie_state
+    if(SIE_MODE(regs)
       && ( (regs->GR_L(1) & CHM_GPR1_ZONE)
         || (regs->GR_L(1) & CHM_GPR1_A) ))
         SIE_INTERCEPT(regs);
@@ -417,7 +431,7 @@ VADR    effective_addr2;                /* Effective address         */
 #if defined(_FEATURE_IO_ASSIST)
     else
     {
-    int zone = regs->sie_state ? regs->siebk->zone :
+    int zone = SIE_MODE(regs) ? regs->siebk->zone :
                                ((regs->GR_L(1) & CHM_GPR1_ZONE) >> 16);
 
         /* Set the measurement block origin address */
@@ -448,12 +462,12 @@ VADR    effective_addr2;                /* Effective address         */
 DEVBLK *dev;                            /* -> device block           */
 ORB     orb;                            /* Operation request block   */
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
 #if defined(_FEATURE_IO_ASSIST)
-    if(regs->sie_state && !regs->sie_pref && !(regs->siebk->ec[0] & SIE_EC0_IOA))
+    if(SIE_STATNB(regs, EC0, IOA) && !regs->sie_pref)
 #endif
         SIE_INTERCEPT(regs);
 
@@ -474,6 +488,12 @@ ORB     orb;                            /* Operation request block   */
         ARCH_DEP(program_interrupt) (regs, PGM_OPERAND_EXCEPTION);
 #endif /*!defined(FEATURE_INCORRECT_LENGTH_INDICATION_SUPPRESSION)*/
 
+#if !defined(FEATURE_MIDAW)                                     /*@MW*/
+    /* Program check if modified indirect data addressing requested */
+    if (orb.flag7 & ORB7_D)                                     
+        ARCH_DEP(program_interrupt) (regs, PGM_OPERAND_EXCEPTION);
+#endif /*!defined(FEATURE_MIDAW)*/                              /*@MW*/
+
     /* Program check if reg 1 bits 0-15 not X'0001' */
     if ( regs->GR_LHH(1) != 0x0001 )
         ARCH_DEP(program_interrupt) (regs, PGM_OPERAND_EXCEPTION);
@@ -525,7 +545,7 @@ int     b2;                             /* Effective addr base       */
 VADR    effective_addr2;                /* Effective address         */
 BYTE    work[32];                       /* Work area                 */
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -553,7 +573,7 @@ int     b2;                             /* Effective addr base       */
 VADR    effective_addr2;                /* Effective address         */
 U32     n;                              /* Integer work area         */
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -587,7 +607,7 @@ VADR    effective_addr2;                /* Effective address         */
 DEVBLK *dev;                            /* -> device block           */
 SCHIB   schib;                          /* Subchannel information blk*/
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -650,12 +670,12 @@ U32     iointid;                        /* I/O interruption ident    */
 int     icode;                          /* Intercept code            */
 RADR    pfx;                            /* Prefix                    */
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
 #if defined(_FEATURE_IO_ASSIST)
-    if(regs->sie_state && !regs->sie_pref && !(regs->siebk->ec[0] & SIE_EC0_IOA))
+    if(SIE_STATNB(regs, EC0, IOA) && !regs->sie_pref)
 #endif
        SIE_INTERCEPT(regs);
 
@@ -695,8 +715,8 @@ RADR    pfx;                            /* Prefix                    */
                 if(icode != SIE_NO_INTERCEPT)
                 {
                     /* Point to SIE copy of PSA in state descriptor */
-                    psa = (void*)(regs->hostregs->mainstor + regs->sie_state + SIE_II_PSA_OFFSET);
-                    STORAGE_KEY(regs->sie_state, regs->hostregs) |= (STORKEY_REF | STORKEY_CHANGE);
+                    psa = (void*)(regs->hostregs->mainstor + SIE_STATE(regs) + SIE_II_PSA_OFFSET);
+                    STORAGE_KEY(SIE_STATE(regs), regs->hostregs) |= (STORKEY_REF | STORKEY_CHANGE);
                 }
                 else
 #endif
@@ -731,13 +751,13 @@ RADR    pfx;                            /* Prefix                    */
     else
     {
 #if defined(_FEATURE_IO_ASSIST)
-        /* If no I/O assisted devices have pending interrupts 
+        /* If no I/O assisted devices have pending interrupts
            then we must intercept */
         SIE_INTERCEPT(regs);
 #endif
         icode = 0;
     }
-    
+
     regs->psw.cc = (icode == 0) ? 0 : 1;
 }
 
@@ -753,12 +773,12 @@ DEVBLK *dev;                            /* -> device block           */
 IRB     irb;                            /* Interruption response blk */
 int     cc;                             /* Condition Code            */
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
 #if defined(_FEATURE_IO_ASSIST)
-    if(regs->sie_state && !regs->sie_pref && !(regs->siebk->ec[0] & SIE_EC0_IOA))
+    if(SIE_STATNB(regs, EC0, IOA) && !regs->sie_pref)
 #endif
         SIE_INTERCEPT(regs);
 
@@ -813,12 +833,12 @@ int     b2;                             /* Base of effective addr    */
 VADR    effective_addr2;                /* Effective address         */
 DEVBLK *dev;                            /* -> device block           */
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
 #if defined(_FEATURE_IO_ASSIST)
-    if(regs->sie_state && !regs->sie_pref && !(regs->siebk->ec[0] & SIE_EC0_IOA))
+    if(SIE_STATNB(regs, EC0, IOA) && !regs->sie_pref)
 #endif
        SIE_INTERCEPT(regs);
 
@@ -867,7 +887,16 @@ ORB     orb;                            /* Operation request blk @IZW*/
 VADR    ccwaddr;                        /* CCW address for start I/O */
 BYTE    ccwkey;                         /* Bits 0-3=key, 4=7=zeroes  */
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
+#if defined(FEATURE_ECPSVM)
+    if((inst[1])!=0x02)
+    {
+        if(ecpsvm_dosio(regs,b2,effective_addr2)==0)
+        {
+            return;
+        }
+    }
+#endif
 
     PRIV_CHECK(regs);
 
@@ -916,7 +945,7 @@ int     b2;                             /* Base of effective addr    */
 VADR    effective_addr2;                /* Effective address         */
 DEVBLK *dev;                            /* -> device block for SIO   */
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -938,6 +967,12 @@ DEVBLK *dev;                            /* -> device block for SIO   */
 
     /* Test the device and set the condition code */
     regs->psw.cc = testio (regs, dev, inst[1]);
+    /* Yield time slice so that device handler may get some time */
+    /* to possibly complete an I/O - to prevent a TIO Busy Loop  */
+    if(regs->psw.cc==2)
+    {
+        sched_yield();
+    }
 
 }
 
@@ -952,7 +987,7 @@ int     b2;                             /* Base of effective addr    */
 VADR    effective_addr2;                /* Effective address         */
 DEVBLK *dev;                            /* -> device block for SIO   */
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -990,12 +1025,12 @@ BYTE    channelid;
 U16     tch_ctl;
 #endif /*defined(_FEATURE_SIE)*/
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
 #if defined(_FEATURE_SIE)
-    if(!regs->sie_state)
+    if(!SIE_MODE(regs))
     {
 #endif /*defined(_FEATURE_SIE)*/
         /* Test for pending interrupt and set condition code */
@@ -1025,7 +1060,7 @@ DEF_INST(store_channel_id)
 int     b2;                             /* Base of effective addr    */
 VADR    effective_addr2;                /* Effective address         */
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -1048,7 +1083,7 @@ int     b2;                             /* Base of effective addr    */
 VADR    effective_addr2;                /* Effective address         */
 int     i;
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -1057,7 +1092,7 @@ int     i;
     effective_addr2 &= 0xFFFF;
 
     /* Hercules has as many channelsets as CPU's */
-    if(effective_addr2 >= MAX_CPU_ENGINES)
+    if(effective_addr2 >= MAX_CPU)
     {
         regs->psw.cc = 3;
         return;
@@ -1078,9 +1113,10 @@ int     i;
 
     /* If the addressed channelset is connected to another
        CPU then return with cc1 */
-    for(i = 0; i < MAX_CPU_ENGINES; i++)
+    for (i = 0; i < MAX_CPU; i++)
     {
-        if(sysblk.regs[i].chanset == effective_addr2)
+        if (IS_CPU_ONLINE(i)
+         && sysblk.regs[i]->chanset == effective_addr2)
         {
             release_lock(&sysblk.intlock);
             regs->psw.cc = 1;
@@ -1110,14 +1146,14 @@ int     b2;                             /* Base of effective addr    */
 VADR    effective_addr2;                /* Effective address         */
 int     i;
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
     SIE_INTERCEPT(regs);
 
     /* Hercules has as many channelsets as CPU's */
-    if(effective_addr2 >= MAX_CPU_ENGINES)
+    if(effective_addr2 >= MAX_CPU)
     {
         regs->psw.cc = 3;
         return;
@@ -1136,13 +1172,14 @@ int     i;
 
     /* If the addressed channelset is connected to another
        CPU then return with cc0 */
-    for(i = 0; i < MAX_CPU_ENGINES; i++)
+    for(i = 0; i < MAX_CPU; i++)
     {
-        if(sysblk.regs[i].chanset == effective_addr2)
+        if (IS_CPU_ONLINE(i)
+         && sysblk.regs[i]->chanset == effective_addr2)
         {
-            if(sysblk.regs[i].cpustate != CPUSTATE_STARTED)
+            if(sysblk.regs[i]->cpustate != CPUSTATE_STARTED)
             {
-                sysblk.regs[i].chanset = 0xFFFF;
+                sysblk.regs[i]->chanset = 0xFFFF;
                 regs->psw.cc = 0;
             }
             else
