@@ -3,12 +3,6 @@
 //          linklist.h          linked-list macros
 //
 ///////////////////////////////////////////////////////////////////////////////////////////
-//
-//  Change History:
-//
-//  05/31/01    2.13.0  Fish    Module created. (Copied from Microsoft Platform SDK)
-//
-//////////////////////////////////////////////////////////////////////////////////////////
 
 #ifndef _LLIST_
 #define _LLIST_
@@ -85,21 +79,17 @@
 */
 //////////////////////////////////////////////////////////////////////////////////////////
 
-
-#if !defined(_WINNT_)
-
-/*   (already defined in cygwin's version of "winnt.h")
+#if !defined( WIN32  ) || \
+   ( defined( _MSVC_ ) && !defined(_WINNT_) && !defined(_WINNT_H) )
 
 typedef struct _LIST_ENTRY
 {
-    struct  _LIST_ENTRY*  Flink;    // forward link; ptr to next link in chain
-    struct  _LIST_ENTRY*  Blink;    // backward link; ptr to previous link in chain
+    struct  _LIST_ENTRY*  Flink;    // ptr to next link in chain
+    struct  _LIST_ENTRY*  Blink;    // ptr to previous link in chain
 }
 LIST_ENTRY, *PLIST_ENTRY;
 
-*/
-
-
+#endif  // !defined(_WINNT_)
 
 //////////////////////////////////////////////////////////////////////////////////////////
 //
@@ -148,21 +138,17 @@ For Example:
     record by coding the following CONTAINING_RECORD macro expression:
 
         MYRECORD*    pMyRecord;     // the variable you wish to point to your record
-        LIST_ENTRY*  pListEntry;    // already points to the LIST_ENTRY field within
+    //  LIST_ENTRY*  pListEntry;    // already points to the LIST_ENTRY field within
                                     // your record (i.e. points to field "gamma")
 
         pMyRecord = CONTAINING_RECORD(pListEntry,MYRECORD,gamma);
 --*/
 
-#define CONTAINING_RECORD(address,type,field)                      \
+#ifndef  CONTAINING_RECORD
+#define  CONTAINING_RECORD( address, type, field )                 \
                                                                    \
     ( (type*) ((char*)(address) - (char*)(&((type*)0)->field)) )
-
-
-//////////////////////////////////////////////////////////////////////////////////////////
-
-
-#endif  // !defined(_WINNT_)
+#endif
 
 
 //////////////////////////////////////////////////////////////////////////////////////////
@@ -294,7 +280,7 @@ Return Value:
 
 //////////////////////////////////////////////////////////////////////////////////////////
 //
-//  VOID  InsertlistTail
+//  VOID  InsertListTail
 //  (
 //      LIST_ENTRY*  head,
 //      LIST_ENTRY*  entry
