@@ -1,7 +1,7 @@
-/* MACHCHK.C    (c) Copyright Jan Jaeger, 2000-2003                  */
+/* MACHCHK.C    (c) Copyright Jan Jaeger, 2000-2005                  */
 /*              ESA/390 Machine Check Functions                      */
 
-/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2003      */
+/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2005      */
 
 /*-------------------------------------------------------------------*/
 /* The machine check function supports dynamic I/O configuration.    */
@@ -17,6 +17,16 @@
 /* external interrupt as defined in the architecture. - 6/8/01 *JJ   */
 /*-------------------------------------------------------------------*/
 
+#include "hstdinc.h"
+
+#if !defined(_HENGINE_DLL_)
+#define _HENGINE_DLL_
+#endif
+
+#if !defined(_MACHCHK_C_)
+#define _MACHCHK_C_
+#endif
+
 #include "hercules.h"
 
 #include "opcode.h"
@@ -82,7 +92,7 @@ void machine_check_crwpend()
     /* Signal waiting CPUs that an interrupt may be pending */
     obtain_lock (&sysblk.intlock);
     ON_IC_CHANRPT;
-    WAKEUP_WAITING_CPUS (ALL_CPUS, CPUSTATE_STARTED);
+    WAKEUP_CPUS_MASK (sysblk.waiting_mask);
     release_lock (&sysblk.intlock);
 
 } /* end function machine_check_crwpend */
@@ -189,15 +199,12 @@ U32     xdmg = 0;
 RADR    fsta = 0;
 
     /* Release mainlock if held */
-#if MAX_CPU_ENGINES > 1
-    /* Unlock the main storage lock if held */
     if (regs->mainlock)
         RELEASE_MAINLOCK(regs);
 #if defined(FEATURE_INTERPRETIVE_EXECUTION)
     if(regs->sie_active && regs->guestregs->mainlock)
         RELEASE_MAINLOCK(regs->guestregs);
 #endif /*defined(FEATURE_INTERPRETIVE_EXECUTION)*/
-#endif /*MAX_CPU_ENGINES > 1*/
 
     /* Exit SIE when active */
 #if defined(FEATURE_INTERPRETIVE_EXECUTION)
@@ -226,7 +233,7 @@ RADR    fsta = 0;
 
     /* Trace the machine check interrupt */
     if (sysblk.insttrace || sysblk.inststep)
-        logmsg (_("HHCCP019I Machine Check code=%16.16llu\n"),
+        logmsg (_("HHCCP019I Machine Check code=%16.16" I64_FMT "u\n"),
                   (long long)mcic);
 
     /* Store the external damage code at PSA+244 */
@@ -260,16 +267,10 @@ RADR    fsta = 0;
 
 #if defined(_ARCHMODE3)
  #undef   _GEN_ARCH
- #if !defined(HAVE_STRSIGNAL)
-    char * strsignal( int sig ) {
-        return sys_siglist[sig];
-    }
- #endif
  #define  _GEN_ARCH _ARCHMODE3
  #include "machchk.c"
 #endif
 
-
 #if !defined(NO_SIGABEND_HANDLER)
 void sigabend_handler (int signo)
 {
@@ -282,14 +283,19 @@ int i;
     if( signo == SIGUSR2 )
     {
     DEVBLK *dev;
-
-        if(tid == sysblk.cnsltid)
+        if ( equal_threads( tid, sysblk.cnsltid ) ||
+             equal_threads( tid, sysblk.socktid ) )
             return;
         for (dev = sysblk.firstdev; dev != NULL; dev = dev->nextdev)
-            if (dev->tid == tid) break;
+            if ( equal_threads( dev->tid, tid ) ||
+                 equal_threads( dev->shrdtid, tid ) )
+                 break;
         if( dev == NULL)
-            logmsg(_("HHCCP020E signal USR2 received for undetermined "
-                     "device\n"));
+        {
+            if (!sysblk.shutdown)
+                logmsg(_("HHCCP020E signal USR2 received for "
+                         "undetermined device\n"));
+        }
         else
             if(dev->ccwtrace)
                 logmsg(_("HHCCP021E signal USR2 received for device "
@@ -297,27 +303,23 @@ int i;
         return;
     }
 
-#ifdef FEATURE_CPU_RECONFIG
-    for (i = 0; i < MAX_CPU_ENGINES; i++)
-#else /*!FEATURE_CPU_RECONFIG*/
-    for (i = 0; i < sysblk.numcpu; i++)
-#endif /*!FEATURE_CPU_RECONFIG*/
+    for (i = 0; i < MAX_CPU; i++)
     {
-        if(sysblk.regs[i].cputid == tid)
+        if ( equal_threads( sysblk.cputid[i], tid ) )
         {
-            regs = sysblk.regs + i;
+            regs = sysblk.regs[i];
             break;
         }
     }
 
-    if(regs == NULL)
+    if (regs == NULL)
     {
         signal(signo, SIG_DFL);
         raise(signo);
         return;
     }
-    
-    if(regs->psw.mach)
+
+    if(MACHMASK(&regs->psw))
     {
 #if defined(_FEATURE_SIE)
         logmsg(_("HHCCP017I CPU%4.4X: Machine check due to host error: %s\n"),
@@ -362,37 +364,33 @@ int i;
         logmsg(_("HHCCP018I CPU%4.4X: Check-Stop due to host error: %s\n"),
           regs->sie_active ? regs->guestregs->cpuad : regs->cpuad,
           strsignal(signo));
-#else /*!defined(_FEAURE_SIE)*/
+#else /*!defined(_FEATURE_SIE)*/
         logmsg(_("HHCCP018I CPU%4.4X: Check-Stop due to host error: %s\n"),
           regs->cpuad, strsignal(signo));
-#endif /*!defined(_FEAURE_SIE)*/
+#endif /*!defined(_FEATURE_SIE)*/
         display_inst(
 #if defined(_FEATURE_SIE)
                      regs->sie_active ? regs->guestregs :
-#endif /*defined(_FEAURE_SIE)*/
-                                                          regs, 
+#endif /*defined(_FEATURE_SIE)*/
+                                                          regs,
 #if defined(_FEATURE_SIE)
           regs->sie_active ? regs->guestregs->ip :
-#endif /*defined(_FEAURE_SIE)*/
+#endif /*defined(_FEATURE_SIE)*/
                                                    regs->ip);
         regs->cpustate = CPUSTATE_STOPPING;
         regs->checkstop = 1;
-        ON_IC_CPU_NOT_STARTED(regs);
+        ON_IC_INTERRUPT(regs);
 
         /* Notify other CPU's by means of a malfuction alert if possible */
         if (!try_obtain_lock(&sysblk.sigplock))
         {
             if(!try_obtain_lock(&sysblk.intlock))
             {
-#ifdef FEATURE_CPU_RECONFIG
-                for (i = 0; i < MAX_CPU_ENGINES; i++)
-#else /*!FEATURE_CPU_RECONFIG*/
-                for (i = 0; i < sysblk.numcpu; i++)
-#endif /*!FEATURE_CPU_RECONFIG*/
-                    if(i != regs->cpuad)
+                for (i = 0; i < MAX_CPU; i++)
+                    if (i != regs->cpuad && IS_CPU_ONLINE(i))
                     {
-                        ON_IC_MALFALT(&sysblk.regs[i]);
-                        sysblk.regs[i].malfcpu[regs->cpuad] = 1;
+                        ON_IC_MALFALT(sysblk.regs[i]);
+                        sysblk.regs[i]->malfcpu[regs->cpuad] = 1;
                     }
                 release_lock(&sysblk.intlock);
             }
