@@ -1,8 +1,18 @@
-/* PLO.C        (c) Copyright Jan Jaeger, 2000-2003                  */
+/* PLO.C        (c) Copyright Jan Jaeger, 2000-2005                  */
 /*              Perform Locked Operation functions codes             */
 
-/* Interpretive Execution - (c) Copyright Jan Jaeger, 1999-2003      */
-/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2003      */
+/* Interpretive Execution - (c) Copyright Jan Jaeger, 1999-2005      */
+/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2005      */
+
+#include "hstdinc.h"
+
+#if !defined(_HENGINE_DLL_)
+#define _HENGINE_DLL_
+#endif
+
+#if !defined(_PLO_C_)
+#define _PLO_C_
+#endif
 
 #include "hercules.h"
 
@@ -72,7 +82,7 @@ VADR op4addr;
                 ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
             op4alet = ARCH_DEP(vfetch4)(effective_addr4 + 68, b4, regs);
             regs->AR(r3) = op4alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
 
         /* Load address of operand 4 */
@@ -165,7 +175,7 @@ VADR op4addr;
                 ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
             op4alet = ARCH_DEP(vfetch4)(effective_addr4 + 68, b4, regs);
             regs->AR(r3) = op4alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
 
         /* Load address of operand 4 */
@@ -433,7 +443,7 @@ VADR op4addr;
                 ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
             op4alet = ARCH_DEP(vfetch4)(effective_addr4 + 68, b4, regs);
             regs->AR(r3) = op4alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
 
         /* Load address of operand 4 */
@@ -571,7 +581,7 @@ VADR op4addr;
                 ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
             op4alet = ARCH_DEP(vfetch4)(effective_addr4 + 68, b4, regs);
             regs->AR(r3) = op4alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
 
         /* Load address of operand 4 */
@@ -682,7 +692,7 @@ VADR op4addr;
                 ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
             op4alet = ARCH_DEP(vfetch4)(effective_addr4 + 68, b4, regs);
             regs->AR(r3) = op4alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
 
         /* Load address of operand 4 */
@@ -783,7 +793,7 @@ VADR op4addr;
                 ARCH_DEP(program_interrupt)(regs, PGM_SPECIFICATION_EXCEPTION);
             op4alet = ARCH_DEP(vfetch4)(effective_addr4 + 68, b4, regs);
             regs->AR(r3) = op4alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
 
         /* Load address of operand 4 */
@@ -843,7 +853,7 @@ VADR op4addr,
             op4alet = ARCH_DEP(vfetch4)(effective_addr4 + 68, b4, regs);
             op6alet = ARCH_DEP(vfetch4)(effective_addr4 + 100, b4, regs);
             regs->AR(r3) = op6alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
 
         /* Load address of operand 4 */
@@ -871,7 +881,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op4alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(vstore4)(op3, op4addr, r3, regs);
 
@@ -879,7 +889,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op6alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(vstore4)(op5, op6addr, r3, regs);
 
@@ -937,7 +947,7 @@ VADR op4addr,
             op4alet = ARCH_DEP(vfetch4)(effective_addr4 + 68, b4, regs);
             op6alet = ARCH_DEP(vfetch4)(effective_addr4 + 100, b4, regs);
             regs->AR(r3) = op6alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
 
         /* Load address of operand 4 */
@@ -965,7 +975,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op4alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(vstore8)(op3, op4addr, r3, regs);
 
@@ -973,7 +983,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op6alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(vstore8)(op5, op6addr, r3, regs);
 
@@ -1027,7 +1037,7 @@ VADR op4addr,
             op4alet = ARCH_DEP(vfetch4)(effective_addr4 + 68, b4, regs);
             op6alet = ARCH_DEP(vfetch4)(effective_addr4 + 100, b4, regs);
             regs->AR(r3) = op6alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
 
         /* Load address of operand 4 */
@@ -1047,7 +1057,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op4alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(vstore8)(op3, op4addr, r3, regs);
 
@@ -1055,7 +1065,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op6alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(vstore8)(op5, op6addr, r3, regs);
 
@@ -1115,7 +1125,7 @@ VADR op4addr,
             op4alet = ARCH_DEP(vfetch4)(effective_addr4 + 68, b4, regs);
             op6alet = ARCH_DEP(vfetch4)(effective_addr4 + 100, b4, regs);
             regs->AR(r3) = op6alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
 
         /* Load address of operand 4 */
@@ -1135,7 +1145,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op4alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(vstorec)(op3, 16-1, op4addr, r3, regs);
 
@@ -1143,7 +1153,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op6alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(vstorec)(op5, 16-1, op6addr, r3, regs);
 
@@ -1201,7 +1211,7 @@ VADR op4addr,
             op6alet = ARCH_DEP(vfetch4)(effective_addr4 + 100, b4, regs);
             op8alet = ARCH_DEP(vfetch4)(effective_addr4 + 132, b4, regs);
             regs->AR(r3) = op8alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
 
         /* Load address of operand 4 */
@@ -1238,7 +1248,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op6alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(validate_operand) (op6addr, r3, 4-1, ACCTYPE_WRITE_SKP, regs);
 
@@ -1246,7 +1256,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op4alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(vstore4)(op3, op4addr, r3, regs);
 
@@ -1254,7 +1264,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op6alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(vstore4)(op5, op6addr, r3, regs);
 
@@ -1262,7 +1272,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op8alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(vstore4)(op7, op8addr, r3, regs);
 
@@ -1325,7 +1335,7 @@ VADR op4addr,
             op6alet = ARCH_DEP(vfetch4)(effective_addr4 + 100, b4, regs);
             op8alet = ARCH_DEP(vfetch4)(effective_addr4 + 132, b4, regs);
             regs->AR(r3) = op8alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
 
         /* Load address of operand 4 */
@@ -1362,7 +1372,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op6alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(validate_operand) (op6addr, r3, 8-1,ACCTYPE_WRITE_SKP, regs);
 
@@ -1370,7 +1380,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op4alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(vstore8)(op3, op4addr, r3, regs);
 
@@ -1378,7 +1388,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op6alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(vstore8)(op5, op6addr, r3, regs);
 
@@ -1386,7 +1396,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op8alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(vstore8)(op7, op8addr, r3, regs);
 
@@ -1445,7 +1455,7 @@ VADR op4addr,
             op6alet = ARCH_DEP(vfetch4)(effective_addr4 + 100, b4, regs);
             op8alet = ARCH_DEP(vfetch4)(effective_addr4 + 132, b4, regs);
             regs->AR(r3) = op8alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
 
         /* Load address of operand 4 */
@@ -1470,7 +1480,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op6alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(validate_operand) (op6addr, r3, 8-1,ACCTYPE_WRITE_SKP, regs);
 
@@ -1478,7 +1488,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op4alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(vstore8)(op3, op4addr, r3, regs);
 
@@ -1486,7 +1496,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op6alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(vstore8)(op5, op6addr, r3, regs);
 
@@ -1494,7 +1504,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op8alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(vstore8)(op7, op8addr, r3, regs);
 
@@ -1559,7 +1569,7 @@ VADR op4addr,
             op6alet = ARCH_DEP(vfetch4)(effective_addr4 + 100, b4, regs);
             op8alet = ARCH_DEP(vfetch4)(effective_addr4 + 132, b4, regs);
             regs->AR(r3) = op8alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
 
         /* Load address of operand 4 */
@@ -1584,7 +1594,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op6alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(validate_operand) (op6addr, r3, 16-1, ACCTYPE_WRITE_SKP, regs);
 
@@ -1592,7 +1602,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op4alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(vstorec)(op3, 16-1, op4addr, r3, regs);
 
@@ -1600,7 +1610,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op6alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(vstorec)(op5, 16-1, op6addr, r3, regs);
 
@@ -1608,7 +1618,7 @@ VADR op4addr,
         if(!REAL_MODE(&regs->psw) && ACCESS_REGISTER_MODE(&regs->psw))
         {
             regs->AR(r3) = op8alet;
-            INVALIDATE_AEA(r3, regs);
+            SET_AEA_AR(regs, r3);
         }
         ARCH_DEP(vstorec)(op7, 16-1, op8addr, r3, regs);
 
