@@ -1,4 +1,4 @@
-/* PRINTER.C    (c) Copyright Roger Bowler, 1999-2003                */
+/* PRINTER.C    (c) Copyright Roger Bowler, 1999-2005                */
 /*              ESA/390 Line Printer Device Handler                  */
 
 /*-------------------------------------------------------------------*/
@@ -6,17 +6,61 @@
 /* System/370 line printer devices.                                  */
 /*-------------------------------------------------------------------*/
 
+#include "hstdinc.h"
+
 #include "hercules.h"
 
 #include "devtype.h"
 
 #include "opcode.h"
 
+/*-------------------------------------------------------------------*/
+/* Ivan Warren 20040227                                              */
+/* This table is used by channel.c to determine if a CCW code is an  */
+/* immediate command or not                                          */
+/* The tape is addressed in the DEVHND structure as 'DEVIMM immed'   */
+/* 0 : Command is NOT an immediate command                           */
+/* 1 : Command is an immediate command                               */
+/* Note : An immediate command is defined as a command which returns */
+/* CE (channel end) during initialisation (that is, no data is       */
+/* actually transfered. In this case, IL is not indicated for a CCW  */
+/* Format 0 or for a CCW Format 1 when IL Suppression Mode is in     */
+/* effect                                                            */
+/*-------------------------------------------------------------------*/
+
+/* Printer Specific : 1403 */
+/* The following are considered IMMEDIATE commands : */
+/* CTL-NOOP, Skip Channel 'n' Immediate, Block Data check , Allow Data Check
+ * Space 1,2,3 Lines Immediate, UCS Gate Load, Load UCS Buffer & Fold,
+ * Load UCS Buffer (No Fold)
+ */
+
+static BYTE printer_immed_commands[256]=
+/*
+ *0 1 2 3 4 5 6 7 8 9 A B C D E F
+*/
+
+{ 0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,0,
+  0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,0,
+  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+  0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,0,
+  0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,
+  0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,0,
+  0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,0,
+  0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,0,
+  0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,0,
+  0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,0,
+  0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,0,
+  0,0,0,1,0,0,0,0,0,0,0,1,0,0,0,0};
+
 /*-------------------------------------------------------------------*/
 /* Internal macro definitions                                        */
 /*-------------------------------------------------------------------*/
 #define LINE_LENGTH     150
-#define SPACE           ((BYTE)' ')
 
 /*-------------------------------------------------------------------*/
 /* Subroutine to open the printer file or pipe                       */
@@ -24,15 +68,19 @@
 static int
 open_printer (DEVBLK *dev)
 {
-int             fd;                     /* File descriptor           */
-int             rc;                     /* Return code               */
-int             pipefd[2];              /* Pipe descriptors          */
 pid_t           pid;                    /* Child process identifier  */
+BYTE            pathname[MAX_PATH];     /* file path in host format  */
+#if !defined( _MSVC_ )
+int             pipefd[2];              /* Pipe descriptors          */
+int             rc;                     /* Return code               */
+#endif
 
     /* Regular open if 1st char of filename is not vertical bar */
     if (dev->filename[0] != '|')
     {
-        fd = open (dev->filename,
+        int fd;
+        hostpath(pathname, dev->filename, sizeof(pathname));
+        fd = open (pathname, O_BINARY |
                     O_WRONLY | O_CREAT | O_TRUNC /* | O_SYNC */,
                     S_IRUSR | S_IWUSR | S_IRGRP);
         if (fd < 0)
@@ -44,13 +92,34 @@ pid_t           pid;                    /* Child process identifier  */
 
         /* Save file descriptor in device block and return */
         dev->fd = fd;
+        dev->ispiped = 0;
         return 0;
     }
 
     /* Filename is in format |xxx, set up pipe to program xxx */
 
+    dev->ispiped = 1;
+
+#if defined( _MSVC_ )
+
+    /* "Poor man's" fork... */
+    pid = w32_poor_mans_fork ( dev->filename+1, &dev->fd );
+    if (pid < 0)
+    {
+        logmsg (_("HHCPR006E %4.4X device initialization error: fork: %s\n"),
+                dev->devnum, strerror(errno));
+        return -1;
+    }
+
+    /* Log start of child process */
+    logmsg (_("HHCPR007I pipe receiver (pid=%d) starting for %4.4X\n"),
+            pid, dev->devnum);
+    dev->ptpcpid = pid;
+
+#else /* !defined( _MSVC_ ) */
+
     /* Create a pipe */
-    rc = pipe (pipefd);
+    rc = create_pipe (pipefd);
     if (rc < 0)
     {
         logmsg (_("HHCPR005E %4.4X device initialization error: pipe: %s\n"),
@@ -64,6 +133,8 @@ pid_t           pid;                    /* Child process identifier  */
     {
         logmsg (_("HHCPR006E %4.4X device initialization error: fork: %s\n"),
                 dev->devnum, strerror(errno));
+        close_pipe ( pipefd[0] );
+        close_pipe ( pipefd[1] );
         return -1;
     }
 
@@ -75,7 +146,7 @@ pid_t           pid;                    /* Child process identifier  */
                 getpid(), dev->devnum);
 
         /* Close the write end of the pipe */
-        close (pipefd[1]);
+        close_pipe ( pipefd[1] );
 
         /* Duplicate the read end of the pipe onto STDIN */
         if (pipefd[0] != STDIN_FILENO)
@@ -85,56 +156,29 @@ pid_t           pid;                    /* Child process identifier  */
             {
                 logmsg (_("HHCPR008E %4.4X dup2 error: %s\n"),
                         dev->devnum, strerror(errno));
-                close (pipefd[0]);
+                close_pipe ( pipefd[0] );
                 _exit(127);
             }
         } /* end if(pipefd[0] != STDIN_FILENO) */
 
         /* Close the original descriptor now duplicated to STDIN */
-        close (pipefd[0]);
+        close_pipe ( pipefd[0] );
 
-        /* Redirect STDOUT to the control panel message pipe */
-        rc = dup2 (fileno(sysblk.msgpipew), STDOUT_FILENO);
-        if (rc != STDOUT_FILENO)
-        {
-            logmsg (_("HHCPR009E %4.4X dup2 error: %s\n"),
-                    dev->devnum, strerror(errno));
-            _exit(127);
-        }
-
-        /* Redirect STDERR to the control panel message pipe */
-        rc = dup2 (fileno(sysblk.msgpipew), STDERR_FILENO);
-        if (rc != STDERR_FILENO)
-        {
-            logmsg (_("HHCPR010E %4.4X dup2 error: %s\n"),
-                    dev->devnum, strerror(errno));
-            _exit(127);
-        }
+        /* Redirect stderr (screen) to hercules log task */
+        dup2(STDOUT_FILENO, STDERR_FILENO);
 
         /* Relinquish any ROOT authority before calling shell */
         SETMODE(TERM);
 
         /* Execute the specified pipe receiver program */
 
-#if defined(WIN32)
+#if defined(OPTION_FISH_STUPID_GUI_PRTSPLR_EXPERIMENT)
         {
             /* The dev=, pid= and extgui= arguments are for informational
                purposes only so the spooler knows who/what it's spooling. */
-
             BYTE  cmdline[256];
-
-            snprintf(cmdline,256,"\"%s\" pid=%d dev=%4.4X extgui=%d",
-                dev->filename+1,getpid(),dev->devnum,
-#ifdef EXTERNALGUI
-                extgui
-#else /*!EXTERNALGUI*/
-                0
-#endif /*EXTERNALGUI*/
-                );
-
-            fclose(stderr);
-            fclose(sysblk.msgpipew);
-
+            snprintf(cmdline,256,"\"%s\" pid=%d dev=%4.4X",
+                dev->filename+1,getpid(),dev->devnum);
             rc = system (cmdline);
         }
 #else
@@ -162,19 +206,23 @@ pid_t           pid;                    /* Child process identifier  */
     /* The parent process continues as the pipe sender */
 
     /* Close the read end of the pipe */
-    close (pipefd[0]);
+    close_pipe ( pipefd[0] );
 
     /* Save pipe write descriptor in the device block */
     dev->fd = pipefd[1];
+    dev->ptpcpid = pid;
+
+#endif /* defined( _MSVC_ ) */
 
     return 0;
+
 } /* end function open_printer */
 
 /*-------------------------------------------------------------------*/
 /* Subroutine to write data to the printer                           */
 /*-------------------------------------------------------------------*/
 static void
-write_buffer (DEVBLK *dev, BYTE *buf, int len, BYTE *unitstat)
+write_buffer (DEVBLK *dev, char *buf, int len, BYTE *unitstat)
 {
 int             rc;                     /* Return code               */
 
@@ -197,7 +245,7 @@ int             rc;                     /* Return code               */
 /*-------------------------------------------------------------------*/
 /* Initialize the device handler                                     */
 /*-------------------------------------------------------------------*/
-static int printer_init_handler (DEVBLK *dev, int argc, BYTE *argv[])
+static int printer_init_handler (DEVBLK *dev, int argc, char *argv[])
 {
 int     i;                              /* Array subscript           */
 
@@ -205,7 +253,7 @@ int     i;                              /* Array subscript           */
     if (argc == 0 || strlen(argv[0]) > sizeof(dev->filename)-1)
     {
         fprintf (stderr,
-                "HHCPR001E File name missing or invalid for printer %4.4X\n",
+                _("HHCPR001E File name missing or invalid for printer %4.4X\n"),
                  dev->devnum);
         return -1;
     }
@@ -213,6 +261,9 @@ int     i;                              /* Array subscript           */
     /* Save the file name in the device block */
     strcpy (dev->filename, argv[0]);
 
+    if(!sscanf(dev->typname,"%hx",&(dev->devtype)))
+        dev->devtype = 0x1403;
+
     /* Initialize device dependent fields */
     dev->fd = -1;
     dev->printpos = 0;
@@ -231,7 +282,7 @@ int     i;                              /* Array subscript           */
             continue;
         }
 
-        fprintf (stderr, "HHCPR002E Invalid argument for printer %4.4X: %s\n",
+        fprintf (stderr, _("HHCPR002E Invalid argument for printer %4.4X: %s\n"),
                 dev->devnum, argv[i]);
         return -1;
     }
@@ -261,8 +312,8 @@ int     i;                              /* Array subscript           */
 /*-------------------------------------------------------------------*/
 /* Query the device definition                                       */
 /*-------------------------------------------------------------------*/
-static void printer_query_device (DEVBLK *dev, BYTE **class,
-                int buflen, BYTE *buffer)
+static void printer_query_device (DEVBLK *dev, char **class,
+                int buflen, char *buffer)
 {
     *class = "PRT";
     snprintf (buffer, buflen, "%s%s%s",
@@ -277,8 +328,23 @@ static void printer_query_device (DEVBLK *dev, BYTE **class,
 /*-------------------------------------------------------------------*/
 static int printer_close_device ( DEVBLK *dev )
 {
+    if (dev->fd < 0) return 0;  /* (nothing to close!) */
+
     /* Close the device file */
-    close (dev->fd);
+    if ( dev->ispiped )
+    {
+#if !defined( _MSVC_ )
+        close_pipe ( dev->fd );
+#else /* defined( _MSVC_ ) */
+        close (dev->fd);
+        /* Log end of child process */
+        logmsg (_("HHCPR011I pipe receiver (pid=%d) terminating for %4.4X\n"),
+                dev->ptpcpid, dev->devnum);
+#endif /* defined( _MSVC_ ) */
+        dev->ptpcpid = 0;
+    }
+    else
+        close (dev->fd);
     dev->fd = -1;
     dev->stopprt = 0;
 
@@ -410,11 +476,11 @@ BYTE            c;                      /* Print character           */
                 if (dev->buf[i-1] != SPACE) break;
 
             /* Append carriage return and line feed(s) */
-            strcpy (dev->buf + i, eor);
+            strcpy ((char *)(dev->buf + i), eor);
             i += strlen(eor);
 
             /* Write print line */
-            write_buffer (dev, dev->buf, i, unitstat);
+            write_buffer (dev, (char *)dev->buf, i, unitstat);
             if (*unitstat != 0) break;
 
         } /* end if(!data-chaining) */
@@ -507,6 +573,9 @@ BYTE            c;                      /* Print character           */
         write_buffer (dev, eor, strlen(eor), unitstat);
         if (*unitstat != 0) break;
 
+    /*
+        *residual = 0;
+    */
         *unitstat = CSW_CE | CSW_DE;
         break;
 
@@ -518,6 +587,9 @@ BYTE            c;                      /* Print character           */
         write_buffer (dev, eor, strlen(eor), unitstat);
         if (*unitstat != 0) break;
 
+    /*
+        *residual = 0;
+    */
         *unitstat = CSW_CE | CSW_DE;
         break;
 
@@ -529,6 +601,9 @@ BYTE            c;                      /* Print character           */
         write_buffer (dev, eor, strlen(eor), unitstat);
         if (*unitstat != 0) break;
 
+    /*
+        *residual = 0;
+    */
         *unitstat = CSW_CE | CSW_DE;
         break;
 
@@ -552,6 +627,9 @@ BYTE            c;                      /* Print character           */
     /*---------------------------------------------------------------*/
     /* BLOCK DATA CHECK                                              */
     /*---------------------------------------------------------------*/
+    /*
+        *residual = 0;
+    */
         *unitstat = CSW_CE | CSW_DE;
         break;
 
@@ -559,6 +637,9 @@ BYTE            c;                      /* Print character           */
     /*---------------------------------------------------------------*/
     /* ALLOW DATA CHECK                                              */
     /*---------------------------------------------------------------*/
+    /*
+        *residual = 0;
+    */
         *unitstat = CSW_CE | CSW_DE;
         break;
 
@@ -570,6 +651,9 @@ BYTE            c;                      /* Print character           */
         write_buffer (dev, eor, strlen(eor), unitstat);
         if (*unitstat != 0) break;
 
+    /*
+        *residual = 0;
+    */
         *unitstat = CSW_CE | CSW_DE;
         break;
 
@@ -581,17 +665,23 @@ BYTE            c;                      /* Print character           */
         write_buffer (dev, eor, strlen(eor), unitstat);
         if (*unitstat != 0) break;
 
+    /*
+        *residual = 0;
+    */
         *unitstat = CSW_CE | CSW_DE;
         break;
 
-    case 0xE3:
+    case 0xE3: case 0xDB:
     /*---------------------------------------------------------------*/
-    /* SKIP TO CHANNEL 12 IMMEDIATE                                  */
+    /* SKIP TO CHANNEL 12 IMMEDIATE (or 11)                          */
     /*---------------------------------------------------------------*/
         eor = dev->crlf ? "\r\n" : "\n";
         write_buffer (dev, eor, strlen(eor), unitstat);
         if (*unitstat != 0) break;
 
+    /*
+        *residual = 0;
+    */
         *unitstat = CSW_CE | CSW_DE;
         break;
 
@@ -634,7 +724,9 @@ BYTE            c;                      /* Print character           */
 
         /* Set fold indicator and return normal status */
         dev->fold = 1;
+    /*
         *residual = 0;
+    */
         *unitstat = CSW_CE | CSW_DE;
         break;
 
@@ -652,7 +744,9 @@ BYTE            c;                      /* Print character           */
 
         /* Reset fold indicator and return normal status */
         dev->fold = 0;
+    /*
         *residual = 0;
+    */
         *unitstat = CSW_CE | CSW_DE;
         break;
 
@@ -704,10 +798,53 @@ BYTE            c;                      /* Print character           */
 } /* end function printer_execute_ccw */
 
 
+#if defined(OPTION_DYNAMIC_LOAD)
+static
+#endif
 DEVHND printer_device_hndinfo = {
-        &printer_init_handler,
-        &printer_execute_ccw,
-        &printer_close_device,
-        &printer_query_device,
-        NULL, NULL, NULL, NULL
+        &printer_init_handler,         /* Device Initialisation      */
+        &printer_execute_ccw,          /* Device CCW execute         */
+        &printer_close_device,         /* Device Close               */
+        &printer_query_device,         /* Device Query               */
+        NULL,                          /* Device Start channel pgm   */
+        NULL,                          /* Device End channel pgm     */
+        NULL,                          /* Device Resume channel pgm  */
+        NULL,                          /* Device Suspend channel pgm */
+        NULL,                          /* Device Read                */
+        NULL,                          /* Device Write               */
+        NULL,                          /* Device Query used          */
+        NULL,                          /* Device Reserve             */
+        NULL,                          /* Device Release             */
+        printer_immed_commands,        /* Immediate CCW Codes        */
+        NULL,                          /* Signal Adapter Input       */
+        NULL,                          /* Signal Adapter Output      */
+        NULL,                          /* Hercules suspend           */
+        NULL                           /* Hercules resume            */
 };
+
+/* Libtool static name colision resolution */
+/* note : lt_dlopen will look for symbol & modulename_LTX_symbol */
+#if !defined(HDL_BUILD_SHARED) && defined(HDL_USE_LIBTOOL)
+#define hdl_ddev hdt1403_LTX_hdl_ddev
+#define hdl_depc hdt1403_LTX_hdl_depc
+#define hdl_reso hdt1403_LTX_hdl_reso
+#define hdl_init hdt1403_LTX_hdl_init
+#define hdl_fini hdt1403_LTX_hdl_fini
+#endif
+
+#if defined(OPTION_DYNAMIC_LOAD)
+HDL_DEPENDENCY_SECTION;
+{
+     HDL_DEPENDENCY(HERCULES);
+     HDL_DEPENDENCY(DEVBLK);
+}
+END_DEPENDENCY_SECTION;
+
+
+HDL_DEVICE_SECTION;
+{
+    HDL_DEVICE(1403, printer_device_hndinfo );
+    HDL_DEVICE(3211, printer_device_hndinfo );
+}
+END_DEVICE_SECTION;
+#endif
