@@ -1,4 +1,4 @@
-/* SIE.C        (c) Copyright Jan Jaeger, 1999-2003                  */
+/* SIE.C        (c) Copyright Jan Jaeger, 1999-2005                  */
 /*              Interpretive Execution                               */
 
 /*      This module contains the SIE instruction as                  */
@@ -8,9 +8,20 @@
 /*      Enterprise Systems Architecture / Extended Configuration     */
 /*      Principles of Operation, SC24-5594-02 and SC24-5965-00       */
 
-/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2003      */
+/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2005      */
+
+#include "hstdinc.h"
 
 // #define SIE_DEBUG
+//
+
+#if !defined(_HENGINE_DLL_)
+#define _HENGINE_DLL_
+#endif
+
+#if !defined(_SIE_C_)
+#define _SIE_C_
+#endif
 
 #include "hercules.h"
 
@@ -43,7 +54,7 @@ static int (* run_sie[GEN_MAXARCH]) (REGS *regs) =
     };
 
 #define GUESTREGS (regs->guestregs)
-#define STATEBK   ((SIEBK*)(GUESTREGS->siebk))
+#define STATEBK   ((SIEBK *)GUESTREGS->siebk)
 
 #define SIE_I_STOP(_guestregs) \
         ((_guestregs)->siebk->v & SIE_V_STOP)
@@ -54,15 +65,142 @@ static int (* run_sie[GEN_MAXARCH]) (REGS *regs) =
 
 #define SIE_I_HOST(_hostregs) IC_INTERRUPT_CPU(_hostregs)
 
+#if defined(SIE_DEBUG_PERFMON)
+#define SIE_PERF_MAXNEG 0x1F
+static int sie_perfmon[0x41 + SIE_PERF_MAXNEG];
+#define SIE_PERFMON(_code) \
+    do { \
+        sie_perfmon[(_code) + SIE_PERF_MAXNEG] ++; \
+    } while(0)
+#define SIE_PERF_PGMINT \
+    (code <= 0 ? code : (((code-1) & 0x3F)+1))
+void *sie_perfmon_disp()
+{
+static char *dbg_name[] = {
+        /* -31 */       "SIE re-dispatch state descriptor",
+        /* -30 */       "SIE exit",
+        /* -29 */       "SIE run",
+        /* -28 */       "SIE runloop 1",
+        /* -27 */       "SIE runloop 2",
+        /* -26 */       "SIE interrupt check",
+        /* -25 */       "SIE execute instruction",
+        /* -24 */       "SIE unrolled execute",
+        /* -23 */       NULL,
+        /* -22 */       NULL,
+        /* -21 */       NULL,
+        /* -20 */       NULL,
+        /* -19 */       NULL,
+        /* -18 */       NULL,
+        /* -17 */       "SIE intercept I/O instruction",
+        /* -16 */       "SIE intercept I/O interrupt pending",
+        /* -15 */       "SIE intercept I/O interrupt",
+        /* -14 */       "SIE intercept PER interrupt",
+        /* -13 */       "SIE validity check",
+        /* -12 */       "SIE intercept external interrupt pending",
+        /* -11 */       "SIE intercept machine check interrupt",
+        /* -10 */       "SIE intercept restart interrupt",
+        /* -9 */        "SIE intercept stop request",
+        /* -8 */        "SIE intercept virtual machine wait",
+        /* -7 */        "SIE intercept I/O interrupt request",
+        /* -6 */        "SIE intercept external interrupt request",
+        /* -5 */        "SIE intercept instruction completion",
+        /* -4 */        "SIE intercept instruction",
+        /* -3 */        "SIE host program interrupt",
+        /* -2 */        "SIE host interrupt",
+        /* -1 */        "SIE no intercept",
+        /*  0 */        "SIE entry",
+        /* 01 */        "SIE intercept Operation exception",
+        /* 02 */        "SIE intercept Privileged-operation exception",
+        /* 03 */        "SIE intercept Execute exception",
+        /* 04 */        "SIE intercept Protection exception",
+        /* 05 */        "SIE intercept Addressing exception",
+        /* 06 */        "SIE intercept Specification exception",
+        /* 07 */        "SIE intercept Data exception",
+        /* 08 */        "SIE intercept Fixed-point-overflow exception",
+        /* 09 */        "SIE intercept Fixed-point-divide exception",
+        /* 0A */        "SIE intercept Decimal-overflow exception",
+        /* 0B */        "SIE intercept Decimal-divide exception",
+        /* 0C */        "SIE intercept HFP-exponent-overflow exception",
+        /* 0D */        "SIE intercept HFP-exponent-underflow exception",
+        /* 0E */        "SIE intercept HFP-significance exception",
+        /* 0F */        "SIE intercept HFP-floating-point-divide exception",
+        /* 10 */        "SIE intercept Segment-translation exception",
+        /* 11 */        "SIE intercept Page-translation exception",
+        /* 12 */        "SIE intercept Translation-specification exception",
+        /* 13 */        "SIE intercept Special-operation exception",
+        /* 14 */        "SIE intercept Pseudo-page-fault exception",
+        /* 15 */        "SIE intercept Operand exception",
+        /* 16 */        "SIE intercept Trace-table exception",
+        /* 17 */        "SIE intercept ASN-translation exception",
+        /* 18 */        "SIE intercept Page access exception",
+        /* 19 */        "SIE intercept Vector/Crypto operation exception",
+        /* 1A */        "SIE intercept Page state exception",
+        /* 1B */        "SIE intercept Page transition exception",
+        /* 1C */        "SIE intercept Space-switch event",
+        /* 1D */        "SIE intercept Square-root exception",
+        /* 1E */        "SIE intercept Unnormalized-operand exception",
+        /* 1F */        "SIE intercept PC-translation specification exception",
+        /* 20 */        "SIE intercept AFX-translation exception",
+        /* 21 */        "SIE intercept ASX-translation exception",
+        /* 22 */        "SIE intercept LX-translation exception",
+        /* 23 */        "SIE intercept EX-translation exception",
+        /* 24 */        "SIE intercept Primary-authority exception",
+        /* 25 */        "SIE intercept LFX-translation exception",
+        /* 26 */        "SIE intercept LSX-translation exception",
+        /* 27 */        "SIE intercept Control-switch exception",
+        /* 28 */        "SIE intercept ALET-specification exception",
+        /* 29 */        "SIE intercept ALEN-translation exception",
+        /* 2A */        "SIE intercept ALE-sequence exception",
+        /* 2B */        "SIE intercept ASTE-validity exception",
+        /* 2C */        "SIE intercept ASTE-sequence exception",
+        /* 2D */        "SIE intercept Extended-authority exception",
+        /* 2E */        "SIE intercept LSTE-sequence exception",
+        /* 2F */        "SIE intercept ASTE-instance exception",
+        /* 30 */        "SIE intercept Stack-full exception",
+        /* 31 */        "SIE intercept Stack-empty exception",
+        /* 32 */        "SIE intercept Stack-specification exception",
+        /* 33 */        "SIE intercept Stack-type exception",
+        /* 34 */        "SIE intercept Stack-operation exception",
+        /* 35 */        NULL,
+        /* 36 */        NULL,
+        /* 37 */        NULL,
+        /* 38 */        "SIE intercept ASCE-type exception",
+        /* 39 */        "SIE intercept Region-first-translation exception",
+        /* 3A */        "SIE intercept Region-second-translation exception",
+        /* 3B */        "SIE intercept Region-third-translation exception",
+        /* 3C */        NULL,
+        /* 3D */        NULL,
+        /* 3E */        NULL,
+        /* 3F */        NULL,
+        /* 40 */        "SIE intercept Monitor event" };
+
+    if(sie_perfmon[SIE_PERF_ENTER+SIE_PERF_MAXNEG])
+    {
+        int i;
+        for(i = 0; i < 0x61; i++)
+            if(sie_perfmon[i])
+                logmsg("%9u: %s\n",sie_perfmon[i],dbg_name[i]);
+        logmsg("%9u: Average instructions/SIE invocation\n",
+            (sie_perfmon[SIE_PERF_EXEC+SIE_PERF_MAXNEG] +
+             sie_perfmon[SIE_PERF_EXEC_U+SIE_PERF_MAXNEG]*7) /
+            sie_perfmon[SIE_PERF_ENTER+SIE_PERF_MAXNEG]);
+    }
+    else
+        logmsg("No SIE performance data\n");
+}
+#else
+#define SIE_PERFMON(_code)
+#endif
+
 #endif /*!defined(_SIE_C)*/
 
 #undef SIE_I_WAIT
 #if defined(_FEATURE_WAITSTATE_ASSIST)
 #define SIE_I_WAIT(_guestregs) \
-        ((_guestregs)->psw.wait && !((_guestregs)->siebk->ec[0] & SIE_EC0_WAIA))
+        (WAITSTATE(&(_guestregs)->psw) && !((_guestregs)->siebk->ec[0] & SIE_EC0_WAIA))
 #else
 #define SIE_I_WAIT(_guestregs) \
-        ((_guestregs)->psw.wait)
+        (WAITSTATE(&(_guestregs)->psw))
 #endif
 
 #undef SIE_I_IO
@@ -70,7 +208,7 @@ static int (* run_sie[GEN_MAXARCH]) (REGS *regs) =
 #define SIE_I_IO(_guestregs) \
         (((_guestregs)->siebk->v & SIE_V_IO) \
            && ((_guestregs)->psw.sysmask \
-                 & ((_guestregs)->psw.ecmode ? PSW_IOMASK : 0xFE) ))
+                 & (ECMODE(&(_guestregs)->psw) ? PSW_IOMASK : 0xFE) ))
 #else /*!defined(FEATURE_BCMODE)*/
 #define SIE_I_IO(_guestregs) \
         (((_guestregs)->siebk->v & SIE_V_IO) \
@@ -91,16 +229,18 @@ int     n;                              /* Loop counter              */
 U16     lhcpu;                          /* Last Host CPU address     */
 int     icode = 0;                      /* Interception code         */
 
-    S(inst, execflag, regs, b2, effective_addr2);
-
-    SIE_MODE_XC_OPEX(regs);
-
-    PRIV_CHECK(regs);
+    S(inst, regs, b2, effective_addr2);
 
     SIE_INTERCEPT(regs);
 
+    PRIV_CHECK(regs);
+
+    SIE_PERFMON(SIE_PERF_ENTER);
+
+#if !defined(FEATURE_ESAME) && !defined(FEATURE_MULTIPLE_CONTROLLED_DATA_SPACE)
     if(!regs->psw.amode || !PRIMARY_SPACE_MODE(&(regs->psw)))
         ARCH_DEP(program_interrupt) (regs, PGM_SPECIAL_OPERATION_EXCEPTION);
+#endif
 
     if((effective_addr2 & (sizeof(SIEBK)-1)) != 0
 #if defined(FEATURE_ESAME)
@@ -124,20 +264,46 @@ int     icode = 0;                      /* Interception code         */
     if(effective_addr2 > regs->mainlim - (sizeof(SIEBK)-1))
     ARCH_DEP(program_interrupt) (regs, PGM_ADDRESSING_EXCEPTION);
 
-    /* Direct pointer to state descriptor block */
-    STATEBK = (void*)(regs->mainstor + effective_addr2);
+    /*
+     * As long as regs->sie_active is off, no serialization is
+     * required for GUESTREGS.  sie_active should always be off here.
+     * Any other thread looking at sie_active holds the intlock.
+     */
+    if (regs->sie_active)
+    {
+        obtain_lock(&sysblk.intlock);
+        regs->sie_active = 0;
+        release_lock(&sysblk.intlock);
+    }
 
-    obtain_lock(&sysblk.intlock);
+     /* Initialize guestregs if first time */
+     if (GUESTREGS == NULL)
+     {
+        GUESTREGS = calloc (sizeof(REGS), 1);
+        if (GUESTREGS == NULL)
+         {
+             logmsg (_("HHCCP079E CPU%4.4X: calloc failed for sie regs: %s\n"),
+                     regs->cpuad, strerror(errno));
+#if !defined(NO_SIGABEND_HANDLER)
+             signal_thread(sysblk.cputid[regs->cpuad], SIGUSR1);
+#endif
+             return;
+         }
+        cpu_init (regs->cpuad, GUESTREGS, regs);
+     }
+
+    /* Direct pointer to state descriptor block */
+    GUESTREGS->siebk = (void*)(regs->mainstor + effective_addr2);
 
 #if defined(FEATURE_ESAME)
-    if(STATEBK->mx & SIE_MX_ESAME)
+    if (STATEBK->mx & SIE_MX_ESAME)
     {
         GUESTREGS->arch_mode = ARCH_900;
         GUESTREGS->sie_guestpi = (SIEFN)&z900_program_interrupt;
         icode = z900_load_psw(GUESTREGS, STATEBK->psw);
     }
 #else /*!defined(FEATURE_ESAME)*/
-    if(STATEBK->m & SIE_M_370)
+    if (STATEBK->m & SIE_M_370)
     {
 #if defined(_370)
         GUESTREGS->arch_mode = ARCH_370;
@@ -148,14 +314,13 @@ int     icode = 0;                      /* Interception code         */
         SIE_SET_VI(SIE_VI_WHO_CPU, SIE_VI_WHEN_SIENT,
           SIE_VI_WHY_370NI, GUESTREGS);
         STATEBK->c = SIE_C_VALIDITY;
-        release_lock(&sysblk.intlock);
         return;
 #endif
     }
 #endif /*!defined(FEATURE_ESAME)*/
     else
 #if !defined(FEATURE_ESAME)
-    if(STATEBK->m & SIE_M_XA)
+    if (STATEBK->m & SIE_M_XA)
 #endif /*!defined(FEATURE_ESAME)*/
     {
         GUESTREGS->arch_mode = ARCH_390;
@@ -167,21 +332,12 @@ int     icode = 0;                      /* Interception code         */
     {
         /* Validity intercept for invalid mode */
         SIE_SET_VI(SIE_VI_WHO_CPU, SIE_VI_WHEN_SIENT,
-          SIE_VI_WHY_MODE, GUESTREGS);
+                   SIE_VI_WHY_MODE, GUESTREGS);
         STATEBK->c = SIE_C_VALIDITY;
-        release_lock(&sysblk.intlock);
         return;
     }
 #endif /*!defined(FEATURE_ESAME)*/
 
-    release_lock(&sysblk.intlock);
-
-#if defined(OPTION_REDUCED_INVAL)
-    INVALIDATE_AIA(GUESTREGS);
-
-    INVALIDATE_AEA_ALL(GUESTREGS);
-#endif
-
     /* Set host program interrupt routine */
     GUESTREGS->sie_hostpi = (SIEFN)&ARCH_DEP(program_interrupt);
 
@@ -201,7 +357,7 @@ int     icode = 0;                      /* Interception code         */
     if(STATEBK->mx & SIE_MX_RRF)
     {
     RADR mso, msl, eso = 0, esl = 0;
-    
+
         if(STATEBK->zone >= FEATURE_SIE_MAXZONES)
         {
             /* Validity intercept for invalid zone */
@@ -210,7 +366,7 @@ int     icode = 0;                      /* Interception code         */
             STATEBK->c = SIE_C_VALIDITY;
             return;
         }
-       
+
         mso = sysblk.zpb[STATEBK->zone].mso << 20;
         msl = (sysblk.zpb[STATEBK->zone].msl << 20) | 0xFFFFF;
         eso = sysblk.zpb[STATEBK->zone].eso << 20;
@@ -224,7 +380,7 @@ int     icode = 0;                      /* Interception code         */
             STATEBK->c = SIE_C_VALIDITY;
             return;
         }
-       
+
         /* Ensure addressing exceptions on incorrect zone defs */
         if(mso > regs->mainlim || msl > regs->mainlim)
             mso = msl = 0;
@@ -255,7 +411,7 @@ int     icode = 0;                      /* Interception code         */
         {
             /* Validity intercept for invalid zone */
             SIE_SET_VI(SIE_VI_WHO_CPU, SIE_VI_WHEN_SIENT,
-              SIE_VI_WHY_AZNNZ, GUESTREGS);
+                       SIE_VI_WHY_AZNNZ, GUESTREGS);
             STATEBK->c = SIE_C_VALIDITY;
             return;
         }
@@ -272,7 +428,7 @@ int     icode = 0;                      /* Interception code         */
         if(GUESTREGS->sie_mso > GUESTREGS->mainlim)
         {
             SIE_SET_VI(SIE_VI_WHO_CPU, SIE_VI_WHEN_SIENT,
-              SIE_VI_WHY_MSDEF, GUESTREGS);
+                       SIE_VI_WHY_MSDEF, GUESTREGS);
             STATEBK->c = SIE_C_VALIDITY;
             return;
         }
@@ -284,7 +440,7 @@ int     icode = 0;                      /* Interception code         */
         /* Load main storage origin */
         FETCH_HW(GUESTREGS->sie_mso,STATEBK->mso);
         GUESTREGS->sie_mso <<= 16;
-    
+
         /* Load main storage extend */
         FETCH_HW(GUESTREGS->mainlim,STATEBK->mse);
         GUESTREGS->mainlim = ((GUESTREGS->mainlim + 1) << 16) - 1;
@@ -295,7 +451,7 @@ int     icode = 0;                      /* Interception code         */
                            | STATEBK->xso[1] << 8
                            | STATEBK->xso[2];
         GUESTREGS->sie_xso *= (XSTORE_INCREMENT_SIZE >> XSTORE_PAGESHIFT);
-    
+
         /* Load expanded storage limit */
         GUESTREGS->sie_xsl = STATEBK->xsl[0] << 16
                            | STATEBK->xsl[1] << 8
@@ -307,7 +463,7 @@ int     icode = 0;                      /* Interception code         */
     if(GUESTREGS->PX > GUESTREGS->mainlim)
     {
         SIE_SET_VI(SIE_VI_WHO_CPU, SIE_VI_WHEN_SIENT,
-          SIE_VI_WHY_PFOUT, GUESTREGS);
+                   SIE_VI_WHY_PFOUT, GUESTREGS);
         STATEBK->c = SIE_C_VALIDITY;
         return;
     }
@@ -317,28 +473,28 @@ int     icode = 0;                      /* Interception code         */
     if(GUESTREGS->sie_scao > regs->mainlim)
     {
         SIE_SET_VI(SIE_VI_WHO_CPU, SIE_VI_WHEN_SIENT,
-          SIE_VI_WHY_SCADR, GUESTREGS);
+                   SIE_VI_WHY_SCADR, GUESTREGS);
         STATEBK->c = SIE_C_VALIDITY;
         return;
     }
 
     /* Validate MSO */
-    if(GUESTREGS->sie_mso)
+    if (GUESTREGS->sie_mso)
     {
         /* Preferred guest must have zero MSO */
-        if(GUESTREGS->sie_pref)
+        if (GUESTREGS->sie_pref)
         {
             SIE_SET_VI(SIE_VI_WHO_CPU, SIE_VI_WHEN_SIENT,
               SIE_VI_WHY_MSONZ, GUESTREGS);
-            STATEBK->c = SIE_C_VALIDITY; 
+            STATEBK->c = SIE_C_VALIDITY;
             return;
         }
 
         /* MCDS guest must have zero MSO */
-        if(STATEBK->mx & SIE_MX_XC)
+        if (STATEBK->mx & SIE_MX_XC)
         {
             SIE_SET_VI(SIE_VI_WHO_CPU, SIE_VI_WHEN_SIENT,
-              SIE_VI_WHY_MSODS, GUESTREGS);
+                       SIE_VI_WHY_MSODS, GUESTREGS);
             STATEBK->c = SIE_C_VALIDITY;
             return;
         }
@@ -347,10 +503,10 @@ int     icode = 0;                      /* Interception code         */
 #if !defined(FEATURE_ESAME)
     /* Reference and Change Preservation Origin */
     FETCH_FW(GUESTREGS->sie_rcpo, STATEBK->rcpo);
-    if(!GUESTREGS->sie_rcpo && !GUESTREGS->sie_pref)
+    if (!GUESTREGS->sie_rcpo && !GUESTREGS->sie_pref)
     {
         SIE_SET_VI(SIE_VI_WHO_CPU, SIE_VI_WHEN_SIENT,
-          SIE_VI_WHY_RCZER, GUESTREGS);
+                   SIE_VI_WHY_RCZER, GUESTREGS);
         STATEBK->c = SIE_C_VALIDITY;
         return;
     }
@@ -359,34 +515,21 @@ int     icode = 0;                      /* Interception code         */
     /* Load the CPU timer */
     FETCH_DW(GUESTREGS->ptimer, STATEBK->cputimer);
 
-    /* Reset the CPU timer pending flag according to its value */
-    if( (S64)GUESTREGS->ptimer < 0 )
-        ON_IC_PTIMER(GUESTREGS);
-    else
-        OFF_IC_PTIMER(GUESTREGS);
-
     /* Load the TOD clock offset for this guest */
     FETCH_DW(GUESTREGS->sie_epoch, STATEBK->epoch);
-    GUESTREGS->todoffset = regs->todoffset + (GUESTREGS->sie_epoch >> 8);
+    GUESTREGS->tod_epoch = regs->tod_epoch + (GUESTREGS->sie_epoch >> 8);
 
     /* Load the clock comparator */
     FETCH_DW(GUESTREGS->clkc, STATEBK->clockcomp);
     GUESTREGS->clkc >>= 8; /* Internal Hercules format */
 
-    /* Reset the clock comparator pending flag according to
-       the setting of the TOD clock */
-    if( (sysblk.todclk + GUESTREGS->todoffset) > GUESTREGS->clkc )
-        ON_IC_CLKC(GUESTREGS);
-    else
-        OFF_IC_CLKC(GUESTREGS);
-
     /* Load TOD Programmable Field */
     FETCH_HW(GUESTREGS->todpr, STATEBK->todpf);
 
     /* Load the guest registers */
-    memcpy(GUESTREGS->gr, regs->gr, 14 * sizeof(U64));
-    memcpy(GUESTREGS->ar, regs->ar, 16 * sizeof(U32));
-    memcpy(GUESTREGS->fpr, regs->fpr, 32 * sizeof(U32));
+    memcpy(GUESTREGS->gr, regs->gr, 14 * sizeof(regs->gr[0]));
+    memcpy(GUESTREGS->ar, regs->ar, 16 * sizeof(regs->ar[0]));
+    memcpy(GUESTREGS->fpr, regs->fpr, 32 * sizeof(regs->fpr[0]));
 #if defined(FEATURE_BINARY_FLOATING_POINT)
     GUESTREGS->fpc =  regs->fpc;
 #endif /*defined(FEATURE_BINARY_FLOATING_POINT)*/
@@ -402,14 +545,17 @@ int     icode = 0;                      /* Interception code         */
         FETCH_W(GUESTREGS->CR(n), STATEBK->cr[n]);
 
     FETCH_HW(lhcpu, STATEBK->lhcpu);
-
-    /* If this is not the last host cpu that dispatched this state
-       descriptor then clear the guest TLB entries */
-         if((regs->cpuad != lhcpu)
-           || (GUESTREGS->sie_state != effective_addr2))
+    /*
+     * If this is not the last host cpu that dispatched this state
+     * descriptor then clear the guest TLB entries
+     */
+    if (regs->cpuad != lhcpu
+     || SIE_STATE(GUESTREGS) != effective_addr2)
     {
+        SIE_PERFMON(SIE_PERF_ENTER_F);
+
         /* Absolute address of state descriptor block */
-        GUESTREGS->sie_state = effective_addr2;
+        SIE_STATE(GUESTREGS) = effective_addr2;
 
         /* Update Last Host CPU address */
         STORE_HW(STATEBK->lhcpu, regs->cpuad);
@@ -419,102 +565,142 @@ int     icode = 0;                      /* Interception code         */
         ARCH_DEP(purge_alb) (GUESTREGS);
     }
 
-    /* Initialise the last fetched instruction pointer */
-    GUESTREGS->ip = GUESTREGS->inst;
-    GUESTREGS->instvalid = 0;
+    /* Initialize interrupt mask and state */
+    SET_IC_MASK(GUESTREGS);
+    SET_IC_INITIAL_STATE(GUESTREGS);
 
+    /* Initialize accelerated address lookup values */
+    SET_AEA_MODE(GUESTREGS);
+    SET_AEA_COMMON(GUESTREGS);
+
+    /* Force instfetch() to be called for the first instruction */
+    GUESTREGS->instvalid = 0;
+    GUESTREGS->ip = GUESTREGS->inst;
+    GUESTREGS->inst[0] = 0;
+    INVALIDATE_AIA(GUESTREGS);
+
+    /*
+     * Do setjmp(progjmp) because translate_addr() may result in
+     * longjmp(progjmp) for addressing exceptions.
+     */
     if(!setjmp(GUESTREGS->progjmp))
     {
-        /* Set SIE active */
+        /*
+         * Set sie_active to 1.  This means other threads may now
+         * access guestregs when holding intlock.
+         * This is the *only* place sie_active is set to one.
+         */
+        obtain_lock(&sysblk.intlock);
         regs->sie_active = 1;
+        release_lock(&sysblk.intlock);
 
         /* Get PSA pointer and ensure PSA is paged in */
         if(GUESTREGS->sie_pref)
+        {
             GUESTREGS->sie_psa = (PSA_3XX*)(GUESTREGS->mainstor + GUESTREGS->PX);
+            GUESTREGS->sie_px = GUESTREGS->PX;
+        }
         else
         {
-        U16 sie_xcode;
-        int sie_private,
-            sie_protect = 0,
-            sie_stid;
-        RADR sie_px;
-    
             if (ARCH_DEP(translate_addr) (GUESTREGS->sie_mso + GUESTREGS->PX,
-                    USE_PRIMARY_SPACE, regs, ACCTYPE_SIE, &sie_px,
-                    &sie_xcode, &sie_private, &sie_protect, &sie_stid))
+                                          USE_PRIMARY_SPACE, regs, ACCTYPE_SIE))
             {
                 SIE_SET_VI(SIE_VI_WHO_CPU, SIE_VI_WHEN_SIENT,
-                  SIE_VI_WHY_PFACC, GUESTREGS);
+                           SIE_VI_WHY_PFACC, GUESTREGS);
                 STATEBK->c = SIE_C_VALIDITY;
+                obtain_lock(&sysblk.intlock);
                 regs->sie_active = 0;
+                release_lock(&sysblk.intlock);
                 return;
             }
-    
+
             /* Convert host real address to host absolute address */
-            sie_px = APPLY_PREFIXING (sie_px, regs->PX);
-            
-            if (sie_protect || sie_px > regs->mainlim)
+            GUESTREGS->sie_px = APPLY_PREFIXING (regs->dat.raddr, regs->PX);
+
+            if (regs->dat.protect || GUESTREGS->sie_px > regs->mainlim)
             {
                 SIE_SET_VI(SIE_VI_WHO_CPU, SIE_VI_WHEN_SIENT,
-                  SIE_VI_WHY_PFACC, GUESTREGS);
+                           SIE_VI_WHY_PFACC, GUESTREGS);
                 STATEBK->c = SIE_C_VALIDITY;
+                obtain_lock(&sysblk.intlock);
                 regs->sie_active = 0;
+                release_lock(&sysblk.intlock);
                 return;
             }
-    
-            GUESTREGS->sie_psa = (PSA_3XX*)(GUESTREGS->mainstor + sie_px);
-    
+            GUESTREGS->sie_psa = (PSA_3XX*)(GUESTREGS->mainstor + GUESTREGS->sie_px);
         }
-    
-#if !defined(FEATURE_ESAME)
-        /* If this is a S/370 guest, and the interval timer is enabled
-           then initialize the timer */
-        if( (STATEBK->m & SIE_M_370)
-         && !(STATEBK->m & SIE_M_ITMOF))
+
+        /* Intialize guest timers */
+        obtain_lock(&sysblk.todlock);
+
+        /* CPU timer */
+        if( (S64)GUESTREGS->ptimer < 0 )
         {
-        S32 itimer,
-            olditimer;
-        U32 residue;
-    
-            obtain_lock(&sysblk.todlock);
-    
+            obtain_lock(&sysblk.intlock);
+            ON_IC_PTIMER(GUESTREGS);
+            release_lock(&sysblk.intlock);
+        }
+
+        /* Clock comparator */
+        if( TOD_CLOCK(GUESTREGS) > GUESTREGS->clkc )
+        {
+            obtain_lock(&sysblk.intlock);
+            ON_IC_CLKC(GUESTREGS);
+            release_lock(&sysblk.intlock);
+        }
+
+#if !defined(FEATURE_ESAME)
+        /* Interval timer if S/370 and timer is enabled */
+        if((STATEBK->m & SIE_M_370) && !(STATEBK->m & SIE_M_ITMOF))
+        {
+            S32 itimer, olditimer;
+            U32 residue;
+
             /* Set the interval timer pending according to the T bit
                in the state control */
             if(STATEBK->s & SIE_S_T)
+            {
+                obtain_lock(&sysblk.intlock);
                 ON_IC_ITIMER(GUESTREGS);
-            else
-                OFF_IC_ITIMER(GUESTREGS);
-    
+                release_lock(&sysblk.intlock);
+            }
+
             /* Fetch the residu from the state descriptor */
             FETCH_FW(residue,STATEBK->residue);
-    
+
             /* Fetch the timer value from location 80 */
             FETCH_FW(olditimer,GUESTREGS->sie_psa->inttimer);
-    
-            /* Bit position 23 of the interval timer is deremented 
-               once for each multiple of 3,333 usecs containded in 
+
+            /* Bit position 23 of the interval timer is decremented
+               once for each multiple of 3,333 usecs containded in
                bit position 0-19 of the residue counter */
             itimer = olditimer - ((residue / 3333) >> 4);
-    
+
             /* Store the timer back */
             STORE_FW(GUESTREGS->sie_psa->inttimer, itimer);
-    
-            release_lock(&sysblk.todlock);
-    
+
             /* Set interrupt flag and interval timer interrupt pending
                if the interval timer went from positive to negative */
             if (itimer < 0 && olditimer >= 0)
+            {
+                obtain_lock(&sysblk.intlock);
                 ON_IC_ITIMER(GUESTREGS);
+                release_lock(&sysblk.intlock);
+            }
         }
 #endif /*!defined(FEATURE_ESAME)*/
-    
-        /* Early exceptions associated with the guest PSW */
+
+        /* Finished with timers */
+        release_lock(&sysblk.todlock);
+
+        /* Early exceptions associated with the guest load_psw() */
         if(icode)
             (GUESTREGS->sie_guestpi) (GUESTREGS, icode);
-    
+
         /* Run SIE in guests architecture mode */
         icode = run_sie[GUESTREGS->arch_mode] (regs);
-    }
+
+    } /* if (setjmp(GUESTREGS->progjmp)) */
 
     ARCH_DEP(sie_exit) (regs, icode);
 
@@ -523,7 +709,7 @@ int     icode = 0;                      /* Interception code         */
     PERFORM_CHKPT_SYNC (regs);
 
     longjmp(regs->progjmp, SIE_NO_INTERCEPT);
-} 
+}
 
 
 /* Exit SIE state, restore registers and update the state descriptor */
@@ -534,22 +720,28 @@ int     n;
 #if defined(SIE_DEBUG)
     logmsg(_("SIE: interception code %d\n"),code);
     ARCH_DEP(display_inst) (GUESTREGS, GUESTREGS->instvalid ?
-                                        GUESTREGS->inst : NULL);
+                                        GUESTREGS->ip : NULL);
 #endif /*defined(SIE_DEBUG)*/
 
+    SIE_PERFMON(SIE_PERF_EXIT);
+    SIE_PERFMON(SIE_PERF_PGMINT);
+
     /* Indicate we have left SIE mode */
+    obtain_lock (&sysblk.intlock);
     regs->sie_active = 0;
+    release_lock (&sysblk.intlock);
 
     /* zeroize interception status */
     STATEBK->f = 0;
 
-    switch(code)
+    switch(code > 0 ? code & ~PGM_PER_EVENT : code)
     {
         case SIE_HOST_INTERRUPT:
            /* If a host interrupt is pending
               then backup the psw and exit */
-            regs->psw.IA -= regs->psw.ilc;
+            regs->psw.IA -= REAL_ILC(regs);
             regs->psw.IA &= ADDRESS_MAXWRAP(regs);
+            VALIDATE_AIA(regs);
             break;
         case SIE_HOST_PGMINT:
             break;
@@ -643,12 +835,13 @@ int     n;
         STORE_W(STATEBK->cr[n], GUESTREGS->CR(n));
 
     /* Update the approprate host registers */
-    memcpy(regs->gr, GUESTREGS->gr, 14 * sizeof(U64));
-    memcpy(regs->ar, GUESTREGS->ar, 16 * sizeof(U32));
-    memcpy(regs->fpr, GUESTREGS->fpr, 32 * sizeof(U32));
+    memcpy(regs->gr, GUESTREGS->gr, 14 * sizeof(regs->gr[0]));
+    memcpy(regs->ar, GUESTREGS->ar, 16 * sizeof(regs->ar[0]));
+    memcpy(regs->fpr, GUESTREGS->fpr, 32 * sizeof(regs->fpr[0]));
 #if defined(FEATURE_BINARY_FLOATING_POINT)
     regs->fpc =  GUESTREGS->fpc;
 #endif /*defined(FEATURE_BINARY_FLOATING_POINT)*/
+    SET_AEA_MODE(regs);
 
     /* Zeroize the interruption parameters */
     memset(STATEBK->ipa, 0, 10);
@@ -664,9 +857,10 @@ int     n;
 #if defined(_FEATURE_PER)
         /* Handle PER or concurrent PER event */
         if( OPEN_IC_PERINT(GUESTREGS)
+          && ECMODE(&GUESTREGS->psw)
           && (GUESTREGS->psw.sysmask & PSW_PERMODE) )
         {
-        PSA *psa;   
+        PSA *psa;
 #if defined(_FEATURE_PER2)
             GUESTREGS->perc |= OPEN_IC_PERINT(GUESTREGS) >> ((32 - IC_CR9_SHIFT) - 16);
             /* Positions 14 and 15 contain zeros if a storage alteration
@@ -677,7 +871,7 @@ int     n;
 
 #endif /*defined(_FEATURE_PER2)*/
             /* Point to PSA fields in state descriptor */
-            psa = (void*)(regs->mainstor + GUESTREGS->sie_state + SIE_IP_PSA_OFFSET);
+            psa = (void*)(regs->mainstor + SIE_STATE(GUESTREGS) + SIE_IP_PSA_OFFSET);
             STORE_HW(psa->perint, GUESTREGS->perc);
             STORE_W(psa->peradr, GUESTREGS->peradr);
         }
@@ -685,20 +879,21 @@ int     n;
         if (IS_IC_PER_IF(GUESTREGS))
             STATEBK->f |= SIE_F_IF;
 
+        /* Reset any pending PER indication */
+        OFF_IC_PER(GUESTREGS);
 #endif /*defined(_FEATURE_PER)*/
 
         /* Update interception parameters in the state descriptor */
-        if(GUESTREGS->inst[0] != 0x44)
+        if(GUESTREGS->ip[0] != 0x44)
         {
             if(GUESTREGS->instvalid)
-                memcpy(STATEBK->ipa, GUESTREGS->inst, GUESTREGS->psw.ilc);
+                memcpy(STATEBK->ipa, GUESTREGS->ip, ILC(GUESTREGS->ip[0]));
         }
         else
         {
         int exilc;
             STATEBK->f |= SIE_F_EX;
-            exilc = (GUESTREGS->exinst[0] < 0x40) ? 2 :
-                    (GUESTREGS->exinst[0] < 0xC0) ? 4 : 6;
+            exilc = ILC(GUESTREGS->exinst[0]);
             memcpy(STATEBK->ipa, GUESTREGS->exinst, exilc);
         }
     }
@@ -711,21 +906,29 @@ int     n;
 /* Execute guest instructions */
 int ARCH_DEP(run_sie) (REGS *regs)
 {
-    int icode;
+    int   icode;    /* SIE longjmp intercept code      */
+    BYTE  oldv;     /* siebk->v change check reference */
+
+    SIE_PERFMON(SIE_PERF_RUNSIE);
 
-    SET_IC_EXTERNAL_MASK(GUESTREGS);
-    SET_IC_MCK_MASK(GUESTREGS);
-    SET_IC_IO_MASK(GUESTREGS);
-    SET_IC_PER_MASK(GUESTREGS);
 #if defined(_FEATURE_PER)
     /* Reset any PER pending indication */
     OFF_IC_PER(GUESTREGS);
 #endif /*defined(_FEATURE_PER)*/
 
     do {
+        SIE_PERFMON(SIE_PERF_RUNLOOP_1);
         if(!(icode = setjmp(GUESTREGS->progjmp)))
             do
             {
+                SIE_PERFMON(SIE_PERF_RUNLOOP_2);
+
+                /* Keep a copy if SIEBK 'v' field for later checks */
+                oldv=GUESTREGS->siebk->v;
+
+                /* Set `execflag' to 0 in case EXecuted instruction did progjmp */
+                GUESTREGS->execflag = 0;
+
                 if(
                     SIE_I_STOP(GUESTREGS)
                  || SIE_I_EXT(GUESTREGS)
@@ -735,13 +938,20 @@ int ARCH_DEP(run_sie) (REGS *regs)
 
                 if( SIE_IC_INTERRUPT_CPU(GUESTREGS) )
                 {
+                    SIE_PERFMON(SIE_PERF_INTCHECK);
+
                     /* Process PER program interrupts */
                     if( OPEN_IC_PERINT(GUESTREGS) )
                         ARCH_DEP(program_interrupt) (GUESTREGS, PGM_PER_EVENT);
 
                     obtain_lock(&sysblk.intlock);
 
-#if MAX_CPU_ENGINES > 1
+                    /* Turn off "possible interrupt" flag */
+                    /* Otherwise we come back here every  */
+                    /* time                               */
+                    /* ISW20041224                        */
+                    OFF_IC_INTERRUPT(GUESTREGS);
+
                     /* Perform broadcasted purge of ALB and TLB if requested
                        synchronize_broadcast() must be called until there are
                        no more broadcast pending because synchronize_broadcast()
@@ -749,7 +959,6 @@ int ARCH_DEP(run_sie) (REGS *regs)
 
                     while ((IS_IC_BROADCAST(regs)))
                         ARCH_DEP(synchronize_broadcast)(regs, 0, 0);
-#endif /*MAX_CPU_ENGINES > 1*/
 
                     if( OPEN_IC_EXTPENDING(GUESTREGS) )
                         ARCH_DEP(perform_external_interrupt) (GUESTREGS);
@@ -763,7 +972,7 @@ int ARCH_DEP(run_sie) (REGS *regs)
 
 #if defined(_FEATURE_WAITSTATE_ASSIST)
                     /* Wait state assist */
-                    if (GUESTREGS->psw.wait && (STATEBK->ec[0] & SIE_EC0_WAIA))
+                    if (WAITSTATE(&GUESTREGS->psw) && (STATEBK->ec[0] & SIE_EC0_WAIA))
                     {
 
                         /* Test for disabled wait PSW and issue message */
@@ -785,8 +994,7 @@ int ARCH_DEP(run_sie) (REGS *regs)
                         }
 
                         INVALIDATE_AIA(GUESTREGS);
-
-                        INVALIDATE_AEA_ALL(GUESTREGS);
+                        SET_AEA_MODE(GUESTREGS);
 
                         {
                         struct timespec waittime;
@@ -796,12 +1004,10 @@ int ARCH_DEP(run_sie) (REGS *regs)
                             waittime.tv_sec = now.tv_sec;
                             waittime.tv_nsec = ((now.tv_usec + 3333) * 1000);
 
-                            sysblk.waitmask |= regs->cpumask;
-
+                            sysblk.waiting_mask |= BIT(regs->cpuad);
                             timed_wait_condition
-                                 (&INTCOND, &sysblk.intlock, &waittime);
-
-                            sysblk.waitmask &= ~regs->cpumask;
+                                 (&regs->intcond, &sysblk.intlock, &waittime);
+                            sysblk.waiting_mask &= ~BIT(regs->cpuad);
                         }
 
                         release_lock (&sysblk.intlock);
@@ -819,37 +1025,54 @@ int ARCH_DEP(run_sie) (REGS *regs)
                                           )
                     break;
 
-
-                GUESTREGS->instvalid = 0;
-
-                INSTRUCTION_FETCH(GUESTREGS->inst, GUESTREGS->psw.IA, GUESTREGS);
-
-                GUESTREGS->instvalid = 1;
+                GUESTREGS->ip = INSTRUCTION_FETCH(GUESTREGS->inst, GUESTREGS->psw.IA, GUESTREGS, 0);
 
 #if defined(SIE_DEBUG)
                 /* Display the instruction */
-                ARCH_DEP(display_inst) (GUESTREGS, GUESTREGS->inst);
+                ARCH_DEP(display_inst) (GUESTREGS, GUESTREGS->ip);
 #endif /*defined(SIE_DEBUG)*/
 
+                SIE_PERFMON(SIE_PERF_EXEC);
                 regs->instcount++;
-                EXECUTE_INSTRUCTION(GUESTREGS->inst, 0, GUESTREGS);
+                EXECUTE_INSTRUCTION(GUESTREGS->ip, GUESTREGS);
 
-#if defined(OPTION_CPU_UNROLL)
 #ifdef FEATURE_PER
                 if (!PER_MODE(GUESTREGS))
 #endif
+                do
                 {
+                    REGS *gregs = GUESTREGS;
+                    SIE_PERFMON(SIE_PERF_EXEC_U);
+                    UNROLLED_EXECUTE(gregs);
+                    UNROLLED_EXECUTE(gregs);
+                    UNROLLED_EXECUTE(gregs);
                     regs->instcount += 7;
-                    UNROLLED_EXECUTE(GUESTREGS);
-                    UNROLLED_EXECUTE(GUESTREGS);
-                    UNROLLED_EXECUTE(GUESTREGS);
-                    UNROLLED_EXECUTE(GUESTREGS);
-                    UNROLLED_EXECUTE(GUESTREGS);
-                    UNROLLED_EXECUTE(GUESTREGS);
-                    UNROLLED_EXECUTE(GUESTREGS);
-                }
-#endif
-            } while( !SIE_I_HOST(regs) );
+                    UNROLLED_EXECUTE(gregs);
+                    UNROLLED_EXECUTE(gregs);
+                    UNROLLED_EXECUTE(gregs);
+                    UNROLLED_EXECUTE(gregs);
+                } while( likely(!SIE_I_HOST(regs)
+                                && GUESTREGS->siebk->v==oldv
+                                && !SIE_IC_INTERRUPT_CPU(GUESTREGS)));
+                /******************************************/
+                /* ABOVE : Keep executing instructions    */
+                /*         in a tight loop until...       */
+                /*  - A Host Interrupt is made pending    */
+                /*  - An external source changes siebk->v */
+                /*  - A guest interrupt is made pending   */
+                /******************************************/
+            } while( unlikely(!SIE_I_HOST(regs)
+                            && !SIE_I_WAIT(GUESTREGS)
+                            && !SIE_I_EXT(GUESTREGS)
+                            && !SIE_I_IO(GUESTREGS)
+                            && !SIE_IC_INTERRUPT_CPU(GUESTREGS)));
+            /******************************************/
+            /* ABOVE : Remain in SIE until...         */
+            /*  - A Host Interrupt is made pending    */
+            /*  - A Sie defined irpt becomes enabled  */
+            /*  - A guest interrupt is made pending   */
+            /******************************************/
+
 
         if(icode == 0 || icode == SIE_NO_INTERCEPT)
         {
@@ -891,7 +1114,7 @@ RADR    effective_addr2;                /* address of state desc.    */
 ZPB     zpb;                            /* Zone Parameter Block      */
 int     zone;                           /* Zone number               */
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -932,7 +1155,7 @@ RADR    mso,                            /* Main Storage Origin       */
         eso,                            /* Expanded Storage Origin   */
         esl;                            /* Expanded Storage Length   */
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
@@ -987,7 +1210,7 @@ U32     iointid;                        /* I/O interruption ident    */
 FWORD   tpziid[3];
 int     zone;                           /* Zone number               */
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
 
     PRIV_CHECK(regs);
 
