@@ -1,4 +1,4 @@
-/* tapesplt.C  (c) Copyright Jay Maynard, 2000-2003                 */
+/* tapesplt.C  (c) Copyright Jay Maynard, 2000-2005                 */
 /*              Split AWSTAPE format tape image                      */
 
 /*-------------------------------------------------------------------*/
@@ -7,6 +7,8 @@
 /* options.                                                          */
 /*-------------------------------------------------------------------*/
 
+#include "hstdinc.h"
+
 #include "hercules.h"
 
 /*-------------------------------------------------------------------*/
@@ -31,14 +33,9 @@ static BYTE vollbl[] = "\xE5\xD6\xD3";  /* EBCDIC characters "VOL"   */
 static BYTE hdrlbl[] = "\xC8\xC4\xD9";  /* EBCDIC characters "HDR"   */
 static BYTE eoflbl[] = "\xC5\xD6\xC6";  /* EBCDIC characters "EOF"   */
 static BYTE eovlbl[] = "\xC5\xD6\xE5";  /* EBCDIC characters "EOV"   */
-static BYTE buf[65500];
-
-SYSBLK sysblk; /* Currently only used for codepage mapping */
+static BYTE buf[65536];
 
 #ifdef EXTERNALGUI
-/* Special flag to indicate whether or not we're being
-   run under the control of the external GUI facility. */
-int  extgui = 0;
 /* Report progress every this many bytes */
 #define PROGRESS_MASK (~0x3FFFF /* 256K */)
 /* How many bytes we've read so far. */
@@ -55,8 +52,8 @@ int             rc;                     /* Return code               */
 int             i;                      /* Array subscript           */
 int             len;                    /* Block length              */
 int             prevlen;                /* Previous block length     */
-BYTE           *infilename;             /* -> Input file name        */
-BYTE           *outfilename;            /* -> Current out file name  */
+char           *infilename;             /* -> Input file name        */
+char           *outfilename;            /* -> Current out file name  */
 int             infd = -1;              /* Input file descriptor     */
 int             outfd = -1;             /* Current out file desc     */
 int             fileno;                 /* Tape file number          */
@@ -69,29 +66,22 @@ int             outfilecount;           /* Current # files copied    */
 int             files2copy;             /* Current # files to copy   */
 BYTE            labelrec[81];           /* Standard label (ASCIIZ)   */
 AWSTAPE_BLKHDR  awshdr;                 /* AWSTAPE block header      */
-char   *scodepage;
-
-    /* set_codepage() uses the logmsg macro which requires msgpipew */
-    sysblk.msgpipew = stdout;
-    if(!sysblk.codepage)
-    {
-        if((scodepage = getenv("HERCULES_CP")))
-            set_codepage(scodepage);
-        else
-            set_codepage("default");
-    }
+BYTE            pathname[MAX_PATH];     /* file path in host format  */
 
+    set_codepage(NULL);
 
 #ifdef EXTERNALGUI
     if (argc >= 1 && strncmp(argv[argc-1],"EXTERNALGUI",11) == 0)
     {
         extgui = 1;
         argc--;
+        setvbuf(stderr, NULL, _IONBF, 0);
+        setvbuf(stdout, NULL, _IONBF, 0);
     }
 #endif /*EXTERNALGUI*/
 
     /* Display the program identification message */
-    display_version (stderr, "Hercules tape split program ");
+    display_version (stderr, "Hercules tape split program ", FALSE);
 
     /* The only argument is the tape image file name */
     if (argc > 3 && argv[1] != NULL)
@@ -105,7 +95,8 @@ char   *scodepage;
     }
 
     /* Open the tape device */
-    infd = open (infilename, O_RDONLY | O_BINARY);
+    hostpath(pathname, infilename, sizeof(pathname));
+    infd = open (pathname, O_RDONLY | O_BINARY);
     if (infd < 0)
     {
         printf ("tapesplt: error opening input file %s: %s\n",
@@ -124,7 +115,8 @@ char   *scodepage;
     {
         outfilename = argv[outfilenum];
         printf ("Writing output file %s.\n", outfilename);
-        outfd = open (outfilename, O_WRONLY | O_CREAT | O_BINARY,
+        hostpath(pathname, outfilename, sizeof(pathname));
+        outfd = open (pathname, O_WRONLY | O_CREAT | O_BINARY,
                         S_IRUSR | S_IWUSR | S_IRGRP);
 
         if (outfd < 0)
