@@ -1,6 +1,8 @@
 /* TIMER.C   */
 
-/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2003      */
+/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2005      */
+
+#include "hstdinc.h"
 
 #include "hercules.h"
 
@@ -9,64 +11,7 @@
 #include "feat390.h"
 #include "feat370.h"
 
-void check_timer_event(void);
-
-/*-------------------------------------------------------------------*/
-/* Update TOD clock                                                  */
-/*                                                                   */
-/* This function updates the TOD clock.                              */
-/*                                                                   */
-/* This function is called by timer_update_thread and by cpu_thread  */
-/* instructions that manipulate any of the timer related entities    */
-/* (clock comparator, cpu timer and interval timer).                 */
-/*                                                                   */
-/* Internal function `check_timer_event' is called which will signal */
-/* any timer related interrupts to the appropriate cpu_thread.       */
-/*                                                                   */
-/* Callers *must* own the todlock and *must not* own the intlock.    */
-/*                                                                   */
-/*-------------------------------------------------------------------*/
-void update_TOD_clock(void)
-{
-struct timeval  tv;         /* Current time              */
-U64     dreg;           /* Double register work area */
-
-    /* Get current time */
-    gettimeofday (&tv, NULL);
-    ADJUST_TOD (tv, sysblk.lasttod);
-
-    /* Load number of seconds since 00:00:00 01 Jan 1970 */
-    dreg = (U64)tv.tv_sec;
-
-    /* Convert to microseconds */
-    dreg = dreg * 1000000 + tv.tv_usec;
-
-#ifdef OPTION_TODCLOCK_DRAG_FACTOR
-    if (sysblk.toddrag > 1)
-        dreg = sysblk.todclock_init +
-        (dreg - sysblk.todclock_init) / sysblk.toddrag;
-#endif /*OPTION_TODCLOCK_DRAG_FACTOR*/
-
-    /* Shift left 4 bits so that bits 0-7=TOD Clock Epoch,
-       bits 8-59=TOD Clock bits 0-51, bits 60-63=zero */
-    dreg <<= 4;
-
-    /* Ensure that the clock does not go backwards and always
-       returns a unique value in the microsecond range */
-    if( dreg > sysblk.todclk)
-        sysblk.todclk = dreg;
-    else sysblk.todclk += 16;
-
-    /* Get the difference between the last TOD saved and this one */
-    sysblk.todclock_diff = (sysblk.todclock_prev == 0 ? 0 :
-                            sysblk.todclk - sysblk.todclock_prev);
-
-    /* Save the old TOD clock value */
-    sysblk.todclock_prev = sysblk.todclk;
-
-    /* Check if timer event has occurred */
-    check_timer_event();
-}
+int ecpsvm_testvtimer(REGS *,int);
 
 /*-------------------------------------------------------------------*/
 /* Check for timer event                                             */
@@ -77,16 +22,17 @@ U64     dreg;           /* Double register work area */
 /* [3] Interval timer                                                */
 /* CPUs with an outstanding interrupt are signalled                  */
 /*                                                                   */
+/* tod_delta is in hercules internal clock format (>> 8)             */
 /*-------------------------------------------------------------------*/
-void check_timer_event(void)
+void update_cpu_timer(U64 tod_delta)
 {
 int             cpu;                    /* CPU counter               */
 REGS           *regs;                   /* -> CPU register context   */
 PSA_3XX        *psa;                    /* -> Prefixed storage area  */
 S32             itimer;                 /* Interval timer value      */
 S32             olditimer;              /* Previous interval timer   */
-#if defined(OPTION_MIPS_COUNTING) && defined(_FEATURE_SIE)
-int             itimer_diff;            /* TOD difference in TU      */
+#if defined(OPTION_MIPS_COUNTING) && ( defined(_FEATURE_SIE) || defined(_FEATURE_ECPSVM) )
+S32             itimer_diff;            /* TOD difference in TU      */
 #endif
 U32             intmask = 0;            /* Interrupt CPU mask        */
 
@@ -94,28 +40,27 @@ U32             intmask = 0;            /* Interrupt CPU mask        */
     obtain_lock (&sysblk.intlock);
 
     /* Check for [1] clock comparator, [2] cpu timer, and
-       [3] interval timer interrupts for each CPU.
-         Note that we do *not* check for the macro
-         _FEATURE_CPU_RECONFIG because we are not running
-         in an ARCH_DEP context */
-    for (cpu = 0; cpu < MAX_CPU_ENGINES; cpu++)
+     * [3] interval timer interrupts for each CPU.
+     */
+    for (cpu = 0; cpu < HI_CPU; cpu++)
     {
         /* Ignore this CPU if it is not started */
-        if((sysblk.regs[cpu].cpumask & sysblk.started_mask) == 0)
+        if (!IS_CPU_ONLINE(cpu)
+         || CPUSTATE_STOPPED == sysblk.regs[cpu]->cpustate)
             continue;
 
         /* Point to the CPU register context */
-        regs = sysblk.regs + cpu;
+        regs = sysblk.regs[cpu];
 
         /*-------------------------------------------*
          * [1] Check for clock comparator interrupt  *
          *-------------------------------------------*/
-        if ((sysblk.todclk + regs->todoffset) > regs->clkc)
+        if (TOD_CLOCK(regs) > regs->clkc)
         {
             if (!IS_IC_CLKC(regs))
             {
                 ON_IC_CLKC(regs);
-                intmask |= regs->cpumask;
+                intmask |= BIT(regs->cpuad);
             }
         }
         else if (IS_IC_CLKC(regs))
@@ -126,10 +71,10 @@ U32             intmask = 0;            /* Interrupt CPU mask        */
         if(regs->sie_active)
         {
         /* Signal clock comparator interrupt if needed */
-            if((sysblk.todclk + regs->guestregs->todoffset) > regs->guestregs->clkc)
+            if(TOD_CLOCK(regs->guestregs) > regs->guestregs->clkc)
             {
                 ON_IC_CLKC(regs->guestregs);
-                intmask |= regs->cpumask;
+                intmask |= BIT(regs->cpuad);
             }
             else
                 OFF_IC_CLKC(regs->guestregs);
@@ -140,7 +85,7 @@ U32             intmask = 0;            /* Interrupt CPU mask        */
          * [2] Decrement the CPU timer for each CPU  *
          *-------------------------------------------*/
 
-        (S64)regs->ptimer -= (S64)sysblk.todclock_diff << 8;
+        regs->ptimer = (S64)regs->ptimer - (tod_delta << 8);
 
         /* Set interrupt flag if the CPU timer is negative */
         if ((S64)regs->ptimer < 0)
@@ -148,7 +93,7 @@ U32             intmask = 0;            /* Interrupt CPU mask        */
             if (!IS_IC_PTIMER(regs))
             {
                 ON_IC_PTIMER(regs);
-                intmask |= regs->cpumask;
+                intmask |= BIT(regs->cpuad);
             }
         }
         else if(IS_IC_PTIMER(regs))
@@ -159,13 +104,13 @@ U32             intmask = 0;            /* Interrupt CPU mask        */
         if(regs->sie_active)
         {
             /* Decrement the guest CPU timer */
-            (S64)regs->guestregs->ptimer -= (S64)sysblk.todclock_diff << 8;
+            regs->guestregs->ptimer = (S64)regs->guestregs->ptimer - (tod_delta << 8);
 
             /* Set interrupt flag if the CPU timer is negative */
             if ((S64)regs->guestregs->ptimer < 0)
             {
                 ON_IC_PTIMER(regs->guestregs);
-                intmask |= regs->cpumask;
+                intmask |= BIT(regs->cpuad);
             }
             else
                 OFF_IC_PTIMER(regs->guestregs);
@@ -176,9 +121,11 @@ U32             intmask = 0;            /* Interrupt CPU mask        */
          * [3] Check for interval timer interrupt    *
          *-------------------------------------------*/
 
-#if defined(OPTION_MIPS_COUNTING) && defined(_FEATURE_SIE)
-        /* Calculate diff in interval timer units */
-        itimer_diff = (int)((3*sysblk.todclock_diff)/625);
+#if defined(OPTION_MIPS_COUNTING) && ( defined(_FEATURE_SIE) || defined(_FEATURE_ECPSVM) )
+        /* Calculate diff in interval timer units plus rounding to improve accuracy */
+        itimer_diff = (S32) ((((6*tod_delta)/625)+1) >> 1);
+        if (itimer_diff <= 0)           /* Handle gettimeofday low */
+            itimer_diff = 1;            /* resolution problems     */
 #endif
 
         if(regs->arch_mode == ARCH_370)
@@ -189,7 +136,7 @@ U32             intmask = 0;            /* Interrupt CPU mask        */
                     /* Decrement the location 80 timer */
             FETCH_FW(itimer,psa->inttimer);
                     olditimer = itimer;
-            
+
                     /* The interval timer is decremented as though bit 23 is
                        decremented by one every 1/300 of a second. This comes
                        out to subtracting 768 (X'300') every 1/100 of a second.
@@ -198,33 +145,48 @@ U32             intmask = 0;            /* Interrupt CPU mask        */
                        comes out to 75 on the Alpha, with its 1024/second
                        tick interval. See 370 POO page 4-29. (ESA doesn't
                        even have an interval timer.) */
-#if defined(OPTION_MIPS_COUNTING) && defined(_FEATURE_SIE)
+#if defined(OPTION_MIPS_COUNTING) && ( defined(_FEATURE_SIE) || defined(_FEATURE_ECPSVM) )
             itimer -= itimer_diff;
 #else
             itimer -= 76800 / CLK_TCK;
 #endif
             STORE_FW(psa->inttimer,itimer);
-        
+
             /* Set interrupt flag and interval timer interrupt pending
                if the interval timer went from positive to negative */
             if (itimer < 0 && olditimer >= 0)
             {
+#if defined(_FEATURE_ECPSVM)
+                regs->rtimerint=1;      /* To resolve concurrent V/R Int Timer Ints */
+#endif
                 ON_IC_ITIMER(regs);
-                intmask |= regs->cpumask;
+                intmask |= BIT(regs->cpuad);
             }
+#if defined(_FEATURE_ECPSVM)
+#if defined(OPTION_MIPS_COUNTING)
+            if(ecpsvm_testvtimer(regs,itimer_diff)==0)
+#else /* OPTION_MIPS_COUNTING */
+            if(ecpsvm_testvtimer(regs,76800 / CLK_TCK)==0)
+#endif /* OPTION_MIPS_COUNTING */
+            {
+                ON_IC_ITIMER(regs);
+                intmask |= BIT(regs->cpuad);
+            }
+#endif /* _FEATURE_ECPSVM */
+
         } /*if(regs->arch_mode == ARCH_370)*/
 
 #if defined(_FEATURE_SIE)
         /* When running under SIE also update the SIE copy */
         if(regs->sie_active)
         {
-            if((regs->guestregs->siebk->m & SIE_M_370)
-              && !(regs->guestregs->siebk->m & SIE_M_ITMOF))
+            if(SIE_STATB(regs->guestregs, M, 370)
+              && SIE_STATNB(regs->guestregs, M, ITMOF))
             {
                 /* Decrement the location 80 timer */
                 FETCH_FW(itimer,regs->guestregs->sie_psa->inttimer);
                 olditimer = itimer;
-            
+
 #if defined(OPTION_MIPS_COUNTING)
                 itimer -= itimer_diff;
 #else
@@ -237,17 +199,18 @@ U32             intmask = 0;            /* Interrupt CPU mask        */
                 if (itimer < 0 && olditimer >= 0)
                 {
                     ON_IC_ITIMER(regs->guestregs);
-                    intmask |= regs->cpumask;
+                    intmask |= BIT(regs->cpuad);
                 }
             }
         }
 #endif /*defined(_FEATURE_SIE)*/
 
+
     } /* end for(cpu) */
 
     /* If a timer interrupt condition was detected for any CPU
        then wake up those CPUs if they are waiting */
-    WAKEUP_WAITING_CPUS (intmask, CPUSTATE_STARTED);
+    WAKEUP_CPUS_MASK (intmask);
 
     release_lock(&sysblk.intlock);
 
@@ -257,7 +220,7 @@ U32             intmask = 0;            /* Interrupt CPU mask        */
 /* TOD clock and timer thread                                        */
 /*                                                                   */
 /* This function runs as a separate thread.  It wakes up every       */
-/* x milliseconds, updates the TOD clock, and decrements the         */
+/* 1 microsecond, updates the TOD clock, and decrements the          */
 /* CPU timer for each CPU.  If any CPU timer goes negative, or       */
 /* if the TOD clock exceeds the clock comparator for any CPU,        */
 /* it signals any waiting CPUs to wake up and process interrupts.    */
@@ -265,7 +228,7 @@ U32             intmask = 0;            /* Interrupt CPU mask        */
 void *timer_update_thread (void *argp)
 {
 #ifdef OPTION_MIPS_COUNTING
-int     msecctr = 0;                    /* Millisecond counter       */
+int     usecctr = 0;                    /* Microsecond counter       */
 int     cpu;                            /* CPU counter               */
 REGS   *regs;                           /* -> CPU register context   */
 U64     prev = 0;                       /* Previous TOD clock value  */
@@ -275,100 +238,130 @@ U64     waittime;                       /* CPU wait time in interval */
 U64     now = 0;                        /* Current time of day (us)  */
 U64     then;                           /* Previous time of day (us) */
 int     interval;                       /* Interval (us)             */
+double  cpupct;                         /* Calculated cpu percentage */
 #endif /*OPTION_MIPS_COUNTING*/
-struct  timeval tv;                     /* Structure for gettimeofday
-                                           and select function calls */
-  UNREFERENCED(argp);
+struct  timeval tv;                     /* Structure for select      */
+
+    UNREFERENCED(argp);
+
+    /* Set root mode in order to set priority */
+    SETMODE(ROOT);
+
+    /* Set timer thread priority */
+    if (setpriority(PRIO_PROCESS, 0, sysblk.todprio))
+        logmsg (_("HHCTT001W Timer thread set priority %d failed: %s\n"),
+                sysblk.todprio, strerror(errno));
+
+    /* Back to user mode */
+    SETMODE(USER);
 
     /* Display thread started message on control panel */
-    logmsg (_("HHCCP012I Timer thread started: tid="TIDPAT", pid=%d\n"),
-            thread_id(), getpid());
+    logmsg (_("HHCTT002I Timer thread started: tid="TIDPAT", pid=%d, "
+            "priority=%d\n"),
+            thread_id(), getpid(), getpriority(PRIO_PROCESS,0));
 
-#ifdef OPTION_TODCLOCK_DRAG_FACTOR
-    /* Get current time */
-    gettimeofday (&tv, NULL);
-
-    /* Load number of seconds since 00:00:00 01 Jan 1970 */
-    sysblk.todclock_init = (U64)tv.tv_sec;
-
-    /* Convert to microseconds */
-    sysblk.todclock_init = sysblk.todclock_init * 1000000 + tv.tv_usec;
-#endif /*OPTION_TODCLOCK_DRAG_FACTOR*/
-
-    while (sysblk.numcpu)
+    while (sysblk.cpus)
     {
         /* Obtain the TOD lock */
         obtain_lock (&sysblk.todlock);
 
         /* Update TOD clock */
-        update_TOD_clock();
+        update_tod_clock();
 
 #ifdef OPTION_MIPS_COUNTING
         /* Calculate MIPS rate and percentage CPU busy */
-
-        /* Get the difference between the last TOD saved and this one */
-        diff = (prev == 0 ? 0 : sysblk.todclk - prev);
-
-        /* Save the old TOD clock value */
-        prev = sysblk.todclk;
+        diff = (prev == 0 ? 0 : tod_clock - prev);
+        prev = tod_clock;
 
         /* Shift the epoch out of the difference for the CPU timer */
         diff <<= 8;
 
-        msecctr += (int)(diff/4096000);
-        if (msecctr > 999)
+        usecctr += (int)(diff/4096);
+        if (usecctr > 999999)
         {
-            /* Access the diffent register contexts with the intlock held */
-            obtain_lock(&sysblk.intlock);
-
-            /* Get current time, we may have had to wait for the intlock */
+            U32  mipsrate = 0;   /* (total for ALL CPUs together) */
+            U32  siosrate = 0;   /* (total for ALL CPUs together) */
+// logmsg("+++ BLIP +++\n"); // (should appear once per second)
+            /* Get current time */
             then = now;
-            gettimeofday (&tv, NULL);
-            now = (U64)tv.tv_sec;
-            now = now * 1000000 + tv.tv_usec;
+            now = hw_clock();
             interval = (int)(now - then);
+            if (interval < 1)
+                interval = 1;
 
-            for (cpu = 0; cpu < MAX_CPU_ENGINES; cpu++)
+#if defined(OPTION_SHARED_DEVICES)
+            sysblk.shrdrate = sysblk.shrdcount;
+            sysblk.shrdcount = 0;
+            siosrate = sysblk.shrdrate;
+#endif
+
+            for (cpu = 0; cpu < HI_CPU; cpu++)
             {
-                /* Point to the CPU register context */
-                regs = sysblk.regs + cpu;
 
-                /* 0% if no cpu thread or first time thru */
-                if (regs->cputid == 0 || then == 0 || regs->waittod == 0)
+                obtain_lock (&sysblk.cpulock[cpu]);
+
+                if (!IS_CPU_ONLINE(cpu))
                 {
-                    regs->mipsrate = regs->siosrate = 0;
-                    regs->cpupct = 0.0;
+                    release_lock(&sysblk.cpulock[cpu]);
                     continue;
                 }
 
-                /* Calculate instructions/millisecond for this CPU */
-                regs->mipsrate =
-                    ((regs->instcount - regs->prevcount)*1000) / interval;
+                regs = sysblk.regs[cpu];
+
+                /* 0% if first time thru */
+                if (then == 0)
+                {
+                    regs->mipsrate = regs->siosrate = 0;
+                    regs->cpupct = 0.0;
+                    release_lock(&sysblk.cpulock[cpu]);
+                    continue;
+                }
+
+                /* Calculate instructions per second for this CPU */
+                regs->mipsrate = (regs->instcount - regs->prevcount);
                 regs->siosrate = regs->siocount;
 
+                /* Ignore wildly high rates probably in error */
+                if (regs->mipsrate > MAX_REPORTED_MIPSRATE)
+                    regs->mipsrate = 0;
+                if (regs->siosrate > MAX_REPORTED_SIOSRATE)
+                    regs->siosrate = 0;
+
+                /* Total for ALL CPUs together */
+                mipsrate += regs->mipsrate;
+                siosrate += regs->siosrate;
+
                 /* Save the instruction counter */
                 regs->prevcount = regs->instcount;
+                regs->siototal += regs->siocount;
                 regs->siocount = 0;
 
                 /* Calculate CPU busy percentage */
                 waittime = regs->waittime;
-                if ((sysblk.waitmask & regs->cpumask) != 0)
+                if (regs->waittod)
                     waittime += now - regs->waittod;
-                regs->cpupct = ((interval - waittime)*1.0) / (interval*1.0);
+                cpupct = ((double)(interval - waittime)) / ((double)interval);
+                if (cpupct < 0.0) cpupct = 0.0;
+                else if (cpupct > 1.0) cpupct = 1.0;
+                regs->cpupct = cpupct;
 
                 /* Reset the wait values */
                 regs->waittime = 0;
-                regs->waittod = now;
+                if (regs->waittod)
+                    regs->waittod = now;
+
+                release_lock(&sysblk.cpulock[cpu]);
 
             } /* end for(cpu) */
 
-            /* Reset the millisecond counter */
-            msecctr = 0;
+            /* Total for ALL CPUs together */
+            sysblk.mipsrate = mipsrate;
+            sysblk.siosrate = siosrate;
 
-            /* Release the intlock */
-            release_lock(&sysblk.intlock);
+            /* Reset the microsecond counter */
+            usecctr = 0;
 
-        } /* end if(msecctr) */
+        } /* end if(usecctr) */
 #endif /*OPTION_MIPS_COUNTING*/
 
         /* Release the TOD lock */
@@ -382,7 +375,9 @@ struct  timeval tv;                     /* Structure for gettimeofday
 
     } /* end while */
 
-    logmsg (_("HHCCP013I Timer thread ended\n"));
+    logmsg (_("HHCTT003I Timer thread ended\n"));
+
+    sysblk.todtid = 0;
 
     return NULL;
 
