@@ -1,8 +1,30 @@
-/* TRACE.C      (c) Copyright Jan Jaeger, 2000-2003                  */
+/* TRACE.C      (c) Copyright Jan Jaeger, 2000-2005                  */
 /*              Implicit tracing functions                           */
 
-/* Interpretive Execution - (c) Copyright Jan Jaeger, 1999-2003      */
-/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2003      */
+/* Interpretive Execution - (c) Copyright Jan Jaeger, 1999-2005      */
+/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2005      */
+
+/*-------------------------------------------------------------------*/
+/* This module contains procedures for creating entries in the       */
+/* system trace table as described in the manuals:                   */
+/* SA22-7201 ESA/390 Principles of Operation                         */
+/* SA22-7832 z/Architecture Principles of Operation                  */
+/*-------------------------------------------------------------------*/
+
+/*-------------------------------------------------------------------*/
+/* Additional credits:                                               */
+/*      ASN-and-LX-reuse facility - Roger Bowler            July 2004*/
+/*-------------------------------------------------------------------*/
+
+#include "hstdinc.h"
+
+#if !defined(_HENGINE_DLL_)
+#define _HENGINE_DLL_
+#endif
+
+#if !defined(_TRACE_C_)
+#define _TRACE_C_
+#endif
 
 #include "hercules.h"
 
@@ -15,6 +37,9 @@
 #if !defined(_TRACE_H)
 #define _TRACE_H
 
+/*-------------------------------------------------------------------*/
+/* Format definitions for trace table entries                        */
+/*-------------------------------------------------------------------*/
 typedef struct _TRACE_F1_BR {
 FWORD   newia24;                        /* Bits 0-7 are zeros        */
 } TRACE_F1_BR;
@@ -29,7 +54,7 @@ BYTE    format;                         /* B'01010010'               */
 BYTE    fmt2;                           /* B'1100' B'0000'           */
 #define TRACE_F3_BR_FM2 0xC0
 HWORD   resv;
-DWORD   newia64;
+DBLWRD  newia64;
 } TRACE_F3_BR;
 
 typedef struct _TRACE_F1_BSG {
@@ -43,7 +68,7 @@ typedef struct _TRACE_F2_BSG {
 BYTE    format;                         /* B'01000010'               */
 #define TRACE_F2_BSG_FMT 0x42
 BYTE    alet[3];
-DWORD   newia;
+DBLWRD  newia;
 } TRACE_F2_BSG;
 
 typedef struct _TRACE_F1_MS {
@@ -70,7 +95,7 @@ BYTE    format;
 BYTE    fmt2;
 #define TRACE_F3_MS_FM2 0x60
 HWORD   resv;
-DWORD   newia;
+DBLWRD  newia;
 } TRACE_F3_MS;
 
 
@@ -98,7 +123,7 @@ BYTE    format;
 BYTE    fmt2;
 #define TRACE_F3_MSB_FM2 0xF0
 HWORD   resv;
-DWORD   newia;
+DBLWRD  newia;
 } TRACE_F3_MSB;
 
 typedef struct _TRACE_F1_PT {
@@ -125,13 +150,13 @@ BYTE    format;
 BYTE    pswkey;
 #define TRACE_F3_PT_FM2 0x0C
 HWORD   newpasn;
-DWORD   r2;
+DBLWRD  r2;
 } TRACE_F3_PT;
 
 typedef struct _TRACE_F1_SSAR {
 BYTE    format;
 #define TRACE_F1_SSAR_FMT 0x10
-BYTE    zero;
+BYTE    extfmt;
 HWORD   newsasn;
 } TRACE_F1_SSAR;
 
@@ -148,9 +173,9 @@ typedef struct _TRACE_F2_TRACE {
 BYTE    format;
 BYTE    extfmt;
 HWORD   tod1631;
-DWORD   tod3279;
+DBLWRD  tod3279;
 FWORD   operand;
-DWORD   regs[16];
+DBLWRD  regs[16];
 } TRACE_F2_TRACE;
 
 typedef struct _TRACE_F1_PR {
@@ -180,7 +205,7 @@ BYTE    pswkey;
 #define TRACE_F3_PR_FM2 0x03
 HWORD   newpasn;
 FWORD   retna;
-DWORD   newia;
+DBLWRD  newia;
 } TRACE_F3_PR;
 
 typedef struct _TRACE_F4_PR {
@@ -210,7 +235,7 @@ BYTE    pswkey;
 #define TRACE_F6_PR_FM2 0x0B
 HWORD   newpasn;
 FWORD   retna;
-DWORD   newia;
+DBLWRD  newia;
 } TRACE_F6_PR;
 
 typedef struct _TRACE_F7_PR {
@@ -219,7 +244,7 @@ BYTE    format;
 BYTE    pswkey;
 #define TRACE_F7_PR_FM2 0x0C
 HWORD   newpasn;
-DWORD   retna;
+DBLWRD  retna;
 FWORD   newia;
 } TRACE_F7_PR;
 
@@ -229,7 +254,7 @@ BYTE    format;
 BYTE    pswkey;
 #define TRACE_F8_PR_FM2 0x0E
 HWORD   newpasn;
-DWORD   retna;
+DBLWRD  retna;
 FWORD   newia;
 } TRACE_F8_PR;
 
@@ -239,7 +264,7 @@ BYTE    format;
 BYTE    pswkey;
 #define TRACE_F9_PR_FM2 0x0F
 HWORD   newpasn;
-DWORD   retna;
+DBLWRD  retna;
 FWORD   newia;
 } TRACE_F9_PR;
 
@@ -256,9 +281,56 @@ BYTE    format;
 #define TRACE_F2_PC_FMT 0x22
 BYTE    pswkey_pcnum_hi;
 HWORD   pcnum_lo;
-DWORD   retna;
+DBLWRD  retna;
 } TRACE_F2_PC;
 
+typedef struct _TRACE_F3_PC {
+BYTE    format;
+#define TRACE_F3_PC_FMT 0x21
+BYTE    pswkey_pcnum_hi;
+HWORD   pcnum_lo;
+FWORD   retna;
+} TRACE_F3_PC;
+
+typedef struct _TRACE_F4_PC {
+BYTE    format;
+#define TRACE_F4_PC_FMT 0x22
+BYTE    pswkey_pcnum_hi;
+HWORD   pcnum_lo;
+DBLWRD  retna;
+} TRACE_F4_PC;
+
+typedef struct _TRACE_F5_PC {
+BYTE    format;
+#define TRACE_F5_PC_FMT 0x22
+BYTE    pswkey;
+#define TRACE_F5_PC_FM2 0x08 
+HWORD   resv;
+FWORD   retna;
+FWORD   pcnum;
+} TRACE_F5_PC;
+
+typedef struct _TRACE_F6_PC {
+BYTE    format;
+#define TRACE_F6_PC_FMT 0x22
+BYTE    pswkey;
+#define TRACE_F6_PC_FM2 0x0A 
+HWORD   resv;
+FWORD   retna;
+FWORD   pcnum;
+} TRACE_F6_PC;
+
+typedef struct _TRACE_F7_PC {
+BYTE    format;
+#define TRACE_F7_PC_FMT 0x23
+BYTE    pswkey;
+#define TRACE_F7_PC_FM2 0x0E 
+HWORD   resv;
+DBLWRD  retna;
+FWORD   pcnum;
+} TRACE_F7_PC;
+
+
 typedef struct _TRACE_F1_TR {
 BYTE    format;
 #define TRACE_F1_TR_FMT 0x70
@@ -279,11 +351,24 @@ HWORD   clk0;
 FWORD   clk16;
 FWORD   clk48;
 FWORD   operand;
-DWORD   reg[16];
+DBLWRD  reg[16];
 } TRACE_F2_TR;
 
 #endif /*!defined(_TRACE_H)*/
 
+/*-------------------------------------------------------------------*/
+/* Reserve space for a new trace entry                               */
+/*                                                                   */
+/* Input:                                                            */
+/*      size    Number of bytes required for trace entry             */
+/*      regs    Pointer to the CPU register context                  */
+/* Output:                                                           */
+/*      abs_guest  Guest absolute address of trace entry (if SIE)    */
+/* Return value:                                                     */
+/*      Absolute address of new trace entry                          */
+/*                                                                   */
+/*      This function does not return if a program check occurs.     */
+/*-------------------------------------------------------------------*/
 static inline RADR ARCH_DEP(get_trace_entry) (RADR *abs_guest, int size, REGS *regs)
 {
 RADR    n;                              /* Addr of trace table entry */
@@ -292,7 +377,7 @@ RADR    n;                              /* Addr of trace table entry */
     n = regs->CR(12) & CR12_TRACEEA;
 
     /* Apply low-address protection to trace entry address */
-    if (ARCH_DEP(is_low_address_protected) (n, 0, regs))
+    if (ARCH_DEP(is_low_address_protected) (n, regs))
     {
 #ifdef FEATURE_SUPPRESSION_ON_PROTECTION
         regs->TEA = (n & STORAGE_KEY_PAGEMASK);
@@ -324,6 +409,17 @@ RADR    n;                              /* Addr of trace table entry */
 } /* end function ARCH_DEP(get_trace_entry) */
 
 
+/*-------------------------------------------------------------------*/
+/* Commit a new trace entry                                          */
+/*                                                                   */
+/* Input:                                                            */
+/*      abs_guest  Guest absolute address of trace entry (if SIE)    */
+/*      raddr   Absolute address of trace entry                      */
+/*      size    Number of bytes reserved for trace entry             */
+/*      regs    Pointer to the CPU register context                  */
+/* Return value:                                                     */
+/*      Updated value for CR12 after committing the trace entry      */
+/*-------------------------------------------------------------------*/
 static inline CREG ARCH_DEP(set_trace_entry) (RADR abs_guest, RADR raddr, int size, REGS *regs)
 {
 #if defined(_FEATURE_SIE)
@@ -339,7 +435,7 @@ RADR abs_host;
     raddr = abs_guest + (raddr - abs_host);
 #endif /*defined(_FEATURE_SIE)*/
 
-    /* Convert trace entry absolue address back to real address */
+    /* Convert trace entry absolute address back to real address */
     raddr = APPLY_PREFIXING (raddr, regs->PX);
 
     /* Return updated value of control register 12 */
@@ -373,7 +469,7 @@ int  size;
         TRACE_F3_BR *tte;
         size = sizeof(TRACE_F3_BR);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
         tte->format = TRACE_F3_BR_FMT;
         tte->fmt2 = TRACE_F3_BR_FM2;
         STORE_HW(tte->resv,0);
@@ -386,7 +482,7 @@ int  size;
         TRACE_F2_BR *tte;
         size = sizeof(TRACE_F2_BR);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
         STORE_FW(tte->newia31,ia | 0x80000000);
     }
     else
@@ -394,13 +490,13 @@ int  size;
         TRACE_F1_BR *tte;
         size = sizeof(TRACE_F1_BR);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
         STORE_FW(tte->newia24,ia & 0x00FFFFFF);
     }
     
     return ARCH_DEP(set_trace_entry) (ag, raddr, size, regs);
     
-} /* end function ARCH_DEP(set_trace_entry) */
+} /* end function ARCH_DEP(trace_br) */
 
 
 #if defined(FEATURE_SUBSPACE_GROUP)
@@ -428,7 +524,7 @@ int  size;
         TRACE_F2_BSG *tte;
         size = sizeof(TRACE_F2_BSG);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
         tte->format = TRACE_F2_BSG_FMT;
         tte->alet[0] = (alet >> 16) & 0xFF; 
         tte->alet[1] = (alet >> 8) & 0xFF; 
@@ -441,7 +537,7 @@ int  size;
         TRACE_F1_BSG *tte;
         size = sizeof(TRACE_F1_BSG);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
         tte->format = TRACE_F1_BSG_FMT;
         tte->alet[0] = ((alet >> 17) & 0x80) | ((alet >> 16) & 0x7F); 
         tte->alet[1] = (alet >> 8) & 0xFF; 
@@ -458,9 +554,10 @@ int  size;
 
 
 /*-------------------------------------------------------------------*/
-/* Form implicit SSAR trace entry                                    */
+/* Form implicit SSAR/SSAIR trace entry                              */
 /*                                                                   */
 /* Input:                                                            */
+/*      ssair   1=SSAIR instruction, 0=SSAR instruction              */
 /*      sasn    Secondary address space number                       */
 /*      regs    Pointer to the CPU register context                  */
 /* Return value:                                                     */
@@ -468,19 +565,20 @@ int  size;
 /*                                                                   */
 /*      This function does not return if a program check occurs.     */
 /*-------------------------------------------------------------------*/
-CREG ARCH_DEP(trace_ssar) (U16 sasn, REGS *regs)
+CREG ARCH_DEP(trace_ssar) (int ssair, U16 sasn, REGS *regs)
 {
 RADR raddr;
 RADR ag;
 int  size;
+BYTE nbit = (ssair ? 1 : 0);
 
     {
         TRACE_F1_SSAR *tte;
         size = sizeof(TRACE_F1_SSAR);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
         tte->format = TRACE_F1_SSAR_FMT;
-        tte->zero = 0;
+        tte->extfmt = 0 | nbit;
         STORE_HW(tte->newsasn,sasn);
     }
 
@@ -493,48 +591,137 @@ int  size;
 /* Form implicit PC trace entry                                      */
 /*                                                                   */
 /* Input:                                                            */
-/*      pcnum   Destination PC number                                */
+/*      pcea    PC instruction effective address (20 or 32 bits)     */
 /*      regs    Pointer to the CPU register context                  */
 /* Return value:                                                     */
 /*      Updated value for CR12 after adding new trace entry          */
 /*                                                                   */
 /*      This function does not return if a program check occurs.     */
 /*-------------------------------------------------------------------*/
-CREG ARCH_DEP(trace_pc) (U32 pcnum, REGS *regs)
+CREG ARCH_DEP(trace_pc) (U32 pcea, REGS *regs)
 {
 RADR raddr;
 RADR ag;
 int  size;
+int  eamode;
+
+    regs->psw.IA &= ADDRESS_MAXWRAP(regs);
+    eamode = regs->psw.amode64;
 
 #if defined(FEATURE_ESAME)
+    if (ASN_AND_LX_REUSE_ENABLED(regs))
+    {
+        if ((pcea & PC_BIT44) && regs->psw.amode64 && regs->psw.IA_H)
+        {
+            /* In 64-bit mode, regardless of resulting mode, when
+               ASN-and-LX-reuse is enabled, 32-bit PC number is used,
+               and bits 0-31 of return address are not all zeros */
+            TRACE_F7_PC *tte;
+            size = sizeof(TRACE_F7_PC);
+            raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
+            tte = (void*)(regs->mainstor + raddr);
+            tte->format = TRACE_F7_PC_FMT;
+            tte->pswkey = regs->psw.pkey | TRACE_F7_PC_FM2 | eamode;
+            STORE_HW(tte->resv, 0x0000);
+            STORE_DW(tte->retna, regs->psw.IA_G | PROBSTATE(&regs->psw));
+            STORE_FW(tte->pcnum, pcea);
+        }
+        else
+        if ((pcea & PC_BIT44) && regs->psw.amode64)
+        {
+            /* In 64-bit mode, regardless of resulting mode, when
+               ASN-and-LX-reuse is enabled, 32-bit PC number is used,
+               and bits 0-31 of return address are all zeros */
+            TRACE_F6_PC *tte;
+            size = sizeof(TRACE_F6_PC);
+            raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
+            tte = (void*)(regs->mainstor + raddr);
+            tte->format = TRACE_F6_PC_FMT;
+            tte->pswkey = regs->psw.pkey | TRACE_F6_PC_FM2 | eamode;
+            STORE_HW(tte->resv, 0x0000);
+            STORE_FW(tte->retna, regs->psw.IA_L | PROBSTATE(&regs->psw));
+            STORE_FW(tte->pcnum, pcea);
+        }
+        else
+        if ((pcea & PC_BIT44))
+        {
+            /* In 24-bit or 31-bit mode, regardless of resulting mode, when
+               ASN-and-LX-reuse is enabled and 32-bit PC number is used */
+            TRACE_F5_PC *tte;
+            size = sizeof(TRACE_F5_PC);
+            raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
+            tte = (void*)(regs->mainstor + raddr);
+            tte->format = TRACE_F5_PC_FMT;
+            tte->pswkey = regs->psw.pkey | TRACE_F5_PC_FM2 | eamode;
+            STORE_HW(tte->resv, 0x0000);
+            STORE_FW(tte->retna, (regs->psw.amode << 31) | regs->psw.IA_L | PROBSTATE(&regs->psw));
+            STORE_FW(tte->pcnum, pcea);
+        }
+        else
+        if(regs->psw.amode64)
+        {
+            /* In 64-bit mode, regardless of resulting mode, when
+               ASN-and-LX-reuse is enabled and 20-bit PC number is used */
+            TRACE_F4_PC *tte;
+            size = sizeof(TRACE_F4_PC);
+            raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
+            tte = (void*)(regs->mainstor + raddr);
+            tte->format = TRACE_F4_PC_FMT;
+            tte->pswkey_pcnum_hi = regs->psw.pkey | ((pcea & 0xF0000) >> 16);
+            STORE_HW(tte->pcnum_lo, pcea & 0x0FFFF);
+            STORE_DW(tte->retna, regs->psw.IA_G | PROBSTATE(&regs->psw));
+        }
+        else
+        {
+            /* In 24-bit or 31-bit mode, regardless of resulting mode, when
+               ASN-and-LX-reuse is enabled and 20-bit PC number is used */
+            TRACE_F3_PC *tte;
+            size = sizeof(TRACE_F3_PC);
+            raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
+            tte = (void*)(regs->mainstor + raddr);
+            tte->format = TRACE_F3_PC_FMT;
+            tte->pswkey_pcnum_hi = regs->psw.pkey | ((pcea & 0xF0000) >> 16);
+            STORE_HW(tte->pcnum_lo, pcea & 0x0FFFF);
+            STORE_FW(tte->retna, (regs->psw.amode << 31) | regs->psw.IA_L | PROBSTATE(&regs->psw));
+        }
+    } /* end ASN_AND_LX_REUSE_ENABLED */
+    else
     if(regs->psw.amode64)
     {
+        /* In 64-bit mode, regardless of resulting mode,
+           when ASN-and-LX-reuse is not enabled */
         TRACE_F2_PC *tte;
         size = sizeof(TRACE_F2_PC);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
         tte->format = TRACE_F2_PC_FMT;
-        tte->pswkey_pcnum_hi = regs->psw.pkey | ((pcnum & 0xF0000) >> 16);
-        STORE_HW(tte->pcnum_lo, pcnum & 0x0FFFF);
-        STORE_DW(tte->retna, regs->psw.IA_G | regs->psw.prob); 
+        tte->pswkey_pcnum_hi = regs->psw.pkey | ((pcea & 0xF0000) >> 16);
+        STORE_HW(tte->pcnum_lo, pcea & 0x0FFFF);
+        STORE_DW(tte->retna, regs->psw.IA_G | PROBSTATE(&regs->psw)); 
     }
     else
 #endif /*defined(FEATURE_ESAME)*/
     {
+        /* In 24-bit or 31-bit mode, regardless of resulting mode,
+           when ASN-and-LX-reuse is not enabled */
         TRACE_F1_PC *tte;
         size = sizeof(TRACE_F1_PC);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
         tte->format = TRACE_F1_PC_FMT;
-        tte->pswkey_pcnum_hi = regs->psw.pkey | ((pcnum & 0xF0000) >> 16);
-        STORE_HW(tte->pcnum_lo, pcnum & 0x0FFFF);
-        STORE_FW(tte->retna, (regs->psw.amode << 31) | regs->psw.IA_L | regs->psw.prob); 
+        tte->pswkey_pcnum_hi = regs->psw.pkey | ((pcea & 0xF0000) >> 16);
+        STORE_HW(tte->pcnum_lo, pcea & 0x0FFFF);
+        STORE_FW(tte->retna, (regs->psw.amode << 31) | regs->psw.IA_L | PROBSTATE(&regs->psw)); 
     }
 
     return ARCH_DEP(set_trace_entry) (ag, raddr, size, regs);
     
 } /* end function ARCH_DEP(trace_pc) */
 
+#if defined(_MSVC_)
+  /* Workaround for "fatal error C1001: INTERNAL COMPILER ERROR" in MSVC */
+  #pragma optimize("",off)
+#endif /*defined(_MSVC_)*/
 
 #if defined(FEATURE_LINKAGE_STACK)
 /*-------------------------------------------------------------------*/
@@ -554,6 +741,8 @@ RADR raddr;
 RADR ag;
 int  size;
 
+    regs->psw.IA &= ADDRESS_MAXWRAP(regs);
+
 #if defined(FEATURE_ESAME)
     if(!regs->psw.amode64 && !newregs->psw.amode64)
 #endif /*defined(FEATURE_ESAME)*/
@@ -561,12 +750,12 @@ int  size;
         TRACE_F1_PR *tte;
         size = sizeof(TRACE_F1_PR);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
         tte->format = TRACE_F1_PR_FMT;
         tte->pswkey = regs->psw.pkey | TRACE_F1_PR_FM2;
         STORE_HW(tte->newpasn, newregs->CR_LHL(4));
         STORE_FW(tte->retna, (newregs->psw.amode << 31)
-                                | newregs->psw.IA_L | newregs->psw.prob);
+                                | newregs->psw.IA_L | PROBSTATE(&newregs->psw));
         STORE_FW(tte->newia, (regs->psw.amode << 31)
                                  | regs->psw.IA_L);
     }
@@ -577,12 +766,12 @@ int  size;
         TRACE_F2_PR *tte;
         size = sizeof(TRACE_F2_PR);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
         tte->format = TRACE_F2_PR_FMT;
         tte->pswkey = regs->psw.pkey | TRACE_F2_PR_FM2;
         STORE_HW(tte->newpasn, newregs->CR_LHL(4));
         STORE_FW(tte->retna, (newregs->psw.amode << 31)
-                                | newregs->psw.IA_L | newregs->psw.prob);
+                                | newregs->psw.IA_L | PROBSTATE(&newregs->psw));
         STORE_FW(tte->newia, (regs->psw.amode << 31)
                                  | regs->psw.IA_L);
     }
@@ -592,12 +781,12 @@ int  size;
         TRACE_F3_PR *tte;
         size = sizeof(TRACE_F3_PR);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
         tte->format = TRACE_F3_PR_FMT;
         tte->pswkey = regs->psw.pkey | TRACE_F3_PR_FM2;
         STORE_HW(tte->newpasn, newregs->CR_LHL(4));
         STORE_FW(tte->retna, (newregs->psw.amode << 31)
-                                | newregs->psw.IA_L | newregs->psw.prob);
+                                | newregs->psw.IA_L | PROBSTATE(&newregs->psw));
         STORE_DW(tte->newia, regs->psw.IA_G);
     }
     else
@@ -606,11 +795,11 @@ int  size;
         TRACE_F4_PR *tte;
         size = sizeof(TRACE_F4_PR);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
         tte->format = TRACE_F4_PR_FMT;
         tte->pswkey = regs->psw.pkey | TRACE_F4_PR_FM2;
         STORE_HW(tte->newpasn, newregs->CR_LHL(4));
-        STORE_FW(tte->retna,  newregs->psw.IA_L | newregs->psw.prob);
+        STORE_FW(tte->retna,  newregs->psw.IA_L | PROBSTATE(&newregs->psw));
         STORE_FW(tte->newia, (regs->psw.amode << 31)
                                  | regs->psw.IA_L);
     }
@@ -620,11 +809,11 @@ int  size;
         TRACE_F5_PR *tte;
         size = sizeof(TRACE_F5_PR);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
         tte->format = TRACE_F5_PR_FMT;
         tte->pswkey = regs->psw.pkey | TRACE_F5_PR_FM2;
         STORE_HW(tte->newpasn, newregs->CR_LHL(4));
-        STORE_FW(tte->retna,  newregs->psw.IA_L | newregs->psw.prob);
+        STORE_FW(tte->retna,  newregs->psw.IA_L | PROBSTATE(&newregs->psw));
         STORE_FW(tte->newia, regs->psw.IA_L);
     }
     else
@@ -633,11 +822,11 @@ int  size;
         TRACE_F6_PR *tte;
         size = sizeof(TRACE_F6_PR);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
         tte->format = TRACE_F6_PR_FMT;
         tte->pswkey = regs->psw.pkey | TRACE_F6_PR_FM2;
         STORE_HW(tte->newpasn, newregs->CR_LHL(4));
-        STORE_FW(tte->retna,  newregs->psw.IA_L | newregs->psw.prob);
+        STORE_FW(tte->retna,  newregs->psw.IA_L | PROBSTATE(&newregs->psw));
         STORE_DW(tte->newia, regs->psw.IA_G);
     }
     else
@@ -646,11 +835,11 @@ int  size;
         TRACE_F7_PR *tte;
         size = sizeof(TRACE_F7_PR);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
         tte->format = TRACE_F7_PR_FMT;
         tte->pswkey = regs->psw.pkey | TRACE_F7_PR_FM2;
         STORE_HW(tte->newpasn, newregs->CR_LHL(4));
-        STORE_DW(tte->retna,  newregs->psw.IA_G | newregs->psw.prob);
+        STORE_DW(tte->retna,  newregs->psw.IA_G | PROBSTATE(&newregs->psw));
         STORE_FW(tte->newia, (regs->psw.amode << 31)
                                  | regs->psw.IA_L);
     }
@@ -660,11 +849,11 @@ int  size;
         TRACE_F8_PR *tte;
         size = sizeof(TRACE_F8_PR);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
         tte->format = TRACE_F8_PR_FMT;
         tte->pswkey = regs->psw.pkey | TRACE_F8_PR_FM2;
         STORE_HW(tte->newpasn, newregs->CR_LHL(4));
-        STORE_DW(tte->retna,  newregs->psw.IA_G | newregs->psw.prob);
+        STORE_DW(tte->retna,  newregs->psw.IA_G | PROBSTATE(&newregs->psw));
         STORE_FW(tte->newia, regs->psw.IA_L);
     }
     else
@@ -673,12 +862,12 @@ int  size;
         TRACE_F9_PR *tte;
         size = sizeof(TRACE_F9_PR);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
         tte->format = TRACE_F9_PR_FMT;
         tte->pswkey = regs->psw.pkey | TRACE_F9_PR_FM2;
         STORE_HW(tte->newpasn, newregs->CR_LHL(4));
-        STORE_DW(tte->retna,  newregs->psw.IA_G | newregs->psw.prob);
-        STORE_DW(tte->newia, regs->psw.IA_L);
+        STORE_DW(tte->retna,  newregs->psw.IA_G | PROBSTATE(&newregs->psw));
+        STORE_DW(tte->newia, regs->psw.IA_G);
     }
 #endif /*defined(FEATURE_ESAME)*/
 
@@ -687,11 +876,16 @@ int  size;
 } /* end function ARCH_DEP(trace_pr) */
 #endif /*defined(FEATURE_LINKAGE_STACK)*/
 
+#if defined(_MSVC_)
+  /* Workaround for "fatal error C1001: INTERNAL COMPILER ERROR" in MSVC */
+  #pragma optimize("",on)
+#endif /*defined(_MSVC_)*/
 
 /*-------------------------------------------------------------------*/
-/* Form implicit PT trace entry                                      */
+/* Form implicit PT/PTI trace entry                                  */
 /*                                                                   */
 /* Input:                                                            */
+/*      pti     1=PTI instruction, 0=PT instruction                  */
 /*      pasn    Primary address space number                         */
 /*      gpr2    Contents of PT second operand register               */
 /*      regs    Pointer to the CPU register context                  */
@@ -700,11 +894,12 @@ int  size;
 /*                                                                   */
 /*      This function does not return if a program check occurs.     */
 /*-------------------------------------------------------------------*/
-CREG ARCH_DEP(trace_pt) (U16 pasn, GREG gpr2, REGS *regs)
+CREG ARCH_DEP(trace_pt) (int pti, U16 pasn, GREG gpr2, REGS *regs)
 {
 RADR raddr;
 RADR ag;
 int  size;
+BYTE nbit = (pti ? 1 : 0);
 
 #if defined(FEATURE_ESAME)
     if(regs->psw.amode64 && gpr2 > 0xFFFFFFFFULL)
@@ -712,9 +907,9 @@ int  size;
         TRACE_F3_PT *tte;
         size = sizeof(TRACE_F3_PT);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
         tte->format = TRACE_F3_PT_FMT;
-        tte->pswkey = regs->psw.pkey | TRACE_F3_PT_FM2;
+        tte->pswkey = regs->psw.pkey | TRACE_F3_PT_FM2 | nbit;
         STORE_HW(tte->newpasn, pasn);
         STORE_DW(tte->r2, gpr2);
     }
@@ -724,9 +919,9 @@ int  size;
         TRACE_F2_PT *tte;
         size = sizeof(TRACE_F2_PT);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
         tte->format = TRACE_F2_PT_FMT;
-        tte->pswkey = regs->psw.pkey | TRACE_F2_PT_FM2;
+        tte->pswkey = regs->psw.pkey | TRACE_F2_PT_FM2 | nbit;
         STORE_HW(tte->newpasn, pasn);
         STORE_FW(tte->r2, gpr2 & 0xFFFFFFFF);
     }
@@ -736,9 +931,9 @@ int  size;
         TRACE_F1_PT *tte;
         size = sizeof(TRACE_F1_PT);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
         tte->format = TRACE_F1_PT_FMT;
-        tte->pswkey = regs->psw.pkey | TRACE_F1_PT_FM2;
+        tte->pswkey = regs->psw.pkey | TRACE_F1_PT_FM2 | nbit;
         STORE_HW(tte->newpasn, pasn);
         STORE_FW(tte->r2, gpr2 & 0xFFFFFFFF);
     }
@@ -767,6 +962,8 @@ RADR raddr;
 RADR ag;
 int  size;
 
+    regs->psw.IA &= ADDRESS_MAXWRAP(regs);
+
     if(!br)
     {
         if(!regs->psw.amode64)
@@ -774,7 +971,7 @@ int  size;
             TRACE_F1_MS *tte;
             size = sizeof(TRACE_F1_MS);
             raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-            tte = (void*)regs->mainstor + raddr;
+            tte = (void*)(regs->mainstor + raddr);
             tte->format = TRACE_F1_MS_FMT;
             tte->fmt2 = TRACE_F1_MS_FM2;
             STORE_HW(tte->resv, 0);
@@ -786,7 +983,7 @@ int  size;
             TRACE_F2_MS *tte;
             size = sizeof(TRACE_F2_MS);
             raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-            tte = (void*)regs->mainstor + raddr;
+            tte = (void*)(regs->mainstor + raddr);
             tte->format = TRACE_F2_MS_FMT;
             tte->fmt2 = TRACE_F2_MS_FM2;
             STORE_HW(tte->resv, 0);
@@ -797,7 +994,7 @@ int  size;
             TRACE_F3_MS *tte;
             size = sizeof(TRACE_F3_MS);
             raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-            tte = (void*)regs->mainstor + raddr;
+            tte = (void*)(regs->mainstor + raddr);
             tte->format = TRACE_F3_MS_FMT;
             tte->fmt2 = TRACE_F3_MS_FM2;
             STORE_HW(tte->resv, 0);
@@ -811,7 +1008,7 @@ int  size;
             TRACE_F1_MSB *tte;
             size = sizeof(TRACE_F1_MSB);
             raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-            tte = (void*)regs->mainstor + raddr;
+            tte = (void*)(regs->mainstor + raddr);
             tte->format = TRACE_F1_MSB_FMT;
             tte->fmt2 = TRACE_F1_MSB_FM2;
             STORE_HW(tte->resv, 0);
@@ -823,7 +1020,7 @@ int  size;
             TRACE_F2_MSB *tte;
             size = sizeof(TRACE_F2_MSB);
             raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-            tte = (void*)regs->mainstor + raddr;
+            tte = (void*)(regs->mainstor + raddr);
             tte->format = TRACE_F2_MSB_FMT;
             tte->fmt2 = TRACE_F2_MSB_FM2;
             STORE_HW(tte->resv, 0);
@@ -834,7 +1031,7 @@ int  size;
             TRACE_F3_MSB *tte;
             size = sizeof(TRACE_F3_MSB);
             raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-            tte = (void*)regs->mainstor + raddr;
+            tte = (void*)(regs->mainstor + raddr);
             tte->format = TRACE_F3_MSB_FMT;
             tte->fmt2 = TRACE_F3_MSB_FM2;
             STORE_HW(tte->resv, 0);
@@ -872,7 +1069,7 @@ U64  dreg;
         TRACE_F1_TR *tte;
         size = sizeof(TRACE_F1_TR);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
 
         /* Calculate the number of registers to be traced, minus 1 */
         n = ( r3 < r1 ) ? r3 + 16 - r1 : r3 - r1;
@@ -881,10 +1078,10 @@ U64  dreg;
         obtain_lock (&sysblk.todlock);
 
         /* Update the TOD clock */
-        update_TOD_clock();
+        update_tod_clock();
 
         /* Retrieve the TOD clock value and shift out the epoch */
-        dreg = (sysblk.todclk << 8) | regs->cpuad;
+        dreg = (TOD_CLOCK(regs) << 8) | regs->cpuad;
 
         /* Release the TOD clock update lock */
         release_lock (&sysblk.todlock);
@@ -939,7 +1136,7 @@ U64  dreg;
         TRACE_F2_TR *tte;
         size = sizeof(TRACE_F2_TR);
         raddr = ARCH_DEP(get_trace_entry) (&ag, size, regs);
-        tte = (void*)regs->mainstor + raddr;
+        tte = (void*)(regs->mainstor + raddr);
 
         /* Calculate the number of registers to be traced, minus 1 */
         n = ( r3 < r1 ) ? r3 + 16 - r1 : r3 - r1;
@@ -948,10 +1145,10 @@ U64  dreg;
         obtain_lock (&sysblk.todlock);
 
         /* Update the TOD clock */
-        update_TOD_clock();
+        update_tod_clock();
 
         /* Retrieve the TOD clock value including the epoch */
-        dreg = sysblk.todclk;
+        dreg = TOD_CLOCK(regs);
 
         /* Release the TOD clock update lock */
         release_lock (&sysblk.todlock);
