@@ -1,16 +1,26 @@
-/* VM.C         (c) Copyright Roger Bowler, 2000-2003                */
+/* VM.C         (c) Copyright Roger Bowler, 2000-2005                */
 /*              ESA/390 VM Diagnose calls and IUCV instruction       */
 
-/* Interpretive Execution - (c) Copyright Jan Jaeger, 1999-2003      */
+/* Interpretive Execution - (c) Copyright Jan Jaeger, 1999-2005      */
 
 /*-------------------------------------------------------------------*/
 /* This module implements miscellaneous diagnose functions           */
 /* described in SC24-5670 VM/ESA CP Programming Services             */
 /* and SC24-5855 VM/ESA CP Diagnosis Reference.                      */
 /*      Modifications for Interpretive Execution (SIE) by Jan Jaeger */
-/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2003      */
+/* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2005      */
 /*-------------------------------------------------------------------*/
 
+#include "hstdinc.h"
+
+#if !defined(_HENGINE_DLL_)
+#define _HENGINE_DLL_
+#endif
+
+#if !defined(_VM_C_)
+#define _VM_C_
+#endif
+
 #include "hercules.h"
 
 #include "opcode.h"
@@ -26,7 +36,6 @@
 /*-------------------------------------------------------------------*/
 /* Internal macro definitions                                        */
 /*-------------------------------------------------------------------*/
-#define SPACE   ((BYTE)' ')
 
 /*-------------------------------------------------------------------*/
 /* Synchronous Block I/O Parameter List                              */
@@ -173,6 +182,8 @@ BYTE            unitstat = 0;           /* Device status             */
 BYTE            chanstat = 0;           /* Subchannel status         */
 BYTE            skey1, skey2;           /* Storage keys of first and
                                            last byte of I/O buffer   */
+//FIXME: code not right for shared devices
+
     UNREFERENCED(r2);
 
     /* Register R1 contains the real address of the parameter list */
@@ -276,7 +287,7 @@ BYTE            skey1, skey2;           /* Storage keys of first and
 #endif /*FEATURE_CHANNEL_SUBSYSTEM*/
 
     /* Return code 5 and condition code 1 if device is busy */
-    if (dev->busy || dev->pending)
+    if (dev->busy || IOPENDING(dev))
     {
         release_lock (&dev->lock);
         regs->GR_L(15) = 5;
@@ -368,9 +379,7 @@ BYTE            skey1, skey2;           /* Storage keys of first and
     } /* end for(blkcount) */
 
     /* Reset the device busy indicator */
-    obtain_lock (&dev->lock);
     dev->busy = 0;
-    release_lock (&dev->lock);
 
     /* Store the block count in the parameter list */
     ioparm.blkcount[0] = (blkcount >> 24) & 0xFF;
@@ -431,6 +440,8 @@ BYTE            accum;                  /* Work area                 */
 BYTE            unitstat = 0;           /* Device status             */
 BYTE            chanstat = 0;           /* Subchannel status         */
 
+//FIXME: code not right for shared devices
+
     UNREFERENCED(r2);
 
     /* Register R1 contains the real address of the parameter list */
@@ -509,7 +520,7 @@ BYTE            chanstat = 0;           /* Subchannel status         */
         return 0;
     }
 
-    /* Obtain the device lock */
+    /* Obtain the interrupt lock */
     obtain_lock (&dev->lock);
 
 #ifdef FEATURE_CHANNEL_SUBSYSTEM
@@ -524,7 +535,7 @@ BYTE            chanstat = 0;           /* Subchannel status         */
 #endif /*FEATURE_CHANNEL_SUBSYSTEM*/
 
     /* Return code 5 and condition code 1 if device is busy */
-    if (dev->busy || dev->pending)
+    if (dev->busy || IOPENDING(dev))
     {
         release_lock (&dev->lock);
         regs->GR_L(15) = 5;
@@ -568,8 +579,7 @@ BYTE            chanstat = 0;           /* Subchannel status         */
 
     /* Clear the interrupt pending and device busy conditions */
     obtain_lock (&dev->lock);
-    dev->pending = 0;
-    dev->busy = 0;
+    dev->busy = dev->pending = 0;
     dev->scsw.flag2 = 0;
     dev->scsw.flag3 = 0;
     release_lock (&dev->lock);
@@ -619,14 +629,19 @@ BYTE            chanstat = 0;           /* Subchannel status         */
 /*-------------------------------------------------------------------*/
 void ARCH_DEP(extid_call) (int r1, int r2, REGS *regs)
 {
-int             i;                      /* Array subscript           */
-int             ver, rel;               /* Version and release number*/
-U32             idaddr;                 /* Address of storage operand*/
-U32             idlen;                  /* Length of storage operand */
-BYTE            buf[40];                /* Extended identification   */
-struct passwd  *ppwd;                   /* Pointer to passwd entry   */
-BYTE           *puser;                  /* Pointer to user name      */
-BYTE            c;                      /* Character work area       */
+int        i;                           /* Array subscript           */
+int        ver, rel;                    /* Version and release number*/
+U32        idaddr;                      /* Address of storage operand*/
+U32        idlen;                       /* Length of storage operand */
+BYTE       buf[40];                     /* Extended identification   */
+#if defined( HAVE_GETLOGIN_R )
+  #if !defined(LOGIN_NAME_MAX)  
+    #define LOGIN_NAME_MAX 100
+  #endif
+char       unam[LOGIN_NAME_MAX+1];      /* User name                 */
+#endif
+char      *puser;                       /* Pointer to user name      */
+BYTE       c;                           /* Character work area       */
 
     /* Load storage operand address from R1 register */
     idaddr = regs->GR_L(r1);
@@ -671,8 +686,13 @@ BYTE            c;                      /* Character work area       */
     buf[15] = regs->cpuad & 0xFF;
 
     /* Bytes 16-23 contain the userid in EBCDIC */
-    ppwd = getpwuid(getuid());
-    puser = (ppwd != NULL ? ppwd->pw_name : "");
+#if defined( HAVE_GETLOGIN_R )
+    memset( unam, 0, sizeof(unam) );
+    VERIFY( getlogin_r ( unam, sizeof(unam) ) == 0 );
+    puser = unam;
+#else
+    puser = "";
+#endif
     for (i = 0; i < 8; i++)
     {
         c = (*puser == '\0' ? SPACE : *(puser++));
@@ -715,7 +735,7 @@ BYTE            c;                      /* Character work area       */
         idlen = sizeof(buf);
 
     /* Store the extended identification code at operand address */
-    ARCH_DEP(vstorec) (buf, idlen-1, idaddr, r1, regs);
+    ARCH_DEP(vstorec) (buf, idlen-1, idaddr, USE_REAL_ADDR, regs);
 
     /* Deduct number of bytes from the R2 register */
     regs->GR_L(r2) -= idlen;
@@ -739,8 +759,11 @@ BYTE    cmdflags;                       /* Command flags             */
 #define CMDFLAGS_RESPONSE       0x40    /* Return response in buffer */
 #define CMDFLAGS_REQPASSW       0x20    /* Prompt for password       */
 #define CMDFLAGS_RESERVED       0x1F    /* Reserved bits, must be 0  */
-BYTE    buf[256];                       /* Command buffer (ASCIIZ)   */
-BYTE    resp[256];                      /* Response buffer (ASCIIZ)  */
+char    buf[256];                       /* Command buffer (ASCIIZ)   */
+char    resp[256];                      /* Response buffer (ASCIIZ)  */
+char    *dresp;                         /* Default response (ASCIIZ) */
+int     freeresp;                       /* Flag to free resp bfr     */
+int     j,k;
 
     /* Obtain command address from R1 register */
     cmdaddr = regs->GR_L(r1);
@@ -764,43 +787,95 @@ BYTE    resp[256];                      /* Response buffer (ASCIIZ)  */
     /* Put machine into stopped state if command length is zero */
     if (cmdlen == 0)
     {
+        regs->opinterv = 0;
         regs->cpustate = CPUSTATE_STOPPED;
-        ON_IC_CPU_NOT_STARTED(regs);
+        ON_IC_INTERRUPT(regs);
         return 0;
     }
 
     /* Obtain the command string from storage */
-    ARCH_DEP(vfetchc) (buf, cmdlen-1, cmdaddr, r1, regs);
+    ARCH_DEP(vfetchc) (buf, cmdlen-1, cmdaddr, USE_REAL_ADDR, regs);
 
-    /* Display the command on the console */
+    /* Translate EBCDIC command to ASCII */
     for (i = 0; i < cmdlen; i++)
+    {
         buf[i] = guest_to_host(buf[i]);
+    }
     buf[i] = '\0';
-    logmsg ("HHC660I %s\n", buf);
+    dresp="";
+    freeresp=0;
+
+    if(buf && *buf)
+    {
+#ifdef FEATURE_HERCULES_DIAGCALLS
+        int shcmd = 0;
+        {
+            char* p = buf;
+            while (*p && isspace(*p)) p++;
+            if ((*(p+0) == 's' || *(p+0) == 'S') &&
+                (*(p+1) == 'h' || *(p+1) == 'H') && 
+                isspace(*(p+2))) shcmd = 1;
+        }
+        if (sysblk.diag8cmd
+            && (!shcmd || !(sysblk.shcmdopt & (SHCMDOPT_DISABLE | SHCMDOPT_NODIAG8)))
+        )
+        {
+            logmsg (_("HHCVM001I *%s* panel command issued by guest\n"), buf);
+            if (cmdflags & CMDFLAGS_RESPONSE)
+            {
+                dresp=log_capture(panel_command,buf);
+                if(dresp!=NULL)
+                {
+                    freeresp=1;
+                }
+                else
+                {
+                    dresp="";
+                }
+            }
+            else
+            {
+                panel_command(buf);
+                logmsg (_("HHCVM002I *%s* command complete\n"), buf);
+            }
+        }
+        else
+            dresp=_("HHCVM003I Host command processing disabled by configuration statement");
+#else
+            dresp=_("HHCVM004E Host command processing not included in engine build");
+#endif
+    }
 
     /* Store the response and set length if response requested */
     if (cmdflags & CMDFLAGS_RESPONSE)
     {
-        strcpy (resp, "HHC661I Command complete");
-        resplen = strlen(resp);
+        if(!freeresp)
+        {
+                strncpy (resp, dresp,256);
+                dresp=resp;
+        }
+        resplen = strlen(dresp);
         for (i = 0; i < resplen; i++)
-            resp[i] = host_to_guest(resp[i]);
+            dresp[i] = host_to_guest(dresp[i]);
 
         respadr = regs->GR_L(r1+1);
         maxrlen = regs->GR_L(r2+1);
 
-        if (resplen <= maxrlen)
+        i=(resplen<=maxrlen) ? resplen : maxrlen;
+        j=0;
+        while(i>0)
         {
-            ARCH_DEP(vstorec) (resp, resplen-1, respadr, r1+1, regs);
-            regs->GR_L(r2+1) = resplen;
-            cc = 0;
-        }
-        else
-        {
-            ARCH_DEP(vstorec) (resp, maxrlen-1, respadr, r1+1, regs);
-            regs->GR_L(r2+1) = resplen - maxrlen;
-            cc = 1;
+            k=(i<255 ? i : 255);
+            ARCH_DEP(vstorec) (&dresp[j], k , respadr+j, USE_REAL_ADDR, regs);
+            i-=k;
+            j+=k;
         }
+        regs->GR_L(r2+1) = (resplen<=maxrlen) ? resplen : resplen-maxrlen;
+        cc = (resplen<=maxrlen) ? 0 : 1;
+    }
+    if(freeresp)
+    {
+        free(dresp);
     }
 
     /* Set R2 register to CP completion code */
@@ -853,18 +928,18 @@ time_t  timeval;                        /* Current time              */
 struct  tm *tmptr;                      /* -> Current time structure */
 U32     bufadr;                         /* Real addr of data buffer  */
 U32     buflen;                         /* Length of data buffer     */
-BYTE    buf[64];                        /* Response buffer           */
+char    buf[64];                        /* Response buffer           */
 BYTE    dattim[64];                     /* Date and time (EBCDIC)    */
 #define DIAG_DATEFMT_SHORT      0x80    /* Date format mm/dd/yy      */
 #define DIAG_DATEFMT_FULL       0x40    /* Date format mm/dd/yyyy    */
 #define DIAG_DATEFMT_ISO        0x20    /* Date format yyyy-mm-dd    */
 #define DIAG_DATEFMT_SYSDFLT    0x10    /* System-wide default format*/
-static  BYTE timefmt[]="%m/%d/%y%H:%M:%S%m/%d/%Y%Y-%m-%d";
+static  char timefmt[]="%m/%d/%y%H:%M:%S%m/%d/%Y%Y-%m-%d";
 
     /* Get the current date and time in EBCDIC */
     timeval = time(NULL);
     tmptr = localtime(&timeval);
-    strftime(dattim, sizeof(dattim), timefmt, tmptr);
+    strftime((char *)dattim, sizeof(dattim), timefmt, tmptr);
     for (i = 0; dattim[i] != '\0'; i++)
         dattim[i] = host_to_guest(dattim[i]);
 
@@ -1028,13 +1103,20 @@ DEF_INST(inter_user_communication_vehicle)
 int     b2;                             /* Effective addr base       */
 VADR    effective_addr2;                /* Effective address         */
 
-    S(inst, execflag, regs, b2, effective_addr2);
+    S(inst, regs, b2, effective_addr2);
+#if defined(FEATURE_ECPSVM)
+    if(ecpsvm_doiucv(regs,b2,effective_addr2)==0)
+    {
+        return;
+    }
+#endif
+
 
     /* Program check if in problem state,
        the IUCV instruction generates an operation exception
        rather then a priviliged operation exception when
        executed in problem state                                 *JJ */
-    if ( regs->psw.prob )
+    if ( PROBSTATE(&regs->psw) )
         ARCH_DEP(program_interrupt) (regs, PGM_OPERATION_EXCEPTION);
 
     SIE_INTERCEPT(regs);
