@@ -1,100 +1,41 @@
 ////////////////////////////////////////////////////////////////////////////////////
 //         w32chan.c           Fish's new i/o scheduling logic
 ////////////////////////////////////////////////////////////////////////////////////
-// (c) Copyright "Fish" (David B. Trout), 2001-2003. Released under the Q Public License
+// (c) Copyright "Fish" (David B. Trout), 2001-2005. Released under the Q Public License
 // (http://www.conmicro.cx/hercules/herclic.html) as modifications to Hercules.
 ////////////////////////////////////////////////////////////////////////////////////
 
-#if defined(HAVE_CONFIG_H)
-#include <config.h>     // (needed 1st to set OPTION_FTHREADS flag appropriately)
-#endif
+#include "hstdinc.h"
 
-#include "featall.h"    // (needed 2nd to set OPTION_FISHIO flag appropriately)
+#define _W32CHAN_C_
+#define _HENGINE_DLL_
 
-#if !defined(OPTION_FISHIO)
-int dummy = 0;
-#else // defined(OPTION_FISHIO)
+#include "hercules.h"
+#include "w32chan.h"
 
-#include <windows.h>    // (standard WIN32)
-#include <errno.h>      // (need "errno")
-#include <stdarg.h>     // (need "va_list", etc)
-#include <stdio.h>      // (need "FILE", "vfprintf", etc)
-#include <malloc.h>     // (need "malloc", etc)
-#include "fthreads.h"   // (need "fthread_create")
-#include "w32chan.h"    // (function prototypes for this module)
-#include "linklist.h"   // (linked list macros)
+#if defined(OPTION_FISHIO)
 
 /////////////////////////////////////////////////////////////////////////////
-// (helper macros...)
+// Internal scheduler helper macros...
 
 #if defined(FISH_HANG)
-
-    #include "fishhang.h"
-
-    #define MyInitializeCriticalSection(lock)       (FishHang_InitializeCriticalSection(__FILE__,__LINE__,(lock)))
-    #define MyDeleteCriticalSection(lock)           (FishHang_DeleteCriticalSection(__FILE__,__LINE__,(lock)))
-    #define MyCreateEvent(sec,man,set,name)         (FishHang_CreateEvent(__FILE__,__LINE__,(sec),(man),(set),(name)))
-    #define MySetEvent(handle)                      (FishHang_SetEvent(__FILE__,__LINE__,(handle)))
-    #define MyResetEvent(handle)                    (FishHang_ResetEvent(__FILE__,__LINE__,(handle)))
-    #define MyWaitForSingleObject(handle,millisecs) (FishHang_WaitForSingleObject(__FILE__,__LINE__,(handle),(millisecs)))
-    #define MyCloseHandle(handle)                   (FishHang_CloseHandle(__FILE__,__LINE__,(handle)))
     #define LockScheduler()                         (FishHang_EnterCriticalSection(__FILE__,__LINE__,&IOSchedulerLock))
     #define LockThreadParms(pThreadParms)           (FishHang_EnterCriticalSection(__FILE__,__LINE__,&pThreadParms->IORequestListLock))
     #define UnlockScheduler()                       (FishHang_LeaveCriticalSection(__FILE__,__LINE__,&IOSchedulerLock))
     #define UnlockThreadParms(pThreadParms)         (FishHang_LeaveCriticalSection(__FILE__,__LINE__,&pThreadParms->IORequestListLock))
-
-#else // !defined(FISH_HANG)
-
-    #define MyInitializeCriticalSection(lock)       (InitializeCriticalSection((lock)))
-    #define MyDeleteCriticalSection(lock)           (DeleteCriticalSection((lock)))
-    #define MyCreateEvent(sec,man,set,name)         (CreateEvent((sec),(man),(set),(name)))
-    #define MySetEvent(handle)                      (SetEvent((handle)))
-    #define MyResetEvent(handle)                    (ResetEvent((handle)))
-    #define MyWaitForSingleObject(handle,millisecs) (WaitForSingleObject((handle),(millisecs)))
-    #define MyCloseHandle(handle)                   (CloseHandle((handle)))
+#else
     #define LockScheduler()                         (EnterCriticalSection(&IOSchedulerLock))
     #define LockThreadParms(pThreadParms)           (EnterCriticalSection(&pThreadParms->IORequestListLock))
     #define UnlockScheduler()                       (LeaveCriticalSection(&IOSchedulerLock))
     #define UnlockThreadParms(pThreadParms)         (LeaveCriticalSection(&pThreadParms->IORequestListLock))
-
-#endif // defined(FISH_HANG)
-
-#define logmsg(fmt...)           \
-{                                \
-    fprintf(ios_msgpipew, fmt);  \
-    fflush(ios_msgpipew);        \
-}
-
-#define IsEventSet(hEventHandle) (WaitForSingleObject(hEventHandle,0) == WAIT_OBJECT_0)
-
-/////////////////////////////////////////////////////////////////////////////
-// Debugging
-
-#if defined(DEBUG) || defined(_DEBUG)
-    #define TRACE(a...) logmsg(a)
-    #define ASSERT(a) \
-        do \
-        { \
-            if (!(a)) \
-            { \
-                logmsg("** Assertion Failed: %s(%d)\n",__FILE__,__LINE__); \
-            } \
-        } \
-        while(0)
-    #define VERIFY(a) ASSERT((a))
-#else
-    #define TRACE(a...)
-    #define ASSERT(a)
-    #define VERIFY(a) ((void)(a))
 #endif
 
 /////////////////////////////////////////////////////////////////////////////
 // i/o scheduler variables...  (some private, some externally visible)
 
-FILE*  ios_msgpipew           = NULL;
-int    ios_arch_mode          = 0;
-int    ios_devthread_priority = THREAD_PRIORITY_NORMAL;
-int    ios_devthread_timeout  = 30;
+int    ios_arch_mode          = 0;          // current architecture mode
+int    ios_devthread_timeout  = 30;         // max device thread wait time
+int*   ios_devthread_prio     = NULL;       // pointer to sysblk.devprio
 
 LIST_ENTRY        ThreadListHeadListEntry;  // anchor for DEVTHREADPARMS linked list
 CRITICAL_SECTION  IOSchedulerLock;          // lock for accessing above list
@@ -111,16 +52,14 @@ int  ios_devtunavail = 0;   // #of times 'idle' thread unavailable
 
 void  InitIOScheduler
 (
-    FILE*  msgpipew,        // (for issuing msgs to Herc console)
     int    arch_mode,       // (for calling execute_ccw_chain)
-    int    devt_priority,   // (for calling fthread_create)
+    int*   devt_prio,       // (ptr to device thread priority)
     int    devt_timeout,    // (MAX_DEVICE_THREAD_IDLE_SECS)
     long   devt_max         // (maximum #of device threads allowed)
 )
 {
-    ios_msgpipew           = msgpipew;
     ios_arch_mode          = arch_mode;
-    ios_devthread_priority = devt_priority;
+    ios_devthread_prio     = devt_prio;
     ios_devthread_timeout  = devt_timeout;
     ios_devtmax            = devt_max;
 
@@ -150,6 +89,7 @@ typedef struct _DevIORequest
 {
     LIST_ENTRY      IORequestListLinkingListEntry;  // (just a link in the chain)
     void*           pDevBlk;                        // (ptr to device block)
+    int*            pnDevPrio;                      // (ptr to device i/o priority)
     unsigned short  wDevNum;                        // (device number for debugging)
 }
 DEVIOREQUEST;
@@ -160,6 +100,7 @@ DEVIOREQUEST;
 DEVTHREADPARMS*  SelectDeviceThread();
 DEVTHREADPARMS*  CreateDeviceThread(unsigned short wDevNum);
 
+void   AdjustThreadPriority(int* pCurPrio, int* pNewPrio);
 void*  DeviceThread(void* pThreadParms);
 void   RemoveDeadThreadsFromList();
 void   RemoveThisThreadFromOurList(DEVTHREADPARMS* pThreadParms);
@@ -168,7 +109,7 @@ void   RemoveThisThreadFromOurList(DEVTHREADPARMS* pThreadParms);
 // Schedule a DeviceThread for this i/o request... (called by the 'startio'
 // function whenever the startio or start subchannel instruction is executed)
 
-int  ScheduleIORequest(void* pDevBlk, unsigned short wDevNum)
+int  ScheduleIORequest(void* pDevBlk, unsigned short wDevNum, int* pnDevPrio)
 {
     /////////////////////////////////////////////////////////////////
     // PROGRAMMING NOTE: The various errors that can occur in this
@@ -189,7 +130,7 @@ int  ScheduleIORequest(void* pDevBlk, unsigned short wDevNum)
 
     if (!pIORequest)
     {
-        logmsg("HHC762I malloc(DEVIOREQUEST) failed; device=%4.4X, strerror=\"%s\"\n",
+        logmsg(_("HHCCP084E malloc(DEVIOREQUEST) failed; device=%4.4X, strerror=\"%s\"\n"),
             wDevNum,strerror(errno));
         return 2;
     }
@@ -197,6 +138,7 @@ int  ScheduleIORequest(void* pDevBlk, unsigned short wDevNum)
     InitializeListLink(&pIORequest->IORequestListLinkingListEntry);
     pIORequest->pDevBlk = pDevBlk;
     pIORequest->wDevNum = wDevNum;
+    pIORequest->pnDevPrio = pnDevPrio;
 
     // Schedule a device_thread to process this i/o request
 
@@ -251,7 +193,7 @@ int  ScheduleIORequest(void* pDevBlk, unsigned short wDevNum)
 
             if (ios_devtmax && ios_devtnbr >= ios_devtmax)  // max threads already created?
             {
-                logmsg("HHC765I *WARNING* max device threads exceeded.\n");
+                logmsg(_("HHCCP085W *WARNING* max device threads exceeded.\n"));
                 ios_devtunavail++;          // (count occurrences)
             }
 
@@ -348,7 +290,7 @@ DEVTHREADPARMS*  CreateDeviceThread(unsigned short wDevNum)
 
     if (!pThreadParms)
     {
-        logmsg("HHC761I malloc(DEVTHREADPARMS) failed; device=%4.4X, strerror=\"%s\"\n",
+        logmsg(_("HHCCP086E malloc(DEVTHREADPARMS) failed; device=%4.4X, strerror=\"%s\"\n"),
             wDevNum,strerror(errno));
         return NULL;    // (error)
     }
@@ -357,7 +299,7 @@ DEVTHREADPARMS*  CreateDeviceThread(unsigned short wDevNum)
 
     if (!pThreadParms->hShutdownEvent)
     {
-        logmsg("HHC763I CreateEvent(hShutdownEvent) failed; device=%4.4X, strerror=\"%s\"\n",
+        logmsg(_("HHCCP087E CreateEvent(hShutdownEvent) failed; device=%4.4X, strerror=\"%s\"\n"),
             wDevNum,strerror(errno));
         free(pThreadParms);
         return NULL;    // (error)
@@ -367,7 +309,7 @@ DEVTHREADPARMS*  CreateDeviceThread(unsigned short wDevNum)
 
     if (!pThreadParms->hRequestQueuedEvent)
     {
-        logmsg("HHC764I CreateEvent(hRequestQueuedEvent) failed; device=%4.4X, strerror=\"%s\"\n",
+        logmsg(_("HHCCP088E CreateEvent(hRequestQueuedEvent) failed; device=%4.4X, strerror=\"%s\"\n"),
             wDevNum,strerror(errno));
         MyCloseHandle(pThreadParms->hShutdownEvent);
         free(pThreadParms);
@@ -383,12 +325,12 @@ DEVTHREADPARMS*  CreateDeviceThread(unsigned short wDevNum)
     pThreadParms->dwThreadID = 0;
 
 #ifdef FISH_HANG
-    if (fthread_create(__FILE__,__LINE__,&dwThreadID,NULL,DeviceThread,pThreadParms,ios_devthread_priority) != 0)
+    if (fthread_create(__FILE__,__LINE__,&dwThreadID,NULL,DeviceThread,pThreadParms) != 0)
 #else
-    if (fthread_create(&dwThreadID,NULL,DeviceThread,pThreadParms,ios_devthread_priority) != 0)
+    if (fthread_create(&dwThreadID,NULL,DeviceThread,pThreadParms) != 0)
 #endif
     {
-        logmsg("HHC760I fthread_create(DeviceThread) failed; device=%4.4X, strerror=\"%s\"\n",
+        logmsg(_("HHCCP089E fthread_create(DeviceThread) failed; device=%4.4X, strerror=\"%s\"\n"),
             wDevNum,strerror(errno));
         MyCloseHandle(pThreadParms->hShutdownEvent);
         MyCloseHandle(pThreadParms->hRequestQueuedEvent);
@@ -408,6 +350,18 @@ DEVTHREADPARMS*  CreateDeviceThread(unsigned short wDevNum)
     return pThreadParms;            // (success)
 }
 
+/////////////////////////////////////////////////////////////////////////////
+//  helper function to set a thread's priority to its proper value
+
+void AdjustThreadPriority(int* pCurPrio, int* pNewPrio)
+{
+    if (*pCurPrio != *pNewPrio)
+    {
+        setpriority(PRIO_PROCESS, 0, *pNewPrio);
+        *pCurPrio = *pNewPrio;
+    }
+}
+
 /////////////////////////////////////////////////////////////////////////////
 // the device_thread itself...  (processes queued i/o request
 // by calling "execute_ccw_chain" function in channel.c...)
@@ -420,11 +374,16 @@ void*  DeviceThread (void* pArg)
     LIST_ENTRY*      pListEntry;        // (work)
     DEVIOREQUEST*    pIORequest;        // ptr to i/o request
     void*            pDevBlk;           // ptr to device block
+    int*             pnDevPrio;         // ptr to device i/o priority
+    int              nCurPrio;          // current thread priority
 
     pThreadParms = (DEVTHREADPARMS*) pArg;
 
     pThreadParms->dwThreadID = GetCurrentThreadId();
 
+    nCurPrio = getpriority(PRIO_PROCESS, 0);
+    AdjustThreadPriority(&nCurPrio,ios_devthread_prio);
+
     for (;;)
     {
         // Wait for an i/o request to be queued...
@@ -433,6 +392,7 @@ void*  DeviceThread (void* pArg)
         MyWaitForSingleObject(pThreadParms->hRequestQueuedEvent,ios_devthread_timeout * 1000);
         InterlockedDecrement(&ios_devtwait);
 
+
         if (IsEventSet(pThreadParms->hShutdownEvent)) break;
 
         // Lock our queue so it doesn't change while we take a look at it...
@@ -461,14 +421,23 @@ void*  DeviceThread (void* pArg)
         UnlockThreadParms(pThreadParms);    // (done with thread parms for now)
 
         pIORequest = CONTAINING_RECORD(pListEntry,DEVIOREQUEST,IORequestListLinkingListEntry);
-        pDevBlk = pIORequest->pDevBlk;      // (this is all we need)
+        pDevBlk   = pIORequest->pDevBlk;    // (need ptr to devblk)
+        pnDevPrio = pIORequest->pnDevPrio;  // (need ptr to devprio)
         free(pIORequest);                   // (not needed anymore)
 
         // Process the i/o request by calling the proper 'execute_ccw_chain'
         // function (based on architectural mode) in source module channel.c
 
+        // Set thread priority to requested device level
+        AdjustThreadPriority(&nCurPrio,pnDevPrio);
+
         call_execute_ccw_chain(ios_arch_mode, pDevBlk); // (process i/o request)
 
+        // Reset thread priority, if necessary
+        if (nCurPrio > *ios_devthread_prio)
+            AdjustThreadPriority(&nCurPrio,ios_devthread_prio);
+
+
         ////////////////////////////////////////////////////////////////////////////
         //
         //                  * * *   I M P O R T A N T   * * *
@@ -699,7 +668,7 @@ char*  PrintDEVIOREQUEST(DEVIOREQUEST* pIORequest, DEVTHREADPARMS* pDEVTHREADPAR
     }
     else pNextDEVIOREQUEST = (DEVIOREQUEST*) &pDEVTHREADPARMS->IORequestListHeadListEntry;
 
-    sprintf(PrintDEVIOREQUESTBuffer,
+    snprintf(PrintDEVIOREQUESTBuffer,sizeof(PrintDEVIOREQUESTBuffer),
         "DEVIOREQUEST @ %8.8X\n"
         "               pDevBlk                       = %8.8X\n"
         "               wDevNum                       = %4.4X\n"
@@ -710,6 +679,8 @@ char*  PrintDEVIOREQUEST(DEVIOREQUEST* pIORequest, DEVTHREADPARMS* pDEVTHREADPAR
             (int)pNextDEVIOREQUEST
         );
 
+    PrintDEVIOREQUESTBuffer[ sizeof(PrintDEVIOREQUESTBuffer) - 1 ] = 0;
+
     return PrintDEVIOREQUESTBuffer;
 }
 
@@ -726,7 +697,7 @@ void  PrintAllDEVIOREQUESTs(DEVTHREADPARMS* pDEVTHREADPARMS)
     {
         pDEVIOREQUEST = CONTAINING_RECORD(pListEntry,DEVIOREQUEST,IORequestListLinkingListEntry);
         pListEntry = pListEntry->Flink;
-        fprintf(stdout,"%s\n",PrintDEVIOREQUEST(pDEVIOREQUEST,pDEVTHREADPARMS));
+        FishHang_Printf("%s\n",PrintDEVIOREQUEST(pDEVIOREQUEST,pDEVTHREADPARMS));
     }
 }
 
@@ -756,7 +727,7 @@ char*  PrintDEVTHREADPARMS(DEVTHREADPARMS* pDEVTHREADPARMS)
     }
     else pNextDEVTHREADPARMS = (DEVTHREADPARMS*) &ThreadListHeadListEntry;
 
-    sprintf(PrintDEVTHREADPARMSBuffer,
+    snprintf(PrintDEVTHREADPARMSBuffer,sizeof(PrintDEVTHREADPARMSBuffer),
 
         "DEVTHREADPARMS @ %8.8X\n"
         "                 dwThreadID                 = %8.8X\n"
@@ -777,6 +748,8 @@ char*  PrintDEVTHREADPARMS(DEVTHREADPARMS* pDEVTHREADPARMS)
             ,(int)pNextDEVTHREADPARMS
         );
 
+    PrintDEVTHREADPARMSBuffer[ sizeof(PrintDEVTHREADPARMSBuffer) - 1 ] = 0;
+
     return PrintDEVTHREADPARMSBuffer;
 }
 
@@ -798,7 +771,7 @@ void  PrintAllDEVTHREADPARMSs()
     }
     else pDEVTHREADPARMS = (DEVTHREADPARMS*) &ThreadListHeadListEntry;
 
-    fprintf(stdout,"\nDEVTHREADPARMS LIST ANCHOR @ %8.8X --> %8.8X\n\n",
+    FishHang_Printf("\nDEVTHREADPARMS LIST ANCHOR @ %8.8X --> %8.8X\n\n",
         (int)&ThreadListHeadListEntry,(int)pDEVTHREADPARMS);
 
     while (pListEntry != &ThreadListHeadListEntry)
@@ -807,7 +780,7 @@ void  PrintAllDEVTHREADPARMSs()
 
         LockThreadParms(pDEVTHREADPARMS);
 
-        fprintf(stdout,"%s\n",PrintDEVTHREADPARMS(pDEVTHREADPARMS));
+        FishHang_Printf("%s\n",PrintDEVTHREADPARMS(pDEVTHREADPARMS));
         PrintAllDEVIOREQUESTs(pDEVTHREADPARMS);
 
         UnlockThreadParms(pDEVTHREADPARMS);
